/******************************************************************************
*                                                                             *
*   Name:  IDMSAMP.C                                                          *
*                                                                             *
*   Copyright : COPYRIGHT IBM CORPORATION, 1995                               *
*               LICENSED MATERIAL - PROGRAM PROPERTY OF IBM                   *
*                                                                             *
*   Description: This program lets user to get the icons from icon files      *
*                first, and then when he select an icon, the name of icon     *
*                is given in animation. He can write the name and the program *
*                checks whether the user written name is correct or not.      *
*                                                                             *
*   Execution Instruction: IDMSAMP                                            *
*                                                                             *
*   Software Requirement: IBM OS/2 Warp, Pen for OS/2 V1.1                    *
*                                                                             *
*   DISCLAIMER OF WARRANTIES.  The following [enclosed] code is               *
*      sample code created by IBM Corporation. This sample code is not        *
*      part of any standard or IBM product and is provided to you solely      *
*      for  the purpose of assisting you in the development of your           *
*      applications.  The code is provided "AS IS", without                   *
*      warranty of any kind.  IBM shall not be liable for any damages         *
*      arising out of your use of the sample code, even if they have been     *
*      advised of the possibility of such damages.                            *
*                                                                             *
*   ATTENTION                                                                 *
*      For each SOM method call, the returned code checking is strongly       *
*      recommended. The result can be checked as follow:                      *
*                                                                             *
*          if (lev._major != NO_EXCEPTION)                                    *
*          {                                                                  *
*             rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;       *
*             Check the return code here                                      *
*             somExceptionFree(&lev);                                         *
*          }                                                                  *
*                                                                             *
*      In this sample program, there are lots of SOM method calls and to      *
*      reduce the size of program, all SOM method calls do not have the       *
*      checking.                                                              *
*                                                                             *
*      In Presentation program, it is not good idea to use DosSleep. In this  *
*      sample program, the Dos API is used only for synchronization purpose.  *
******************************************************************************/

#define  INCL_PM
#define  INCL_DOS
#define  INCL_GPI
#define  INCL_WIN
#define  INCL_DOSMEMMGR
#define  INCL_DOSSEMAPHORES
#define  INCL_WINMESSAGEMGR
#define  SOMF_NIL 0

#include <os2.h>
#include <pmwp.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <som.h>

#define  INCL_IDM
#define  INCL_IDO
#include <penpm.h>
#include "idmsamp.h"

/**************************************************************************
*                                                                         *
*  Name       : main                                                      *
*                                                                         *
*  Description: Main program of idmsamp                                   *
*                                                                         *
**************************************************************************/
int main( USHORT argCnt, PCHAR argVals[] )
{
   HMQ   hmq;
   QMSG  qmsg;
   RECTL rectl;
   LONG lHeight, lWidth, lx, ly;
   ULONG flFlags = FCF_TITLEBAR   | FCF_SIZEBORDER  |
                   FCF_MINMAX     | FCF_SHELLPOSITION |
                   FCF_TASKLIST   | FCF_MENU | FCF_ICON;

   /**********************************************************
   *  Initialize SOM local environment and global variables. *
   **********************************************************/
   SOM_InitEnvironment( &lev );

   lBoardColor = CLR_DARKGRAY;
   lBoardCheck = ID_BOARD_DARKGRAY;

   lAnimateColor = CLR_DARKRED;
   lAnimateCheck = ID_ANIMATE_DARKRED;

   lDrawColor = CLR_DARKCYAN;
   lDrawCheck = ID_DRAW_DARKCYAN;

   lDrawInkColor = CLR_BLACK;
   lDrawInkCheck = ID_DRAW_INK_BLACK;

   lAnimateInkCheck = ID_ANIMATE_INK_WHITE;
   lAnimateInkColor = CLR_WHITE;

   display_string[0] = 0;
   hab = WinInitialize(0);
   hmq = WinCreateMsgQueue(hab, 0);

   /*************************************************************
   *  Register classes for client, animation, and write windows *
   **************************************************************/
   WinRegisterClass(hab, szAPPNAME,     ClientWndProc,  0L, 0);
   WinRegisterClass(hab, szANIMATENAME, AnimateWndProc, 0L, 0);
   WinRegisterClass(hab, szWRITENAME,   WriteWndProc,   0L, 0);

   hwndFrame = WinCreateStdWindow(HWND_DESKTOP,
                                  0L          ,
                                  &flFlags    ,
                                  szAPPNAME   ,
                                  szAPPNAME   ,
                                  0L          ,
                                  hNULL       ,
                                  ID_MAINWND  ,
                                  &hwndClient );
   if (hwndFrame)
   {

      /* position, size, and make the window visible */
      WinQueryWindowRect(HWND_DESKTOP, &rectl);

      /* Window Size and Height will be 3/4s of the desktops width */
      lWidth = (rectl.xRight * 3) / 4;
      lHeight = (rectl.yTop * 3) / 4;

      /* position 1/8 right and 1/8 up */
      lx = (rectl.xRight / 8);
      ly = (rectl.yTop / 8);

      WinSetWindowPos(hwndFrame, HWND_TOP, lx, ly,
                      lWidth, lHeight,                      /* size parms */
                      SWP_ZORDER | SWP_MOVE | SWP_SIZE |
                      SWP_SHOW | SWP_ACTIVATE );

      while( WinGetMsg( hab, (PQMSG) &qmsg, hNULL, 0L, 0L ) )
      {
         WinDispatchMsg( hab, (PQMSG) &qmsg );
      }
      DosCloseMutexSem(hMtx);
   }

   /* Clean up my private environment */
   SOM_UninitEnvironment( &lev );

   WinDestroyWindow( hwndFrame );

   WinDestroyMsgQueue( hmq );
   WinTerminate( hab );

   if (hfileDoodleFile)
       DosClose (hfileDoodleFile);

   return (0);
}  /* of main */

/**************************************************************************
*                                                                         *
*  Name       : ClineWndProc                                              *
*                                                                         *
*  Description: Client window procedure.                                  *
*                                                                         *
*               This window creates three windows while processing the    *
*               WM_CREATE message. The three windows are container window *
*               for icons, animate window, and write window.              *
*                                                                         *
**************************************************************************/
MRESULT EXPENTRY ClientWndProc(HWND hwnd, ULONG  msg, MPARAM mp1, MPARAM mp2)
{
HPS      hps;
RECTL    rcl;
ULONG    src, srcm;
APIRET   apiret;
SWP      swpTop, swpBot;
CHAR     err[60];
SIZEL    sizlPage;
SHORT    sHeight, sWidth;
static ULONG ulAnimationCheck;
LONG     rc;
ULONG    ulAction;
PSZ      pszShowMe = "Show Me";
PSZ      pszTryIt =  "Try It ";

    switch( msg )
    {
       case WM_CREATE:
       {
       CNRINFO CnrInfo;
       SIZEL   sizlPage;
       ULONG   flFlags = FCF_BORDER;

           /**********************************************************************/
           /*  Check to make sure Pen for OS/2 is up before we do anything else. */
           /**********************************************************************/
           if( WrtWaitActive( WRT_IMMEDIATE_RETURN ) )
           {
              /******************************************************************/
              /* If Pen for OS/2 is NOT active then put up a message box and    */
              /* terminate the program.                                         */
              /******************************************************************/
              ShowMessageBox(hwnd, ID_MSG_NO_PEN);
              WinPostMsg( hwnd, WM_CLOSE, NULL, NULL );
              return((MRESULT) TRUE);
           }

           /*************************************************************
           *  Create the icon container window.                         *
           *************************************************************/
           hwndCntr = WinCreateWindow(hwnd, WC_CONTAINER, "Icon List",
                        WS_VISIBLE | CCS_AUTOPOSITION | CCS_READONLY | CCS_SINGLESEL,
                        0, 0, 0, 0,
                        hwnd, HWND_TOP, ID_ICONS, NULL, (PVOID) NULL );
           CnrInfo.cb            = sizeof( CNRINFO );
           CnrInfo.flWindowAttr  = CV_ICON;
           WinSendMsg(hwndCntr, CM_SETCNRINFO,
                      MPFROMP( &CnrInfo ), MPFROMLONG( CMA_FLWINDOWATTR ) );

           /*************************************************************
           *  Create the animation window.                              *
           *************************************************************/
           hwndAnimateFrame = WinCreateStdWindow(hwnd, WS_VISIBLE, &flFlags,
                                             szANIMATENAME, szANIMATENAME,
                                             0L, hNULL, 0, &hwndAnimateClient );

           /*************************************************************
           *  Create the write window                                   *
           *************************************************************/
           hwndWriteFrame = WinCreateStdWindow(hwnd, WS_VISIBLE, &flFlags,
                                           szWRITENAME, szWRITENAME, 0L,
                                           hNULL, 0, &hwndWriteClient );

           pobjDoodle    = IDODoodleNew();
           pobjAllDoodle = IDODoodleNew();
           if (!pobjDoodle || !pobjAllDoodle)
           {
              ShowMessageBox(hwnd, ID_MSG_SOM_CREATION_FAIL);
              WinPostMsg (hwnd, WM_QUIT, 0, 0);
              return (MRESULT) TRUE;
           }
           /*************************************************************
           *  Initialize the client window and set animation speed.     *
           *************************************************************/
           WinPostMsg (hwnd, WM_INITMYWINDOW, 0, 0);
           WinPostMsg (hwnd, WM_COMMAND, (PVOID) ID_ANIM_SPEED1, 0);

           /*****************************************************/
           /*  Open file                                        */
           /*****************************************************/
           rc =  DosOpen(DOODLE_FILE,
                         &hfileDoodleFile,
                         &ulAction,
                         0,
                         FILE_NORMAL,
                         FILE_OPEN,
                         (ULONG) OPEN_ACCESS_READONLY |
                         OPEN_SHARE_DENYREADWRITE | OPEN_FLAGS_FAIL_ON_ERROR,
                         (PEAOP2) NULL);
           if (rc)
           {
               ShowMessageBox(hwnd, ID_MSG_OPEN_FILE_FAIL);
               WinPostMsg (hwnd, WM_QUIT, 0, 0);
               return (MRESULT) TRUE;
           }

           WinPostMsg (hwnd, WM_COMMAND, (PVOID) IDM_OPEN, 0);
           return (MRESULT)FALSE;
       }  /* of WM_CREATE */

       case WM_INITMYWINDOW:
       {
       IDR_SYMSET   idrSymset;

          hwndMenu = WinWindowFromID(hwndFrame, FID_MENU);

          CheckIt(lBoardCheck);
          CheckIt(lAnimateCheck);
          CheckIt(lDrawCheck);
          CheckIt(lDrawInkCheck);
          CheckIt(lAnimateInkCheck);

          objIDM = IDMObjectNew();
          if (!objIDM)
          {
              ShowMessageBox(hwnd, ID_MSG_SOM_CREATION_FAIL);
              WinPostMsg (hwnd, WM_QUIT, 0, 0);
              return (MRESULT) TRUE;
          }

          _idmSetOwner(objIDM, &lev, hwndWriteClient);
          if (lev._major != NO_EXCEPTION)
          {
             rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
             somExceptionFree(&lev);
          }

          _idmSetHps(objIDM, &lev, hpsWrite);
          if (lev._major != NO_EXCEPTION)
          {
             rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
             somExceptionFree(&lev);
          }

          draw_ink_color (CLR_WHITE);

          _idmSetInkingStyle(objIDM, &lev, IDM_INK_WINDOW);
          if (lev._major != NO_EXCEPTION)
          {
             rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
             somExceptionFree(&lev);
          }

          _idmSetIDMOp(objIDM, &lev, IDM_TEXT);
          if (lev._major != NO_EXCEPTION)
          {
             rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
             somExceptionFree(&lev);
          }

          /************************************************************
          *  Limit the symbol set to alphabet hi and digits           *
          ************************************************************/
          idrSymset.cbSize = sizeof(IDR_SYMSET);
          idrSymset.ulSetMask = IDR_SYMSET_ALPHAHI | IDR_SYMSET_DIGIT;
          idrSymset.pSyms = NULL;
          idrSymset.ulSymCnt = 0;
          _idmSetTextSymbolSubset(objIDM, &lev, &idrSymset);
          if (lev._major != NO_EXCEPTION)
          {
             rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
             somExceptionFree(&lev);
          }

          hpointerPen = WinLoadPointer(HWND_DESKTOP, NULLHANDLE, ID_BLACKPEN_PTR);
          _idmSetWritingPtr (objIDM, &lev, hpointerPen);
          if (lev._major != NO_EXCEPTION)
          {
             rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
             somExceptionFree(&lev);
          }

          WrtSetSysValue( HWND_DESKTOP, PPMSV_GESTURE_TIMEOUT, 5000L);

          hbitSmile = GpiLoadBitmap(hpsWrite, NULLHANDLE, ID_SMILE, 32, 32);
          if (!hbitSmile)
          {
             ShowMessageBox(hwnd, ID_MSG_BAD_BITMAP_FILE);
          }
       }
       return (MRESULT) 0;

       case WM_SIZE:
       {
       SWP  swp;
       int  y_32nd;
       int  x_16th;
       int  draw_yBottom, draw_yHeight;
       int  animate_yBottom, animate_yHeight;
       int  xWidth;

          WinQueryWindowPos(hwnd, &swp);

          /*
           *  Size the container to fill the upper half of the client window
           */
          WinSetWindowPos(hwndCntr, HWND_TOP,
                          10, swp.cy/2, swp.cx - 20, (swp.cy/2)- 10,
                          SWP_SHOW | SWP_MOVE | SWP_SIZE | SWP_ZORDER );
          x_16th  = (int) ((double) swp.cx / 16.0);
          y_32nd = (int) ((double) swp.cy / 32.0);

          draw_yBottom = y_32nd;
          draw_yHeight = y_32nd * 7;
          animate_yBottom = draw_yBottom + draw_yHeight + (y_32nd * 2);
          animate_yHeight = (y_32nd * 4);

          xWidth = x_16th * 14;
          WinSetWindowPos(hwndAnimateFrame, HWND_TOP,
                          x_16th, animate_yBottom, xWidth, animate_yHeight,
                          SWP_SHOW | SWP_MOVE | SWP_SIZE | SWP_ZORDER );
          WinSetWindowPos(hwndWriteFrame, HWND_TOP,
                          x_16th, draw_yBottom, xWidth, draw_yHeight,
                          SWP_SHOW | SWP_MOVE | SWP_SIZE | SWP_ZORDER );
          WinPostMsg ( hwnd, ID_REPAINT, 0, 0);
          return (MRESULT)TRUE;
       }  /* of WM_SIZE */

       case WM_PAINT:
       {
           hps = WinBeginPaint(hwnd, hNULL, &rcl);
           WinQueryWindowRect(hwnd, &rcl);
           WinFillRect(hps, &rcl, lBoardColor);

           WinQueryWindowPos (hwndCntr, &swpTop);
           WinQueryWindowPos (hwndAnimateFrame, &swpBot);

           sHeight = swpTop.y - (swpBot.y + swpBot.cy);
           sWidth =  swpBot.cx;

           rcl.xLeft = swpBot.x;
           rcl.xRight = swpBot.x + sWidth;
           rcl.yTop = swpTop.y - (0.1 * sHeight);
           rcl.yBottom = (swpBot.y + swpBot.cy);

           WinDrawText (hps, strlen(pszShowMe), pszShowMe, &rcl,
                contrast (lBoardColor), lBoardColor, DT_LEFT);

           memcpy (&swpTop, &swpBot, sizeof (swpTop));
           WinQueryWindowPos (hwndWriteFrame, &swpBot);

           sHeight = swpTop.y - (swpBot.y + swpBot.cy);
           sWidth =  swpBot.cx;

           rcl.xLeft = swpBot.x;
           rcl.xRight = swpBot.x + sWidth;
           rcl.yTop = swpTop.y - (0.1 * sHeight);
           rcl.yBottom = (swpBot.y + swpBot.cy) + (0.1 * sHeight);

           WinDrawText (hps, strlen(pszTryIt), pszTryIt, &rcl,
                contrast (lBoardColor), lBoardColor, DT_LEFT);

           WinEndPaint(hps);

           if ((display_string[0]) && !fOutS)
           {
              fOutS = TRUE;
              out_string(hwnd, display_string, ulAnimationSpeed);
              fOutS = FALSE;
           }

           break;
       }  /* of WM_PAINT */

       case WM_COMMAND:
       {
       SHORT what = SHORT1FROMMP (mp1);

          switch( what )
          {
             case IDM_OPEN:
                 FileOpen( hwnd, mp2 );
                 break;

             case ID_ANIM_SPEED1:
             case ID_ANIM_SPEED2:
                 UnCheckIt (ulAnimationCheck);
                 if (what == ID_ANIM_SPEED1)
                 {
                    ulAnimationSpeed = SPEED_FAST;
                 }
                 else
                 {
                    ulAnimationSpeed = SPEED_MED;
                 }
                 ulAnimationCheck = what;
                 CheckIt (ulAnimationCheck);
                 break;

             case ID_QUIT:
                 WinPostMsg (hwnd, WM_QUIT, 0, 0);

             default:  /* process all color related menu items */
                 return menu_cmd(hwnd, msg, mp1, mp2);
          }
          return (MRESULT) TRUE;
       }  /* of case WM_COMMAND */

       case WM_CONTROL:
       {
       PNOTIFYRECORDENTER pinfo;
       PFUNCTIONREC       pRecInfo;

           switch(SHORT2FROMMP(mp1))
           {
              case CN_ENTER:
                 pinfo = (PNOTIFYRECORDENTER) mp2;
                 pRecInfo = (PFUNCTIONREC) pinfo->pRecord;
                 if ((ULONG) pRecInfo)
                 {
                    int  i, nLen;

                    strcpy (actual_string, pRecInfo->CntrRecord.pszIcon);
                    nLen = strlen(actual_string);
                    for (i = 0; i < nLen; i++)
                    {
                       if (*(actual_string + i) == '.')
                       {
                          break;
                       }
                       if (!isupper(*(actual_string + i)))
                       {
                          ShowMessageBox(hwnd, ID_MSG_NO_LOWER_CASE);
                          return(MRESULT) NULL;
                       }
                    }
                    out_string (hwnd, pRecInfo->CntrRecord.pszIcon, ulAnimationSpeed);
                 }
                 else
                     return (MRESULT) NULL;
              break;
           }
           return (MRESULT)TRUE;
       }   /* of WM_CONTROL */

       case ID_REPAINT:
       {
           WinQueryWindowRect( hwnd, &rcl);
           WinInvalidateRect ( hwnd, &rcl, (BOOL) NULL);

           WinQueryWindowRect( hwndCntr, &rcl);
           WinInvalidateRect ( hwndCntr, &rcl, (BOOL) NULL);

           WinQueryWindowRect( hwndAnimateClient, &rcl);
           WinInvalidateRect ( hwndAnimateClient, &rcl, (BOOL) NULL);

           WinQueryWindowRect( hwndWriteClient, &rcl);
           WinInvalidateRect ( hwndWriteClient, &rcl, (BOOL) NULL);
           break;
       }   /* of case ID_REPAINT */

       case WM_CLOSE:
       {
          _somFree(objIDM);
          _somFree(pobjDoodle);
          _somFree(pobjAllDoodle);
          GpiDestroyPS(hpsAnimate);

          /* **** GO THRU **** */
       }

       default:
           return WinDefWindowProc( hwnd, msg, mp1, mp2 );
    }
    return (MRESULT) FALSE;
}   /* of Client Window Proc */

/**************************************************************************
*                                                                         *
*  Name       : AnimateWndProc                                            *
*                                                                         *
*  Description: Animate window procedure.                                 *
*                                                                         *
*               From the client window menu item, the color of this       *
*               window and color of text can be changed.                  *
*                                                                         *
**************************************************************************/
MRESULT EXPENTRY AnimateWndProc( HWND hwnd, ULONG  msg, MPARAM mp1, MPARAM mp2 )
{
HPS     hps;
RECTL   rcl;
POINTL  HotSpot;
ULONG   src;
APIRET  apiret;
SWP     swpMy;
CHAR    err[60];
SIZEL   sizlPage;
RECTL   top_rect;
static BOOL fFirst = TRUE;

   switch( msg )
   {
      case WM_CREATE:
      {
         hdcAnimate   = WinOpenWindowDC(hwnd);
         sizlPage.cx = 0;
         sizlPage.cy = 0;
         hpsAnimate = GpiCreatePS( hab, hdcAnimate, &sizlPage,
                                   PU_PELS | GPIA_ASSOC );
         GpiSetBackMix(hpsAnimate, BM_OVERPAINT);
         DosCreateMutexSem((PSZ) (NULL), &hMtx, 0, 0);
         break;
      }  /* of case WM_CREATE */

      case WM_PAINT:
      {
         hps = WinBeginPaint( hwnd, 0L, &top_rect );

         if (fFirst)
         {
            fFirst = FALSE;
            out_string(hwnd, "", SPEED_FAST);
         }
         if ((display_string[0]) && !fOutS)
         {
            fOutS = TRUE;
            out_string(hwnd, display_string, ulAnimationSpeed);
            fOutS = FALSE;
         }

         WinEndPaint(hps);
         break;
      }  /* of case WM_PAINT */

      default:
         return WinDefWindowProc( hwnd, msg, mp1, mp2 );
   }
   return (MRESULT) NULL;
}  /* of AnimateWndProc( HWND hwnd, ULONG  msg, MPARAM mp1, MPARAM mp2 ) */

/**************************************************************************
*                                                                         *
*  Name       : WriteWndProc                                              *
*                                                                         *
*  Description: Write window procedure.                                   *
*                                                                         *
*               From the client window menu item, the color of this       *
*               window and color of text can be changed.                  *
*               User writes spelling in this window and WM_RESULT message *
*               sent from Pen system is processed.                        *
*                                                                         *
**************************************************************************/
MRESULT EXPENTRY WriteWndProc( HWND hwnd, ULONG  msg, MPARAM mp1, MPARAM mp2 )
{
HPS       hps;
RECTL     rcl;
POINTL    HotSpot;
ULONG     src;
APIRET    apiret;
SWP       swpMy;
CHAR      err[128];
SIZEL     sizlPage;
RECTL     top_rect;
static RECTL rect;

   switch( msg )
   {
      case WM_CREATE:
      {
         hdcWrite   = WinOpenWindowDC(hwnd);
         sizlPage.cx = 0;
         sizlPage.cy = 0;
         hpsWrite  = GpiCreatePS(hab, hdcWrite, &sizlPage, PU_PELS | GPIA_ASSOC);
         GpiSetBackMix(hpsWrite, BM_OVERPAINT);
         WinSetActiveWindow(HWND_DESKTOP, hwnd);
         break;
      }  /* of case WM_CREATE */

      case WM_TOUCHDOWN:
         _idmStart(objIDM, &lev);
         if (lev._major != NO_EXCEPTION)
         {
            rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
            somExceptionFree(&lev);
         }
         break;

      case WM_PAINT:
         hps = WinBeginPaint(hwnd, 0L, &top_rect);

         /* Determine size of our window */
         src = WinQueryWindowPos( hwnd, &swpMy );
         ClearWindowArea(hwnd, hpsWrite, TRUE, lDrawColor);
         WinEndPaint(hps);
         break;

      case WM_SIZE:
         WinQueryWindowRect(hwndWriteClient, &rect);
         break;

      case WM_RESULT:
      {
      PIDM_RESULTDATA   pRD;
      PSZ psz;
      POINTL ptl;

         pRD = (PIDM_RESULTDATA) mp2;
         if (pRD->ulResultType != RES_TEXT)
             break;
         if (! *actual_string)
         {
             display_string[0] = 0;
             ShowMessageBox(hwnd, ID_MSG_SELECT_ANOTHER_ICON);
             break;
         }

         if ( strs_the_same (pRD->Data.pszText, display_string) )
         {
             display_string[0] = 0;
             actual_string[0]  = 0;

             ClearWindowArea(hwnd, hpsWrite, TRUE, lDrawColor);

             ptl.y =  (rect.yTop / 2) - 16;
             if (rect.xRight - rect.xLeft > (32 * 4))
             {
                 ptl.x =  (rect.xRight / 2) - 64;
                 WinDrawBitmap(hpsWrite, hbitSmile, &rect, &ptl, 0, 0, DBM_NORMAL);
                 DosSleep(0);
                 ptl.x =  (rect.xRight / 2) - 16;
                 WinDrawBitmap(hpsWrite, hbitSmile, &rect, &ptl, 0, 0, DBM_NORMAL);
                 DosSleep(0);
                 ptl.x =  (rect.xRight / 2) + 32;
                 WinDrawBitmap(hpsWrite, hbitSmile, &rect, &ptl, 0, 0, DBM_NORMAL);
             }
             else
             {
                 ptl.x =  (rect.xRight / 2) - 16;
                 WinDrawBitmap(hpsWrite, hbitSmile, &rect, &ptl, 0, 0, DBM_NORMAL);
             }
             DosSleep(2000);
             WinQueryWindowRect(hwndAnimateClient, &rect);
             ClearWindowArea(hwndAnimateClient, hpsAnimate, TRUE, lAnimateColor);
         }
         else
         {
             DosBeep(480, 100);
             DosBeep(79, 500);
             sprintf (err,
                "Uh-oh!\n\nYou wrote: \"%s\"\nInstead of \"%s\".",
                pRD->Data.pszText, actual_string);
             WinMessageBox (HWND_DESKTOP, hwnd, err,
                "IDMSAMP, Try-It Result", 0,
                MB_OK | MB_ERROR | MB_MOVEABLE);
         }

         ClearWindowArea(hwnd, hpsWrite, TRUE, lDrawColor);
         WinQueryWindowRect(hwndWriteClient, &rect);
         break;
      }  /* of case WM_RESULT */

      default:
         return WinDefWindowProc( hwnd, msg, mp1, mp2 );
   }
   return (MRESULT)NULL;
}  /* of WriteWndProc( HWND hwnd, ULONG  msg, MPARAM mp1, MPARAM mp2 ) */

/**************************************************************************
*                                                                         *
*  Name       : LoadFunctRecord                                           *
*                                                                         *
*  Description: From the file dialog box, user selects icon files and     *
*               the icons are loaded to container window here.            *
*                                                                         *
*  Parameters: hwndCntr : Container window handle                         *
*              pszText: text given under icon in container window         *
*              pFileName: icon file name                                  *
*                                                                         *
**************************************************************************/
PFUNCTIONREC LoadFunctRecord(HWND hwndCntr, PSZ pszText, PSZ pFileName)
{
RECORDINSERT  riRecord;
PFUNCTIONREC  pRecInfo;
ULONG         ulExtra = sizeof( FUNCTIONREC ) - sizeof( RECORDCORE );
ULONG         ulTmp;

   pRecInfo =(PFUNCTIONREC) WinSendMsg(hwndCntr, CM_ALLOCRECORD,
                                      MPFROMLONG(ulExtra), MPFROMSHORT(1) );
   pRecInfo->CntrRecord.pszIcon = pRecInfo->szText;
   pRecInfo->CntrRecord.pszText = pRecInfo->szText;
   pRecInfo->CntrRecord.pszTree = pRecInfo->szText;
   pRecInfo->CntrRecord.pszName = pRecInfo->szText;

   pRecInfo->CntrRecord.hptrIcon = WinLoadFileIcon(pFileName, FALSE);
   {
      /* Cutoff the '.ico' in the filename */
      char *psz;
      psz = strchr(pszText, '.');
      if(psz)
          *psz = 0;

      /* Strip directory information */
      ulTmp = strlen (pszText);
      while (ulTmp > 0)
      {
         if (pszText[ulTmp-1] == '\\')
             break;
         --ulTmp;
      }
      strcpy( pRecInfo->CntrRecord.pszName, &pszText[ulTmp] );
   }

   pRecInfo->CntrRecord.flRecordAttr = CRA_RECORDREADONLY;

   riRecord.cb                = sizeof( RECORDINSERT );
   riRecord.pRecordOrder      = (PRECORDCORE)CMA_END;
   riRecord.fInvalidateRecord = TRUE;
   riRecord.zOrder            = CMA_BOTTOM;
   riRecord.pRecordParent     = NULL;
   riRecord.cRecordsInsert    = 1;

   WinSendMsg(hwndCntr, CM_INSERTRECORD, MPFROMP(pRecInfo), MPFROMP(&riRecord));
   return pRecInfo;
}  /* of LoadFunctRecord(HWND hwndCntr, PSZ pszText, PSZ pFileName) */

/**************************************************************************
*                                                                         *
*  Name       : out_string                                                *
*                                                                         *
*  Description: When an icon in container window is selected by user,     *
*               its name is given in animation window.                    *
*                                                                         *
*  Parameters: psz: name text                                             *
*              ulSpeed: speed set in client window menu                   *
*                                                                         *
**************************************************************************/
VOID out_string(HWND hwnd, PSZ psz, ULONG ulSpeed)
{
SWP   swp;
RECTL r, top_rect;
LONG  lCharWidth;
PSZ   ret_psz;
char  buf[TEXT_LEN];
/* HPS   hps; */
int   ndx, pad1, pad2;
int   len = strlen(psz);

   /* if the string is too small, pad it with spaces */
   if (len <= 8)
   {
      pad1 = pad2 = (10 - len) / 2;
      ndx = 0;
      while(pad1--)
         buf[ndx++] = ' ';
      while (*psz)
      {
         buf[ndx++] = *psz;
         psz++;
      }
      while(pad2--)
         buf[ndx++] = ' ';
      buf[ndx] = 0;
      psz = buf;
   }
   strcpy(display_string, psz);

   WinQueryWindowPos(hwndAnimateClient, &swp);
   ClearWindowArea(hwndAnimateClient, hpsAnimate, TRUE, lAnimateColor);

   r.xLeft = r.yBottom = 0;
   r.yTop = swp.cy;
   r.xRight = lCharWidth = (swp.cx / strlen(psz));


   while (*psz)
   {
      if (ret_psz = AnimateChar(hwnd, hpsAnimate, *psz, &r, lAnimateInkColor,
                                ulSpeed, hfileDoodleFile ))
      {
          ShowMessageBox(hwnd, ID_MSG_INTERNAL_ERROR);
          break;
      }

      r.xRight += lCharWidth;
      r.xLeft  += lCharWidth;
      psz++;
   }
}  /* of out_string(HWND hwnd, PSZ psz, ULONG ulSpeed) */

/**************************************************************************
*                                                                         *
*  Name       : strs_the_same                                             *
*                                                                         *
*  Description: Compare the string written by user and the name of icon.  *
*               The user written string may have a blank and the blank    *
*               will be removed before comparison. The comparison is      *
*               case insensitive.                                         *
*                                                                         *
*  Parameters:  S1: user written string                                   *
*               S2: name of icon                                          *
*                                                                         *
*  Returns: TRUE if the strings are same, otherwise FALSE.                *
*                                                                         *
**************************************************************************/
int strs_the_same(PSZ s1, PSZ s2)
{
char buf1[TEXT_LEN], buf2[TEXT_LEN];
int  ndx;

    /* remove any spaces */
    ndx = 0;
    while(*s1)
    {
       if (*s1 != ' ') buf1[ndx++] = *s1;
       s1++;
    }
    buf1[ndx] = 0;

    ndx = 0;
    while(*s2)
    {
       if (*s2 != ' ') buf2[ndx++] = *s2;
       s2++;
    }
    buf2[ndx] = 0;

    if (!strcmpi(buf1, buf2))
       return 1;

    return 0;
}   /* of strs_the_same(PSZ s1, PSZ s2) */

/**************************************************************************
*                                                                         *
*  Name       : FileOpen(hwnd, mp2)                                       *
*                                                                         *
*  Description: Processes the File menu's Open item                       *
*                                                                         *
*  Concepts:  Called whenever OPEN from the FILE menu is selected         *
*                                                                         *
*             Calls the standard file open dialog to get the              *
*             file name.  The file name is passed onto DosOpen            *
*             which returns the handle to the file.  The file             *
*             input procedure is called and then the file handle          *
*             is closed.                                                  *
*                                                                         *
*  Parameters :  mp2      = second message parameter                      *
*                                                                         *
*  Return     :  [none]                                                   *
*                                                                         *
**************************************************************************/
VOID FileOpen(HWND hwnd, MPARAM mp2)
{
static  FILEDLG *pfdg = NULL;
static  char    pszDrive[3];
HFILE   hfIn;
SWP     swp;
ULONG   ulAction;
CHAR    szTitle[MESSAGELEN], szButton[MESSAGELEN];
PAPSZ   papszGlobalFileList;
PSZ     pszPath, pszName, pPath;
HWND    hwndFileInfo;
ULONG   i = 0, ulLen;

   /*
    ** Setup the standard file dialog parameters...
    */

   if (! pfdg)
   {
       if (! (pfdg = malloc (sizeof (FILEDLG))))
       {
           ShowMessageBox(hwnd, ID_MSG_MEMORY_ERROR);
           return;
       }

       memset(pfdg, 0, sizeof(FILEDLG) );
       pfdg->cbSize         = sizeof(FILEDLG);
       pfdg->pszIType       = (PSZ)NULL;
       pfdg->papszITypeList = (PAPSZ)NULL;
       pfdg->pszIDrive      = (PSZ)NULL;
       pfdg->papszIDriveList= (PAPSZ)NULL;
       pfdg->sEAType        = (SHORT)0;
       pfdg->papszFQFilename= (PAPSZ)NULL;
       pfdg->ulFQFCount     = 0L;
       pfdg->fl = FDS_MULTIPLESEL | FDS_OPEN_DIALOG | FDS_PRELOAD_VOLINFO | FDS_CENTER;
       pfdg->pszTitle       = "Which Pictures?";
       pfdg->pszOKButton    = "Ok";
   }
   strcpy (pfdg->szFullFile, "*.ico");      /* Type of file */

   /*
    ** Get the file(s)
    */
   hwndFileInfo = WinFileDlg(HWND_DESKTOP, hwndClient, pfdg);

   if (pfdg->lReturn == DID_CANCEL)
      return;

   if( ! hwndFileInfo )
   {
       ShowMessageBox(hwnd, ID_MSG_FILE_HANDLE_ERROR);
       return;
   }
   papszGlobalFileList = pfdg->papszFQFilename;

   /*
    *  Remove all icons from container
    */
   WinSendMsg( hwndCntr, CM_REMOVERECORD, MPFROMP( 0 ), MPFROMP( 0 ) );

   /*
    *  Update current working directory, find all icon files
    *  and insert them in the container.
    */
   ulLen = strlen( pfdg->szFullFile );
   pPath = pfdg->szFullFile + ulLen;
   while ( (*pPath != '\\') && (ulLen > 0) )
   {
      pPath--;
      ulLen--;
   }
   if (ulLen > 0)
   {
      pszName = (pPath + 1);
      pPath = malloc( ulLen );
      strncpy( pPath, pfdg->szFullFile, ulLen );
   }
   else
      pPath = 0;

  /* Save drive letter if applicable */
   if (pfdg->szFullFile[1] == ':')
       strncpy (pszDrive, pfdg->szFullFile, 2);
   else
       pszDrive[0] = 0;

   pszPath = pPath;
   if ( pszDrive[0] )
       pfdg->pszIDrive = pszDrive;

   DosSetCurrentDir( pszPath );

   if (papszGlobalFileList)
   {
       while ((ULONG) *papszGlobalFileList[i] )
       {
           LoadFunctRecord(hwndCntr, *papszGlobalFileList[i],
                           *papszGlobalFileList[i]);
           ++i;
       }
   }
   out_string(hwnd, "", SPEED_FAST);
}   /* of FileOpen   */

/**************************************************************************
*                                                                         *
*  Name       : Paint3DLines                                              *
*                                                                         *
*  Description: For the animation and write windows, give extra lines     *
*               for the boundary of the windows, so that image of three   *
*               dimension can be given.                                   *
*                                                                         *
*  Parameters:  hPS: presentation space handle                            *
*               pRectl: Rectangular of window                             *
*               ulSense: inward, outward, gray, balck, or while line      *
*                                                                         *
*  Returns: TRUE                                                          *
*                                                                         *
**************************************************************************/
BOOL Paint3dLines( HPS hPS, PRECTL pRectl, ULONG ulSense )
{
POINTL p;
RECTL  rClient;
LONG   ulLTColor, ulBRColor;

   rClient = *pRectl;                           /* Get a copy of the rect  */

   switch( ulSense )                            /* Setup Colors based on sense */
   {
     case OUTWARD:
        ulLTColor = CLR_WHITE;                   /* Left/Top      */
        ulBRColor = CLR_BLACK;                   /* Bottom/Right  */
        break;

     case INWARD:
        ulLTColor = CLR_BLACK;                   /* Left/Top      */
        ulBRColor = CLR_WHITE;                   /* Bottom/Right  */
        break;

     case BLACK_LINES:
        ulLTColor = CLR_BLACK;                   /* Left/Top      */
        ulBRColor = CLR_BLACK;                   /* Bottom/Right  */
        break;

     case WHITE_LINES:
        ulLTColor = CLR_WHITE;                   /* Left/Top      */
        ulBRColor = CLR_WHITE;                   /* Bottom/Right  */
        break;

     case GRAY_LINES:
        ulLTColor = CLR_PALEGRAY;                /* Left/Top      */
        ulBRColor = CLR_PALEGRAY;                /* Bottom/Right  */
        break;
   }  /* of switch(ulSense) */

   GpiSetColor( hPS, ulLTColor );     /* Color for left vert and top horz  */

   p.x = rClient.xLeft;                         /* Move to lower left corner */
   p.y = rClient.yBottom;
   GpiMove( hPS, &p );

   p.x = rClient.xLeft;                         /* Line to upper left corner */
   p.y = rClient.yTop;
   GpiLine( hPS, &p );

   p.x = rClient.xRight;                        /* Line to upper right corner */
   p.y = rClient.yTop;
   GpiLine( hPS, &p );

   GpiSetColor( hPS, ulBRColor );     /* Color for right vert and bot horz */

   p.x = rClient.xRight;                        /* Line to lower right corner */
   p.y = rClient.yBottom;
   GpiLine( hPS, &p );

   p.x = rClient.xLeft;                         /* Line to lower left  corner */
   p.y = rClient.yBottom;
   GpiLine( hPS, &p );

   return( TRUE );
}  /* of BOOL Paint3dLines( HPS hPS, PRECTL pRectl, ULONG ulSense ) */

/**************************************************************************
*                                                                         *
*  Name       : ClearWindowArea                                           *
*                                                                         *
*  Description: Clear the animation or write window before a new text     *
*               (icon anme or user written) is given.                     *
*                                                                         *
*  Parameters:  hwnd: animation or write window handle                    *
*               hps:  presentation space handle                           *
*               fFocus: focus to the window                               *
*               color: color of window                                    *
*                                                                         *
**************************************************************************/
VOID  ClearWindowArea( HWND hwnd, HPS hps, BOOL fFocus, LONG color )
{
RECTL rctl;

   WinQueryWindowRect( hwnd, (PRECTL)&rctl );
   WinFillRect( hps, &rctl , color );

   WinInflateRect(hab, &rctl, -1L, -1L );
   Paint3dLines( hps, &rctl, OUTWARD );

   if(fFocus)
   {
      WinInflateRect(hab, &rctl, -1L, -1L );
      Paint3dLines( hps, &rctl, GRAY_LINES );
      WinInflateRect(hab, &rctl, -1L, -1L );
      Paint3dLines( hps, &rctl, GRAY_LINES );
      WinInflateRect(hab, &rctl, -1L, -1L );
      Paint3dLines( hps, &rctl, INWARD );
   }
}  /* of VOID  ClearWindowArea( HWND hwnd, HPS hps, BOOL fFocus, LONG color ) */

/**************************************************************************
*                                                                         *
*  Name       : draw_ink_color                                            *
*                                                                         *
*  Description: Set the color of inking in write window.                  *
*                                                                         *
*  Parameters:  color: color of ink                                       *
*                                                                         *
**************************************************************************/
VOID draw_ink_color (LONG color)
{
    UnCheckIt (lDrawInkColor);

    _idmSetInkColor(objIDM, &lev, color);
    if (lev._major != NO_EXCEPTION)
    {
       rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
       somExceptionFree(&lev);
    }

    lDrawInkColor = color;
    CheckIt (color);
}   /* of VOID draw_ink_color (LONG color) */

/**************************************************************************
*                                                                         *
*  Name       : new_box                                                   *
*                                                                         *
*  Description: Depending upon the alphabet, adjust the base line,        *
*               height, left, and right of box.                           *
*                                                                         *
*  Parameters:  prectl: rectangular of box                                *
*               c:      character(alphabet, digit, period, or '-')        *
*                                                                         *
**************************************************************************/
VOID new_box(PRECTL prectl, CHAR c)
{
int           letter_type, bottom, top;
double        xRight, xLeft, yBottom, yTop;
double        dBaseline, dY, dHeight;
static double MaximumDescender = 6.0 / 20.0;
static double XHeight          = 7.0 / 20.0;
static double EmHeight         = 9.0 / 20.0;
static double LowerCaseDescent = 4.0 / 20.0;

   xRight = (double) prectl->xRight;
   xLeft  = (double) prectl->xLeft;
   yBottom = (double) prectl->yBottom;
   yTop = (double) prectl->yTop;

   dY = yTop - yBottom;
   dBaseline = yBottom + (dY * MaximumDescender);

   switch (c)
   {
      case 'a':
      case 'c':
      case 'e':
      case 'i':
      case 'm':
      case 'n':
      case 'o':
      case 'r':
      case 's':
      case 'u':
      case 'v':
      case 'w':
      case 'x':
      case 'z':
      case '+':
      case '=':
      case '*':
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
         dHeight = dBaseline + (dY * XHeight);
         break;

      case 'b':
      case 'd':
      case 'f':
      case 'h':
      case 'k':
      case 'l':
      case 't':
         break;

      case 'g':
      case 'j':
      case 'p':
      case 'q':
      case 'y':
         dHeight    = dBaseline + (dY * XHeight);
         dBaseline -= (dY * LowerCaseDescent);
         break;

      case '.':
         prectl->xLeft += (((prectl->xRight - prectl->xLeft)/2)-3);
         prectl->xRight = prectl->xLeft + 6;
         prectl->yBottom += ((prectl->yTop - prectl->yBottom) / 5);
         prectl->yTop = prectl->yBottom + 6;
         return;
         break;

      case '-':
         prectl->yBottom += ((prectl->yTop - prectl->yBottom) / 5);
         prectl->yBottom += (((prectl->yTop - prectl->yBottom) / 2) - 2);
         prectl->yTop = prectl->yBottom + 4;
         return;
         break;

      default:
         dHeight = dBaseline + (EmHeight * dY);
         break;
   }

   prectl->yBottom = (LONG) dBaseline;
   prectl->yTop    = (LONG) dHeight;
   prectl->xLeft   += ((prectl->xRight - prectl->xLeft) / 6);
   prectl->xRight  -= ((prectl->xRight - prectl->xLeft) / 6);
}  /* of VOID new_box(PRECTL prectl, CHAR c) */

/**************************************************************************
*                                                                         *
*  Name       : AnimateChar                                               *
*                                                                         *
*  Description: Give a character of the icon name in animation window.    *
*                                                                         *
*  Parameters:  hps: presentation space handle                            *
*               c:   character                                            *
*               pr:  rectangular                                          *
*               lColor: color of the character                            *
*               lspeed: speed set by user                                 *
*               hfile: file handle                                        *
*                                                                         *
*  Returns: NULL if success, otherwise "Get doodle failed"                *
*                                                                         *
**************************************************************************/
PSZ AnimateChar(HWND hwnd, HPS hps, CHAR c, PRECTL pr, LONG lColor,
                ULONG lSpeed, HFILE hfile)
{
ULONG      ulNumStrokes;
IDOStroke  *pStroke;
RECTL      r;
RECTL      *prectl = &r;
char       c2[2];

   if (c == ' ') return NULL;
   if (c == '.') return NULL;

   memcpy(prectl, pr, sizeof(RECTL));
   new_box(prectl, c);

   c2[0] = c; c2[1] = 0;
   if ( _idoCount(pobjAllDoodle, &lev))
   {
      _idoFreeStrokes (pobjAllDoodle, &lev);
      if (lev._major != NO_EXCEPTION)
      {
         rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
         somExceptionFree(&lev);
      }
   }
   if (!GetDoodle(hwnd, c2, pobjAllDoodle, hfile))
      return "Get doodle failure";

   ScaleAndPlace(pobjAllDoodle, hps, prectl, lColor);

   WinRequestMutexSem(hMtx, SEM_INDEFINITE_WAIT);
   pobjDoodle = pobjAllDoodle;
   AnimateDoodle(pobjDoodle, hps, lSpeed, lColor);
   DosReleaseMutexSem(hMtx);

   _idoRemoveAll(pobjDoodle, &lev);
   if (lev._major != NO_EXCEPTION)
   {
      rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
      somExceptionFree(&lev);
   }

   return NULL;
}  /* of PSZ AnimateChar(HPS hps, CHAR c, PRECTL pr, LONG lColor, ... */

/**************************************************************************
*                                                                         *
*  Name       : CheckIt, UnCheckIt                                        *
*                                                                         *
*  Description: Check and Uncheck the color or speed in menu item.        *
*                                                                         *
*  Parameters:  sId: Check or Uncheck the menu item                       *
*                                                                         *
**************************************************************************/
VOID CheckIt(SHORT sId)
{
   WinSendMsg(hwndMenu, MM_SETITEMATTR,
              MPFROM2SHORT(sId, TRUE),
              MPFROM2SHORT(MIA_CHECKED, MIA_CHECKED) );
}

VOID UnCheckIt(SHORT sId)
{
   WinSendMsg(hwndMenu, MM_SETITEMATTR,
              MPFROM2SHORT(sId, TRUE),
              MPFROM2SHORT(MIA_CHECKED, ~MIA_CHECKED) );
}

/**************************************************************************
*                                                                         *
*  Name       : map_id_to_color                                           *
*                                                                         *
*  Description: From the user selected menu item, get the color.          *
*                                                                         *
*  Parameters:  id: id of menu item                                       *
*                                                                         *
*  Return: color from menu id                                             *
*                                                                         *
**************************************************************************/
LONG map_id_to_color(SHORT id)
{
ULONG i;
LONG  lColor;

   lColor = CLR_WHITE;                 /* Default color */
   for (i = 0; i < 3 * NCOLOR; i++)
   {
      if (cmMapColor[i].sMenuColorID == id)
      {
         lColor = cmMapColor[i].lColor;
         break;
      }
   }
   return(lColor);
}

/**************************************************************************
*                                                                         *
*  Name       : contrast                                                  *
*                                                                         *
*  Description: From the user selected menu item, get the contrast color. *
*                                                                         *
*  Parameters:  color: color                                              *
*                                                                         *
*  Return: contrast color from color                                      *
*                                                                         *
**************************************************************************/
SHORT contrast (LONG color)
{
ULONG i;
SHORT sColor;

   sColor = CLR_BLACK;                 /* Default color */
   for (i = 0; i < NCOLOR; i++)
   {
      if (ccMatch[i].lColor == color)
      {
         sColor = ccMatch[i].sContrastColor;
         break;
      }
   }
   return(sColor);
}

/**************************************************************************
*                                                                         *
*  Name       : menu_cmd                                                  *
*                                                                         *
*  Description: Process menu command. When new color is selected, uncheck *
*               old color and check the new color.                        *
*                                                                         *
*  Parameters:  hwnd: client window handle                                *
*               msg:  message                                             *
*               mp1, mp2: MPARAM of client window                         *
*                                                                         *
**************************************************************************/
MRESULT menu_cmd( HWND   hwnd, ULONG  msg, MPARAM mp1, MPARAM mp2 )
{
ULONG          i;
SHORT          Id;

   Id = SHORT1FROMMP(mp1);
   switch (Id)
   {
      case ID_BOARD_WHITE      :
      case ID_BOARD_BLACK      :
      case ID_BOARD_RED        :
      case ID_BOARD_BLUE       :
      case ID_BOARD_PINK       :
      case ID_BOARD_GREEN      :
      case ID_BOARD_CYAN       :
      case ID_BOARD_YELLOW     :
      case ID_BOARD_BROWN      :
      case ID_BOARD_PALEGRAY   :
      case ID_BOARD_DARKGRAY   :
      case ID_BOARD_DARKRED    :
      case ID_BOARD_DARKBLUE   :
      case ID_BOARD_DARKPINK   :
      case ID_BOARD_DARKGREEN  :
      case ID_BOARD_DARKCYAN   :
      {
         UnCheckIt(lBoardCheck);
         lBoardCheck = Id;
         CheckIt(lBoardCheck);
         lBoardColor = map_id_to_color(Id);
         WinSendMsg(hwndClient, ID_REPAINT, (MPARAM) 0, (MPARAM) 0);
         break;
      }

      case ID_ANIMATE_WHITE      :
      case ID_ANIMATE_BLACK      :
      case ID_ANIMATE_RED        :
      case ID_ANIMATE_BLUE       :
      case ID_ANIMATE_PINK       :
      case ID_ANIMATE_GREEN      :
      case ID_ANIMATE_CYAN       :
      case ID_ANIMATE_YELLOW     :
      case ID_ANIMATE_BROWN      :
      case ID_ANIMATE_PALEGRAY   :
      case ID_ANIMATE_DARKGRAY   :
      case ID_ANIMATE_DARKRED    :
      case ID_ANIMATE_DARKBLUE   :
      case ID_ANIMATE_DARKPINK   :
      case ID_ANIMATE_DARKGREEN  :
      case ID_ANIMATE_DARKCYAN   :
      {
         UnCheckIt(lAnimateCheck);
         lAnimateCheck = Id;
         CheckIt(lAnimateCheck);
         lAnimateColor = map_id_to_color(Id);
         WinSendMsg(hwndAnimateClient, WM_PAINT, (MPARAM) 0, (MPARAM) 0);
         break;
      }

      case ID_DRAW_WHITE      :
      case ID_DRAW_BLACK      :
      case ID_DRAW_RED        :
      case ID_DRAW_BLUE       :
      case ID_DRAW_PINK       :
      case ID_DRAW_GREEN      :
      case ID_DRAW_CYAN       :
      case ID_DRAW_YELLOW     :
      case ID_DRAW_BROWN      :
      case ID_DRAW_PALEGRAY   :
      case ID_DRAW_DARKGRAY   :
      case ID_DRAW_DARKRED    :
      case ID_DRAW_DARKBLUE   :
      case ID_DRAW_DARKPINK   :
      case ID_DRAW_DARKGREEN  :
      case ID_DRAW_DARKCYAN   :
      {
         UnCheckIt(lDrawCheck);
         lDrawCheck = Id;
         CheckIt(lDrawCheck);
         lDrawColor = map_id_to_color(Id);
         WinSendMsg(hwndWriteClient, WM_PAINT, (MPARAM) 0, (MPARAM) 0);
         break;
      }

      case ID_ANIMATE_INK_WHITE:
      case ID_ANIMATE_INK_BLACK:
      case ID_ANIMATE_INK_RED:
      case ID_ANIMATE_INK_BLUE:
      case ID_ANIMATE_INK_PINK:
      case ID_ANIMATE_INK_GREEN:
      case ID_ANIMATE_INK_CYAN:
      case ID_ANIMATE_INK_YELLOW:
      case ID_ANIMATE_INK_BROWN:
      case ID_ANIMATE_INK_PALEGRAY:
      case ID_ANIMATE_INK_DARKGRAY:
      case ID_ANIMATE_INK_DARKRED:
      case ID_ANIMATE_INK_DARKBLUE:
      case ID_ANIMATE_INK_DARKPINK:
      case ID_ANIMATE_INK_DARKGREEN:
      case ID_ANIMATE_INK_DARKCYAN:
      {
         for (i = 0; i < NCOLOR; i++)
         {
            if (Id == acmColor[i].sAnimateInkColor)
            {
               UnCheckIt(lAnimateInkCheck);
               lAnimateInkCheck = Id;
               CheckIt(lAnimateInkCheck);
               lAnimateInkColor = acmColor[i].sColor;
               WinSendMsg(hwndAnimateClient, WM_PAINT, (MPARAM) 0, (MPARAM) 0);
               break;
            }
         }
         break;
      }

      case ID_DRAW_INK_WHITE:
      case ID_DRAW_INK_BLACK:
      case ID_DRAW_INK_RED:
      case ID_DRAW_INK_BLUE:
      case ID_DRAW_INK_PINK:
      case ID_DRAW_INK_GREEN:
      case ID_DRAW_INK_CYAN:
      case ID_DRAW_INK_YELLOW:
      case ID_DRAW_INK_BROWN:
      case ID_DRAW_INK_PALEGRAY:
      case ID_DRAW_INK_DARKGRAY:
      case ID_DRAW_INK_DARKRED:
      case ID_DRAW_INK_DARKBLUE:
      case ID_DRAW_INK_DARKPINK:
      case ID_DRAW_INK_DARKGREEN:
      case ID_DRAW_INK_DARKCYAN:
      {
         for (i = 0; i < NCOLOR; i++)
         {
            if (Id == dcmColor[i].sDrawInkColor)
            {
               draw_ink_color(dcmColor[i].lColor);
               break;
            }
         }
         break;
      }

      default:
          break;
   }
   return (MRESULT) 0;
}   /* of MRESULT menu_cmd(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) */

/**************************************************************************
*                                                                         *
*  Name       : GetDoodle                                                 *
*                                                                         *
*  Description: Read stroke data from file.                               *
*                                                                         *
*  Parameters:  pChar: Name of icon(signature)                            *
*               pDoodleObject: pointer to doodle object                   *
*               hf: stroke file handle                                    *
*                                                                         *
*  Return:   TRUE if seccessful, otherwise FALSE                          *
*                                                                         *
**************************************************************************/
int GetDoodle(HWND hwnd, PCHAR pChar, IDODoodle *pDoodleObject, HFILE hf)
{
BOOL         bFound;
ULONG        ulStrokeIndex;             /* stroke indexer              */
BOOL         rc, bFill;
IDOStroke    *pStrokeObject;
ULONG        ulNewPtr;                  /* file pointer                */
ULONG        ulBytesRead;
ULONG        *ulEachStrokeLength;       /* array of stroke lengths     */
ULONG        BufLen, ulSize;
APPLICATION  App;
PSKETCHSTROKEDEF pStrokeData = NULL;

    /*********************************************/
    /*  Find the symbol.                         */
    /*********************************************/
    DosSetFilePtr(hf, 0, FILE_BEGIN, &ulNewPtr);   /* beginning of file */
    bFound = FALSE;
    while( !bFound)
    {
       DosRead(hf, (VOID *) &App, sizeof(APPLICATION), &ulBytesRead);
       if (ulBytesRead != sizeof(APPLICATION) )
       {
          return FALSE;
       }
       else
       {
          if ( (strcmp(App.szName, (PSZ)pChar) ) == 0)
          {
             bFound = TRUE;
          }
          else
          {
             /************************************************/
             /* Advance the file pointer past the signature. */
             /************************************************/
             DosSetFilePtr(hf, App.ulSignatureSize, FILE_CURRENT, &ulNewPtr);
          }
       }
    } /* of while( !bFound) */

    /*********************************************************/
    /* Malloc enough to retrieve the lengths of each stroke. */
    /*********************************************************/
    ulEachStrokeLength = (ULONG *) calloc(App.ulStrokeTotal, sizeof(ULONG) );

    /**************************************************/
    /* Read All stroke lengths into array of lengths. */
    /**************************************************/
    DosRead(hf, (VOID *) ulEachStrokeLength, (sizeof(ULONG) * App.ulStrokeTotal), &ulBytesRead);
    if ( ulBytesRead != (sizeof(ULONG) * App.ulStrokeTotal) )
    {
       free(ulEachStrokeLength);
       return(FALSE);
    }

    /**************************************/
    /* First stroke to the last stroke -  */
    /* Read each stroke from the file.    */
    /**************************************/
    for (ulStrokeIndex = 0; ulStrokeIndex < App.ulStrokeTotal; ulStrokeIndex++)
    {
        /*******************************************************/
        /* Allocate a buffer (non-shared) based on that size.  */
        /*******************************************************/
        pStrokeData = malloc(ulEachStrokeLength[ulStrokeIndex]);
        if (!pStrokeData)
        {
            free(ulEachStrokeLength);
            return(FALSE);
        }

        memset(pStrokeData , '\0',ulEachStrokeLength[ulStrokeIndex]);
        /**************************************************/
        /* Read the stroke from the file into the buffer. */
        /**************************************************/
        DosRead(hf,
                pStrokeData,
                ulEachStrokeLength[ulStrokeIndex],
                &ulBytesRead);

        /**********************************************/
        /* The following code is necessary for Stroke */
        /* Data pointer fix ups after each read.      */
        /**********************************************/

        /*************************************************************
        *  The SYMDB.DAT file has stroke information of all upper    *
        *  case alphabets. Its format is as follow:                  *
        *     CHAR    *name,                                         *
        *     ULONG   number of strokes,                             *
        *     ULONG   size of stroke data of this alphabet,          *
        *     ULONG   size of stroke 1,                              *
        *     ULONG   size of stroke 2, repeated for the number      *
        *                               of strokes,                  *
        *     SKETCHSTROKEDEF  stroke1, repeated for the number      *
        *     SKETCHSTROKEDEF  stroke2, of strokes,                  *
        *                                                            *
        *  Before the stroke data is written to disk, it is flattened*
        *  using WrtCopyStrokeData call.                             *
        *************************************************************/

        /*************************************************************
        *  When the stroke data was kept in disk, it was flattened.  *
        *  So, unflatten to adjust the offsets to actual pointers.   *
        *************************************************************/
        pStrokeData->pStroke = (PSTROKEDATA) ((PBYTE) pStrokeData +
                                   sizeof(SKETCHSTROKEDEF));
        WrtCopyStrokeData(pStrokeData->pStroke, 0, CSD_UNFLATTEN);

        if ( ulBytesRead != ulEachStrokeLength[ulStrokeIndex])
        {
            free(ulEachStrokeLength);
            free(pStrokeData);
            return(FALSE);
        }

        pStrokeObject = IDOStrokeNew ( );
        if (!pStrokeObject)
        {
            ShowMessageBox(hwnd, ID_MSG_SOM_CREATION_FAIL);
            return FALSE;
        }

        _idoSetStrokeData (pStrokeObject,
                           &lev,
                           pStrokeData->pStroke,
                           (HWND) NULL,
                           0UL);
        if (lev._major != NO_EXCEPTION)
        {
           rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
           somExceptionFree(&lev);
        }

        _idoAddLast ( pDoodleObject,
                      &lev,
                      pStrokeObject );
        if (lev._major != NO_EXCEPTION)
        {
           rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
           somExceptionFree(&lev);
        }

        free(pStrokeData);
    } /* end for loop - completed reading all strokes */

    /**************************************************/
    /* Free the array of stroke lengths.              */
    /**************************************************/
    free(ulEachStrokeLength);

    return TRUE;
} /* of GetDoodle */

/**************************************************************************
*                                                                         *
*  Name       : AnimateDoodle                                             *
*                                                                         *
*  Description: Read stroke data from file.                               *
*                                                                         *
*  Parameters:                                                            *
*               pDoodle: pointer to doodle object                         *
*               hps: presentation space handle                            *
*               ulTime: animation speed                                   *
*               lColor: color of doodle                                   *
*                                                                         *
**************************************************************************/
VOID AnimateDoodle(IDODoodle *pDoodle, HPS hps, ULONG ulTime, LONG lColor )
{
ULONG       ulNumStrokes = 0;
ULONG       i;
ULONG       j;
ULONG       BufLen;
IDOStroke   *pStroke;
IDOStroke   *pNextStroke;
PSTROKEDATA pStrokeData;
PPOINTL     p;
ULONG       speedfactor;

   speedfactor = MAXTIME - ulTime;
   ulNumStrokes = _idoCount ( pDoodle, &lev );
   if (lev._major != NO_EXCEPTION)
   {
      rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
      somExceptionFree(&lev);
   }
   pStroke = _idoFirst ( pDoodle, &lev );
   if (lev._major != NO_EXCEPTION)
   {
      rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
      somExceptionFree(&lev);
   }

   i = j = 0;
   while ( i < ulNumStrokes )
   {
   QMSG   qmsg;

       BufLen = 0;
       pStrokeData = NULL;
       _idoGetScaledData ( pStroke,
                           &lev,
                           pStrokeData,
                           &BufLen );
        if (lev._major != NO_EXCEPTION)
        {
           rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
           somExceptionFree(&lev);
        }
        pStrokeData = (PSTROKEDATA) malloc(BufLen);
        if (pStrokeData)
        {
            _idoGetScaledData ( pStroke,
                                &lev,
                                pStrokeData,
                                &BufLen );
            if (lev._major != NO_EXCEPTION)
            {
               rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
               somExceptionFree(&lev);
            }

           GpiSetColor ( hps, lColor );
           for ( j = 0;
                 j < (pStrokeData->ulNumPoints - 1); j++ )
           {
              p = pStrokeData->pXY + j;
              GpiMove ( hps, p );
              GpiLine ( hps, p + 1);

              if ( ulTime && (! (i % speedfactor)) )
              {
                 DosSleep ( ulTime );
              }
           }  /* of for ( p = pStrokeData->pXY, j = 0 )                    */

           while (WinPeekMsg(hab, &qmsg, NULL, 0, 0, PM_NOREMOVE))
           {
              if (qmsg.msg == WM_QUIT)
              {
                 break;
              }
              WinGetMsg(hab, &qmsg, NULL, 0, 0);
              WinDispatchMsg(hab, &qmsg);
           }
           free(pStrokeData);
           if (qmsg.msg == WM_QUIT)
           {
              break;
           }
        }

       pNextStroke = pStroke;
       pStroke = _idoAfter ( pDoodle, &lev, pNextStroke );
       if (lev._major != NO_EXCEPTION)
       {
          rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
          somExceptionFree(&lev);
       }
       if (!pStroke)
       {
          break;
       }
       i++;
   }  /* of while ( i < ulNumStrokes ) */
}  /* of AnimateDoodle */

/**************************************************************************
*                                                                         *
*  Name       : ScaleAndPlace                                             *
*                                                                         *
*  Description:                                                           *
*                                                                         *
*  Parameters:                                                            *
*               pDoodle: pointer to doodle object                         *
*               hps: presentation space handle                            *
*               pRecT:                                                    *
*               lColor: color of doodle                                   *
*                                                                         *
**************************************************************************/
VOID ScaleAndPlace(IDODoodle *pDoodle, HPS hps, PRECTL pRect, LONG lColor )
{
RECTL      bbox;
ULONG      ulX, ulY;
LONG       lX,  lY;
ULONG      ulNumStrokes = 0;
ULONG      i = 0;
IDOStroke  *pStroke;
IDOStroke  *pNextStroke;

    _idoGetScaledBoundingBox(pDoodle,
                             &lev,
                             &bbox,
                             0 );
    if (lev._major != NO_EXCEPTION)
    {
       rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
       somExceptionFree(&lev);
    }

    ulX = ((pRect->xRight-pRect->xLeft) * 1000) / (bbox.xRight - bbox.xLeft);
    ulY = ((pRect->yTop-pRect->yBottom) * 1000) / (bbox.yTop - bbox.yBottom);

    ulNumStrokes = _idoCount ( pDoodle, &lev );
    if (lev._major != NO_EXCEPTION)
    {
       rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
       somExceptionFree(&lev);
    }
    pStroke = _idoFirst ( pDoodle, &lev );
    if (lev._major != NO_EXCEPTION)
    {
       rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
       somExceptionFree(&lev);
    }
    while ( i < ulNumStrokes )
    {
       _idoSetInkColor ( pStroke, &lev, lColor );
        if (lev._major != NO_EXCEPTION)
        {
           rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
           somExceptionFree(&lev);
        }
       _idoScaleStroke ( pStroke, &lev, ulX, ulY, IDO_SCALE_FRESH, 0 );
        if (lev._major != NO_EXCEPTION)
        {
           rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
           somExceptionFree(&lev);
        }

       pNextStroke = pStroke;
       pStroke = _idoAfter ( pDoodle, &lev, pNextStroke );
        if (lev._major != NO_EXCEPTION)
        {
           rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
           somExceptionFree(&lev);
        }
       i++;
    }  /* of while ( i < ulNumStrokes ) */

    _idoGetScaledBoundingBox ( pDoodle, &lev, &bbox, 0 );
    if (lev._major != NO_EXCEPTION)
    {
       rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
       somExceptionFree(&lev);
    }
    lX = pRect->xLeft - bbox.xLeft;
    lY = pRect->yBottom - bbox.yBottom;

    _idoSetOffset ( pDoodle, &lev, lX, lY );
    if (lev._major != NO_EXCEPTION)
    {
       rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
       somExceptionFree(&lev);
    }
}   /* of ScaleAndPlace */

/*****************************************************************************/
/*  Name:   ShowMessageBox                                                   */
/*                                                                           */
/*  Description : Displays Messages                                          */
/*                                                                           */
/*  Concepts : Displays the Info/warning/error message box with the message  */
/*             given in idMsg retrived from the message table                */
/*             Called whever a Info/warning/error occurs and a message wishes*/
/*             to be displayed to the user.                                  */
/*                                                                           */
/*  Parameters :  hwnd      - window handle of the owner of the              */
/*                            message box                                    */
/*                idMsg     - id of message to be retrieved from             */
/*                            resource file                                  */
/*                                                                           */
/*  Returns: USHORT                                                          */
/*                                                                           */
/*****************************************************************************/
USHORT ShowMessageBox(HWND hwnd, USHORT idMsg)
{
       CHAR    szText[TEXT_LEN];

static USHORT  ulAlarm = WA_ERROR;
static USHORT  ulStyle = MB_OK | MB_ERROR | MB_MOVEABLE;

    WinLoadString(hab, 0L, idMsg, TEXT_LEN, (PSZ) szText);
    WinAlarm(HWND_DESKTOP, ulAlarm);
    return(WinMessageBox(HWND_DESKTOP, hwnd, szText, NULL, IDD_MSGBOX, ulStyle));
}
