/******************************************************************************
*                                                                             *
*   Name:  IDOSAMP.C                                                          *
*                                                                             *
*   Copyright : COPYRIGHT IBM CORPORATION, 1995                               *
*               LICENSED MATERIAL - PROGRAM PROPERTY OF IBM                   *
*                                                                             *
*   Description: This program lets user to draw any stroke in a window,       *
*                select one stroke and change attributes of the stroke, and   *
*                redisplay the strokes with changed attributes.               *
*                                                                             *
*                No recognition of stroke is needed in this program whether   *
*                the stroke is gesture or alphabets. So no IDM object is      *
*                created in this program.                                     *
*                                                                             *
*   Execution Instruction: IDOSAMP                                            *
*                                                                             *
*   Software Requirement: IBM OS/2 V2.1, Pen for OS/2 V1.1                    *
*                                                                             *
*   DISCLAIMER OF WARRANTIES.  The following [enclosed] code is               *
*      sample code created by IBM Corporation. This sample code is not        *
*      part of any standard or IBM product and is provided to you solely      *
*      for  the purpose of assisting you in the development of your           *
*      applications.  The code is provided "AS IS", without                   *
*      warranty of any kind.  IBM shall not be liable for any damages         *
*      arising out of your use of the sample code, even if they have been     *
*      advised of the possibility of such damages.                            *
*                                                                             *
*   ATTENTION                                                                 *
*      For each SOM method call, the returned code checking is strongly       *
*      recommended. The result can be checked as follow:                      *
*                                                                             *
*          if (lev._major != NO_EXCEPTION)                                    *
*          {                                                                  *
*             rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;       *
*             Check the return code here                                      *
*             somExceptionFree(&lev);                                         *
*          }                                                                  *
*                                                                             *
*      In this sample program, there are lots of IDO method calls and to      *
*      reduce the size of program, all ido method call do not have the        *
*      checking.                                                              *
******************************************************************************/

#define    INCL_PM
#define    INCL_GPIPRIMATIVES
#define    INCL_DOSMODULEMGR
#define    INCL_DOSPROCESS
#define    INCL_DOSDATETIME
#define    INCL_IDO
#define    SOMF_NIL 0

#include   <os2.h>
#include   <string.h>
#include   <stdlib.h>
#include   <penpm.h>
#include   "IDOSAMP.h"

/*****************************************************************************
*                                                                            *
*   Subroutine Name : main                                                   *
*                                                                            *
*   Function: Main procedure to create the client window, process the window *
*             messages, and exits the program.                               *
*                                                                            *
*   Parameters: None.                                                        *
*                                                                            *
*****************************************************************************/
int main(VOID)
{
   ULONG flFrameFlags = FCF_STANDARD & ~FCF_ACCELTABLE;

   QMSG         qmsg;
   HMQ          hmq;
   CHAR         achClass[] = "IDOSAMP";

   hab = WinInitialize(0UL);
   hmq = WinCreateMsgQueue(hab, 0);

   /* Initialize my private environment */
   SOM_InitEnvironment( &lev );

   WinRegisterClass(hab,
                    achClass,
                    ClientWinProc,
                    CS_SYNCPAINT,
                    0);

   hwndFrame  = WinCreateStdWindow(HWND_DESKTOP,
                                   WS_VISIBLE,
                                   &flFrameFlags,
                                   achClass,
                                   NULL,
                                   0UL,
                                   0UL,
                                   ID_IDOSAMP,
                                   NULL);

   if (hwndFrame)
   {
      while (WinGetMsg(hab, &qmsg, 0UL, 0UL, 0UL))
         WinDispatchMsg( hab, &qmsg );
   }

   /* Clean up my private environment */
   SOM_UninitEnvironment( &lev );

   WinDestroyWindow(hwndFrame);
   WinDestroyMsgQueue( hmq );
   WinTerminate( hab );
   DosExit(1, 0);
}  /* of int main(VOID) */

/*****************************************************************************
*                                                                            *
*   Subroutine Name : ClientWinProc                                          *
*                                                                            *
*   Function: Window procedure of the main client window.                    *
*             Process all messages and menu items in this window.            *
*                                                                            *
*   Parameters: HWND    hwnd                                                 *
*               ULONG   msg                                                  *
*               MAPRAM  mp1                                                  *
*               MPARAM  mp2                                                  *
*                                                                            *
*****************************************************************************/
MRESULT EXPENTRY ClientWinProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
static HPS        hps = (HPS) NULL;
static HWND       hwndMenu;
CHAR              achMsg[80];
static SIZEL      sizel;
static INT        cyChar, cxChar, cyClient;
LONG              lYCoord;
static BOOL       fRemove, fStrokeAdded, fNewInkAttr;
static HPOINTER   hPtr, hCptr;
static ATTRCHG    acData;
static IDOInkAttr *pInkAttr;
static INKERDATA  InkerData, InkerDataDoodle, InkerDataInker, InkerDataT;
static IDODoodle  *pIDODoodle;
static IDOStroke  *pIDOStroke, *idoS, *idoStrokeChanged;
static IDOInker   *pIDOInker;
static ULONG      ulRedrawOpt = REDRAW_DEF;
static SHORT      sRedrawMenu = IDO_REDRAW_DEF;
static BOOL       fFirst = TRUE;
static BOOL       fErase = FALSE;
static BOOL       fClear = TRUE;
ULONG             ulNPoints;
PSTROKEDATA       pStrokeData;
INT               i;
ULONG             ulBufLen;
POINTL            ptl;
APIRET            rc;

   switch (msg)
   {
      case WM_CREATE:
      {
      FONTMETRICS  fm;
      HPS          hpsPaint;

         /********************************************************************
         *  The APIs being used in this procedure depend on the PenPM sub-   *
         *  system being correctly initialized.  Thus we check to make sure  *
         *  PenPM is up before we do anything else.                          *
         ********************************************************************/
         if(WrtWaitActive(WRT_IMMEDIATE_RETURN))
         {
            /*************************************************************
            *  If PenPM is NOT active then put up a message box and      *
            *  return TRUE to terminate the program.                     *
            *************************************************************/
            PutString(hwnd, STR_1, TRUE);
            return((MRESULT) TRUE);
         }

         /***********************************************************
         *  Get font information from presentation space            *
         ***********************************************************/
         hpsPaint = WinGetPS(hwnd);
         GpiQueryFontMetrics(hpsPaint, (LONG) sizeof(fm), &fm);
         cxChar = fm.lEmInc;
         cyChar = fm.lMaxDescender + fm.lLowerCaseAscent;
         WinReleasePS(hpsPaint);

         /***********************************************************
         *  Put checkmark to default drawing option.                *
         ***********************************************************/
         hwndMenu = WinWindowFromID( WinQueryWindow(hwnd, QW_PARENT),
                                     FID_MENU);
         WinSendMsg(hwndMenu, MM_SETITEMATTR,
                    MPFROM2SHORT(sRedrawMenu, TRUE),
                    MPFROM2SHORT(MIA_CHECKED, MIA_CHECKED));

         /***************************************************************
         * Create an IDO Inker object which will be used to draw        *
         * strokes and doodle at WM_STROKE and WM_PAINT messages.       *
         ***************************************************************/
         pIDOInker = IDOInkerNew();

         /***********************************************************
         *  Create a presentation space and associate it with Inker *
         ***********************************************************/
         hab = WinQueryAnchorBlock(hwnd);
         hdc = WinOpenWindowDC(hwnd);
         DevQueryCaps(hdc, CAPS_WIDTH,  1L, &sizel.cx);
         DevQueryCaps(hdc, CAPS_HEIGHT, 1L, &sizel.cy);
         hps = GpiCreatePS(hab,
                           hdc,
                           &sizel,
                           PU_PELS | GPIT_NORMAL | GPIF_LONG | GPIA_ASSOC);
         _idoSetHps(pIDOInker, &lev, hps);

         /******************************************************************
         *  Create an Inker Attribute object with different color and line *
         *  width. This Attribute object will be associated with doodle    *
         *  object. When the doodle is redrawn, this new attribute will    *
         *  be used.                                                       *
         ******************************************************************/
         InkerData.cbSize = InkerDataDoodle.cbSize = InkerDataInker.cbSize = sizeof(INKERDATA);
         InkerData.hps = InkerDataDoodle.hps = InkerDataInker.hps = hps;
         InkerDataDoodle.flAttrMask = InkerDataInker.flAttrMask =
             IDO_COLOR | IDO_WIDTH | IDO_MIX | IDO_LINETYPE;
         InkerDataDoodle.lColor = CLR_BLUE;
         InkerDataDoodle.lMix = InkerDataInker.lMix = FM_DEFAULT;
         InkerData.lLineType = InkerDataDoodle.lLineType = InkerDataInker.lLineType = LINETYPE_DEFAULT;
         InkerData.ulWidth = InkerDataDoodle.ulWidth = InkerDataInker.ulWidth = 2;
         pInkAttr = IDOInkAttrNew();
         _idoSetInkerData(pInkAttr,
                          &lev,
                          &InkerDataDoodle,
                          NULL);

         if (lev._major != NO_EXCEPTION)
         {
            somExceptionFree(&lev);
         }

         /********************************************************************
         * Create an instance of DOODLE object and associate it with default *
         * inker data.                                                       *
         ********************************************************************/
         pIDODoodle = IDODoodleNew();
         _idoSetInkAttr(pIDODoodle, &lev, pInkAttr);
         if (lev._major != NO_EXCEPTION)
         {
            somExceptionFree(&lev);
         }

         /******************************************************************
         *  Default Inker color is RED                                     *
         ******************************************************************/
         InkerDataInker.lColor = CLR_RED;
         InkerData.flAttrMask = IDO_COLOR | IDO_WIDTH | IDO_MIX | IDO_LINETYPE;
         InkerData.lColor = CLR_PALEGRAY;
         InkerData.lMix = FM_XOR;
         _idoSetInkerData(pIDOInker,
                          &lev,
                          &InkerDataInker,
                          (PINKERDATA) NULL);
         if (lev._major != NO_EXCEPTION)
         {
            somExceptionFree(&lev);
         }
         hCptr = WinLoadPointer(HWND_DESKTOP, NULLHANDLE, ID_CROSS);
         WinPostMsg(hwnd,
                    WM_COMMAND,
                    MPFROMSHORT(IDO_MCDOODLE),
                    NULL);
         return((MRESULT) FALSE);
      }  /* of WM_CREATE */

      case WM_SIZE:
      {
         /***********************************************************
         *  Get the size of the client window                       *
         ***********************************************************/
         sizel.cx = SHORT1FROMMP(mp2);
         sizel.cy = cyClient = SHORT2FROMMP(mp2);
         return((MRESULT) FALSE);
      }

      case WM_TOUCHDOWN:
      {
         if (fClear)
         {
            fClear = FALSE;
            WinInvalidateRect(hwnd, NULL, FALSE);
         }
         WinSetCapture(HWND_DESKTOP, hwnd);
         ptl.x = SHORT1FROMMP(mp1);
         ptl.y = SHORT2FROMMP(mp1);
         _idoSetPoint(pIDOInker, &lev, &ptl);
         if (lev._major != NO_EXCEPTION)
         {
            somExceptionFree(&lev);
         }

         return((MRESULT) TDN_INFINITE);
      }  /* of case WM_TOUCHDOWN */

      case WM_STROKE:
      {
         ULONG      ulNumPoints;
         PPOINTL    pPoints;
         ULONG      rc;
         IDOInkAttr *pIDOInkAttrChg;       /* InkAttr of selected stroke */

         WinSetCapture(HWND_DESKTOP, NULLHANDLE);

         /***********************************************************
         *  Retrieve stroke data. This part of code is not needed   *
         *  if NULL is used as buffer pointer in idoclsCreateStroke *
         *  method. This code is given only as a sample.            *
         ***********************************************************/
         pStrokeData = NULL;
         ulBufLen = 0;
         WrtQueryStrokeData( (PBYTE) pStrokeData,
                             &ulBufLen,
                             hwnd,
                             QSD_STANDARDIZED_RES,
                             0, 0, 0);
         pStrokeData = malloc(ulBufLen);
         if (!pStrokeData)
         {
            PutString(hwnd, STR_2, TRUE);
            return((MRESULT) FALSE);
         }
         else
         {
            pStrokeData->cbStructSize = sizeof (STROKEDATA);
            WrtQueryStrokeData( (PBYTE) pStrokeData,
                                &ulBufLen,
                                hwnd,
                                QSD_STANDARDIZED_RES,
                                0, 0, 0);

            /***********************************************************
            *  Create a stroke object and set stroke data to it        *
            ***********************************************************/
            pIDOStroke = IDOStrokeNew();
            _idoSetStrokeData(pIDOStroke,
                              &lev,
                              pStrokeData,
                              hwnd,
                              0L);
         }

         /*************************************************************
         *  Check whether the fSelect flag is set. If so, then do not *
         *  collect the stroke. instead get the (x,y) coordinate of   *
         *  tapping spot and find the user selected stroke.           *
         *************************************************************/
         if (fSelect || fRemove)
         {
         BOOL fFound;
         INT  nGapX, nGapY;

             /*********************************************************
             *  Check whether the stroke is tap or not                *
             *********************************************************/
             if (!_idoIsTap(pIDOStroke, &lev))
             {
                PutString(hwnd, STR_5, TRUE);
                free(pStrokeData);
                _somFree(pIDOStroke);
                fSelect = FALSE;
                return(FALSE);
             }

             /*********************************************************
             *  Get the (x,y) coordinate of the Tap point             *
             *********************************************************/
             rc = 0;
             ulBufLen = 0;
             _idoGetScaledPoints(pIDOStroke,
                                 &lev,
                                 &ulNumPoints,
                                 NULL,
                                 &ulBufLen);
             if (lev._major != NO_EXCEPTION)
             {
                somExceptionFree(&lev);
             }

             pPoints = (PPOINTL) malloc(ulBufLen);
             _idoGetScaledPoints(pIDOStroke,
                                    &lev,
                                    &ulNumPoints,
                                    pPoints,
                                    &ulBufLen);
             if (lev._major != NO_EXCEPTION)
             {
                somExceptionFree(&lev);
             }
             ptl.x = pPoints->x;
             ptl.y = pPoints->y;
             free(pPoints);

             fFound = FALSE;
             nGapX = GAP_X;
             nGapY = GAP_Y;

             while (!fFound)
             {
                idoStrokeChanged = _idoFirst(pIDODoodle, &lev);
                if (lev._major != NO_EXCEPTION)
                {
                   rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                   PutString(hwnd, STR_9, TRUE);
                   return((MRESULT) FALSE);
                }
                while (idoStrokeChanged)
                {
                   ulBufLen = 0;
                   _idoGetScaledPoints(idoStrokeChanged,
                                       &lev,
                                       &ulNumPoints,
                                       NULL,
                                       &ulBufLen);
                   if (lev._major != NO_EXCEPTION)
                   {
                      somExceptionFree(&lev);
                   }

                   pPoints = (PPOINTL) malloc(ulBufLen);
                   if (pPoints)
                   {
                      _idoGetScaledPoints(idoStrokeChanged,
                                          &lev,
                                          &ulNumPoints,
                                          pPoints,
                                          &ulBufLen);
                      if (lev._major != NO_EXCEPTION)
                      {
                         somExceptionFree(&lev);
                      }
                      for (i = 0; i < ulNumPoints; ++i)
                      {  /* use 2 variables for GAP and increase it to find stroke */
                         if((abs(pPoints[i].x - ptl.x) <= nGapX)  &&
                            (abs(pPoints[i].y - ptl.y) <= nGapY))
                         {
                            /**************************************************
                            *  The user selected stroke is found              *
                            **************************************************/
                            fFound = TRUE;
                            fModify = TRUE;
                            break;
                         }
                      }
                      free(pPoints);
                      if(fFound)
                         break;
                   }  /* of if(pStroke)  */
                   else
                   {
                      PutString(hwnd, STR_2, TRUE);
                      return((MRESULT) FALSE);
                   }
                   idoStrokeChanged = _idoAfter(pIDODoodle, &lev, idoStrokeChanged);
                   if (lev._major != NO_EXCEPTION)
                   {
                      somExceptionFree(&lev);
                   }
                }  /* of while(!idoStrokeChanged)    */
                nGapX++;
                nGapY++;
             }  /* of while (!fFound) */

             /*******************************************************
             *  Draw the selected stroke in pale gray.              *
             *******************************************************/
             pIDOInkAttrChg = _idoGetStrokeInkAttr(idoStrokeChanged,
                                                   &lev);
             if (lev._major != NO_EXCEPTION)
             {
                somExceptionFree(&lev);
             }

             if (pIDOInkAttrChg)
             {
                acData.nLWidth = _idoGetWidth(pIDOInkAttrChg, &lev);
                acData.nColor  = _idoGetColor(pIDOInkAttrChg, &lev);
                acData.nLType  = _idoGetLineType(pIDOInkAttrChg, &lev);
                acData.nSMix   = _idoGetMix(pIDOInkAttrChg, &lev);

                /*****************************************************
                *  Get offset of stroke                              *
                *****************************************************/
                _idoGetStrokeOffset(idoStrokeChanged,
                                    &lev,
                                    &acData.nX,
                                    &acData.nY);
                if (lev._major != NO_EXCEPTION)
                {
                   somExceptionFree(&lev);
                }

                _idoSetMix(pIDOInkAttrChg, &lev, FM_XOR);
                _idoDrawStroke(idoStrokeChanged, &lev, hps);
                _idoSetMix(pIDOInkAttrChg, &lev, FM_OVERPAINT);
                _idoSetColor(pIDOInkAttrChg, &lev, CLR_PALEGRAY);
                _idoDrawStroke(idoStrokeChanged, &lev, hps);
                fNewInkAttr = FALSE;
             }
             else
             {
                /*****************************************************
                *  Get InkAttr from Doodle                           *
                *****************************************************/
                pIDOInkAttrChg = _idoGetInkAttr(pIDODoodle, &lev);
                if (lev._major != NO_EXCEPTION)
                {
                   somExceptionFree(&lev);
                }
                acData.nLWidth = _idoGetWidth(pIDOInkAttrChg, &lev);
                acData.nColor  = _idoGetColor(pIDOInkAttrChg, &lev);
                acData.nLType  = _idoGetLineType(pIDOInkAttrChg, &lev);
                acData.nSMix   = _idoGetMix(pIDOInkAttrChg, &lev);
                acData.nX = acData.nY = 0;

                /*****************************************************
                *  Create a InkAttr and associate with stroke        *
                *****************************************************/
                pIDOInkAttrChg = IDOInkAttrNew();

                _idoSetInkerData(pIDOInkAttrChg,
                                 &lev,
                                 &InkerData,
                                 NULL);
                _idoSetStrokeInkAttr(idoStrokeChanged,
                                     &lev,
                                     pIDOInkAttrChg);
                if (lev._major != NO_EXCEPTION)
                {
                   somExceptionFree(&lev);
                }
                _idoDrawStroke(idoStrokeChanged, &lev, hps);
                _idoSetMix(pIDOInkAttrChg, &lev, FM_OVERPAINT);
                _idoDrawStroke(idoStrokeChanged, &lev, hps);
                fNewInkAttr = TRUE;
             }

             if (fSelect)
             {
                WinPostMsg(hwnd,
                           WM_COMMAND,
                           MPFROMSHORT(IDO_CHANGE),
                           NULL);
             }
             else
             {
                /***********************************************************
                *  Give a message box to confirm deletion                  *
                ***********************************************************/
                if (WinMessageBox(HWND_DESKTOP, hwnd,
                                  "Do you want to remove this stroke?",
                                  "Confirmation", 0,
                                  MB_YESNO | MB_ICONQUESTION | MB_MOVEABLE)
                      == MBID_YES)
                {
                   /*******************************************************
                   *  Remove strokes from doodle                          *
                   *******************************************************/
                   _idoSetStrokeInkAttr(idoStrokeChanged, &lev, NULL);
                   _somFree(pIDOInkAttrChg);
                   _idoRemove(pIDODoodle,
                              &lev,
                              idoStrokeChanged);
                   if (lev._major != NO_EXCEPTION)
                   {
                      somExceptionFree(&lev);
                   }
                   idoStrokeChanged = NULLHANDLE;
                }
                else
                {
                   if (!fNewInkAttr)
                   {
                      /*****************************************************
                      *  Restore the color and set mix                     *
                      *****************************************************/
                      _idoSetWidth(pIDOInkAttrChg, &lev, acData.nLWidth);
                      _idoSetLineType(pIDOInkAttrChg, &lev, acData.nLType);
                      _idoSetColor(pIDOInkAttrChg, &lev, acData.nColor);
                      _idoSetMix(pIDOInkAttrChg, &lev, acData.nSMix);
                   }
                   else
                   {
                      /*****************************************************
                      *  Remove the InkAttr attached to this stroke.       *
                      *****************************************************/
                      _idoSetStrokeInkAttr(idoStrokeChanged, &lev, NULL);
                      _somFree(pIDOInkAttrChg);
                   }
                }
                fRemove = FALSE;
                WinSetPointer(HWND_DESKTOP, hPtr);
                WinInvalidateRect(hwnd, NULL, FALSE);
             }
         }   /* of if(fSelect || fRemove)  */
         else
         {
         /**************************************************************
         *  Addition of new stroke in doodle.                          *
         **************************************************************/
         ULONG   ulRedraw;

            ulBufLen = 0;
            /***********************************************************
            *  Add the stroke data to the doodle object.               *
            ***********************************************************/
            _idoAddLast(pIDODoodle, &lev, pIDOStroke);
            if (lev._major != NO_EXCEPTION)
            {
               somExceptionFree(&lev);
            }

            /***********************************************************
            *  Draw the doolde                                         *
            ***********************************************************/
            if (ulRedrawOpt == REDRAW_INK)
            {
               _idoSetInkerData(pIDOInker, &lev, &InkerDataInker, (PINKERDATA) NULL);
               if (lev._major != NO_EXCEPTION)
               {
                  somExceptionFree(&lev);
               }
               ulRedraw = IDO_INK_INKER;
            }
            else if (ulRedrawOpt == REDRAW_DEF)
            {
               ulRedraw = IDO_INK_DEFAULT;
            }
            else if (ulRedrawOpt == REDRAW_DOO)
            {
               ulRedraw = IDO_INK_DOODLE;
            }
            _idoInkerDraw(pIDODoodle, &lev, pIDOInker, ulRedraw);
            if (lev._major != NO_EXCEPTION)
            {
               somExceptionFree(&lev);
            }

            if (!fStrokeAdded)
            {
               /*********************************************************
               *  Enable MODIFY menu item after addition of a stroke.   *
               *********************************************************/
               WinSendMsg(hwndMenu,
                          MM_SETITEMATTR,
                          MPFROM2SHORT(IDO_SELECT, TRUE),
                          MPFROM2SHORT(MIA_DISABLED, FALSE));
               WinSendMsg(hwndMenu,
                          MM_SETITEMATTR,
                          MPFROM2SHORT(IDO_MRSTROKE, TRUE),
                          MPFROM2SHORT(MIA_DISABLED, FALSE));
               fStrokeAdded = TRUE;
            }
         }  /* of else */
         /***********************************************************
         *  Now, the stroke object copies my stroke data, so I can  *
         *  free it.                                                *
         ***********************************************************/
         free(pStrokeData);
         return((MRESULT) STK_STROKE_PROCESSED);
      }  /* of WM_STROKE */

      case WM_PAINT:
      {
      ULONG   ulRedraw;
      HPS     hpsPaint;

         hpsPaint = WinBeginPaint(hwnd, (HPS) NULL, NULL);
         GpiErase(hps);

         if (fFirst)
         {
            /* skip 1 line from top */
            ptl.x = cxChar;
            lYCoord = cyClient;
            for (i = 0; i < NLINES; i++)
            {
               lYCoord = lYCoord - cyChar;
               ptl.y = lYCoord;
               GpiCharStringAt(hpsPaint,
                               &ptl,
                               (LONG) strlen(achInit[i]),
                               achInit[i]);
            }
            fFirst = FALSE;
         }
         else
         {
            if (ulRedrawOpt == REDRAW_INK)
            {
               _idoSetInkerData(pIDOInker, &lev, &InkerDataInker, (PINKERDATA) NULL);
               if (lev._major != NO_EXCEPTION)
               {
                  somExceptionFree(&lev);
               }

               ulRedraw = IDO_INK_INKER;
            }
            else if (ulRedrawOpt == REDRAW_DEF)
            {
               ulRedraw = IDO_INK_DEFAULT;
            }
            else if (ulRedrawOpt == REDRAW_DOO)
            {
               ulRedraw = IDO_INK_DOODLE;
            }
            _idoInkerDraw(pIDODoodle, &lev, pIDOInker, ulRedraw);
            if (lev._major != NO_EXCEPTION)
            {
               somExceptionFree(&lev);
            }

            if (fRemove && idoStrokeChanged)
            {
                IDOInkAttr    *pInkAttrTmp;

                pInkAttrTmp = _idoGetStrokeInkAttr(idoStrokeChanged, &lev);
                _idoDrawStroke(idoStrokeChanged, &lev, hps);
                _idoSetMix(pInkAttrTmp, &lev, FM_OVERPAINT);
                _idoDrawStroke(idoStrokeChanged, &lev, hps);
                if (lev._major != NO_EXCEPTION)
                {
                   somExceptionFree(&lev);
                }
            }
         }
         WinEndPaint(hpsPaint);
         return((MRESULT) FALSE);
      }  /* of WM_PAINT */

      case WM_COMMAND:
      {
         SHORT    sMsg;

         sMsg = SHORT1FROMMP(mp1);
         switch(sMsg)
         {
            case IDO_MCDOODLE:
            {
               /***********************************************************
               *  To start a new doodle, remove any strokes it has        *
               ***********************************************************/
               RemoveStrokes(pIDODoodle);

               /*********************************************************
               *  (Re)set the offset to zero                            *
               *********************************************************/
               acData.nX = acData.nY = 0;
               _idoSetOffset(pIDODoodle,
                             &lev,
                             acData.nX,
                             acData.nY);
               if (lev._major != NO_EXCEPTION)
               {
                  somExceptionFree(&lev);
               }
               /*******************************************************
               * Disable SELECT and REMOVE stroke menuitems           *
               *******************************************************/
               WinSendMsg(hwndMenu,
                          MM_SETITEMATTR,
                          MPFROM2SHORT(IDO_SELECT, TRUE),
                          MPFROM2SHORT(MIA_DISABLED, MIA_DISABLED));
               WinSendMsg(hwndMenu,
                          MM_SETITEMATTR,
                          MPFROM2SHORT(IDO_MRSTROKE, TRUE),
                          MPFROM2SHORT(MIA_DISABLED, MIA_DISABLED));
               fModify = FALSE;
               fSelect = FALSE;
               fStrokeAdded = FALSE;
               if (!fErase)
               {
                  fErase = TRUE;
               }
               else
               {
                  WinInvalidateRect(hwnd, NULL, FALSE);
               }
               return (MRESULT) FALSE;
            }  /* of case IDO_MCDOODLE */

            case IDO_MDOODLE:
            case IDO_MINKER:
            {
               fSelect = FALSE;
               if (sMsg == IDO_MDOODLE)
               {
                  acData.fAttrChg = TRUE;
                  acData.nLWidth  = InkerDataDoodle.ulWidth;
                  acData.nColor   = InkerDataDoodle.lColor;
                  acData.nLType   = InkerDataDoodle.lLineType;
                  acData.nSMix    = InkerDataDoodle.lMix;
                  _idoGetOffset(pIDODoodle,
                                &lev,
                                &acData.nX,
                                &acData.nY);
                  if (lev._major != NO_EXCEPTION)
                  {
                     somExceptionFree(&lev);
                  }
               }
               else
               {
                  acData.fAttrChg = FALSE;
                  acData.nLWidth  = InkerDataInker.ulWidth;
                  acData.nColor   = InkerDataInker.lColor;
                  acData.nLType   = InkerDataInker.lLineType;
                  acData.nSMix    = InkerDataInker.lMix;
               }

               if(WinDlgBox(HWND_DESKTOP,
                            hwnd,
                            AttrDlgProc,
                            (HMODULE) NULLHANDLE,
                            IDD_ATTRIBUTE,
                            &acData))
               {
                   if (sMsg == IDO_MDOODLE)
                   {
                   IDOInkAttr *pIDOInkAttrTemp;

                       InkerDataDoodle.ulWidth   = acData.nLWidth;
                       InkerDataDoodle.lColor    = acData.nColor;
                       InkerDataDoodle.lLineType = acData.nLType;
                       InkerDataDoodle.lMix      = acData.nSMix;
                       _idoSetOffset(pIDODoodle,
                                     &lev,
                                     acData.nX,
                                     acData.nY);
                       if (lev._major != NO_EXCEPTION)
                       {
                          somExceptionFree(&lev);
                       }
                       acData.nX = acData.nY = 0;
                       pIDOInkAttrTemp = _idoGetInkAttr(pIDODoodle, &lev);
                       _idoSetInkerData(pIDOInkAttrTemp,
                                        &lev,
                                        &InkerDataDoodle,
                                        (PINKERDATA) NULL);
                       if (lev._major != NO_EXCEPTION)
                       {
                          somExceptionFree(&lev);
                       }
                   }
                   else
                   {
                       InkerDataInker.ulWidth   = acData.nLWidth;
                       InkerDataInker.lColor    = acData.nColor;
                       InkerDataInker.lLineType = acData.nLType;
                       InkerDataInker.lMix      = acData.nSMix;
                       _idoSetInkerData(pIDOInker,
                                        &lev,
                                        &InkerDataInker,
                                        (PINKERDATA) NULL);
                       if (lev._major != NO_EXCEPTION)
                       {
                          somExceptionFree(&lev);
                       }
                   }
                   WinInvalidateRect(hwnd, NULL, FALSE);
               }
               break;
            }  /* of case IDO_MDOODLE, IDO_MINKER */

            case IDO_REDRAW_DEF:
            case IDO_REDRAW_DOO:
            case IDO_REDRAW_INK:
            {
               switch(sMsg)
               {
                   case IDO_REDRAW_DEF:
                   {
                      ulRedrawOpt = REDRAW_DEF;
                      break;
                   }

                   case IDO_REDRAW_DOO:
                   {
                      ulRedrawOpt = REDRAW_DOO;
                      break;
                   }

                   case IDO_REDRAW_INK:
                   {
                      ulRedrawOpt = REDRAW_INK;
                      break;
                   }
               }

               fSelect = FALSE;

               /*********************************************************
               *  If any drawing option changed, change the check mark. *
               *********************************************************/
               WinSendMsg(hwndMenu, MM_SETITEMATTR,
                          MPFROM2SHORT(sRedrawMenu, TRUE),
                          MPFROM2SHORT(MIA_CHECKED, 0));
               WinSendMsg(hwndMenu, MM_SETITEMATTR,
                          MPFROM2SHORT(sMsg, TRUE),
                          MPFROM2SHORT(MIA_CHECKED, MIA_CHECKED));
               sRedrawMenu = sMsg;

               /***********************************************************
               *  Draw the doolde                                         *
               ***********************************************************/
               WinInvalidateRect(hwnd, NULL, FALSE);
               break;
            }  /* of case IDO_REDRAW_DEF, IDO_REDRAW_DOO, and IDO_REDRAW_INK */

            case IDO_SELECT:
            {
               fSelect = TRUE;
               PutString(hwnd, STR_3, FALSE);
               hPtr = WinQueryPointer(HWND_DESKTOP);
               WinSetPointer(HWND_DESKTOP, hCptr);
               break;
            }  /* of IDO_SELECT */

            case IDO_MRSTROKE:
            {
               fSelect = FALSE;
               /*********************************************************
               *  Remove a stroke from doodle                           *
               *********************************************************/
               fRemove = TRUE;
               PutString(hwnd, STR_7, FALSE);
               hPtr = WinQueryPointer(HWND_DESKTOP);
               WinSetPointer(HWND_DESKTOP, hCptr);
               break;
            }

            case IDO_CHANGE:
            {
            INKERDATA    InkerDataT;
            IDOInkAttr   *pIDOInkAttrTmp;

               if (fModify)
               {  /* There is user delected stroke to modify */
                  fModify = FALSE;
                  acData.fAttrChg = TRUE;
                  if(WinDlgBox(HWND_DESKTOP,
                               hwnd,
                               AttrDlgProc,
                               (HMODULE) NULLHANDLE,
                               IDD_ATTRIBUTE,
                               &acData))
                  {
                     /****************************************************
                     *  Change offset data if user selects OK button.    *
                     ****************************************************/
                     _idoSetStrokeOffset(idoStrokeChanged,
                                         &lev,
                                         acData.nX,
                                         acData.nY);
                     if (lev._major != NO_EXCEPTION)
                     {
                        somExceptionFree(&lev);
                     }
                     fNewInkAttr = FALSE;
                  }
                  acData.nX = acData.nY = 0;
                  pIDOInkAttrTmp = _idoGetStrokeInkAttr(idoStrokeChanged,
                                                        &lev);
                  if (lev._major != NO_EXCEPTION)
                  {
                     somExceptionFree(&lev);
                  }

                  /*****************************************************
                  *  Put the new/saved attributes. If user selects     *
                  *  CANCEL button, restore the attributes in acData   *
                  *  bacause, the selected stroke is given in gray.    *
                  *****************************************************/
                  if (!fNewInkAttr)
                  {
                     _idoSetColor(pIDOInkAttrTmp, &lev, acData.nColor);
                     _idoSetLineType(pIDOInkAttrTmp, &lev, acData.nLType);
                     _idoSetMix(pIDOInkAttrTmp, &lev, acData.nSMix);
                     _idoSetWidth(pIDOInkAttrTmp, &lev, acData.nLWidth);
                  }
                  else
                  {
                     /*****************************************************
                     *  Remove the InkAttr attached to this stroke.       *
                     *****************************************************/
                     _idoSetStrokeInkAttr(idoStrokeChanged,
                                          &lev,
                                          NULL);
                     if (lev._major != NO_EXCEPTION)
                     {
                        somExceptionFree(&lev);
                     }
                     _somFree(pIDOInkAttrTmp);
                  }
                  WinInvalidateRect(hwnd, NULL, FALSE);
                  fSelect = FALSE;
               }
               else
               {
                   PutString(hwnd, STR_6, TRUE);
                   return((MRESULT) FALSE);
               }
               return (MRESULT) FALSE;
            }  /* of case IDO_CHANGE */

            case IDO_EXIT:
               WinPostMsg(hwnd, WM_CLOSE, NULL, NULL);
               return (MRESULT) FALSE;
         }

         return (MRESULT) FALSE;
      }  /* of case WM_COMMAND */

      case WM_MOUSEMOVE:
      {
         /*****************************************************************
         *  Handlde this message to keep the arrow or cross pointer       *
         *****************************************************************/
         if (fRemove)
         {
            WinSetPointer(HWND_DESKTOP, hCptr);
         }
         break;
      }

      case WM_CLOSE:
         /******************************************************************
         *  Clean up all allocated memory for all objects and stroke data  *
         ******************************************************************/
         GpiDestroyPS(hps);
         WinDestroyPointer(hCptr);
         RemoveStrokes(pIDODoodle);
         _somFree(pIDOInker);
         _somFree(pIDODoodle);

         /***  GO THRU  ***/

      default:
         return(WinDefWindowProc(hwnd, msg, mp1, mp2));

   } /* end of switch */

   return((MRESULT) FALSE);
}  /* of ClientWinProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) */

MRESULT EXPENTRY AttrDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
INT               i, j;
CHAR              achTemp[80];
static ATTRCHG    *pAC;
static INKERDATA  *pInkerData;

   switch(msg)
   {
      case WM_INITDLG:
      {
         pAC = (ATTRCHG *) PVOIDFROMMP(mp2);

         if (!pAC->fAttrChg)
         {
            pAC->nX = pAC->nY = 0;
            for (i = X_OFFSET; i < OFFSETTEXT + 1; i++)
            {
               WinEnableControl(hwnd, i, FALSE);
            }
         }
         /**********************************************************
         *  Set for line attributes of color, line width, set mix, *
         *  line type, and offsets.                                *
         **********************************************************/
         for (i = 0; i < NCOLOR; i++)
         {
            if (pAC->nColor == LColor[i].nLineColor)
            {
               j = LColor[i].nLColor;
               break;
            }
         }
         WinSendDlgItemMsg(hwnd, j, BM_SETCHECK,
                           MPFROM2SHORT(TRUE, 0),
                           MPFROMSHORT(0));

         for (i = 0; i < NWIDTH; i++)
         {
            if (pAC->nLWidth == LWidth[i].nLineWidth)
            {
               j = LWidth[i].nLWidth;
               break;
            }
         }
         WinSendDlgItemMsg(hwnd, j, BM_SETCHECK,
                           MPFROM2SHORT(TRUE, 0),
                           MPFROMSHORT(0));

         for (i = 0; i < NTYPE; i++)
         {
            if (pAC->nLType == LType[i].nLineType)
            {
               j = LType[i].nLType;
               break;
            }
         }
         WinSendDlgItemMsg(hwnd, j, BM_SETCHECK,
                           MPFROM2SHORT(TRUE, 0),
                           MPFROMSHORT(0));

         for (i = 0; i < NSETMIX; i++)
         {
            if (pAC->nSMix == LSetMix[i].nLineSMix)
            {
               j = LSetMix[i].nLSMix;
               break;
            }
         }
         WinSendDlgItemMsg(hwnd, j, BM_SETCHECK,
                           MPFROM2SHORT(TRUE, 0),
                           MPFROMSHORT(0));

         WinSetDlgItemShort(hwnd, X_OFFSET, (SHORT) pAC->nX, TRUE);
         WinSetDlgItemShort(hwnd, Y_OFFSET, (SHORT) pAC->nY, TRUE);
         break;
      }  /* of case WM_INITDLG */

      case WM_COMMAND:
      {
         switch(SHORT1FROMMP(mp1))
         {
            case IDD_OK:
            {
               /************************************************************
               *  Query the user set attributes of color, line width, line *
               *  type, and set mix in dialog box and then convert the     *
               *  values into the ones defined in pmgpi.h.                 *
               ************************************************************/
               /* First check line color */
               for (i = IDC_WHITE; i < IDC_PALEGRAY + 1; ++i)
               {
                  if((INT) WinSendDlgItemMsg(hwnd, i, BM_QUERYCHECK,
                                             (MPARAM) 0,
                                             (MPARAM) 0) == 1)
                     {
                        for (j = 0; j < NCOLOR; j++)
                        {
                           if (LColor[j].nLColor == i)
                           {
                              nColor = LColor[j].nLineColor;
                              break;
                           }
                        }
                        break;
                     }
               }

               /* Next check line width */
               for (i = IDLW_1; i < IDLW_2 + 1; ++i)
               {
                  if((INT) WinSendDlgItemMsg(hwnd, i, BM_QUERYCHECK,
                                             (MPARAM) 0,
                                             (MPARAM) 0) == 1)
                     {
                        for (j = 0; j < NWIDTH; j++)
                        {
                           if (LWidth[j].nLWidth == i)
                           {
                              nWidth = LWidth[j].nLineWidth;
                              break;
                           }
                        }
                        break;
                     }
               }

               /* Next check line type */
               for (i = IDLT_DEFAULT; i < IDLT_INVISIBLE + 1; ++i)
               {
                  if((INT) WinSendDlgItemMsg(hwnd, i, BM_QUERYCHECK,
                                             (MPARAM) 0,
                                             (MPARAM) 0) == 1)
                     {
                        for (j = 0; j < NTYPE; j++)
                        {
                           if (LType[j].nLType == i)
                           {
                              nType = LType[j].nLineType;
                              break;
                           }
                        }
                        break;
                     }
               }

               /* Finally check line set mixe */
               for (i = IDSM_DEFAULT; i < IDSM_ONE + 1; ++i)
               {
                  if((INT) WinSendDlgItemMsg(hwnd, i, BM_QUERYCHECK,
                                             (MPARAM) 0,
                                             (MPARAM) 0) == 1)
                     {
                        for (j = 0; j < NSETMIX; j++)
                        {
                           if (LSetMix[j].nLSMix == i)
                           {
                              nSMix = LSetMix[j].nLineSMix;
                              break;
                           }
                        }
                        break;
                     }
               }

               if (pAC->fAttrChg)
               {
                  SHORT  sTemp;

                  WinQueryDlgItemShort(hwnd, X_OFFSET, &sTemp, TRUE);
                  pAC->nX = (LONG) sTemp;
                  WinQueryDlgItemShort(hwnd, Y_OFFSET, &sTemp, TRUE);
                  pAC->nY = (LONG) sTemp;
               }

               /************************************************************
               *  Update InkerData to have correct attributes              *
               ************************************************************/
               pAC->nColor  = nColor;
               pAC->nLWidth = nWidth;
               pAC->nSMix   = nSMix;
               pAC->nLType  = nType;

               WinDismissDlg(hwnd, TRUE);
               return((MRESULT) FALSE);
            }  /* of case IDD_OK */

            case IDD_CANCEL:
            {
               WinDismissDlg(hwnd, FALSE);
               return((MRESULT) FALSE);
            }
         }
         break;
      }  /* of case WM_COMMAND */

      default:
         return(WinDefDlgProc(hwnd, msg, mp1, mp2));
   }
}  /* of AttrDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2) */

/******************************************************************************
*                                                                             *
* Function Name: RemoveStrokes                                                *
*                                                                             *
* Description :                                                               *
*                                                                             *
*   Remove strokes in doodle                                                  *
*                                                                             *
* Input: hwnd    : Window handle                                              *
*        nStr_ID : String ID                                                  *
*        sts     : TRUE for Error title, FALSE for Info title                 *
*                                                                             *
* Output:                                                                     *
*                                                                             *
******************************************************************************/
VOID RemoveStrokes(IDODoodle *pIDODoodle)
{
IDOStroke *idoS;

   if (pIDODoodle)
   {
       /*******************************************
       *  Remove any existing stroke and doodle.  *
       *******************************************/
       idoS = _idoFirst(pIDODoodle, &lev);
       if (lev._major != NO_EXCEPTION)
       {
          somExceptionFree(&lev);
       }
       while (idoS)
       {
          IDOInkAttr    *pInkAttrTmp;

          /******************************************************
          *  If an Ink Attribute is associated with this stroke *
          *  object, free the Ink Attribute object first.       *
          ******************************************************/
          pInkAttrTmp = _idoGetStrokeInkAttr(idoS, &lev);
          if (lev._major != NO_EXCEPTION)
          {
             somExceptionFree(&lev);
          }
          if (pInkAttrTmp)
          {
             _idoSetStrokeInkAttr(idoS, &lev, NULL);
             if (lev._major != NO_EXCEPTION)
             {
                somExceptionFree(&lev);
             }
             _somFree(pInkAttrTmp);
          }

          idoS = _idoAfter(pIDODoodle, &lev, idoS);
          if (lev._major != NO_EXCEPTION)
          {
             somExceptionFree(&lev);
          }
       }  /* of while(!idoS)    */

       /********************************************************
       *  Now, free all stroke objects in the doodle object. *
       ********************************************************/
       _idoFreeStrokes(pIDODoodle, &lev);
       if (lev._major != NO_EXCEPTION)
       {
          somExceptionFree(&lev);
       }
   }   /* of if(pIDODoodle)  */
}  /* of RemoveStrokes(IDODoodle *pIDODoodle) */

/******************************************************************************
*                                                                             *
* Function Name: PutString                                                    *
*                                                                             *
* Description :                                                               *
*                                                                             *
*   Load string by WinLoadString.                                             *
*                                                                             *
* Input: hwnd    : Window handle                                              *
*        nStr_ID : String ID                                                  *
*        sts     : TRUE for Error title, FALSE for Info title                 *
*                                                                             *
* Output:                                                                     *
*                                                                             *
******************************************************************************/
VOID  PutString(HWND hwnd, ULONG nStr_ID, BOOL sts)
{
CHAR       achMsg[80];

    WinLoadString(hab, 0UL, nStr_ID, 80, achMsg);
    WinMessageBox( HWND_DESKTOP,
                   hwnd,
                   achMsg,
                   sts ? NULL : "Information",
                   1,
                   MB_OK );
}
