/***************************************************************************
*                                                                          *
*   Name:  IDRSAMP.C                                                       *
*                                                                          *
*   Copyright : COPYRIGHT IBM CORPORATION, 1995                            *
*               LICENSED MATERIAL - PROGRAM PROPERTY OF IBM                *
*                                                                          *
*   Description:                                                           *
*      This program gives examples of using the IDR Text class.            *
*      User can set the reco metrics, symbol set, and language model.      *
*                                                                          *
*   Execution Instruction: IDRSAMP                                         *
*                                                                          *
*   Software Requirement: IBM WARP and Pen for OS/2 V1.1                   *
*                                                                          *
*  DISCLAIMER OF WARRANTIES.  The following [enclosed] code is             *
*      sample code created by IBM Corporation. This sample code is not     *
*      part of any standard or IBM product and is provided to you solely   *
*      for  the purpose of assisting you in the development of your        *
*      applications.  The code is provided "AS IS", without                *
*      warranty of any kind.  IBM shall not be liable for any damages      *
*      arising out of your use of the sample code, even if they have been  *
*      advised of the possibility of such damages.                         *
*                                                                          *
*                                                                          *
*   ATTENTION                                                              *
*      For each SOM method call, the returned code checking is strongly    *
*      recommended. The result can be checked as follow:                   *
*                                                                          *
*          if (lev._major != NO_EXCEPTION)                                 *
*          {                                                               *
*             rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;    *
*             Check the return code here;                                  *
*             somFreeException(&lev);                                      *
*          }                                                               *
*      In this program all returned codes (rc) are not checked.            *
*                                                                          *
***************************************************************************/
#define  INCL_WIN
#define  INCL_GPI
#define  INCL_DOS
#include <os2.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <som.h>

/* include the header file for the IDR classes */
#define INCL_IDR
#include <penpm.h>

#include "idrsamp.h"

int main ( VOID )
{
HMQ   hmq;
QMSG  qmsg;
ULONG flCreate;
CHAR  pchTemp;
LONG  i, j;

   hab = WinInitialize ( 0 );
   hmq = WinCreateMsgQueue ( hab, 0 );

   /* Initialize my private environment */
   SOM_InitEnvironment( &lev );

   WinRegisterClass ( hab,
                      "IDRclient",
                      ClientWinProc,
                      CS_SIZEREDRAW,
                      0 );

   flCreate = FCF_STANDARD & ~FCF_ACCELTABLE;

   hwndFrame = WinCreateStdWindow ( HWND_DESKTOP,
                                    WS_VISIBLE,
                                    &flCreate,
                                    "IDRclient",
                                    "IDR Window",
                                    0UL,
                                    0UL,
                                    ID_IDRSAMP,
                                    NULL );

   while ( WinGetMsg ( hab, &qmsg, (HWND)0, 0, 0 ) )
         WinDispatchMsg  ( hab, &qmsg );

   /* Clean up my private environment */
   SOM_UninitEnvironment( &lev );

   WinDestroyWindow ( hwndFrame );
   WinDestroyMsgQueue ( hmq );
   WinTerminate ( hab );
   return 0;
}

/************************************************************************
*                                                                       *
* Function Name: ClientWinProc                                          *
*                                                                       *
* Description :                                                         *
*                                                                       *
*   Main window procedure for this IDR sample program                   *
*                                                                       *
************************************************************************/
MRESULT EXPENTRY ClientWinProc ( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
        HPS             hps;
static  IDRGesture      *IDRGestureObj;       /* Gesture object for testing */
static  IDRText         *IDRTextObj;          /* Text object for testing    */
static  IDR_RECOMETRICS idrMetrics;
static  BOOL            fFirst = TRUE;
static  LONG            fMode;
static  ULONG           ulTextRecoForm, ulGestureRecoForm;
static  PRECODATA       pRDB;
static  HWND            hwndMenu;
        LONG            i, j;
static  LONG            cyChar, cxChar, cyClient;
static  SIZEL           sizel;
static  ULONG           ulNumDev, ulXPels, ulYPels;
        POINTL          ptl;
        LONG            lYCoord;

   switch( msg )
   {
      case WM_TOUCHDOWN:
      {
         WinSendMsg(hwndMenu,
                    MM_SETITEMATTR,
                    MPFROM2SHORT(ID_TEXT, TRUE),
                    MPFROM2SHORT(MIA_DISABLED, MIA_DISABLED));
         WinSendMsg(hwndMenu,
                    MM_SETITEMATTR,
                    MPFROM2SHORT(ID_GESTURE, TRUE),
                    MPFROM2SHORT(MIA_DISABLED, MIA_DISABLED));
         WinSendMsg(hwndMenu,
                    MM_SETITEMATTR,
                    MPFROM2SHORT(ID_SETS_MENU, TRUE),
                    MPFROM2SHORT(MIA_DISABLED, MIA_DISABLED));
         return (MRESULT)(TDN_INFINITE | TDN_LEAVE_INK);
      }

      case WM_STROKE:
      {
         if (fMode & TEXT_MODE)
         {
             _idrAddStroke( IDRTextObj,
                            &lev,
                            NULL,
                            hwnd );
             if (lev._major != NO_EXCEPTION)
             {
                rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                somExceptionFree(&lev);
             }
         }
         if (fMode & GESTURE_MODE)
         {
             _idrAddStroke( IDRGestureObj,
                            &lev,
                            NULL,
                            hwnd );
             if (lev._major != NO_EXCEPTION)
             {
                rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                somExceptionFree(&lev);
             }
         }
         return (MRESULT) STK_STROKE_PROCESSED;
      }

      case IDR_RESULT:
      {
         WinDlgBox( hwnd, hwnd, ResultDlgProc,
                    0, ID_RESULT_DLG, mp2 );
         return (MRESULT)FALSE;
      }  /* of IDR_RESULT */

      case WM_SIZE:
      {
         /***********************************************************
         *  Get the size of the client window                       *
         ***********************************************************/
         sizel.cx = SHORT1FROMMP(mp2);
         sizel.cy = cyClient = SHORT2FROMMP(mp2);
         return((MRESULT) FALSE);
      }  /* of case WM_SIZE */

      case WM_COMMAND:
      {
         switch( SHORT1FROMMP( mp1 ) )
         {
            case ID_TEXT:
            {
               if (fMode & TEXT_MODE)
               {
                  fMode &= ~TEXT_MODE;
                  WinSendMsg(hwndMenu, MM_SETITEMATTR,
                             MPFROM2SHORT(ID_TEXT, TRUE),
                             MPFROM2SHORT(MIA_CHECKED, 0));
                  ulTextRecoForm = idrMetrics.ulRecoForm;
                  WinSendMsg(hwndMenu,
                             MM_SETITEMATTR,
                             MPFROM2SHORT(ID_SETS_MENU, TRUE),
                             MPFROM2SHORT(MIA_DISABLED, MIA_DISABLED));
               }
               else
               {
                  fMode |= TEXT_MODE;
                  WinSendMsg(hwndMenu, MM_SETITEMATTR,
                             MPFROM2SHORT(ID_TEXT, TRUE),
                             MPFROM2SHORT(MIA_CHECKED, MIA_CHECKED));
                  WinSendMsg(hwndMenu,
                             MM_SETITEMATTR,
                             MPFROM2SHORT(ID_SETS_MENU, TRUE),
                             MPFROM2SHORT(MIA_DISABLED, FALSE));
               }
               if (!((fMode & TEXT_MODE) || (fMode & GESTURE_MODE)))
               {
                  PutString(hwnd, STR_9, FALSE);
                  WinPostMsg(hwnd, WM_COMMAND, MPFROMSHORT(ID_TEXT), NULL);
                  return(FALSE);
               }
               WinInvalidateRect(hwnd, NULL, FALSE);
               break;
            }

            case ID_GESTURE:
            {

               Text_GetMetrics(&idrMetrics);
               if (fMode & GESTURE_MODE)
               {
                  fMode &= ~GESTURE_MODE;
                  WinSendMsg(hwndMenu, MM_SETITEMATTR,
                             MPFROM2SHORT(ID_GESTURE, TRUE),
                             MPFROM2SHORT(MIA_CHECKED, 0));
                  idrMetrics.ulRecoForm &= ~ulGestureRecoForm;
               }
               else
               {
                  fMode |= GESTURE_MODE;
                  WinSendMsg(hwndMenu, MM_SETITEMATTR,
                             MPFROM2SHORT(ID_GESTURE, TRUE),
                             MPFROM2SHORT(MIA_CHECKED, MIA_CHECKED));
               }

               if (!fMode)
               {
                  PutString(hwnd, STR_9, TRUE);
                  return(FALSE);
               }
               WinInvalidateRect(hwnd, NULL, FALSE);
               return(FALSE);
            }

            case ID_METRICS:
            {
               /* Setup IDR_RECOMETRICS */
               Text_GetMetrics(&idrMetrics);
               if (WinDlgBox( HWND_DESKTOP,
                              hwnd,
                              MetricsDlgProc,
                              0,
                              ID_METRICS_DLG,
                              &idrMetrics ))
               {
                  Text_SetMetrics(hwnd, IDRTextObj, ulNumDev, ulXPels, ulYPels, &idrMetrics);
                  WinInvalidateRect(hwnd, NULL, FALSE);
               }
               return( FALSE );
            }  /* of ID_METRICS */

            case ID_SYMSETS:
            {
               /* Setup Symbol Sets */
               if (WinDlgBox( HWND_DESKTOP,
                              hwnd,
                              SymsetsDlgProc,
                              0,
                              ID_SYMSETS_DLG,
                              &idrSymset ))
               {
                  _idrSetSymbolSubset(IDRTextObj, &lev, &idrSymset);
                  if (lev._major != NO_EXCEPTION)
                  {
                     rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                     somExceptionFree(&lev);
                  }
               }
               WinInvalidateRect(hwnd, NULL, FALSE);
               return( FALSE );
            }  /* of ID_SYMSETS */

            case ID_LM:
            {
               /* Dictionary */
               if (WinDlgBox( HWND_DESKTOP,
                              hwnd,
                              DictionaryDlgProc,
                              0,
                              ID_LM_DLG,
                              &lModel ))
               {
                   _idrSetDictionaryCoercion(IDRTextObj,
                                             &lev,
                                             lModel.ulCoercion);
                   if (lev._major != NO_EXCEPTION)
                   {
                      rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                      somExceptionFree(&lev);
                   }
                   _idrSetDictionaryPolicy(IDRTextObj,
                                           &lev,
                                           lModel.ulPolicy);
                   if (lev._major != NO_EXCEPTION)
                   {
                      rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                      somExceptionFree(&lev);
                   }
               }
               WinInvalidateRect(hwnd, NULL, FALSE);
               return( FALSE );
            }  /* of case ID_LM */

            case ID_RECO:
            {
               if (fMode & GESTURE_MODE)
               {
                  pRDB = _idrRecoStrokes(IDRGestureObj, &lev);
                  if (lev._major != NO_EXCEPTION)
                  {
                     rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                     somExceptionFree(&lev);
                  }
                  else
                  {
                     WinDlgBox(hwnd, hwnd, GestureDlgProc,
                               0, ID_GESTURE_DLG, pRDB);
                  }
               }
               if (fMode & TEXT_MODE)
               {
                  _idrRecoInput( IDRTextObj,
                                 &lev,
                                 hwnd,
                                 1 );
                  if (lev._major != NO_EXCEPTION)
                  {
                     rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                     somExceptionFree(&lev);
                  }
               }
               WinSendMsg(hwndMenu,
                          MM_SETITEMATTR,
                          MPFROM2SHORT(ID_TEXT, TRUE),
                          MPFROM2SHORT(MIA_DISABLED, FALSE));
               WinSendMsg(hwndMenu,
                          MM_SETITEMATTR,
                          MPFROM2SHORT(ID_GESTURE, TRUE),
                          MPFROM2SHORT(MIA_DISABLED, FALSE));
               WinSendMsg(hwndMenu,
                          MM_SETITEMATTR,
                          MPFROM2SHORT(ID_SETS_MENU, TRUE),
                          MPFROM2SHORT(MIA_DISABLED, FALSE));

               WinInvalidateRect( hwnd, NULL, FALSE );
               return( FALSE );
            }  /* of case ID_RECO */

            case ID_CLEAR:
            {
               _idrClearStrokes( IDRTextObj, &lev );
               if (lev._major != NO_EXCEPTION)
               {
                  rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                  somExceptionFree(&lev);
               }
               WinInvalidateRect( hwnd, NULL, FALSE );
               return( FALSE );
            }  /* of case ID_CLEAR */

            case ID_QUIT:
            {
               WinPostMsg( hwnd, WM_CLOSE, 0, 0 );
               return( FALSE );
            }  /* of case ID_QUIT */
         }
         break;
      }  /* of case WM_COMMAND */

      case WM_PAINT:
      {
         hps = WinBeginPaint ( hwnd , (HPS) 0, NULL );
         GpiErase ( hps );
         if (fFirst)
         {
            /* skip 1 line from top */
            ptl.x = cxChar;
            lYCoord = cyClient;
            for (i = 0; i < NUM_INITIAL_LINES; i++)
            {
               lYCoord = lYCoord - cyChar;
               ptl.y = lYCoord;
               GpiCharStringAt(hps,
                               &ptl,
                               (LONG) strlen(achInit[i]),
                               achInit[i]);
            }
            fFirst = FALSE;
         }  /* of if(fFirst) */
         else
         {
            if (fMode & TEXT_MODE)
            {
               if (idrMetrics.ulRecoForm & IDR_RECOFORM_BOXED)
               {
                  GiveBoxedForm(hwnd,
                                hps,
                                sizel,
                                idrMetrics);
               }
               else if (idrMetrics.ulRecoForm & IDR_RECOFORM_LINED)
               {
                  GiveLinedForm(hwnd,
                                hps,
                                sizel,
                                idrMetrics);
               }

               /***************************************************************
               *  else if (idrMetrics.ulRecoForm & IDR_RECOFORM_UNLINED)      *
               *  {                                                           *
               *     For unlined form, user can write any place in window.    *
               *  }                                                           *
               ***************************************************************/
            }

         }  /* of else */
         WinEndPaint ( hps );
         return (MRESULT) NULL;
      }  /* of WM_PAINT */

      case WM_CREATE:
      {
        PIDR_SYM    pidrSym = NULL;
        ULONG       ulSymbols, ulSymCnt;
        FONTMETRICS fm;

         /*******************************************************************
         *  The APIs being used in this procedure depend on the PenPM sub-  *
         *  system being correctly initialized.  Thus we check to make sure *
         *  PenPM is up before we do anything else.                         *
         *******************************************************************/
         if(WrtWaitActive(WRT_IMMEDIATE_RETURN))
         {
            /*************************************************************
            *  If PenPM is NOT active then put up a message box and      *
            *  return TRUE to terminate the program.                     *
            *************************************************************/
            PutString(hwnd, STR_1, TRUE);
            return((MRESULT) TRUE);
         }

         /************************************************************
         *  To change from screen metrics to sensor metrics and back *
         *  the device info is needed.                               *
         ************************************************************/
         WrtSi.cbStructSize = sizeof(WRTSYSTEMINFO);
         WrtQuerySystemCaps(&WrtSi);
         ulXPels = WrtSi.ulDisplayXPels;
         ulYPels = WrtSi.ulDisplayYPels;
         ulNumDev  = 0;
         WrtLi.cbStructSize = sizeof(WRTLOCATORDEVINFO);
         WrtQueryLocatorCaps( &ulNumDev, &WrtLi );

         fMode = TEXT_MODE;
         hwndMenu = WinWindowFromID(WinQueryWindow(hwnd, QW_PARENT),
                                    FID_MENU);
         WinSendMsg(hwndMenu, MM_SETITEMATTR,
                    MPFROM2SHORT(ID_TEXT, TRUE),
                    MPFROM2SHORT(MIA_CHECKED, MIA_CHECKED));

         /***********************************************************
         *  Get font information from presentation space            *
         ***********************************************************/
         hps = WinGetPS(hwnd);
         GpiQueryFontMetrics(hps, (LONG) sizeof(fm), &fm);
         cxChar = fm.lEmInc;
         cyChar = fm.lMaxDescender + fm.lLowerCaseAscent;
         WinReleasePS(hps);

         /* Create Object for IDRText and IDRGesture*/
         IDRTextObj = IDRTextNew();
         IDRGestureObj = IDRGestureNew();
         _idrSetErrorHwnd(IDRTextObj, &lev, hwnd);
         _idrSetErrorHwnd(IDRGestureObj, &lev, hwnd);

         /* Put default values for the IDR_RECOMETRICS data */
         idrMetrics.cbSize = sizeof(IDR_RECOMETRICS);
         ulTextRecoForm = idrMetrics.ulRecoForm = IDR_RECOFORM_UNLINED;
         ulGestureRecoForm = IDR_RECOFORM_SINGLE_SYM;
         idrMetrics.ulSpaceDisable = IDR_SPACE_ENABLE | IDR_NEWLINE_ENABLE;
         idrMetrics.ulDirection = IDR_DIRECT_RIGHT_DOWN;
         idrMetrics.ptlOrigin.x = 0;
         idrMetrics.ptlOrigin.y = 0;
         idrMetrics.ptlOffset.x = 30;
         idrMetrics.ptlOffset.y = 30;
         idrMetrics.ptlBox.x    = 30;
         idrMetrics.ptlBox.y    = 36;
         idrMetrics.ptlGray.x   = 10;
         idrMetrics.ptlGray.y   = 10;
         idrMetrics.lOffsetBline = 10;
         idrMetrics.ulRowCount  = 3;
         idrMetrics.ulColCount  = 7;

         /* Set metrics */
         if (!Text_SetMetrics(hwnd, IDRTextObj, ulNumDev, ulXPels, ulYPels, &idrMetrics))
         {
            return((MRESULT) TRUE);
         }

         /***********************************************************
         *  Setup All symbol Sets                                   *
         ***********************************************************/
         idrSymset.cbSize = sizeof(IDR_SYMSET);
         ulSymbols = IDR_SYMSET_FULL;
         idrSymset.ulSetMask = ulSymbols;
         idrSymset.pSyms = NULL;
         idrSymset.ulSymCnt = 0;

         /***********************************************************
         *  Setup Language Model                                    *
         ***********************************************************/
         lModel.ulCoercion = IDR_LM_DEFAULT;       /* Use defaults */
         lModel.ulPolicy = IDR_TXTLM_DEFAULT;

         return (MRESULT) FALSE;
      }  /* of case WM_CREATE */

      case IDR_ERROR:
      {
      IDF_PROB  *pidfProb;

         /***************************************************************
         *  There is a problem in recognition engine and ther is NO way *
         *  to solve this problem here. Just give the error infomation  *
         *  and exit.                                                   *
         ***************************************************************/

         pidfProb = PVOIDFROMMP(mp2);
         WinDlgBox(HWND_DESKTOP,
                   hwnd,
                   IDRErrorDlgProc,
                   0,
                   ID_ERROR_DLG,
                   pidfProb);
         WinPostMsg(hwnd, WM_CLOSE, NULL, NULL);
         break;
      }

      case WM_CLOSE:
      {
         if (IDRTextObj)    _somFree(IDRTextObj);
         if (IDRGestureObj) _somFree(IDRGestureObj);

         /* ***** GO THROUGH ***** */
      }  /* of WM_CLOSE */

      default:
         return WinDefWindowProc( hwnd, msg, mp1, mp2 );
   }  /* of switch( msg ) */

   return (MRESULT) FALSE;
}  /* of ClientWinProc */

/************************************************************************
*                                                                       *
* Function Name: MetricsDlgProc                                         *
*                                                                       *
* Description :                                                         *
*                                                                       *
*   Dialog procedure for Metrics                                        *
*                                                                       *
************************************************************************/
MRESULT EXPENTRY MetricsDlgProc ( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
ULONG                  size;
CHAR                   ptxtbuf[CCHMAXPATH];
static IDR_RECOMETRICS *pMetrics;
LONG                   i;

    switch( msg )
    {
        case WM_COMMAND:
           switch( SHORT1FROMMP( mp1 ) )
           {
              case ID_METRICS_DONE:
              {
                 WinQueryDlgItemShort(hwnd, ID_ORIGIN_X, (PSHORT) &pMetrics->ptlOrigin.x, FALSE);
                 WinQueryDlgItemShort(hwnd, ID_ORIGIN_Y, (PSHORT) &pMetrics->ptlOrigin.y, FALSE);
                 WinQueryDlgItemShort(hwnd, ID_OFFSET_X, (PSHORT) &pMetrics->ptlOffset.x, FALSE);
                 WinQueryDlgItemShort(hwnd, ID_OFFSET_Y, (PSHORT) &pMetrics->ptlOffset.y, FALSE);
                 WinQueryDlgItemShort(hwnd, ID_BOX_X,    (PSHORT) &pMetrics->ptlBox.x, FALSE);
                 WinQueryDlgItemShort(hwnd, ID_BOX_Y,    (PSHORT) &pMetrics->ptlBox.y, FALSE);
                 WinQueryDlgItemShort(hwnd, ID_GRAY_X,   (PSHORT) &pMetrics->ptlGray.x, FALSE);
                 WinQueryDlgItemShort(hwnd, ID_GRAY_Y,   (PSHORT) &pMetrics->ptlGray.y, FALSE);
                 WinQueryDlgItemShort(hwnd, ID_BASELINE, (PSHORT) &pMetrics->lOffsetBline, FALSE);
                 WinQueryDlgItemShort(hwnd, ID_ROW,      (PSHORT) &pMetrics->ulRowCount, FALSE);
                 WinQueryDlgItemShort(hwnd, ID_COL,      (PSHORT) &pMetrics->ulColCount, FALSE);
                 for (i = ID_FORM_BOXED; i < ID_FORM_UNLINED + 1 ; i++)
                 {
                    if ((INT) WinSendDlgItemMsg(hwnd,
                                                i,
                                                BM_QUERYCHECK,
                                                (MPARAM) 0,
                                                (MPARAM) 0) == 1)
                    {
                       break;
                    }
                 }
                 switch(SHORT1FROMMP(i))
                 {
                    case ID_FORM_BOXED:
                       pMetrics->ulRecoForm = IDR_RECOFORM_BOXED;
                       break;

                    case ID_FORM_LINED:
                       pMetrics->ulRecoForm = IDR_RECOFORM_LINED;
                       break;

                    case ID_FORM_UNLINED:
                       pMetrics->ulRecoForm = IDR_RECOFORM_UNLINED;
                       break;
                 }
                 pMetrics->ulSpaceDisable =
                     WinSendDlgItemMsg(hwnd,
                                       ID_DISABLE_SPACE,
                                       BM_QUERYCHECK,
                                       NULL,
                                       NULL) ?
                        IDR_SPACE_DISABLE : IDR_SPACE_ENABLE;

                 pMetrics->ulSpaceDisable |=
                     WinSendDlgItemMsg(hwnd,
                                       ID_DISABLE_ENTER,
                                       BM_QUERYCHECK,
                                       NULL,
                                       NULL) ?
                        IDR_NEWLINE_DISABLE : IDR_NEWLINE_ENABLE;

                 WinDismissDlg( hwnd, TRUE );
                 break;
              }

              case ID_METRICS_CANCEL:
                 WinDismissDlg( hwnd, FALSE );
                 break;
           }
           return (MRESULT)FALSE;

        case WM_INITDLG:
        {
            SHORT  sTemp;

            pMetrics = PVOIDFROMMP(mp2);
            WinSetDlgItemShort(hwnd, ID_ORIGIN_X, (USHORT) pMetrics->ptlOrigin.x, FALSE);
            WinSetDlgItemShort(hwnd, ID_ORIGIN_Y, (USHORT) pMetrics->ptlOrigin.y, FALSE);
            WinSetDlgItemShort(hwnd, ID_OFFSET_X, (USHORT) pMetrics->ptlOffset.x, FALSE);
            WinSetDlgItemShort(hwnd, ID_OFFSET_Y, (USHORT) pMetrics->ptlOffset.y, FALSE);
            WinSetDlgItemShort(hwnd, ID_BOX_X,    (USHORT) pMetrics->ptlBox.x, FALSE);
            WinSetDlgItemShort(hwnd, ID_BOX_Y,    (USHORT) pMetrics->ptlBox.y, FALSE);
            WinSetDlgItemShort(hwnd, ID_GRAY_X,   (USHORT) pMetrics->ptlGray.x, FALSE);
            WinSetDlgItemShort(hwnd, ID_GRAY_Y,   (USHORT) pMetrics->ptlGray.y, FALSE);
            WinSetDlgItemShort(hwnd, ID_BASELINE, (USHORT) pMetrics->lOffsetBline, FALSE);
            WinSetDlgItemShort(hwnd, ID_ROW,      (USHORT) pMetrics->ulRowCount, FALSE);
            WinSetDlgItemShort(hwnd, ID_COL,      (USHORT) pMetrics->ulColCount, FALSE);
            sTemp = ID_FORM_UNLINED;
            if (pMetrics->ulRecoForm & IDR_RECOFORM_BOXED)
            {
               sTemp = ID_FORM_BOXED;
            }
            else if (pMetrics->ulRecoForm & IDR_RECOFORM_LINED)
            {
               sTemp = ID_FORM_LINED;
            }
            WinSendDlgItemMsg(hwnd,
                              sTemp,
                              BM_SETCHECK,
                              MPFROM2SHORT(TRUE, 0),
                              NULL);
            if (pMetrics->ulSpaceDisable & IDR_SPACE_DISABLE)
            {
               WinSendDlgItemMsg(hwnd, ID_DISABLE_SPACE, BM_SETCHECK,
                                 MPFROM2SHORT(TRUE, 0), NULL);
            }
            if (pMetrics->ulSpaceDisable & IDR_NEWLINE_DISABLE)
            {
               WinSendDlgItemMsg(hwnd, ID_DISABLE_ENTER, BM_SETCHECK,
                                 MPFROM2SHORT(TRUE, 0), NULL);
            }

            break;
        }

        default:
            return WinDefDlgProc( hwnd, msg, mp1, mp2 );
    }
    return(FALSE);
}   /* of MetricsDlgProc */

/************************************************************************
*                                                                       *
* Function Name: SymsetsDlgProc                                         *
*                                                                       *
* Description :                                                         *
*                                                                       *
*   Dialog procedure for Symbol Set                                     *
*                                                                       *
************************************************************************/
MRESULT EXPENTRY SymsetsDlgProc ( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
ULONG             ulSymbols, ulSymCnt;
static IDR_SYMSET *pIdrSymset;
static BOOL       fFullSym;
       PIDR_SYM   pIdrSym;
       LONG       i;

    switch( msg )
    {
        case WM_COMMAND:
           switch( SHORT1FROMMP( mp1 ) )
           {
              case ID_SYMSETS_DONE:
              {
                 ulSymbols = 0;
                 fFullSym = TRUE;
                 for (i = 0; i < NUM_SYMSETS ; i++)
                 {
                    if (WinSendDlgItemMsg(hwnd,
                                          i + ID_SYMSETS_ALPHALO,
                                          BM_QUERYCHECK,
                                          (MPARAM) 0,
                                          (MPARAM) 0))
                    {
                       ulSymbols |= ulSymSets[i][0];
                    }
                    else
                    {
                        fFullSym = FALSE;
                    }
                 }
                 if (fFullSym)
                 {
                    ulSymbols = IDR_SYMSET_FULL;
                 }

                 idrSymset.cbSize = sizeof(IDR_SYMSET);
                 idrSymset.ulSetMask = ulSymbols;
                 idrSymset.ulSymCnt = 0;
                 idrSymset.pSyms = NULL;
                 WinDismissDlg( hwnd, TRUE );
                 break;
              }

              case ID_SYMSETS_CANCEL:
                 WinDismissDlg( hwnd, FALSE );
                 break;
           }
           return (MRESULT)FALSE;

        case WM_CONTROL:
        {
        SHORT  sTemp;

           sTemp = SHORT1FROMMP(mp1);
           if ((sTemp >= ID_SYMSETS_ALPHALO) && (sTemp <= ID_SYMSETS_SPECIAL))
           {
              if (WinSendDlgItemMsg(hwnd, sTemp, BM_QUERYCHECK, 0, 0))
              {
                 fFullSym = TRUE;
                 for (i = 0; i < NUM_SYMSETS ; i++)
                 {
                    if (!WinSendDlgItemMsg(hwnd,
                                          i + ID_SYMSETS_ALPHALO,
                                          BM_QUERYCHECK,
                                          (MPARAM) 0,
                                          (MPARAM) 0))
                    {
                        fFullSym = FALSE;
                    }
                 }
                 if (fFullSym)
                 {
                    WinSendDlgItemMsg(hwnd,
                                      ID_SYMSETS_FULL,
                                      BM_SETCHECK,
                                      MPFROM2SHORT(TRUE, 0),
                                      NULL);
                 }
              }
              else
              {
                 WinSendDlgItemMsg(hwnd,
                                   ID_SYMSETS_FULL,
                                   BM_SETCHECK,
                                   MPFROM2SHORT(FALSE, 0),
                                   NULL);
              }
           }
           else if (sTemp == ID_SYMSETS_FULL)
           {
              if (WinSendDlgItemMsg(hwnd, sTemp, BM_QUERYCHECK, 0, 0))
              {
                 for (i = 0; i <= NUM_SYMSETS; i++)
                 {
                    WinSendDlgItemMsg(hwnd,
                                      i + ID_SYMSETS_ALPHALO,
                                      BM_SETCHECK,
                                      MPFROM2SHORT(TRUE, 0),
                                      NULL);
                 }
              }
           }
           break;
        }  /* of case WM_CONTROL */

        case WM_INITDLG:
        {
            pIdrSymset = PVOIDFROMMP(mp2);
            if (pIdrSymset->ulSetMask == IDR_SYMSET_FULL)
            {
               for (i = 0; i <= NUM_SYMSETS; i++)
               {
                  WinSendDlgItemMsg(hwnd,
                                    i + ID_SYMSETS_ALPHALO,
                                    BM_SETCHECK,
                                    MPFROM2SHORT(TRUE, 0),
                                    NULL);
               }
            }
            else
            {
               for (i = 0; i < NUM_SYMSETS; i++)
               {
                  if (pIdrSymset->ulSetMask & ulSymSets[i][0])
                  {
                     WinSendDlgItemMsg(hwnd,
                                       i + ID_SYMSETS_ALPHALO,
                                       BM_SETCHECK,
                                       MPFROM2SHORT(TRUE, 0),
                                       NULL);
                  }
               }
            }
            break;
        }

        default:
           return WinDefDlgProc( hwnd, msg, mp1, mp2 );
    }
    return(FALSE);
}   /* of SymsetsDlgProc */

/************************************************************************
*                                                                       *
* Function Name: DictionaryDlgProc                                      *
*                                                                       *
* Description :                                                         *
*                                                                       *
*   Dialog procedure for Language Model                                 *
*                                                                       *
************************************************************************/
MRESULT EXPENTRY DictionaryDlgProc ( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
ULONG              ulPolicy, ulCoercion;
static PLANGMODEL  plModel;
LONG               i;

    switch( msg )
    {
        case WM_COMMAND:
           switch( SHORT1FROMMP( mp1 ) )
           {
              case ID_LM_DONE:
              {
                 ulPolicy = ulCoercion = 0;
                 for (i = 0; i < NUM_COERCION ; i++)
                 {
                    if (WinSendDlgItemMsg(hwnd,
                                          i + ID_LM_ADVICE,
                                          BM_QUERYCHECK,
                                          (MPARAM) 0,
                                          (MPARAM) 0))
                    {
                       ulCoercion |= ulLMCoercion[i];
                    }
                 }
                 for (i = 0; i < NUM_POLICY ; i++)
                 {
                    if (WinSendDlgItemMsg(hwnd,
                                          i + ID_TXT_WRDCASE,
                                          BM_QUERYCHECK,
                                          (MPARAM) 0,
                                          (MPARAM) 0))
                    {
                       ulPolicy |= ulLMPolicy[i];
                    }
                 }
                 plModel->ulCoercion = ulCoercion;
                 plModel->ulPolicy   = ulPolicy;
                 WinDismissDlg( hwnd, TRUE );
                 break;
              }

              case ID_LM_CANCEL:
                 WinDismissDlg( hwnd, FALSE );
                 break;
           }
           return (MRESULT)FALSE;

        case WM_INITDLG:
        {
            plModel = PVOIDFROMMP(mp2);
            ulCoercion = plModel->ulCoercion;
            if (!ulCoercion)
            {
                WinSendDlgItemMsg(hwnd,
                                  ID_LM_NONE,
                                  BM_SETCHECK,
                                  MPFROM2SHORT(TRUE, 0),
                                  NULL);
            }
            else
            {
               for (i = 0; i < NUM_COERCION; i++)
               {
                   if (plModel->ulCoercion & ulLMCoercion[i])
                   {
                       WinSendDlgItemMsg(hwnd,
                                         i + ID_LM_ADVICE,
                                         BM_SETCHECK,
                                         MPFROM2SHORT(TRUE, 0),
                                         NULL);
                   }
               }
            }
            for (i = 0; i < NUM_POLICY; i++)
            {
                if (plModel->ulPolicy & ulLMPolicy[i])
                {
                    WinSendDlgItemMsg(hwnd,
                                      i + ID_TXT_WRDCASE,
                                      BM_SETCHECK,
                                      MPFROM2SHORT(TRUE, 0),
                                      NULL);
                }
            }
            break;
        }

        default:
           return WinDefDlgProc( hwnd, msg, mp1, mp2 );
    }
    return(FALSE);
}   /* of DictionaryDlgProc */

/************************************************************************
*                                                                       *
* Function Name: ResultDlgProc                                          *
*                                                                       *
* Description :                                                         *
*                                                                       *
*   Dialog procedure for Text result data                               *
*                                                                       *
************************************************************************/
MRESULT EXPENTRY ResultDlgProc ( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
static  IDRText  *pIDRTextResObj;
        IDR_RECOMETRICS   idrRMetrics;
        PHCLUSTER         pCls;
static  CHAR     achTop[132];      /* Top level result      */
        CHAR     achNext[132];     /* Next level result     */
        CHAR     achBox[64];
        ULONG    ulSize, ulTop, ulNext;
static  ULONG    ulBoxId;          /* Box number for Box type   */
static  ULONG    ulResultNo;       /* result number             */
static  SHORT    sMaxNumRes;      /* Maxinum number of results */
static  ULONG    ulClusterCnt;
static  CHAR     cBoxChar;
static  ULONG    ulMaxBox; /* max results allowed */
        PVOID    pBuf;
        SHORT    sBoxNo;
        LONG     i, j;

   switch( msg )
   {
      case WM_COMMAND:
      {
          switch( SHORT1FROMMP( mp1 ) )
          {

             case ID_DONE:
             {
                WinDismissDlg(hwnd, FALSE);
                break;
             }

             case ID_NEXT:
             {
             ULONG  ulResult;
             SHORT  sRes;
             BOOL   fResErr;

                /**********************************************************
                *  The cluster handles and the number of results of  each *
                *  cluster are  in clData[0..ulClusterCnt]                *
                **********************************************************/
                ++ulResultNo;
                if ((SHORT) ulResultNo == sMaxNumRes)
                {
                   ulResultNo = 0;
                }
                sRes = ulResultNo;
                ++sRes;
                j = 0;
                for (i = 0; i < ulClusterCnt; i++)
                {
                   fResErr = FALSE;
                   ulResult = ulResultNo;
                   if (ulResult > clData[i].ulNumRes)
                   {
                      ulResult = 0;
                   }

                   ulSize = 0;
                   somExceptionFree(&lev);
                   _idrGetResultInfo(pIDRTextResObj,
                                     &lev,
                                     clData[i].hCls,
                                     ulResult,
                                     NULL,
                                     &ulSize);
                   if (lev._major != NO_EXCEPTION)
                   {
                      rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                      if (rc == IDRERR_INVALID_RESULT_NUMBER)
                      {
                         /*************************************************
                         *  If the result number error is returned, get   *
                         *  the size of top result.                       *
                         *************************************************/
                         fResErr = TRUE;
                         ulSize = 0;
                         _idrGetResultInfo(pIDRTextResObj,
                                           &lev,
                                           clData[i].hCls,
                                           0,
                                           NULL,
                                           &ulSize);
                      }
                      somExceptionFree(&lev);
                   }

                   /*******************************************************
                   *  Put limit on the max number of results.             *
                   *******************************************************/
                   if (pBuf = malloc(ulSize))
                   {
                      ((PIDR_TXTCLUSTER) pBuf)->cbSize = sizeof(IDR_TXTCLUSTER);
                      _idrGetResultInfo(pIDRTextResObj,
                                        &lev,
                                        clData[i].hCls,
                                        fResErr ? 0 : ulResult,
                                        (PIDR_TXTCLUSTER) pBuf,
                                        &ulSize);
                      if (lev._major != NO_EXCEPTION)
                      {
                         rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                         somExceptionFree(&lev);
                      }
                      j += GetSymStringProc(((PIDR_TXTCLUSTER) pBuf)->pSyms,
                                            ((PIDR_TXTCLUSTER) pBuf)->ulSymCnt,
                                            &achNext[j]);
                      free(pBuf);
                   }
                   else
                   {
                       PutString(hwnd, STR_2, TRUE);
                       return((MRESULT) TRUE);
                   }
                }  /* of for (i = 0; i < ulClusterCnt; i++) */
                WinSetDlgItemShort(hwnd, NEXT_RES_NO, sRes, TRUE);
                WinSendDlgItemMsg(hwnd, NEXT_RES, EM_SETTEXTLIMIT,
                                  MPFROM2SHORT((SHORT) j, 0), NULL);
                WinSetDlgItemText(hwnd, NEXT_RES, achNext);

                break;
             }  /* of ID_NEXT */

             case ID_NEXTB:
             {
                /**********************************************************
                *  Get results from next Box                              *
                **********************************************************/
                ++ulBoxId;
                if (ulBoxId == ulMaxBox)
                {
                  ulBoxId = 0;
                }
                sBoxNo = ulBoxId;
                sBoxNo++;

                WinSetDlgItemShort(hwnd, BOX_NO, sBoxNo, TRUE);
                ulSize = 0;
                somExceptionFree(&lev);
                _idrGetBoxResult(pIDRTextResObj,
                                 &lev,
                                 ulBoxId,
                                 NULL,
                                 &ulSize);
                if (lev._major != NO_EXCEPTION)
                {
                   rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                   somExceptionFree(&lev);
                }

                if (!ulSize)
                {
                   /*******************************************************
                   *  This means that this box is blank.                  *
                   *******************************************************/
                   WinSetDlgItemText(hwnd, BOX_RES, " ");
                }
                else
                {
                   if (pBuf = malloc(ulSize))
                   {
                      ((PIDR_TXTBOX) pBuf)->cbSize = sizeof(IDR_TXTBOX);
                      _idrGetBoxResult(pIDRTextResObj,
                                       &lev,
                                       ulBoxId,
                                       (PIDR_TXTBOX) pBuf,
                                       &ulSize);
                      if (lev._major != NO_EXCEPTION)
                      {
                         rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                         somExceptionFree(&lev);
                      }
                   }
                   else
                   {
                       PutString(hwnd, STR_2, TRUE);
                       return((MRESULT) FALSE);
                   }

                   ulSize = GetSymStringProc(((PIDR_TXTBOX) pBuf)->pSyms,
                                             ((PIDR_TXTBOX) pBuf)->ulSymCnt,
                                             achBox);
                   achBox[ulSize] = '\0';
                   WinSendDlgItemMsg(hwnd, BOX_RES, EM_SETTEXTLIMIT,
                                     MPFROM2SHORT((SHORT) ulSize, 0), NULL);
                   WinSetDlgItemText(hwnd, BOX_RES, achBox);
                   free(pBuf);
                }
                break;
             }  /* of ID_NEXTB */
          }  /* of switch( SHORT1FROMMP( mp1 ) ) */
          break;
      }   /* of case WM_COMMAND */

      case WM_INITDLG:
      {
      BOOL  fTextResult = FALSE;

          ulResultNo = 0;
          WinShowWindow(hwnd, FALSE);

          /**********************************************************
          *  Get IDRText object result handle                       *
          **********************************************************/
          pIDRTextResObj = PVOIDFROMMP(mp2);

          /**********************************************************
          *  If it is not boxed mode, then disable the Box related  *
          *  fields and buttons.                                    *
          **********************************************************/
          if (idrGMetrics.ulRecoForm & IDR_RECOFORM_BOXED)
          {
             ulBoxId = 0;
             sBoxNo = ulBoxId + 1;

             /*******************************************************
             *  Put Box Number                                      *
             *******************************************************/
             WinSetDlgItemShort(hwnd, BOX_NO, sBoxNo, TRUE);

             /*******************************************************
             *  Get Box Result (Top Result)                         *
             *******************************************************/
             ulSize = 0;
             pBuf = NULL;
             _idrGetBoxResult(pIDRTextResObj,
                              &lev,
                              ulBoxId,
                              NULL,
                              &ulSize);
             if (lev._major != NO_EXCEPTION)
             {
                rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                somExceptionFree(&lev);
             }
             if (ulSize)
             {
                /******************************************************
                *  The first box is not blank.                        *
                ******************************************************/
                if (pBuf = malloc(ulSize))
                {
                   fTextResult = TRUE;
                   ((PIDR_TXTBOX) pBuf)->cbSize = sizeof(IDR_TXTBOX);
                   _idrGetBoxResult(pIDRTextResObj,
                                    &lev,
                                    ulBoxId,
                                    (PIDR_TXTBOX) pBuf,
                                    &ulSize);
                   if (lev._major != NO_EXCEPTION)
                   {
                      rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                      somExceptionFree(&lev);
                   }
                }
                else
                {
                    PutString(hwnd, STR_2, TRUE);
                    return((MRESULT) FALSE);
                }
                ulSize = GetSymStringProc(((PIDR_TXTBOX) pBuf)->pSyms,
                                          ((PIDR_TXTBOX) pBuf)->ulSymCnt,
                                          achBox);
             }
             achBox[ulSize] = '\0';
             WinSendDlgItemMsg(hwnd, BOX_RES, EM_SETTEXTLIMIT,
                               MPFROM2SHORT((SHORT) ulSize, 0), NULL);
             WinSetDlgItemText(hwnd, BOX_RES, achBox);
             if (pBuf) free(pBuf);
          }  /* of if (idrRMetrics.ulRecoForm & IDR_RECOFORM_BOXED) */
          else
          {
             /************************************************************
             *  This is NOT boxed mode. Disable BOX related data items.  *
             ************************************************************/
             WinEnableControl(hwnd, BOX_R1, FALSE);
             WinEnableControl(hwnd, BOX_NO, FALSE);
             WinEnableControl(hwnd, BOX_R2, FALSE);
             WinEnableControl(hwnd, BOX_RES, FALSE);
             WinEnableControl(hwnd, ID_NEXTB, FALSE);
          }

          /**********************************************************
          * Call idrTopResult for to get result                     *
          **********************************************************/
          ulSize = 0;
          _idrGetTopResult(pIDRTextResObj,
                           &lev,
                           NULL,
                           &ulSize);
          if (lev._major != NO_EXCEPTION)
          {
             rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
             somExceptionFree(&lev);
          }

          /************************************************************
          *  For boxed mode, the ulSize has the number of boxes which *
          *  have data.                                               *
          ************************************************************/
          ulMaxBox = ulSize;
          ulTop = 0;
          if (ulSize)
          {
             fTextResult = TRUE;
             if (pBuf = malloc(ulSize * sizeof(IDR_SYM)))
             {
                _idrGetTopResult(pIDRTextResObj,
                                 &lev,
                                 (PIDR_SYM) pBuf,
                                 &ulSize);
                if (lev._major != NO_EXCEPTION)
                {
                   rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                   if (rc == IDRERR_NO_RESULTS_AVAIL)
                   {
                       somExceptionFree(&lev);
                       WinPostMsg(hwnd, WM_COMMAND, MPFROMSHORT(ID_DONE), NULL);
                       return(FALSE);
                   }
                   somExceptionFree(&lev);
                }
             }
             else
             {
                 PutString(hwnd, STR_2, TRUE);
                 return((MRESULT) FALSE);
             }
             ulTop = GetSymStringProc((PIDR_SYM) pBuf,
                                      ulSize,
                                      achTop);
          }
          achTop[ulTop] = '\0';
          WinSetDlgItemText(hwnd, TOP_RES, achTop);
          free(pBuf);

          /**********************************************************
          *  Clean the Next Result field                            *
          **********************************************************/
          WinSetDlgItemText(hwnd, NEXT_RES, " ");

          /**********************************************************
          *  Get the number of Results                              *
          **********************************************************/
          ulClusterCnt = 0;
          _idrGetClusterHandles(pIDRTextResObj,
                                &lev,
                                NULL,
                                &ulClusterCnt);
          if (lev._major != NO_EXCEPTION)
          {
             rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
             if (rc == IDRERR_NO_RESULTS_AVAIL)
             {
                 somExceptionFree(&lev);
                 WinPostMsg(hwnd, WM_COMMAND, MPFROMSHORT(ID_DONE), NULL);
                 return(FALSE);
             }
          }

          sMaxNumRes = 0;
          if (ulClusterCnt)
          {
             if (pCls = malloc(ulClusterCnt * sizeof(HCLUSTER)))
             {
                fTextResult = TRUE;
                _idrGetClusterHandles(pIDRTextResObj,
                                      &lev,
                                      pCls,
                                      &ulClusterCnt);
                if (lev._major != NO_EXCEPTION)
                {
                   rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                   if (rc == IDRERR_NO_RESULTS_AVAIL)
                   {
                       somExceptionFree(&lev);
                       WinPostMsg(hwnd, WM_COMMAND, MPFROMSHORT(ID_DONE), NULL);
                       return(FALSE);
                   }
                }
                for (i = 0; i < ulClusterCnt; i++)
                {
                   clData[i].hCls = pCls[i];

                   /****************************************************
                   *  Get the number of result from each cluster       *
                   ****************************************************/
                   clData[i].ulNumRes = _idrGetResultCount(pIDRTextResObj,
                                                           &lev,
                                                           pCls[i]);
                   if (lev._major != NO_EXCEPTION)
                   {
                      rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
                      somExceptionFree(&lev);
                   }
                   sMaxNumRes = max(sMaxNumRes, clData[i].ulNumRes);
                }

                /********************************************************
                *  Put limit to maximum number of results               *
                ********************************************************/
                WinSetDlgItemShort(hwnd, NUM_RES, sMaxNumRes, TRUE);
                free(pCls);
             }
             else
             {
                 PutString(hwnd, STR_2, TRUE);
                 return((MRESULT) TRUE);
             }
          }

          if (!fTextResult)
          {
             /********************************************************
             *  NO Text result available. Exit this dialog box.      *
             ********************************************************/
             WinDismissDlg(hwnd, FALSE);
             return(FALSE);
          }

          WinShowWindow(hwnd, TRUE);

          break;
      }   /* of case WM_INITDLG */

      default:
          return WinDefDlgProc( hwnd, msg, mp1, mp2 );
   }   /* of switch(msg) */

   return FALSE;

}   /* of ResultDlgProc */

/************************************************************************
*                                                                       *
* Function Name: GestureDlgProc                                         *
*                                                                       *
* Description :                                                         *
*                                                                       *
*   Dialog procedure for Gesture result data                            *
*                                                                       *
************************************************************************/
MRESULT EXPENTRY GestureDlgProc ( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
RECODATA   rcData;
PRECODATA  prcData;
CHAR       achName[80];
APIRET     rc;

   switch( msg )
   {
      case WM_COMMAND:
      {
          switch( SHORT1FROMMP( mp1 ) )
          {
             case ID_DONE:
             {
                WinDismissDlg(hwnd, FALSE);
                break;
             }
          }  /* of switch( SHORT1FROMMP( mp1 ) ) */
          break;
      }   /* of case WM_COMMAND */

      case WM_INITDLG:
      {
         SHORT  sTemp;

         prcData = PVOIDFROMMP(mp2);
         rc = RedRecoNameFromID(prcData->hReco,
                                prcData->id,
                                achName);
         if (rc != RED_NO_ERROR)
         {
            WinPostMsg(hwnd, WM_COMMAND, MPFROMSHORT(ID_DONE), NULL);
         }
         WinSetDlgItemText(hwnd, GES_NAME, achName);
         sTemp = prcData->virtual_id;
         WinSetDlgItemShort(hwnd, V_ID, sTemp, TRUE);
         sTemp = prcData->char_code;
         WinSetDlgItemShort(hwnd, C_ID, sTemp, TRUE);
         break;
      }   /* of case WM_INITDLG */

      default:
          return WinDefDlgProc( hwnd, msg, mp1, mp2 );
   }   /* of switch(msg) */

   return FALSE;

}   /* of GestureDlgProc */

/************************************************************************
*                                                                       *
* Function Name: IDRErrorDlgProc                                        *
*                                                                       *
* Description :                                                         *
*                                                                       *
*   Dialog procedure for IDR_ERROR message handling                     *
*                                                                       *
************************************************************************/
MRESULT EXPENTRY IDRErrorDlgProc (HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
IDF_PROB   *pIdfErr;
SHORT      sRc;
CHAR       achText[80];

   switch(msg)
   {
      case WM_INITDLG:
      {
         pIdfErr = PVOIDFROMMP(mp2);
         sRc = (SHORT) pIdfErr->ulErrorCode;
         WinSetDlgItemShort(hwnd, ID_ERROR_RC, sRc, TRUE);
         strcpy(achText, pIdfErr->pszClassName);
         WinSetDlgItemText(hwnd, ID_ERROR_CLASS, achText);
         strcpy(achText, pIdfErr->pszMethodName);
         WinSetDlgItemText(hwnd, ID_ERROR_METHOD, achText);
         return(FALSE);
      }

      case WM_COMMAND:
         switch(SHORT1FROMMP(mp1))
         {
            case ID_ERROR_DONE:
               return(FALSE);
         }

      default:
          return WinDefDlgProc( hwnd, msg, mp1, mp2 );
   }
}  /* of IDRErrorDlgProc */

/*************************************************************************
*                                                                        *
* Function Name: GiveBoxedForm                                           *
*                                                                        *
* Description :                                                          *
*                                                                        *
*   Give Boxed form based upon in IDR_RECOMETRICS                        *
*                                                                        *
* Input: hwnd    : Window handle                                         *
*        hps     : Presentation space handle                             *
*        sizel   : size of client window                                 *
*        Metrics : IDR_RECOMETRICS                                       *
*                                                                        *
* Output:                                                                *
*                                                                        *
*************************************************************************/
VOID GiveBoxedForm(HWND hwnd, HPS hps, SIZEL sizel, IDR_RECOMETRICS Metrics)
{
LONG   i, j, k, l;
ULONG  nXChars, nYChars;     /* Number of character in X and Y directions */
ULONG  ulX, ulY, ulXC, ulYC;
POINTL p[128];               /* Max 64 characters in a row or a column    */

CHAR   temp[120];

   /*********************************************************************
   *  Assume that there is no conflict bwtween the Metrics data.        *
   *  First calculate the number of characters in (x,y) coordinate and  *
   *  check whether this is small than what user wants.                 *
   *********************************************************************/
   nXChars = (sizel.cx - Metrics.ptlOffset.x - Metrics.ptlOrigin.x) /
                 (Metrics.ptlBox.x + Metrics.ptlGray.x);
   nYChars = (sizel.cy - Metrics.ptlOffset.y - Metrics.ptlOrigin.y) /
                 (Metrics.ptlBox.y + Metrics.ptlGray.y);

   if (fFirstMsg &&
       (nXChars < Metrics.ulColCount || nYChars < Metrics.ulRowCount))
   {
      fFirstMsg = FALSE;
      PutString(hwnd, STR_11, TRUE);
   }

   nXChars = min(nXChars, Metrics.ulColCount);
   nYChars = min(nYChars, Metrics.ulRowCount);

   /******************************************************************
   *  Get the (x,y) coordinate of the left buttom box                *
   ******************************************************************/
   ulXC = ulX = Metrics.ptlOrigin.x + Metrics.ptlOffset.x + Metrics.ptlGray.x;
   ulYC = ulY = Metrics.ptlOrigin.y + Metrics.ptlOffset.y;

   /*******************************************************************
   *  Draw horizontal lines for top, baseline, and buttom of each box *
   *******************************************************************/
   for (i = 0; i < 3 * nYChars; i++)
   {
      j = i / 3;
      k = i - j * 3;                           /* k = mod(i, 3) */
      switch (k)
      {
         case 0: ulYC += Metrics.ptlGray.y;
                 break;
         case 1: ulYC += Metrics.lOffsetBline;
                 break;
         case 2: ulYC += Metrics.ptlBox.y - Metrics.lOffsetBline;
                 break;
      }

      k = 0;
      for (j = 0; j < nXChars; j++)
      {
         p[j].y = ulYC;
         p[k].x = ulXC + j * (Metrics.ptlBox.x + Metrics.ptlGray.x);
         k++;
         p[k].x = p[k - 1].x + Metrics.ptlBox.x;
         k++;
      }
      for (j = nXChars; j < 2 * nXChars; j++)
      {
         p[j].y = ulYC;
      }
      GpiPolyLineDisjoint(hps,
                          2 * nXChars,
                          p);
   }

   /******************************************************************
   *  Draw vertical lines for left and right side of each box        *
   ******************************************************************/
   ulXC = Metrics.ptlOrigin.x + Metrics.ptlOffset.x;

   for (i = 0; i < 2 * nXChars; i++)
   {
      j = i / 2;
      k = i - j * 2;                      /* k = mod(i, 2) */
      ulXC += !k ? Metrics.ptlGray.x : Metrics.ptlBox.x;
      ulYC = ulY;
      for (j = 0; j < 2 * nYChars; j++)
      {
         l = j / 2;
         k = j - l * 2;                   /* k = mod(j, 2) */
         p[j].x = ulXC;
         ulYC += !k ? Metrics.ptlGray.y : Metrics.ptlBox.y;
         p[j].y = ulYC;
      }
      GpiPolyLineDisjoint(hps,
                          2 * nYChars,
                          p);
   }
}  /* of GiveBoxedForm */

/**************************************************************************
*                                                                         *
* Function Name: GiveLinedForm                                            *
*                                                                         *
* Description :                                                           *
*                                                                         *
*   Give Lined form based upon in IDR_RECOMETRICS                         *
*                                                                         *
* Input: hwnd    : Window handle                                          *
*        hps     : Presentation space handle                              *
*        sizel   : size of client window                                  *
*        Metrics : IDR_RECOMETRICS                                        *
*                                                                         *
* Output:                                                                 *
*                                                                         *
**************************************************************************/
VOID GiveLinedForm(HWND hwnd, HPS hps, SIZEL sizel, IDR_RECOMETRICS Metrics)
{
LONG   i, j, k;
ULONG  nXChars, nYChars;     /* Number of character in x and Y directions */
ULONG  ulX, ulY, ulWidth;
POINTL ptl, ptl2;

   /********************************************************************
   *  Assume that there is no conflict bwtween the Metrics data.       *
   *  First calculate the number of characters in (x,y) coordinate and *
   *  check whether this is small than what user wants.                *
   ********************************************************************/
   nXChars = (sizel.cx - Metrics.ptlOffset.x - Metrics.ptlOrigin.x) /
                 (Metrics.ptlBox.x + Metrics.ptlGray.x);
   nYChars = (sizel.cy - Metrics.ptlOffset.y - Metrics.ptlOrigin.y) /
                 (Metrics.ptlBox.y + Metrics.ptlGray.y);

   if (fFirstMsg &&
       (nXChars < Metrics.ulColCount || nYChars < Metrics.ulRowCount))
   {
      fFirstMsg = FALSE;
      PutString(hwnd, STR_11, TRUE);
   }

   nXChars = min(nXChars, Metrics.ulColCount);
   nYChars = min(nYChars, Metrics.ulRowCount);
   ulWidth = nXChars * (Metrics.ptlBox.x + Metrics.ptlGray.x);

   /******************************************************************
   *  Get the (x,y) coordinate of the left buttom box                *
   ******************************************************************/
   ptl.x = Metrics.ptlOrigin.x + Metrics.ptlOffset.x + Metrics.ptlGray.x;
   ptl2.x = ptl.x + ulWidth;
   ptl.y = Metrics.ptlOrigin.y + Metrics.ptlOffset.y + Metrics.ptlGray.y;

   /******************************************************************
   *  Give the lines                                                 *
   ******************************************************************/
   for (i = 0; i < 3 * nYChars; i++)
   {
        j = i / 3;
        k = i - j * 3;                     /* k = moc(i, 3)  */
        GpiMove(hps, &ptl);
        ptl2.y = ptl.y;
        if (k == 1)
        {
           GpiSetColor(hps, CLR_RED);
        }
        else
        {
           GpiSetColor(hps, CLR_BLACK);
        }
        GpiLine(hps, &ptl2);
        switch(k)
        {
           case 0:
             ptl.y += Metrics.lOffsetBline;
             break;

           case 1:
             ptl.y += Metrics.ptlBox.y - Metrics.lOffsetBline;
             break;

           case 2:
             ptl.y += Metrics.ptlGray.y;
             break;
        }
   }

   ptl.x = Metrics.ptlOrigin.x + Metrics.ptlOffset.x + Metrics.ptlGray.x;
   for (i = 0; i < 2; i++)
   {
      ptl.y = Metrics.ptlOrigin.y + Metrics.ptlOffset.y + Metrics.ptlGray.y;
      if (i == 1)
      {
         ptl.x += ulWidth;
      }
      for (j = 0; j < nYChars; j++)
      {
         GpiMove(hps, &ptl);
         ptl.y += Metrics.ptlBox.y;
         GpiLine(hps, &ptl);
         ptl.y += Metrics.ptlGray.y;
      }
   }
}  /* of GiveLinedForm */

/***************************************************************************
*                                                                          *
* Function Name: GetSymStringProc                                          *
*                                                                          *
* Description :  idrGetTopResult and idrGetResultInfo have IDR_TXTSYM data *
*                and the symbols will be returned as PSZ.                  *
*                                                                          *
* Input: pIDRSym : Pointer to IDR_SYM data                                 *
*        ulSymCnt: Number of IDR_SYM data                                  *
*        pRes    : pointer to returned Text data                           *
*                                                                          *
* Output:                                                                  *
*        pRes    : points the result Text                                  *
*                                                                          *
* Returned:                                                                *
*        number of Text characters.                                        *
*                                                                          *
***************************************************************************/
ULONG GetSymStringProc(PIDR_SYM pIDRSym, ULONG ulSymCnt, PSZ pRes)
{
ULONG      ulRes = 0;
ULONG      ulStatus;
LONG       i;

   for (i = 0; i < ulSymCnt; i++)
   {
      ulStatus = pIDRSym[i].usSymType;
      switch (ulStatus)
      {
         case IDR_TYPE_ASCII:
            *(pRes + ulRes++) = pIDRSym[i].usSymValue;
            break;

         case IDR_TYPE_CTRL:
            *(pRes + ulRes++) = ' ';
            break;

         case IDR_TYPE_UNKNOWN:
            *(pRes + ulRes++) = '*';  /* Put * for unknown symbol         */
            break;
      }
   }
   return(ulRes);
}  /* of GetSymStringProc */

/************************************************************************
*                                                                       *
* Function Name: Text_GetMetrics                                        *
*                                                                       *
* Description :                                                         *
*   Change from sensor metrics to screen metrics                        *
*                                                                       *
* Input: hwnd    : Window handle                                        *
*        IDRTextObj : IDRText object                                    *
*        pMetrics: Address of RECOMETRICS                               *
*                                                                       *
* Output:                                                               *
*        pMetrics: Converted recometrics                                *
*                                                                       *
* Returns: FALSE if no pen device is installed                          *
*                                                                       *
************************************************************************/
BOOL Text_GetMetrics(IDR_RECOMETRICS *pMetrics)
{
   pMetrics->ulRecoForm =     idrGMetrics.ulRecoForm;
   pMetrics->ulSpaceDisable = idrGMetrics.ulSpaceDisable;
   pMetrics->ulDirection =    idrGMetrics.ulDirection;
   pMetrics->ulRecoForm =     idrGMetrics.ulRecoForm;
   pMetrics->ulSpaceDisable = idrGMetrics.ulSpaceDisable;
   pMetrics->ulDirection =    idrGMetrics.ulDirection;
   pMetrics->ptlOrigin.x =    idrGMetrics.ptlOrigin.x;
   pMetrics->ptlOrigin.y =    idrGMetrics.ptlOrigin.y;
   pMetrics->ptlOffset.x =    idrGMetrics.ptlOffset.x;
   pMetrics->ptlOffset.y =    idrGMetrics.ptlOffset.y;
   pMetrics->ptlBox.x    =    idrGMetrics.ptlBox.x;
   pMetrics->ptlBox.y    =    idrGMetrics.ptlBox.y;
   pMetrics->ptlGray.x   =    idrGMetrics.ptlGray.x;
   pMetrics->ptlGray.y   =    idrGMetrics.ptlGray.y;
   pMetrics->lOffsetBline =   idrGMetrics.lOffsetBline;
   pMetrics->ulRowCount  =    idrGMetrics.ulRowCount;
   pMetrics->ulColCount  =    idrGMetrics.ulColCount;

   return(TRUE);
}  /* of Text_GetMetrics */

/************************************************************************
*                                                                       *
* Function Name: Text_SetMetrics                                        *
*                                                                       *
* Description :                                                         *
*   Change from screen metrics to sensor metrics                        *
*                                                                       *
* Input: hwnd    : Window handle                                        *
*        IDRTextObj : IDRText object                                    *
*        pMetrics: Address of RECOMETRICS                               *
*                                                                       *
* Output:                                                               *
*        pMetrics: Converted recometrics                                *
*                                                                       *
* Returns: FALSE if no pen device is installed                          *
*                                                                       *
************************************************************************/
BOOL Text_SetMetrics(HWND hwnd, IDRText *IDRTextObj, ULONG ulNumDev,
            ULONG ulDspXpels, ULONG ulDspYpels, IDR_RECOMETRICS *pMetrics)
{
ULONG             ulLocId;
ULONG             ulStdXpts,  ulStdYpts;
IDR_RECOMETRICS   SensorMetrics;
BOOL              fFound = FALSE;

   idrGMetrics.ulRecoForm =     pMetrics->ulRecoForm;
   idrGMetrics.ulSpaceDisable = pMetrics->ulSpaceDisable;
   idrGMetrics.ulDirection =    pMetrics->ulDirection;
   idrGMetrics.ptlOrigin.x =    pMetrics->ptlOrigin.x;
   idrGMetrics.ptlOrigin.y =    pMetrics->ptlOrigin.y;
   idrGMetrics.ptlOffset.x =    pMetrics->ptlOffset.x;
   idrGMetrics.ptlOffset.y =    pMetrics->ptlOffset.y;
   idrGMetrics.ptlBox.x    =    pMetrics->ptlBox.x;
   idrGMetrics.ptlBox.y    =    pMetrics->ptlBox.y;
   idrGMetrics.ptlGray.x   =    pMetrics->ptlGray.x;
   idrGMetrics.ptlGray.y   =    pMetrics->ptlGray.y;
   idrGMetrics.lOffsetBline =   pMetrics->lOffsetBline;
   idrGMetrics.ulRowCount  =    pMetrics->ulRowCount;
   idrGMetrics.ulColCount  =    pMetrics->ulColCount;

   for( ulLocId = 1; ulLocId <= ulNumDev; ++ulLocId )
   {
     WrtQueryLocatorCaps(&ulLocId, &WrtLi);
     if (WrtLi.ulLocatorType == LT_PEN)
     {
        fFound = TRUE;
        break;
     }
   }  /* of for(... */

   if (!fFound)
   {
       PutString(hwnd, STR_8, TRUE);
       return(FALSE);
   }

   ulStdXpts = WrtLi.ulStandardXpts;
   ulStdYpts = WrtLi.ulStandardYpts;

   /*********************************************************************
   *  Convert from screen metrics to sensor metrics                     *
   *********************************************************************/
   SensorMetrics.ptlOrigin.x = (pMetrics->ptlOrigin.x * ulStdXpts) / ulDspXpels;
   SensorMetrics.ptlOrigin.y = (pMetrics->ptlOrigin.y * ulStdYpts) / ulDspYpels;
   SensorMetrics.ptlOffset.x = (pMetrics->ptlOffset.x * ulStdXpts) / ulDspXpels;
   SensorMetrics.ptlOffset.y = (pMetrics->ptlOffset.y * ulStdYpts) / ulDspYpels;
   SensorMetrics.ptlGray.x = (pMetrics->ptlGray.x * ulStdXpts) / ulDspXpels;
   SensorMetrics.ptlGray.y = (pMetrics->ptlGray.y * ulStdYpts) / ulDspYpels;
   SensorMetrics.ptlBox.x = (pMetrics->ptlBox.x * ulStdXpts) / ulDspXpels;
   SensorMetrics.ptlBox.y = (pMetrics->ptlBox.y * ulStdYpts) / ulDspYpels;
   SensorMetrics.lOffsetBline = (pMetrics->lOffsetBline * ulStdYpts) / ulDspYpels;

   SensorMetrics.cbSize = pMetrics->cbSize;
   SensorMetrics.ulRecoForm = pMetrics->ulRecoForm;
   SensorMetrics.ulSpaceDisable = pMetrics->ulSpaceDisable;
   SensorMetrics.ulDirection = pMetrics->ulDirection;
   SensorMetrics.ulRowCount = pMetrics->ulRowCount;
   SensorMetrics.ulColCount = pMetrics->ulColCount;

   _idrSetMetrics(IDRTextObj, &lev, &SensorMetrics);
   if (lev._major != NO_EXCEPTION)
   {
      rc = ((IDF_PROB *) somExceptionValue(&lev))->ulErrorCode;
      somExceptionFree(&lev);
   }

   /*********************************************************************
   * For unlined mode, all these data are lost. So restore for next use *
   *********************************************************************/
   pMetrics->ptlOrigin.x =    idrGMetrics.ptlOrigin.x;
   pMetrics->ptlOrigin.y =    idrGMetrics.ptlOrigin.y;
   pMetrics->ptlOffset.x =    idrGMetrics.ptlOffset.x;
   pMetrics->ptlOffset.y =    idrGMetrics.ptlOffset.y;
   pMetrics->ptlBox.x    =    idrGMetrics.ptlBox.x;
   pMetrics->ptlBox.y    =    idrGMetrics.ptlBox.y;
   pMetrics->ptlGray.x   =    idrGMetrics.ptlGray.x;
   pMetrics->ptlGray.y   =    idrGMetrics.ptlGray.y;
   pMetrics->lOffsetBline =   idrGMetrics.lOffsetBline;
   pMetrics->ulRowCount  =    idrGMetrics.ulRowCount;
   pMetrics->ulColCount  =    idrGMetrics.ulColCount;
   return(TRUE);
}  /* of Text_SetMetrics */

/************************************************************************
*                                                                       *
* Function Name: PutString                                              *
*                                                                       *
* Description :                                                         *
*                                                                       *
*   Load string by WinLoadString.                                       *
*                                                                       *
* Input: hwnd    : Window handle                                        *
*        nStr_ID : String ID                                            *
*        sts     : TRUE for Error title, FALSE for Info title           *
*                                                                       *
* Output:                                                               *
*                                                                       *
************************************************************************/
VOID  PutString(HWND hwnd, ULONG nStr_ID, BOOL sts)
{
CHAR       achMsg[120];

    WinLoadString(hab, 0UL, nStr_ID, 120, achMsg);
    WinMessageBox( HWND_DESKTOP,
                   hwnd,
                   achMsg,
                   sts ? NULL : "Choose a Reco Type",
                   1,
                   MB_OK );
}
