/****************************** Module Header ******************************\
*                                                                          *
* Copyright (c) 1995 IBM Corporation                                       *
*                                                                          *
* Module Name: penidf.h                                                    *
*                                                                          *
*                                                                          *
* Input Data Framework (IDF) interface defines used by                     *
*   all parts of Pen for OS/2.                                             *
\**************************************************************************/

#ifndef PENIDF_H_INCLUDED
#define PENIDF_H_INCLUDED


/**************************************************************************
* Parse user defines                                                      *
**************************************************************************/
#ifdef INCL_IDF
  #ifndef INCL_IDM
    #define INCL_IDM
  #endif
#endif
#ifdef INCL_IDM
  #ifndef INCL_IDR
    #define INCL_IDR
  #endif
  #ifndef INCL_IDO
    #define INCL_IDO
  #endif
#endif





/**************************************************************************
* Common defines section                                                  *
**************************************************************************/


/**************************************************************************
* Defines from IDO                                                        *
**************************************************************************/


/********************************/
/* INKERDATA structres          */
/********************************/
typedef struct _INKERDATA
{
      ULONG       cbSize;
      HPS         hps;
      HBITMAP     hBitmap;
      LONG        lColor;
      LONG        lBackColor;
      LONG        lMix;
      LONG        lBackMix;
      LONG        lLineType;
      ULONG       ulWidth;
      ULONG       ulEraserWidth;
      ULONG       flAttrMask;
} INKERDATA;
typedef INKERDATA *PINKERDATA;

/********************************/
/* INKERDATA.flAttrMask flags   */
/********************************/
#define IDO_COLOR         0x0001L
#define IDO_BACKCOLOR     0x0002L
#define IDO_MIX           0x0004L
#define IDO_BACKMIX       0x0008L
#define IDO_WIDTH         0x0010L
#define IDO_LINETYPE      0x0040L
#define IDO_HPS           0x0200L
#define IDO_HBITMAP       0x0400L
#define IDO_ERASERWIDTH   0x0800L


/************************************/
/* INKERDATA.ulWidth default define */
/************************************/
#define IDO_ERASER_WIDTH_DEFAULT   8


/************************************/
/* IDOStroke type defines           */
/************************************/
#define IDO_STROKE_NORMAL    0x0000
#define IDO_STROKE_ERASER    0x0001

/************************************/
/* IDOStroke scaling defines        */
/************************************/
#define IDO_SCALE_FRESH      0x0000
#define IDO_SCALE_CONTINUE   0x0001

/************************************/
/* IDO inking defines               */
/************************************/
#define IDO_INK_DEFAULT      0x0000
#define IDO_INK_INKER        0x0001
#define IDO_INK_DOODLE       0x0002




/**************************************************************************
* Defines from IDR                                                        *
**************************************************************************/


/**********************************************************/
/* All IDR structures must be packed on 2-byte boundaries */
/**********************************************************/
#pragma pack(2)

/********************************/
/* IDR_SYM.usSymType values     */
/********************************/
#define IDR_TYPE_ASCII          (USHORT)(0x0002)
#define IDR_TYPE_CTRL           (USHORT)(0xffff)
#define IDR_TYPE_SYSGESTURE     (USHORT)(0x8001)
#define IDR_TYPE_LETTERGESTURE  (USHORT)(0x8002)

#define IDR_TYPE_UNKNOWN        (USHORT)(0x7000)

/********************************************************/
/* IDR_SYM.usSymValue values for use with IDR_TYPE_CTRL */
/********************************************************/
#define IDR_CTRL_NODATA         (0x0000)
#define IDR_CTRL_UNKNOWN        (0x0001)
#define IDR_CTRL_NEWLINE        (0x0002)



/********************************************************/
/* IDR_SYM structure                                    */
/********************************************************/
typedef struct _IDR_SYM         /* Basic symbol */
{
  USHORT    usSymType;          /* Symbol type, see IDR_TYPE_xxx */
  USHORT    usSymValue;         /* Symbol value, code point or virtual id */
}   IDR_SYM,
   *PIDR_SYM,
  **PPIDR_SYM;


/********************************************************/
/* IDR_SYMSET.ulSetMask defines                         */
/* The following are valid for both text and gesture    */
/* symbol subsets                                       */
/********************************************************/
#define IDR_SYMSET_FULL        (0x10000000)    /* full symbol set; this is the default */
#define IDR_SYMSET_SYMLIST     (0x00000000)    /* symbol list, not predefined type */

/********************************************************/
/* IDR_SYMSET.ulSetMask defines                         */
/* The following are only valid for text symbol subsets */
/********************************************************/
#define IDR_SYMSET_ALPHALO     (0x00000001)   /* lower-case alphabetic text symbols */
#define IDR_SYMSET_ALPHAHI     (0x00000002)   /* upper-case alphabetic text symbols */
#define IDR_SYMSET_DIGIT       (0x00000004)   /* digits               0123456789    */
#define IDR_SYMSET_PUNCTUATION (0x00000008)   /* punctuation symbols  !-;`'"?()&.,:\  */
#define IDR_SYMSET_MATH        (0x00000010)   /* mathematical symbols %^*()-+={}<>,/. */
#define IDR_SYMSET_MONETARY    (0x00000020)   /* monetary symbols     ,.$ or local  */
#define IDR_SYMSET_SPECIAL     (0x00001000)   /* special symbols      #&<>@[\]^{}~  */


/********************************************************/
/* IDR_SYMSET structure                                 */
/********************************************************/
typedef struct _IDR_SYMSET /* Symbol set */
{
  ULONG cbSize;                 /* Size of this structure */
  ULONG ulSetMask;              /* Symbol subset mask */
  ULONG ulSymCnt;               /* Count of symbols in the symbol list */
  IDR_SYM *pSyms;               /* Pointer to array of symbols */
}  IDR_SYMSET,
  *PIDR_SYMSET,
 **PPIDR_SYMSET;



/********************************************************/
/* IDR_RECOMETRICS.ulRecoForm defines                   */
/* Also used in IDR_TXTCLUSTER.ulResultType             */
/********************************************************/
#define IDR_RECOFORM_SINGLE_STROKE   (0x00000001)
#define IDR_RECOFORM_SINGLE_SYM      (0x00000002)
#define IDR_RECOFORM_BOXED           (0x00000004)
#define IDR_RECOFORM_LINED           (0x00000008)
#define IDR_RECOFORM_UNLINED         (0x00000010)

/********************************************************/
/* IDR_RECOMETRICS.ulSpaceDisable defines               */
/* These defines can tell the engine to ignore spaces   */
/* between strokes, and so treat all strokes as one word*/
/********************************************************/
#define IDR_SPACE_ENABLE         (0x00000000)
#define IDR_SPACE_DISABLE        (0x00000001)
#define IDR_NEWLINE_ENABLE       (0x00000000)
#define IDR_NEWLINE_DISABLE      (0x00000010)

/********************************************************/
/* IDR_RECOMETRICS.ulDirection defines                  */
/* Describes the expected direction of writing          */
/********************************************************/
#define IDR_DIRECT_RIGHT_DOWN    (0x00000001) /* Left->right, top->bottom */


/********************************************************/
/* IDR_RECOMETRICS structure                            */
/********************************************************/
typedef struct _IDR_RECOMETRICS /* Recognition engine metrics */
{
  ULONG           cbSize;           /* IDR_RECOMETRICS size                */
  ULONG           ulRecoForm;       /* Type of reco segmentation           */
  ULONG           ulSpaceDisable;   /* Don't return any whitespace         */
  ULONG           ulDirection;      /* Direction of "normal" writing       */
  POINTL          ptlOrigin;        /* Origin                              */
  POINTL          ptlOffset;        /* Offset of "gray" area from Origin   */
  POINTL          ptlBox;           /* Box extents (width & height)        */
  POINTL          ptlGray;          /* "Gray" area between boxes           */
  LONG            lOffsetBline;     /* Baseline offset from bottom of Box  */
  ULONG           ulRowCount;       /* Row count, 0 == indeterminate       */
  ULONG           ulColCount;       /* Column count, 0 == indeterminate    */
}  IDR_RECOMETRICS,
  *PIDR_RECOMETRICS,
 **PPIDR_RECOMETRICS;

/* Notes about IDR_RECOMETRICS
 * The required fields depends on the ulRecoForm. If the reco form is:
 *   IDR_RECOFORM_UNLINED  - only the first four fields need to be set;
 *                           cbSize, ulRecoForm, ulSpaceDisable, ulDirection
 *   IDR_RECOFORM_LINED
 *         or              - all fields must be set
 *   IDR_RECOFORM_BOXED
 */




/********************************************************/
/* IDR_GESTINFO structure                               */
/********************************************************/
typedef struct _IDR_GESTINFO /* Gesture Information structure */
{
  ULONG         cbSize;         /* IDR_GESTINFO size          */
  CHAR          szGestName[MAX_RECO_NAME_LENGTH+1];  /* Name of the gesture shape */
  IDR_SYM       Symbol;         /* Symbol used in symset struct */
  RECOID        EventId;        /* ID of reco event */
}   IDR_GESTINFO,
   *PIDR_GESTINFO,
  **PPIDR_GESTINFO;

/*
**  Gesture information structure -- contains information about the mapping
**  of a gesture to other Pen for OS/2 system constructs.
**  If the symbol's type is \"IDR_TYPE_SYSGESTURE\" the symbol's value is
**  the virtual id.  If the type is \"IDR_TYPE_LETTERGESTURE\" the value is
**  the character.
*/



/****************************/
/* HCLUSTER structure       */
/****************************/
typedef VOID   *HCLUSTER,              /* Handle to a cluster */
               **PHCLUSTER;



/********************************************************/
/* IDR_TXTBOX structure                                 */
/********************************************************/
typedef struct _IDR_TXTBOX
{
   ULONG cbSize;                /* Size of the structure. */
   ULONG ulSymCnt;              /* Count of IDR_SYMs in pSyms. */
   PIDR_SYM pSyms;              /* Recognition results for this box.  This
                                   provides all alternatives available for this
                                   box, from most likely to least likely.*/
   ULONG ulStrokeIdCnt;         /* Stroke count returned. */
   PULONG pStrokeId;            /* Pointer to an array of stroke ids. */
   ULONG ulLocation;            /* Location information. This is the box id. */
   ULONG ulReserved;            /* Reserved. */
   PVOID pReserved;             /* Reserved. */
} IDR_TXTBOX,
 *PIDR_TXTBOX,
 **PPIDR_TXTBOX;



/********************************************************/
/* IDR_TXTCLUSTER structure                             */
/********************************************************/
typedef struct _IDR_TXTCLUSTER  /* Text cluster descriptor */
{
  ULONG cbSize;                /* Size of the structure. */
  HCLUSTER hCluster;           /* Handle to the cluster. */
  ULONG ulResultType;          /* Type of result.  Uses IDR_RECOFORM_xxx. */
  ULONG ulSymCnt;              /* Count of IDR_SYMs in pSyms. */
  PIDR_SYM pSyms;              /* Recognition results of this cluster. */
  ULONG ulStrokeIdCnt;         /* Stroke count returned. */
  PULONG pStrokeId;            /* Pointer to an array of stroke ids. */
  ULONG ulLocation;            /* Location information.  Depends on
                                  ulResultType.  For Boxed mode, this is
                                  the starting box id.  For Lined mode,
                                  this is the row id.  For Unlined mode,
                                  this is the baseline. */
  RECTL rclBoundBox;           /* Bounding box for strokes, in lined and
                                  unline modes. */
  ULONG ulReserved1;           /* Reserved. */
  PVOID pReserved1;            /* Reserved. */
  ULONG ulReserved2;           /* Reserved. */
  PVOID pReserved2;            /* Reserved. */
} IDR_TXTCLUSTER, *PIDR_TXTCLUSTER,
  **PPIDR_TXTCLUSTER;



/********************************************************/
/* Language modeling coercion flags                     */
/********************************************************/
#define IDR_LM_NONE            (0x00000000)   /* Disable it */
#define IDR_LM_ADVISE          (0x00000001)   /* Enable it */
#define IDR_LM_FORCE           (0x00000002)   /* Suggest results */
#define IDR_LM_REJECT          (0x00000004)   /* Constrain results */

#define IDR_LM_DEFAULT         (0x00000001)   /* Default setting for the coercion */
#define IDR_LM_MASK            (0x00000007)   /* Mask for all valid flags */
#define IDR_LM_ERROR           (0xffffffff)   /* Use to signal error conditions */


/********************************************************/
/* Language modeling policy flags                       */
/********************************************************/
#define IDR_TXTLM_NONE         (0x00000000)   /* No text-specific policy */
#define IDR_TXTLM_WRDCASE      (0x00000001)   /* Advise recognition to all upper */
                                              /* case, all lower case, or first letter */
                                              /* capitalized. */
#define IDR_TXTLM_SENTCASE     (0x00000002)   /* Advise recognition toward proper */
                                              /* sentence capitalization. */
#define IDR_TXTLM_ALPHANUM     (0x00000004)   /* Advise recognition to group alphabetic */
                                              /* and digits separately. */
#define IDR_TXTLM_BRACKETS     (0x00000008)


#define IDR_TXTLM_DEFAULT      (0x0000000f)   /* Default setting for the policies */
#define IDR_TXTLM_MASK         (0x0000000f)   /* Mask for all valid text policy flags */
#define IDR_TXTLM_ERROR        (0xffffffff)   /* Use to signal error conditions */


/*************************************/
/* Done packing on 2-byte boundaries */
/*************************************/
#pragma pack()





/**************************************************************************
* Defines from IDM                                                        *
**************************************************************************/


/********************************/
/* IDMCTLDATA.ulInkStyle values */
/********************************/
#define IDM_INK_SYSTEM  0x00000000      /* Use system inking       */
#define IDM_INK_WINDOW  0x00000001      /* Use Window Inking       */
#define IDM_INKTYPE_BITMASK (IDM_INK_SYSTEM | IDM_INK_WINDOW)

#define IDM_REMOVE_INK  0x00000000      /* Remove ink at end of operation */
#define IDM_LEAVE_INK   0x00000010      /* Leave ink at end of operation */
#define IDM_REMOVEINK_BITMASK (IDM_REMOVE_INK | IDM_LEAVE_INK)

#define IDM_INKSTYLE_BITMASK (IDM_INKTYPE_BITMASK | IDM_REMOVEINK_BITMASK)

/**********************************/
/* IDMCTLDATA.ulIDMOp values      */
/**********************************/
#define IDM_GESTURE     0x00000001      /* Reco incoming strokes as gestures */
#define IDM_TEXT        0x00000002      /* Reco incoming strokes are text */
#define IDM_COLLECTSTROKE 0x00000004    /* Only collect incoming strokes */

/***********************************/
/* IDMCTLDATA.ulDoodlePrePr values */
/***********************************/
#define DPP_NOTHING     0x00000000      /* No preprocessing of doodle        */
#define DPP_DISPLAY     0x00000001      /* Display contents of doodle in hps */
#define DPP_RECO        0x00000002      /* Send doodle contents to recognizer*/
#define DPP_ALL (DPP_DISPLAY | DPP_RECO)

/***********************************/
/* IDMCTLDATA.ulEOWevents values   */
/***********************************/
#define EOW_EXITPROX    0x00000001      /* WM_EXIT_PROXIMITY-Stop collecting */
#define EOW_LIFTOFF     0x00000002      /* WM_LIFTOFF - Stop collecting      */
#define EOW_WRTTIMEOUT  0x00000004      /* WM_WRITING_TIMEOUT-Stop collecting*/
#define EOW_ALL (EOW_EXITPROX | EOW_LIFTOFF | EOW_WRTTIMEOUT)

/***********************************/
/* IDMCTLDATA.pRclInclude values   */
/***********************************/
#define IDM_DEFAULT_INCLUDE (PRECTL)  0L  /* include rectangle=window size   */
#define IDM_NO_INCLUDE      (PRECTL) -1L  /* include rectangle=entire screen */

/***********************************/
/* IDMCTLDATA.ulNotifyFlags values */
/***********************************/
#define IDM_WRITING_MODE     0x00000001 /* Send IDM_WRITING_MODE         */
#define IDM_STATE_CHANGE     0x00000002 /* Send IDM_STATE_CHANGE         */
#define IDM_ENTER_PROXIMITY  0x00000004 /* Send IDM_ENTER_PROXIMITY      */
#define IDM_EXIT_PROXIMITY   0x00000008 /* Send IDM_EXIT_PROXIMITY       */
#define IDM_TOUCHDOWN        0x00000010 /* Send IDM_TOUCHDOWN            */
#define IDM_LIFTOFF          0x00000020 /* Send IDM_LIFTOFF              */
#define IDM_ALL (IDM_WRITING_MODE | IDM_STATE_CHANGE | \
                 IDM_ENTER_PROXIMITY | IDM_EXIT_PROXIMITY | IDM_TOUCHDOWN | \
                 IDM_LIFTOFF)

/********************************/
/* IDMCTLDATA.ulTapFlags values */
/********************************/
/* These flags have no effect when ulIDMOp is set to IDM_GESTURES */
#define IDM_FIRST_TAP_MOUSE   0x00000001    /* If first stroke is a tap, process as mouse emulation     */
#define IDM_FIRST_TAP_STROKE  0x00000002    /* If first stroke is a tap, process as a stroke            */
#define IDM_CHECK_ACTIVE      0x00000003    /* If window not active, treat first tap as mouse emulation */
#define IDM_CHECK_FOCUS       0x00000004    /* If window doesn't have focus, treat first tap as mouse emulation */
#define IDM_PROCESS_FLAGS     0x0000000F    /* If window doesn't have focus, treat first tap as mouse emulation */
#define IDM_REDUCE_TAP_LENGTH 0x80000000    /* Reduce tap length on first stroke                        */

/***************************************************************/
/* IDM States and Notification Sub-mesages of IDM_STATE_CHANGE */
/***************************************************************/
#define IDM_STOPPED          0x00000001  /* IDM stopped, no collect, no reco*/
#define IDM_STARTED          0x00000002  /* IDM started, receiving msgs     */
#define IDM_START_COLLECTING 0x00000004  /* IDM collecting strokes          */
#define IDM_COLLECTING       0x00000004  /* IDM collecting strokes          */
#define IDM_START_RECOING    0x00000008  /* IDM adding strokes to be recoed */
#define IDM_RECOING          0x00000008  /* IDM adding strokes or recoing   */
#define IDM_STOP_COLLECTING  0x00000010  /* IDM stopped collecting strokes  */
#define IDM_STOP_RECOING     0x00000020  /* IDM done recoing strokes        */

/****************************************************************/
/* MRESULTs from WM_RESULT message - choose one from next three */
/****************************************************************/
#define IDM_HANDLE_RESULT     0x00000000  /* Handle result for application   */
#define IDM_DO_NOTHING        0x00000001  /* Application handled WM_RESULT   */
#define IDM_HANDLE_ALT_RESULT 0x00000002  /* Handle alternate result for app */
#define IDM_HANDLE_MASK       0x00000003

/*******************************************************************/
/* More MRESULTs from WM_RESULT message - choose one from next two */
/*******************************************************************/
#define IDM_SEND_WM_RECO      0x00000000  /* Send WM_RECO to application     */
#define IDM_DONT_SEND_WM_RECO 0x00000010  /* Dont send WM_RECO to app        */

/*******************************************************************************/
/* More MRESULTs from WM_RESULT message - only set if IDM_SEND_WM_RECO was set */
/*******************************************************************************/
#define IDM_ROUTE_WM_RECO     0x00000020  /* Route WM_RECO up owner chain    */
                                          /* if window proc doesn't handle it*/

/***********************************************/
/* idmStop - ulStopIndicator flags (bit field) */
/***********************************************/
#define SI_ABORT         0x00000001  /* Stop IDM, no IDM_NOTIFY/WM_RESULT   */
#define SI_STOPNOW       0x00000002  /* Stop IDM, send IDM_NOTIFY/WM_RESULT */
#define SI_LAST          SI_STOPNOW  /* used for parameter checking; == last */

/********************************************************************/
/* Define for setting default writing, proximity, & eraser pointers */
/********************************************************************/
#define IDM_DEFAULT_PTR ((ULONG) -1)

/********************************************************************/
/* Define for _idmGetDoodle() and _idmresGetDoodle option           */
/********************************************************************/
#define IDM_DOODLE_RELEASE    0x00000001


/**********************************************/
/* IDM_RESULTDATA.ulResultType defines        */
/**********************************************/
#define RES_NULL                                   0x00000000
#define RES_TEXT                                   0x00000001
#define RES_GESTURE                                0x00000002
#define RES_STROKE                                 0x00000004
#define RES_ERROR                                  0x00000008
#define RES_TAP                                    0x00000010
#define RES_DOUBLETAP                              0x00000020
#define RES_PAUSETIMEOUT                           0x00000040
#define RES_GESTURE_NOT_IN_WINDOW                  0x00000080
#define RES_GESTURE_USER_MAPPED                    0x00000100
#define RES_IDM_WAS_STOPPED                        0x00000200
#define RES_WRITING_CANCELED_USER_ACTION           0x00000400   /*  User held pen motionless or hit Esc key   */
#define RES_WRITING_CANCELED_USER_ERROR            0x00000800   /*  Stroke buffer full, ink buffer full, etc  */
#define RES_WRITING_CANCELED_SYSTEM_ERROR          0x00001000   /*  Sysmodal popup, screen grp switch etc     */
#define RES_WRITING_CANCELED_APPL_ACTION           0x00002000   /*  Appl cancelled via IDM API                */
#define RES_WRITING_CANCELED_MASK                  0x00003C00   /*  Mask for testing writing cancelled        */



/********************************************************************/
/* PIDM_RECTLIST is an array of exclude rectangles                  */
/********************************************************************/
typedef struct
{
  ULONG  ulRectlCnt;          /* Number of rectangles in list      */
  RECTL  RectList[1];         /* Array of exclude rectangles       */
} IDM_RECTLIST;
typedef IDM_RECTLIST *PIDM_RECTLIST;


/********************************************************************/
/* The following structures can be found in the file idmobj.h       */
/*    IDM_RESULTDATA                                                */
/*    IDM_NOTIFYDATA                                                */
/*    IDM_NOTIFYSTATEDATA                                           */
/*    IDMCTLDATA                                                    */
/********************************************************************/





/**************************************************************************
* Error defines section                                                   *
**************************************************************************/

/* IDM error codes */
#define IDMERR_FIRST   0x7200
#define IDMERR_LAST    0x72FF

#define IDMERR_INVALID_STATE                   0x7201   /* Can't set IDM variables while IDM
                                                          in collect state */
#define IDMERR_CANNOT_START_NOW                0x7202   /* IDM can only be started while
                                                          processing a WM_TOUCHDOWN message
                                                          (or WM_WRITING_MODE message, for
                                                          compatibility layer) */
#define IDMERR_NO_OWNER                        0x7203   /* hwndOwner has not been specified */
#define IDMERR_ANOTHER_IDM_STARTED             0x7204   /* another instance of IDM is working
                                                           on behalf of the specified window*/
#define IDMERR_NOT_STARTED                     0x7205   /* this instance of IDM is not working
                                                           on behalf of any window */
#define IDMERR_INVALID_OPERATION               0x7206   /* invalid combination of IDM
                                                           operations requested.  For example,
                                                           IDM_COLLECTSTROKE & IDM_GESTURE */
#define IDMERR_INTERNAL_ERROR                  0x7207   /* internal IDM error */
#define IDMERR_NO_HPS                          0x7208   /* window inking requested, but no hps
                                                           has been specified */
#define IDMERR_ALREADY_STARTED                 0x7209   /* attempt to start the IDM but it's
                                                           already started */
#define IDMERR_INVALID_OBJECT                  0x720A   /* the object specified is not
                                                           descended from IDMTextDoodle
                                                           object */
#define IDMERR_OUTSIDE_INCLUDE_RECT            0x720B   /* attempt to start IDM but
                                                           touchdown point is outside of the
                                                           include rectangle */
#define IDMERR_INSIDE_EXCLUDE_RECT             0x720C   /* attempt to start IDM but
                                                           touchdown point is inside of an
                                                           exclude rectangle */
#define IDMERR_RECOING_IN_ANOTHER_WINDOW       0x720D   /* attempt to start IDM while
                                                           IDM is recoing for another
                                                           window */
#define IDMERR_NO_DOODLE                       0x720E   /* request to paint doodle, no doodle
                                                           exists */
#define IDMERR_NOT_WINDOW_INKING               0x720F   /* request to paint doodle, not using
                                                           window inking style */
#define IDMERR_NOTHING_TO_RECO                 0x7210   /* request to reco, but there is
                                                           nothing reco */
#define IDMERR_DEVICE_NOT_ENABLED_FOR_WRITING  0x7211   /* Pointing device was not pause
                                                           enabled by user */
#define IDMERR_INVALID_STOP_INDICATOR          0x7212   /* Invalid parameter */
#define IDMERR_CTLDATA_POINTER_NULL            0x7213   /* control data pointer == 0 */
#define IDMERR_INVALID_INKING_STYLE            0x7214   /* Invalid parameter */
#define IDMERR_INVALID_IDMOP                   0x7215   /* Invalid parameter */
#define IDMERR_INVALID_DOODLEPREPR             0x7216   /* Invalid parameter */
#define IDMERR_INVALID_EOWEVENT                0x7217   /* Invalid parameter */
#define IDMERR_INVALID_NOTIFY_FLAG             0x7218   /* Invalid parameter */
#define IDMERR_POINTER_EXCEPTION               0x7219   /* Invalid pointer   */
#define IDMERR_INVALID_RECTL                   0x721A   /* Invalid parameter */
#define IDMERR_MEM_ALLOC_FAILED                0x721B   /* memory allocation failure */
#define IDMERR_BUF_TOO_SMALL                   0x721C   /* zero length buffer or buffer too small */
#define IDMERR_INVALID_AUX_FLAGS               0x721D   /* invalid parameter */
#define IDMERR_INVALID_STROKETYPE              0x721E   /* invalid parameter */
#define IDMERR_ERASER_AND_SYS_INKING           0x721F   /* invalid parameter */
#define IDMERR_INVALID_CBSIZE                  0x7220   /* cbsize not set or incorrect size */
#define IDMERR_ERASER_NOT_IN_COLLECTSTROKE     0x7221   /* Can't set stroketype to Eraser
                                                           unless IDM Operation includes
                                                           IDM_COLLECTSTROKE */
#define IDMERR_NO_TEXT_RESULT_OBJECT           0x7222   /* result not available for this IDM operation */
#define IDMERR_NO_GESTURE_OBJECT               0x7223   /* result not available for this IDM operation */
#define IDMERR_INVALID_GETDOODLE_OPTION        0x7224   /* invalid parameter */
#define IDMERR_INVALID_TAP_FLAG                0x7225   /* invalid parameter */
#define IDMERR_OBJECT_CREATION_FAILURE         0x7226   /* unable to create a SOM object */
#define IDMERR_GET_BUFFER_SIZE                 0x7227   /* method should have returned IDxERR_BUF_TOO_SMALL */
                                                        /* (i.e., _idrGetTopResult) but it didn't */
#define IDMERR_NO_APP_DOODLE                   0x7228   /* no application owned doodle */
#define IDMERR_DOODLE_NOT_RELEASED             0x7229   /* Application gets the doodle from idm thru
                                                           idmGetDoodle without asking the doodle to
                                                           be released and now is trying to set the
                                                           doodle as userdoodle */

#define IDMERR_LAST 0x72FF

/* IDR error codes */
#define IDRERR_FIRST             0x7300
#define IDRERR_LAST              0x73ff

#define IDRERR_ERROR                   0x7300        /* General failure */
#define IDRERR_BUF_TOO_SMALL           0x7301        /* Input buffer too small */
#define IDRERR_NO_RESULTS_AVAIL        0x7302        /* No results available */
#define IDRERR_ENGINE_ERROR            0x7303        /* Reco engine returned an error */
#define IDRERR_INVALID_PARM            0x7304        /* Invalid parameter */
#define IDRERR_INVALID_STATE           0x7305        /* Invalid state to issue this call */
#define IDRERR_INVALID_HANDLE          0x7306        /* Invalid cluster handle */
#define IDRERR_INVALID_COERCION_FLAG   0x7307        /* Invalid coercion flag */
#define IDRERR_INVALID_POLICY_FLAG     0x7308        /* Invalid policy flag */
#define IDRERR_INVALID_SYMSET          0x7309        /* Invalid symset structure */
#define IDRERR_INVALID_SYMSET_FLAG     0x730A        /* Invalid symset flag specified */
#define IDRERR_STRUCT_NOT_INIT         0x730B        /* Structure was not properly initialized */
                                                     /*   structure's size field must be */
                                                     /*   initialized prior to call */
#define IDRERR_POINTER_EXCEPTION       0x730C        /* Invalid pointer caused an exception */
#define IDRERR_BOXID_NOT_FOUND         0x730D        /* Invalid box id for this result */
#define IDRERR_INVALID_RESULT_NUMBER   0x730E        /* Invalid result number for this cluster */
#define IDRERR_MEM_ALLOC_FAILED        0x730F        /* Internal memory allocation failed */
#define IDRERR_INVALID_RECOFORM        0x7310        /* Found an invalid recoform/result form */


/* IDO error codes */
#define  IDOERR_FIRST                  0x7400
#define  IDOERR_LAST                   0x749F

#define  IDOERR_INVALID_OBJECT         0x7401
#define  IDOERR_INVALID_HPS            0x7402
#define  IDOERR_NO_STROKE_DATA         0x7403
#define  IDOERR_INST_FAILED            0x7404
#define  IDOERR_NOT_SUPPORTED          0x7405
#define  IDOERR_INVALID_PARM           0x7406
#define  IDOERR_INVALID_BUFFER         0x7407
#define  IDOERR_MEMORY_ALLOC           0x7408
#define  IDOERR_INVALID_POINTER        0x7409
#define  IDOERR_BUF_TOO_SMALL          0x740A
#define  IDOERR_DATA_NOT_AVAIL         0x740B
#define  IDOERR_INVALID_COLOR          0x740C
#define  IDOERR_INVALID_LINETYPE       0x740D
#define  IDOERR_INVALID_WIDTH          0x740E
#define  IDOERR_INVALID_FM             0x740F
#define  IDOERR_INVALID_BM             0x7410
#define  IDOERR_CREATE_FAIL            0x7411
#define  IDOERR_NO_STROKES             0x7412
#define  IDOERR_STROKE_NOT_FOUND       0x7413
#define  IDOERR_NO_MEMORY_PS           0x7414
#define  IDOERR_NO_INTERSECTION        0x7415
#define  IDOERR_INVALID_STROKE         0x7416
#define  IDOERR_INVALID_BITMAP         0x7417
#define  IDOERR_ZERO_POINTS            0x7418
#define  IDOERR_GPISETATTRS            0x7419
#define  IDOERR_GPIMOVE                0x741A
#define  IDOERR_GPIPOLYLINE            0x741B
#define  IDOERR_GPIBITBLT              0x741C
#define  IDOERR_WINFILLRECT            0x741D
#define  IDOERR_INVALID_SCALE          0x741E
#define  IDOERR_POINTER_EXCEPTION      0x741F
#define  IDOERR_COORDINATE_UNDERFLOW   0x7420
#define  IDOERR_COORDINATE_OVERFLOW    0x7421
#define  IDOERR_GPILINE                0x7422

/* IDF error defines */
#define  IDFERR_FIRST                  0x74A0
#define  IDFERR_LAST                   0x74FF

#define  IDFERR_OBJECT_NOT_FOUND       0x74A0
#define  IDFERR_NULL_POINTER           0x74A1
#define  IDFERR_DUPLICATE_OBJ          0x74A2

/* end of error defines section */



/**************************************************************************
* Bring in object headers according to user defines                       *
**************************************************************************/

#ifdef INCL_IDR
  #include <idrgest.h>
  #include <idrtext.h>
#endif
#ifdef INCL_IDO
  #include <idostrk.h>
  #include <idodood.h>
#endif
#ifdef INCL_IDM
  #include <idmobj.h>
  #include <idmres.h>
#endif



#endif /* PENIDF_H_INCLUDED */
