/****************************** Module Header ******************************\
*                                                                          *
* Copyright (c) 1994 IBM Corporation                                       *
*                                                                          *
* Module Name: penpm.h                                                     *
*                                                                          *
\***************************************************************************/

#ifndef PENPMH_INCLUDED
#define PENPMH_INCLUDED


LONG    APIENTRY WrtQuerySysValue(HWND DeskTop, ULONG  id);
APIRET  APIENTRY WrtSetSysValue(HWND DeskTop, ULONG  id, LONG Val);
APIRET  APIENTRY WrtReadSysValue(HWND DeskTop, ULONG id);
APIRET  APIENTRY WrtWriteSysValue(HWND DeskTop, ULONG  id);

#define RECO_MAX_COORD   4

typedef ULONG    RECOID, HRECO;
typedef RECOID   *PRECOID;
typedef HRECO    *PHRECO;

typedef struct _RECODATA
  {
  ULONG  cbSize;              /* size of struc                       */
  RECOID id;                  /* ID if reco event                    */
  HRECO  hReco;               /* Reco subsystem handle               */
  ULONG  virtual_id;          /* virtual id of event                 */
  ULONG  char_code;           /* character code of event             */
  RECTL  bound_box;           /* box that bounds gesture             */
  POINTL ptlHotSpot;          /* Hot Spot X/Y screen coordinate      */
  HWND   hwnd;                /* Hwnd under hot spot                 */
  HWND   hwndActive;          /* Active window                       */
  HWND   hwndFocus;           /* Window that has the focus           */
  CHAR   *pszCommand;         /* Cmd and args gesture mapped to      */
  CHAR   *pszCmdArgs;
  CHAR   *pszPrefixCmd;       /* Action to be done before Command    */
  CHAR   *pszDescription;     /* Contents of description field       */
  ULONG  ulMappingInfo;       /* Flags detailing where gesture map   */
  } RECODATA;                 /*                       was found     */
typedef RECODATA *PRECODATA;

/* Flag bits for the mapping info ULONG */

#define RED_DEFAULT_MAP     0x00000001
#define RED_USER_MAP        0x00000002

#define RED_INSTANCE_MAP    0x00000010
#define RED_SESSION_MAP     0x00000040
#define RED_SYSTEM_MAP      0x00000080

#define MAX_RECO_NAME_LENGTH   11  /* Maximum characters length for  */
                                   /* Reco subsystem and event names */


/* ------------------------------- */ /*  Found in RECODATA.pszCommand when     */
/*  Built-In Reco Command Strings  */ /*  gestures are assigned to builtin cmds */
/* ------------------------------- */ /*  See object gesture settings page      */
#define BIC_NULLCMD      "NULL"       /* Null assignment/No-op   */
#define BIC_RESTMAX      "RESTMAX"    /* Restore/maximize window */
#define BIC_RESTMIN      "RESTMIN"    /* Restore/minimize window */
#define BIC_MAXIMIZE     "MAXIMIZE"   /* Maximize window         */
#define BIC_MINIMIZE     "MINIMIZE"   /* Minimize window         */
#define BIC_WINCLOSE     "WINCLOSE"   /* Close window            */
#define BIC_TASKLIST     "TASKLIST"   /* Bring up tasklist       */
#define BIC_MENUSEL      "MENUSEL"    /* Select menu item        */
#define BIC_KEYPLAY      "KEYPLAY"    /* Send keystrokes         */
#define BIC_OPEN         "OPEN"       /* Open object             */
#define BIC_SYSMENU      "SYSMENU"    /* Select Sysmenu item     */
#define BIC_CONTEXT      "CONTEXT"    /* Popup context menu      */
#define BIC_GESTHELP     "GESTHELP"   /* Open gesture settings page */
#define BIC_OBJMOVE      "OBJMOVE"    /* Move object             */
#define BIC_OBJCOPY      "OBJCOPY"    /* Copy object             */
#define BIC_OBJLINK      "OBJLINK"    /* Shadow object           */
#define BIC_OBJDRAG      "OBJDRAG"    /* Drag object             */



#define MAX_STROKE_POINTS 8000

/*************************************************************/
/* AUXPOINTDATA structure returned by WrtQueryAuxPointData() */
/*                                                           */
/*  AUXPOINTDATA is a static data structure available on     */
/*  each WM_MOUSEMOVE.                                       */
/*************************************************************/

typedef struct  _AUXPOINTDATA  /* apd */
  {
  USHORT         cbStructSize;   /* Structure size                    */
  ULONG          ulTimestamp;    /* Point timestamp                   */
  USHORT         usFlags;        /* Point flags                       */
  USHORT         usUser;         /* For program use                   */
  SHORT          sScrZ;          /* Normalize Z axis value (4K to -4K)*/
                                 /* Height - 0 t0 4096                */
                                 /* Pressure 0 to -4096               */
  SHORT          sDevZ;          /* Device Z axis value is device     */
                                 /*   dependant. See locator          */
                                 /*   caps structure.                 */
  SHORT          sAngle;         /* Stylus angle to XY (0-90 degrees) */
  SHORT          sRotation;      /* Stylus barrel rotation (0-360 deg)*/
  USHORT         usButton;       /* Mask containing button status     */
                                 /* 1 - button down, 0 - button up    */
  UCHAR          OEM[8];         /* OEM specific data                 */
  } AUXPOINTDATA;
typedef AUXPOINTDATA *PAUXPOINTDATA; /* papd */

/* AUXPOINTDATA.usFlags Equates */
#define APD_MODEMASK         0x0003   /* Mode mask (see below)          */
#define APD_IDLEMODE              0   /* Stylus is idle                 */
#define APD_PROXMODE              1   /* Stylus is in proximity         */
#define APD_CONTACTMODE           2   /* Stylus is in contact           */
#define APD_FILTERED         0x0004   /* Mouse event not passed to PM   */


APIRET APIENTRY WrtQueryPointAuxData(PAUXPOINTDATA  papdInfo);


/*****************************************************************/
/* AUXDATADESC structure used to build the AUXDATAINFO structure */
/*****************************************************************/

typedef struct  _AUXDATADESC  /* add */
  {
  ULONG          id;             /* Aux data type identity (See ADT_*) */
  ULONG          offset;         /* Field offset                 */
  ULONG          cb;             /* Aux data type byte count     */
  } AUXDATADESC;
typedef AUXDATADESC *PAUXDATADESC; /* padd */

/* AUXDATADESC.id Equates  */
#define ADT_TIMESTAMP   0        /* Point timestamp.             */
#define ADT_FLAGS       1        /* Point flags                  */
#define ADT_SCREENZ     2        /* Normalized Z axis value      */
#define ADT_DEVZ        3        /* Device Z axis value          */
#define ADT_ANGLE       4        /* Stylus angle to XY           */
#define ADT_ROTATION    5        /* Stylus rotation              */
#define ADT_BUTTON      6        /* Button status mask           */
#define ADT_USER        7        /* For program use              */
#define ADT_OEM         16       /* OEM specific data            */


/*****************************************************************/
/* AUXDATAINFO structure pointed to by STROKEDATA.pAuxInfo       */
/*****************************************************************/

typedef struct  _AUXDATAINFO  /* adi */
  {
  ULONG          cbAuxDesc;      /* Size of entire AUXDATAINFO   */
                                 /* structure                    */
  ULONG          ulNumElements;  /* Number of data elements in   */
                                 /* aux data packet              */
  ULONG          ulAuxSize;      /* Size of the aux data packet  */
                                 /* for each point in stroke.    */
                                 /* This size is the sum of all  */
                                 /* AUXDATADESC.cb fields        */
  AUXDATADESC    addAuxDesc[1];  /* Array of AUXDATADESC structures */
                                 /* There is one structure for   */
                                 /* each requested auxiliary     */
                                 /* data element.                */
  } AUXDATAINFO;
typedef AUXDATAINFO *PAUXDATAINFO; /* padi */

/*****************************************************************/
/* AUXSTROKEDATA structure pointed to by STROKEDATA.pAuxData     */
/*****************************************************************/

typedef struct  _AUXSTROKEDATA  /* asd */
  {
  UCHAR          bAuxData[1][1]; /* An array of structures       */
                                 /* containing auxiliary data    */
                                 /* (one structure per X/Y coord)*/
                                 /* Each structure contains an   */
                                 /* array of aux data elements as*/
                                 /* described by the AUXDATAINFO */
                                 /* structure.                   */
  } AUXSTROKEDATA;
typedef AUXSTROKEDATA *PAUXSTROKEDATA; /* pasd */


/*********************************************************/
/* STROKEDATA structure returned by WrtQueryStrokeData() */
/*********************************************************/

typedef struct _STROKEDATA  /* sd */
  {
  ULONG          cbStructSize;      /* Size of stroke data header    */
  ULONG          ulSource;          /* See Wrt Event Sources (LT_*)  */
  ULONG          ulSeq;             /* Sequence number wraps at 4G   */
  ULONG          ulResolution;      /* Points per Inch               */
  ULONG          ulSampleRate;      /* Points per Second             */
  RECTL          rclCell;           /* Rectangle Around Stroke       */
  ULONG          ulNumPoints;       /* Number of points in stroke    */
  ULONG          flStrokeFlags;     /* Stroke flags. (See SBF_*)     */
  ULONG          ulXExtent;         /* X extent used to scale coords */
  ULONG          ulYExtent;         /* Y extent used to scale coords */
  ULONG          ulSensorResolution;/* Sensor resolution in dots/inch*/
  ULONG          ulSensorXExtent;   /* Extent of sensor X axis       */
  ULONG          ulSensorYExtent;   /* Extent of sensor Y axis       */
  ULONG          ulTimestamp;       /* Timestamp of first point in stroke */
  ULONG          ulDevTimestampRes; /* Resolution of device TS in usecs   */
  PPOINTL        pXY;               /* pointer to X/Y POINTL array   */
  PAUXDATAINFO   pAuxInfo;          /* See AUXDATAINFO               */
  PAUXSTROKEDATA pAuxData;          /* See AUXPOINTDATA              */
  ULONG          ulUser1;
  ULONG          ulUser2;
  struct _STROKEDATA *psdFwd;       /* Recommended use-Forward chain ptr  */
  struct _STROKEDATA *psdBack;      /* Recommended use-Backward chain ptr */
  ULONG          ulInkColor;        /* Ink color                     */
  ULONG          ulInkWidth;        /* Ink width in pels             */
  ULONG          ulInkMix;          /* Foregound mix type            */
  ULONG          ulLineType;        /* Line type attribute           */
  ULONG          ulLocatorFlags;    /* Pointing device flags (See LTYPE_*) */
  PUCHAR         pszDriverName;     /* Pointer to name of controlling device driver */
  PUCHAR         pszDeviceName;     /* Pointer to name of pointing device  */
  ULONG          cbStrokeBufferSize;/* Size of entire stroke buffer as requested */
  POINTL         ptlOrigin;         /* Window origin used when mapping points */
  ULONG          flAuxData;         /* Aux data requested (See ADF_*)*/
  } STROKEDATA;
typedef STROKEDATA *PSTROKEDATA; /* psd */

/* Stroke Buffer Flags */
#define SBF_SCREEN          0x00000001  /* 1=Screen coords 0=hi res coord   */
#define SBF_SCALE           0x00000001  /* 1=Scaled coords 0=hi res coord   */
#define SBF_INKED           0x00000002  /* Stroke was inked                 */
#define SBF_INK_NOT_REMOVED 0x00000004  /* Stroke left inked                */
#define SBF_TD_LO_SAME_HWND 0x00000008  /* Touchdown/liftoff in same hwnd   */
#define SBF_STROKE_OUTSIDE_HWND 0x00000010 /* Stroke left the boundaries    */
#define SBF_POINTS_LOST     0x00000040  /* Points were lost (HW overrun)    */
#define SBF_BUFFER_OVERRUN  0x00000080  /* Points were lost (SB overrun)    */
#define SBF_MBUTTON_MASK    0x00001C00  /* Mask for isolating mouse button  */
#define SBF_MBUTTON1        0x00000400  /* Mouse button 1                   */
#define SBF_MBUTTON2        0x00000800  /* Mouse button 2                   */
#define SBF_MBUTTON3        0x00001000  /* Mouse button 3                   */
#define SBF_TAP             0x00002000  /* Stroke was a tap                 */
#define SBF_DBLTAP          0x00004000  /* Stroke was a double tap          */
#define SBF_MIGRATED        0x01000000  /* Stroke buffer migrated to new format */
#define SBF_FLAT_STRUCTURE  0x02000000  /* Buffer pointers changed to offsets */
#define SBF_OTHER_RES       0x04000000  /* Coords are in specified resolution */
#define SBF_DISPLAY_RES     0x08000000  /* Coords are display resolution    */
#define SBF_SENSOR_RES      0x10000000  /* Coords are sensor resolution     */
#define SBF_STANDARD_RES    0x20000000  /* Coords are standard resolution   */
#define SBF_COORDS_MAPPED   0x40000000  /* Coords were mapped to specified window */
#define SBF_NEGATIVE_COORDS 0x80000000  /* Contains negative window coords  */

APIRET APIENTRY WrtQueryStrokeData(PBYTE    pStrokeBuf,
                                   PULONG   pBufLen,
                                   HWND     hwnd,
                                   ULONG    flConvopt,   /* See QSD_* */
                                   ULONG    ulXOutput,
                                   ULONG    ulYOutput,
                                   ULONG    flAuxData);  /* See ADF_* */
/* flConvopt equates    */
#define QSD_NO_CNV           0x00000000  /* No Conversion (sensor resolution)   */
#define QSD_SCALE            0x00000001  /* Scale to specified resolution       */
#define QSD_STANDARDIZED_RES 0x00000002  /* Scale to standardize resolution     */
#define QSD_RESOLUTION       0x00000004  /* Scale factor specified in dots/inch */

/*****************************************************************************/
/*                                                                           */
/*   Sample Stroke Buffer            Aux Data Desc Array                     */
/*                                     .00000048  Size of AuxDataInfo        */
/*    Ŀ               . 00000003  Number of Elements         */
/*      Stroke Data               .   00000008  Size of Aux data packet    */
/*        Header                .     00000000  Desc (Timestamp)           */
/*     (STROKEDATA)           .       00000000  Offset                     */
/*    Ĵ       .         00000004  Size                       */
/*          X/Y           .           00000001  Desc (Flags)               */
/*      Coord Array     .             00000004  Offset                     */
/*                    .               00000002  Size                       */
/*    Ĵ                 00000008  Desc (User)                */
/*        Aux Data                    00000006  Offset                     */
/*         Info               . . . . 00000002  Size                       */
/*     (AUXDATAINFO)  . . . .                                              */
/*    Ĵ. . . . .     Aux Data Array (for each X/Y coord)     */
/*                            .                                            */
/*        Aux Data              . . . 000348F8  AuxData{0} - timestamp     */
/*         Array                      0022                   flags         */
/*                                    0000                   user          */
/*    (AUXSTROKEDATA)                 00034919  AuxData{1} - timestamp     */
/*    Ĵ .               0062                   flags         */
/*      Driver Name     .             0000                   user          */
/*    Ĵ     .           0003493A  AuxData{2} - timestamp     */
/*      Device Name         .         0022                   flags         */
/*             .       0000                   user          */
/*                                .     0003495B  AuxData{3} - timestamp     */
/*                                  .   0062                   flags         */
/*                                    . 0000                   user          */
/*                                       *                                   */
/*                                       *                                   */
/*                                       *                                   */
/*****************************************************************************/


APIRET APIENTRY WrtCopyStrokeData(PSTROKEDATA  pStrokeBuf,
                                  PBYTE        pMem,
                                  ULONG        flCopyopt);
/* flCopyopt equates    */
#define CSD_COPY             0x00000001  /* Copy stroke data to buffer          */
#define CSD_FLATTEN          0x00000002  /* Change buffer pointers to offsets   */
#define CSD_UNFLATTEN        0x00000004  /* Change buffer offsets to pointers   */
#define CSD_MIGRATE          0x00000008  /* Migrate stroke data to current format*/


APIRET  APIENTRY WrtMapPointLong(HWND     hwnd,
                                 PPOINTL  pXY,
                                 ULONG    flParms,     /* See MP_*  */
                                 ULONG    ulXInput,
                                 ULONG    ulYInput,
                                 ULONG    ulXOutput,
                                 ULONG    ulYOutput,
                                 ULONG    ulCount
                                 );

#define MP_SCALE        0x00000000    /* Scale coordinates (Default)              */
#define MP_NO_SCALE     0x00000001    /* Do not scale coordinates                 */
#define MP_RESOLUTION   0x00000002    /* Input and output expressed in resolution */


APIRET  APIENTRY WrtSetStrictEmulation(HWND    hwnd,
                                       ULONG   option);
#define SSE_RESET     0x00000000    /* Reset strict emulation for app*/
#define SSE_SET       0x00000001    /* Set strict emulation for appl */


APIRET APIENTRY WrtSetDefaultMessageBehavior(HWND    hwnd,
                                             ULONG   message,
                                             ULONG   option);
/* message id values */
#define DMI_RECO             0x00000004   /* WM_RECO message            */
#define DMI_ENTER_PROXIMITY  0x00000020   /* WM_ENTER_PROXIMITY message */
#define DMI_EXIT_PROXIMITY   0x00000040   /* WM_EXIT_PROXIMITY message  */
#define DMI_ENTER_WINDOW     0x00020000   /* WM_ENTER_WINDOW message    */
#define DMI_EXIT_WINDOW      0x00040000   /* WM_EXIT_WINDOW message     */

/* option values */        /* WinDefWindowProc and WinDlgWindowProc behavior */
#define DMB_FALSE            0x0000    /*  Returns FALSE from the specified  */
                                       /*  message.                          */
#define DMB_ROUTE_OWNER      0x0001    /*  Route specified message to the    */
                                       /*  owner window.                     */



APIRET  APIENTRY WrtControlDisplayBacklight(ULONG  option, ULONG displayID);
#define CDB_LIGHT_ON    0x00000001  /* Turn backlight on.            */
#define CDB_LIGHT_OFF   0x00000002  /* Turn backlight off.           */

/*********************************************************/
/*      PenPM System Pointer Definition                  */
/*                                                       */
/*  These pointers may be loaded via WinLoadSysPointer.  */
/*                                                       */
/*********************************************************/
#define  SPTR_PEN_BLOB   SPTR_PENFIRST+0  /* Blob pointer   */
#define  SPTR_PEN_ERASER SPTR_PENFIRST+1  /* Eraser pointer */
#define  SPTR_PEN_PENCIL SPTR_PENFIRST+2  /* Pencil pointer */



/************************************************/
/*      PenPM Messages Definition Section       */
/*                                              */
/************************************************/

#ifndef WM_PENFIRST
#define WM_PENFIRST             0x04C0        /* These equates taken    */
#endif

#ifndef WM_PENLAST
#define WM_PENLAST              0x04FF        /* from private pmwin.h   */
#endif

#define WM_PEN_MSG   WM_PENFIRST

#define WM_TOUCHDOWN           WM_PEN_MSG + 0x000
#define WM_LIFTOFF             WM_PEN_MSG + 0x001
#define WM_RECO                WM_PEN_MSG + 0x002
#define WM_RECO_COMMAND        WM_PEN_MSG + 0x003
#define WM_SENSOR_MOVE         WM_PEN_MSG + 0x004

#define WM_ENTER_PROXIMITY     WM_PEN_MSG + 0x005
#define WM_EXIT_PROXIMITY      WM_PEN_MSG + 0x006
#define WM_WRT_DEV_CAPS_CHANGE WM_PEN_MSG + 0x007
#define WM_WRT_SYSVALUECHANGED WM_PEN_MSG + 0x008

#define WM_PAUSE_TIMEOUT       WM_PEN_MSG + 0x00C
#define WM_STROKE              WM_PEN_MSG + 0x00D
#define WM_TAP                 WM_PEN_MSG + 0x00E
#define WM_DBL_TAP             WM_PEN_MSG + 0x00F
#define WM_ENTER_WINDOW        WM_PEN_MSG + 0x011
#define WM_EXIT_WINDOW         WM_PEN_MSG + 0x012
#define WM_WRITING_MODE        WM_PEN_MSG + 0x013
#define WM_RESULT              WM_PEN_MSG + 0x014
#define WM_WRX_CONFIG_CHANGE   WM_PEN_MSG + 0x015 /* mp1=type of object (idt_???)
                                                  mp2=key to new default */
#define IDR_RESULT             WM_PEN_MSG + 0x016
#define IDR_ERROR              WM_PEN_MSG + 0x017
#define IDM_NOTIFY             WM_PEN_MSG + 0x01A
#define IDM_ERROR              WM_PEN_MSG + 0x01B
#define WM_PEN_HXM             WM_PEN_MSG + 0x01C

/**************************************/
/*   WM_TOUCHDOWN MRESULT Values      */
/*   -------------------------------  */
#define TDN_DEFAULT_DELAY    0x00000000  /* Delay Button 1 down (default) */
#define TDN_IMMEDIATE        0x00000001  /* Immediate BUTxDOWN            */
#define TDN_INFINITE         0x00000002  /* Never Issue BUTxDOWN          */
#define TDN_DELAY_BITMASK TDN_DEFAULT_DELAY+TDN_IMMEDIATE+TDN_INFINITE
                                         /*                               */
#define TDN_INK_STROKE       0x00000000  /* Ink Stroke (Default)          */
#define TDN_NO_INK_STROKE    0x00000010  /* Don't Ink Stroke              */
#define TDN_LEAVE_INK        0x00000020  /* Leave Stroke Inked at liftoff */
#define TDN_INK_BITMASK  TDN_INK_STROKE+TDN_NO_INK_STROKE+TDN_LEAVE_INK
                                         /*                               */
#define TDN_HIFREQ_MOUSEMOVE 0x00000040  /* Regenerate coalesced mouse moves */
#define TDN_SENSOR_MOVE      0x00000080  /* Generate WM_SENSOR_MOVE msgs  */



/*************************************/
/*  WM_LIFTOFF MRESULT Values        */
/*  -----------------------------    */
#define LO_DEFAULT           0x00000000  /* App didn't process stroke,*/
                                         /*   do gesture processing   */
#define LO_STROKE_PROCESSED  0x00000001  /* App processed stroke,     */
                                         /*   take no other action    */
#define LO_EMULATE_MOUSE     0x00000002  /* App processed stroke,     */
                                         /*   perform mouse emulation */


/*************************************/
/*  WM_STROKE MRESULT Values         */
/*  -----------------------------    */
#define STK_DEFAULT          0x00000000  /* App didn't process stroke,*/
                                         /*   do gesture processing   */
#define STK_STROKE_PROCESSED 0x00000001  /* App processed stroke,     */
                                         /*   take no other action    */


/**************************************/
/*  WM_PAUSE_TIMEOUT MRESULT Values   */
/*  --------------------------------  */

#define PTO_DEFAULT          0x00000000  /* Allow mouse B1 emulation     */
#define PTO_CONTINUE         0x00000001  /* Remain in stroke capture mode*/

#define PTO_NO_MAP           0x00000002  /* Same as PTO_DEFAULT except   */
                                         /* tap pause and dbl tap pause  */
                                         /* are not mapped.              */
#define PTO_BUTTON1          0x00000004  /* Generate mouse button 1 down */
#define PTO_BUTTON2          0x00000008  /* Generate mouse button 2 down */
#define PTO_BUTTON3          0x00000010  /* Generate mouse button 3 down */
#define PTO_BUTTON_MASK      0x0000001C

#define PTO_SHIFT_AUG        0x00000020  /* Augment button down w/shift key   */
#define PTO_CTRL_AUG         0x00000040  /* Augment button down w/control key */
#define PTO_ALT_AUG          0x00000080  /* Augment button down w/alt key     */
#define PTO_AUG_MASK         0x000000E0


/**************************************/
/*  WM_WRITING_MODE MRESULT Values    */
/*  --------------------------------  */

#define WRM_DEFAULT          0x00000000  /* Default return            */


/********************************************/
/*   WM_TAP and WM_DBL_TAP MRESULT Values   */
/*  --------------------------------------  */

#define TP_TAP_PROCESSED    0x00000001  /* App processed tap message,   */
                                        /*   take no other action       */
#define TP_DEFAULT          0x00000000  /* Mouse B1 emulation is performed */
                                        /* except if TDN_INFINITE was   */
                                        /* returned at touchdown        */

#define TP_BUTTON1_CLICK    0x00000004  /* Generate mouse button 1 click*/
#define TP_BUTTON2_CLICK    0x00000008  /* Generate mouse button 2 click*/
#define TP_BUTTON3_CLICK    0x00000010  /* Generate mouse button 3 click*/
#define TP_BUTTON_MASK      0x0000001C

#define TP_SHIFT_AUG        0x00000020  /* Augment click with shift key */
#define TP_CTRL_AUG         0x00000040  /* Augment click with control key */
#define TP_ALT_AUG          0x00000080  /* Augment click with alt key */
#define TP_AUG_MASK         0x000000E0



/*************************************/
/*  WM_RECO MRESULT Values           */
/*  --------------------------       */
#define RECO_MAP             0         /* App didn't process event, */
                                       /*   Pen will map event      */
#define RECO_NOMAP           1         /* App doesn't want mapping  */
#define RECO_PROCESSED       1         /* App processed reco event, */


typedef struct _WRTEVENTDATA   /* ed */
  {
  ULONG     cbStructSize;       /* Structure size                        */
  USHORT    usKCFlags;          /* KC_* flags from mouse message         */
  USHORT    usHitTest;          /* Hit test results from mouse message.  */
                                /* See PM Message WM_HITTEST (HT_*) results*/
  POINTL    ptlDisplay;         /* Display X/Y coordinate                */
  POINTL    ptlSensor;          /* Sensor X/Y coordinate                 */
  POINTL    ptlStandard;        /* Standard X/Y coordinate               */
  ULONG     flEventStatus;      /* Event status                          */
  ULONG     ulStrokeSeq;        /* Stroke sequence number                */
  ULONG     ulStrokeSize;       /* Number of stroke points (Liftoff only)*/
  ULONG     ulLocatorID;        /* ID of pointing device                 */
  ULONG     ulLocatorType;      /* Type of pointing device (See LT_*)    */
  ULONG     ulLocatorFlags;     /* Pointing device flags   (See LTYPE_*) */
  ULONG     ulValidityFlags;    /* Aux data validity flags (See ADF_*)   */
  ULONG     ulButtonStatus;     /* Bit mask indicating button status for */
                                /* up to 16 configured buttons.          */
                                /* (1 - button down, 0 - button up)      */
  HWND      hwndUnderPtr;       /* Hwnd under pointer                    */
  } WRTEVENTDATA;
typedef WRTEVENTDATA *PWRTEVENTDATA; /* ped */


/*******************************************/
/*   WRT_*  Event and State Indicators     */
/*                                         */
/*******************************************/

/* WRTEVENTDATA.flEventStatus Equates */
#define WRT_STROKE_AVAIL     0x00000001   /* Stroke is avail for retrieval   */
#define WRT_INKING           0x00000002   /* Stroke was inked                */
                                          /* Note: The next three flags are  */
                                          /*       only valid if the Stroke  */
                                          /*       Avail bit is set.         */
#define WRT_INK_NOT_REMOVED  0x00000004   /* Stroke left inked as requested  */
#define WRT_TD_LO_SAME_HWND  0x00000008   /* Touchdown/liftoff in same hwnd  */
#define WRT_STROKE_OUTSIDE_HWND 0x00000010 /* Stroke left the boundaries     */
                                           /* of the TD window's hwnd.       */
#define WRT_POINTS_LOST      0x00000040   /* Points were lost (HW overrun)   */
#define WRT_BUFFER_OVERRUN   0x00000080   /* Points lost (Stroke buffer overrun)*/
#define WRT_NON_DEVICE_EVENT 0x00000100   /* Mouse event not generated by device*/
#define WRT_MBUTTON_MASK     0x00001C00   /* Mask for isolating mouse butto  */
#define WRT_MBUTTON1         0x00000400   /* Mouse button 1                  */
#define WRT_MBUTTON2         0x00000800   /* Mouse button 2                  */
#define WRT_MBUTTON3         0x00001000   /* Mouse button 3                  */
#define WRT_TAP              0x00002000   /* Stroke is a tap                 */
#define WRT_DBLTAP           0x00004000   /* Stroke is a double tap          */
#define WRT_TAP_PAUSE        0x00008000   /* Tap pause event                  */
#define WRT_DBLTAP_PAUSE     0x00010000   /* Double tap pause event           */
#define WRT_MOUSE_EMULATION  0x00020000   /* Mouse emulation mode            */
                                          /* Note: The next five flags are   */
                                          /*   only valid for WM_ENTER_WINDOW*/
                                          /*   and WM_ENTER_WINDOW messages  */
#define WRT_ENTEXT_TOP       0x08000000   /* Entered/exited window from top  */
#define WRT_ENTEXT_BOTTOM    0x10000000   /* Entered/exited window from bottom */
#define WRT_ENTEXT_RIGHT     0x20000000   /* Entered/exited window from right  */
#define WRT_ENTEXT_LEFT      0x40000000   /* Entered/exited window from left   */
#define WRT_ENTEXT_Z         0x80000000   /* Entered/exited window from Z axis */
#define WRT_ENTEXT_MASK      0xF8000000   /* Enter/exit mask               */


/*******************************************/
/*   Virtual Event IDs                     */
/*     Intent-based are individual values; */
/*     User/letter events covered by one.  */
/*******************************************/

#define VE_TOGGLESELECTION      (1)
#define VE_HELP                 (2)
#define VE_SPACE                (3)
#define VE_EDIT                 (4)
#define VE_DELETE               (5)
#define VE_PASTE                (6)
#define VE_SCRATCHOUT           (7)
#define VE_DELETECHAR           (8)
#define VE_INSERT               (9)
#define VE_CUT                  (10)
#define VE_COPY                 (11)
#define VE_ASSIGNMENTS          (12)
#define VE_UNDO                 (13)
#define VE_ENTER                (14)
#define VE_TAB                  (15)
#define VE_SCROLLUP             (16)
#define VE_SCROLLDOWN           (17)
#define VE_SCROLLLEFT           (18)
#define VE_SCROLLRIGHT          (19)
#define VE_SCROLLUPMORE         (20)
#define VE_SCROLLDOWNMORE       (21)
#define VE_SCROLLLEFTMORE       (22)
#define VE_SCROLLRIGHTMORE      (23)
#define VE_SCROLLUPMAX          (24)
#define VE_SCROLLDOWNMAX        (25)
#define VE_SCROLLLEFTMAX        (26)
#define VE_SCROLLRIGHTMAX       (27)
#define VE_BACKSPACE            (28)
#define VE_EXTENDSELECTION      (29)
#define VE_CONTEXTMENU          (30)
#define VE_BOUNDSELECTIONLEFT   (31)
#define VE_BOUNDSELECTIONRIGHT  (32)
#define VE_PICKUP               (33)
#define VE_PICKUPCANCEL         (34)
#define VE_PLACEDEFAULT         (35)
#define VE_PLACEMOVE            (36)
#define VE_PLACECOPY            (37)
#define VE_PLACESHADOW          (38)
#define VE_UPPERCASE            (39)
#define VE_LOWERCASE            (40)

#define VE_LETTERGESTURE          (100)   /* Single value covers multiple. */

APIRET  APIENTRY RedSetRecoPointer(HRECO hReco, RECOID id);

/* string types for WrtQueryDeviceNames */
typedef CHAR STR_32[32];     /* str32 */
typedef STR_32 *PSTR_32;

/* defines for all device types */
#define DT_LOCATOR  0x03          /* locator device  */
#define DT_BUTTON   0x04          /* button device   */
#define DT_DISPLAY  0x05          /* display device  */

APIRET APIENTRY WrtQueryEventData(PWRTEVENTDATA  pedInfo);

APIRET APIENTRY WrtQueryInputDeviceNames(PSZ     pszDriverName,
                                         PULONG  ulNumDevices,
                                         PSTR_32  aDeviceName,
                                         PULONG  aDeviceType,
                                         PULONG  aDeviceID);

APIRET APIENTRY WrtEnumInputDrivers(PULONG pulNumDrivers, PSTR_32 aDriverNames);

typedef struct _WRTSYSTEMINFO       /* si */
  {
  ULONG     cbStructSize;      /* Structure size                      */
  ULONG     ulDisplayXPels;    /* Number of display X pixels          */
  ULONG     ulDisplayYPels;    /* Number of display Y pixels          */
  ULONG     ulDisplayXRes;     /* Display resolution                  */
  ULONG     ulDisplayYRes;     /* Display resolution                  */
  ULONG     ulStandardXRes;    /* Standardized dots per inch X axis   */
  ULONG     ulStandardYRes;    /* Standardized dots per inch Y axis   */
  ULONG     ulNumDrivers;      /* Number of registered pen drivers    */
  ULONG     ulNumLocators;     /* Number of registered pointing devs  */
  ULONG     ulNumButtons;      /* Number of registered button device  */
  ULONG     ulNumDisplays;     /* Number of registered display devs   */
  } WRTSYSTEMINFO;
typedef WRTSYSTEMINFO *PWRTSYSTEMINFO; /* psi */

APIRET APIENTRY WrtQuerySystemCaps(PWRTSYSTEMINFO  psiInfo);

typedef struct _WRTLOCATORDEVINFO   /* ldi */
  {
  ULONG     cbStructSize;       /* Structure size                        */
  ULONG     ulLocatorID;        /* ID of pointing device                 */
  ULONG     ulLocatorType;      /* Type of pointing device               */
  ULONG     ulLocatorFlags;     /* Pointing device flags                 */
  ULONG     ulValidityFlags;    /* Aux data validity flags               */
  ULONG     ulOEMValidityCnt;   /* Count of valid OEM bytes (Reserved for Pen)*/
  ULONG     ulSensorHeight;     /* Sensor height in thousands of inches  */
  ULONG     ulSensorWidth;      /* Sensor width in thousands of inches   */
  ULONG     ulNumMouButtons;    /* Number of mouse buttons emulated      */
  ULONG     ulNumButtons;       /* Number of buttons on pointing device  */
  ULONG     ulButtonMask;       /* Mask giving IDs of barrel buttons     */
  ULONG     ulSensorXpts;       /* Number of sensor X axis units         */
  ULONG     ulSensorYpts;       /* Number of sensor Y axis units         */
  LONG      lSensorZptsP;       /* Number of sensor Z axis pressure units*/
  ULONG     ulSensorZptsH;      /* Number of sensor Z axis height units  */
  ULONG     ulSensorXres;       /* Sensor X resolution (points/inch)     */
  ULONG     ulSensorYres;       /* Sensor Y resolution (points/inch)     */
  ULONG     ulStandardXpts;     /* Number of Standardized X axis units   */
  ULONG     ulStandardYpts;     /* Number of Standardized Y axis units   */
  ULONG     ulSampleRate;       /* Sample rate of stylus                 */
  ULONG     ulMaxSampleRate;    /* Maximum possible point rate supported */
  ULONG     ulTimestampRes;     /* Dev timestamp resolution in microsecs */
  UCHAR     pszDriverName[32];  /* Name of controlling device driver     */
  UCHAR     pszDeviceName[32];  /* Name of pointing device               */
  } WRTLOCATORDEVINFO;
typedef WRTLOCATORDEVINFO *PWRTLOCATORDEVINFO; /* pldi */

/* WRTLOCATORDEVINFO.ulLocatorType Values */
#define LT_MOUSE              0x00000001 /* Mouse                     */
#define LT_PEN                0x00000002 /* Stylus                    */
#define LT_TOUCH              0x00000003 /* Finger touch              */
#define LT_OTHER              0x00000004 /* Other misc pointing device*/

/* WRTLOCATORDEVINFO.ulLocatorFlags Values */
#define LTYPE_TETHERED        0x00000001 /* Pointing device is tethered */
#define LTYPE_PROXIMITY_EVENT 0x00000002 /* Proximity events reported   */
#define LTYPE_TILT            0x00000004 /* Device reports angle to XY  */
#define LTYPE_ROTATION        0x00000008 /* Device reports rotation     */
#define LTYPE_OEMDATA         0x00000010 /* Device reports extra OEMata */
#define LTYPE_TIMESTAMP       0x00000020 /* Device reports hi res timestamp*/
#define LTYPE_PRESSURE        0x00000040 /* Device reports pressure     */
#define LTYPE_HEIGHT          0x00000080 /* Device reports height       */
#define LTYPE_PIN             0x00000100 /* Pen contains PIN            */
#define LTYPE_ERASURE         0x00000200 /* Erasure end of stylus reported */
#define LTYPE_INTEG_SENSOR    0x00000400 /* Sensor integrated in display*/
#define LTYPE_BATTERY         0x00000800 /* Pen contains battery        */
#define LTYPE_RELATIVE        0x00001000 /* Relative pointing device    */
                                         /* Sensor coords from relative */
                                         /* devices are offsets from    */
                                         /* last reported point         */
#define LTYPE_ABSOLUTE        0x00002000 /* Absolute pointing device    */
#define LTYPE_FIXED_POINT_RATE 0x00004000 /* Device point rate is not adjustable  */


/* WRTLOCATORDEVINFO.ulValidityFlags Values */
#define ADF_TIMESTAMP         0x00000001 /* Point timestamp                 */
#define ADF_FLAGS             0x00000002 /* Point flags reported (See APD_*)*/
#define ADF_SCREENZ           0x00000004 /* +/-Z axis value normalized to screen  */
#define ADF_DEVZ              0x00000008 /* +/-Z axis value as reported by device */
#define ADF_ANGLE             0x00000010 /* Stylus angle to XY              */
#define ADF_ROTATION          0x00000020 /* Stylus barrel rotation          */
#define ADF_BUTTON            0x00000040 /* Button status                   */
#define ADF_OEM               0x00000080 /* OEM specific data byte cnt      */
#define ADF_OEM_COUNT_MASK    0x000F0000 /* OEM valid count                 */
#define ADF_USER              0x00000100 /* This bit is not HW related and is*/
                                         /* always set. For application use.*/
#define ADF_ALL               0X000001FF /* All auxiliary fields are valid  */

APIRET APIENTRY WrtQueryLocatorCaps(PULONG              ulNumLocators,
                                    PWRTLOCATORDEVINFO  pldiInfo);

typedef struct _WRTBUTTONDEVINFO    /* bdi */
  {
  ULONG     cbStructSize;       /* Structure size                    */
  ULONG     ulButtonID;         /* ID of button device               */
  ULONG     ulNumButtons;       /* Number of configured buttons      */
  ULONG     ulButtonMask;       /* 1=barrel,  0=non-barrel button    */
  UCHAR     pszDriverName[32];  /* Name of controlling device driver */
  UCHAR     pszDeviceName[32];  /* Name of button device             */
  } WRTBUTTONDEVINFO;
typedef WRTBUTTONDEVINFO *PWRTBUTTONDEVINFO;   /* pbdi */

APIRET APIENTRY WrtQueryButtonCaps(PULONG             ulNumButtons,
                                   PWRTBUTTONDEVINFO  pbdiInfo);

typedef struct _WRTDISPLAYDEVINFO     /* ddi */
  {
  ULONG     cbStructSize;      /* Structure size                    */
  ULONG     ulDisplayID;       /* ID of display device              */
  ULONG     ulDisplayType;     /* Type of display                   */
  ULONG     ulDisplayXPels;    /* Number of display X pixels        */
  ULONG     ulDisplayYPels;    /* Number of display Y pixels        */
  ULONG     ulDisplayXRes;     /* Display pels per inch X axis      */
  ULONG     ulDisplayYRes;     /* Display pels per inch Y axis      */
  ULONG     ulDisplayHeight;   /* Height of display in thousandsths */
  ULONG     ulDisplayWidth;    /* Width of display in thousandsths  */
  UCHAR     pszDriverName[32]; /* Name of controlling device driver */
  UCHAR     pszDeviceName[32]; /* Name of display device            */
  } WRTDISPLAYDEVINFO;
typedef WRTDISPLAYDEVINFO *PWRTDISPLAYDEVINFO;   /* pddi */

/* WRTDISPLAYDEVINFO.ulDisplayType Values */
#define DISPTYPE_LCD        0x0001   /* 1 - LCD,      0 - not LC   */
#define DISPTYPE_CRT        0x0002   /* 1 - CRT,      0 - not CR   */
#define DISPTYPE_BLANKING   0x0004   /* Display performs blankin   */
#define DISPTYPE_COLOR      0x0008   /* 1 - Color,    0 - Monochme */
#define DISPTYPE_ATTACHED   0x0010   /* 1 - Attached, 0 - Portab   */
#define DISPTYPE_NO_KBD     0x0020   /* No Keyboard Attached       */
#define DISPTYPE_BATTERY    0x0040   /* 1 - Battery,  0 - AC pow   */
#define DISPTYPE_SPEAKER    0x0080   /* 1 - Speaker integrated     */
#define DISPTYPE_MICROPHONE 0x0100   /* 1 - Microphone integrate   */

APIRET APIENTRY WrtQueryDisplayCaps(PULONG              ulDisplayID,
                                    PWRTDISPLAYDEVINFO  pddiInfo);


APIRET APIENTRY WrtWaitActive( ULONG ulTimeout );

/* WrtWaitActive Timeout equates */
#define WRT_INDEFINITE_WAIT     -1L
#define WRT_IMMEDIATE_RETURN     0L

APIRET APIENTRY WrtQueryWindowPointRate (HWND hwnd, PULONG pulRate, PULONG pflFlags);
#define QWPR_USER   0x0001     /* Point rate set by user via object override */
#define QWPR_APP    0x0002     /* Point rate set by application.             */

APIRET APIENTRY WrtSetWindowPointRate (HWND hwnd, ULONG ulRate );
/* Valid range of point rate settings */
#define PPMSV_POINTRATE_MAX  200
#define PPMSV_POINTRATE_MIN   40
#define PPMSV_POINTRATE_DEF  200


APIRET APIENTRY VkpCloseKb( HWND hwnd );
APIRET APIENTRY VkpHideKeyboard( HWND hwnd );
APIRET APIENTRY VkpLoadKeyboard( PSZ pszKbd, HWND hwnd );
APIRET APIENTRY VkpDeleteKeyboard( PSZ pszKbd, HWND hwnd );
APIRET APIENTRY VkpIsKbRunning( PBOOL fRunning );
APIRET APIENTRY VkpIsKbHidden( PBOOL fHidden );
APIRET APIENTRY VkpQueryKbPos( PSWP pswp );
APIRET APIENTRY VkpSetKbPos( LONG x, LONG y, LONG cx, LONG cy,
                             ULONG fs, HWND hwnd );


/****************************/
/* PPMSV_* System Value IDs */
/****************************/

/* Not stored in Profile */
#define  PPMSV_VERSION                  0    /*  PenPM version number        */
#define  PPMSV_RELEASE                  1
#define  PPMSV_LEVEL                    2
#define  PPMSV_SUSPEND                  3    /*  Suspend PenPM processing    */

/* Stored in Profile */
#define  PPMSV_PEN_PAUSE               13    /*  Pen pause T.O value         */
#define  PPMSV_TOUCH_PAUSE             14    /*  Touch pause T.O value       */
#define  PPMSV_MOUSE_PAUSE             15    /*  Mouse pause T.O value       */
#define  PPMSV_OTHER_PAUSE             16    /*  Other pause T.O value       */

#define  PPMSV_PAUSEBEEPFREQ           17    /*  Pause T.O. beep freq        */
#define  PPMSV_RECO_BEEP_ENABLE        18    /*  Enable beep for no reco map */
#define  PPMSV_MINGESTUREPOINTS        19    /*  Minimum gesture points      */
#define  PPMSV_RECOPTRTIMER            20    /*  Timer display time          */
#define  PPMSV_GESTURE_CANCEL_TIMEOUT  21    /*  Cancel gesture timeout      */
#define  PPMSV_GESTURE_TIMEOUT         22    /*  Inter-stroke gesture timeout*/
#define  PPMSV_STANDARDIZED_RESOLUTION 23    /*                              */

#define  PPMSV_CHARACTER_BOX_DX        24    /*  Character box dx and dy are */
#define  PPMSV_CHARACTER_BOX_DY        25    /*   read-only variables        */

#define  PPMSV_TOUCH_PAUSE_ENABLE      26    /*  Touch pause T.O enable      */
#define  PPMSV_PEN_PAUSE_ENABLE        27    /*  Pen pause T.O enable        */
#define  PPMSV_MOUSE_PAUSE_ENABLE      28    /*  Mouse pause T.O enable      */
#define  PPMSV_OTHER_PAUSE_ENABLE      29    /*  Other pause T.O enable      */

#define  PPMSV_RESERVED_FIRST          30    /*  Reserved                    */
#define  PPMSV_RESERVED_LAST           39    /*  Reserved                    */

#define  PPMSV_INK_WIDTH               40    /*  Width of system ink in pels */

#define  PPMSV_HW_PAD_STYLE            41    /*  Appearence style of HW pop-up*/
#define  PPMSV_HW_PAD_CONFIRM          42    /*  Pop up HWX for confirmation */

#define  PPMSV_SIZEABLE_CHARACTER_BOX_DX 43  /*  Sizeable character box width  */
#define  PPMSV_SIZEABLE_CHARACTER_BOX_DY 44  /*  Sizeable character box height */

#define  PPMSV_BASELINE_LEADING        45    /*  Space above baseline        */
#define  PPMSV_HW_PAD_POINT_SIZE       46    /*  Point size of edit pad font   */

#define  PPMSV_TAP_PAUSE_MAPPING       47    /*  Result of tap pause event   */
#define  PPMSV_DBLTAP_PAUSE_MAPPING    48    /*  Result of tap pause event   */

#define  PPMSV_BOXED_POINT_SIZE        49    /*  Point size of edit pad font - boxed    */
#define  PPMSV_LINED_POINT_SIZE        50    /*  Point size of edit pad font - lined    */
#define  PPMSV_UNLINED_POINT_SIZE      51    /*  Point size of edit pad font - unlined  */

#define  PPMSV_LINED_DY                52    /*  Space between lines in edit pad    */

#define  PPMSV_INCLUDE_RECT            53    /*  Enable include rectangle            */
#define  PPMSV_INCLUDE_RECT_CX         54    /*  Width of include rectangle in .001" */
#define  PPMSV_INCLUDE_RECT_CY         55    /*  Height of include rectangle         */

#define  PPMSV_WRITING_TIMEOUT         56    /*  Inter-stroke writing timeout for text */

#define  PPMSV_SPARE_FIRST             57    /*  Reserved                    */
#define  PPMSV_SPARE_LAST              62    /*  Reserved                    */

#define  PPMSV_PROFILE_FIRST           13
#define  PPMSV_PROFILE_LAST            62

#define  PPMSV_NUM_PPMSV               63
#define  PPMSV_PROTECTED               PPMSV_LEVEL


/***********************************/
/* PPMSV_* Variables Range Values  */
/***********************************/

#define  PPMSV_TOUCH_PAUSE_MIN             150
#define  PPMSV_TOUCH_PAUSE_MAX            2000
#define  PPMSV_TOUCH_PAUSE_DEF             300
#define  PPMSV_PEN_PAUSE_MIN               150
#define  PPMSV_PEN_PAUSE_MAX              2000
#define  PPMSV_PEN_PAUSE_DEF               300
#define  PPMSV_MOUSE_PAUSE_MIN             150
#define  PPMSV_MOUSE_PAUSE_MAX            2000
#define  PPMSV_MOUSE_PAUSE_DEF             400
#define  PPMSV_OTHER_PAUSE_MIN             150
#define  PPMSV_OTHER_PAUSE_MAX            3500
#define  PPMSV_OTHER_PAUSE_DEF             200

#define  PPMSV_GESTURE_CANCEL_TIMEOUT_MIN  500
#define  PPMSV_GESTURE_CANCEL_TIMEOUT_MAX 3000
#define  PPMSV_GESTURE_CANCEL_TIMEOUT_DEF 1200
#define  PPMSV_GESTURE_TIMEOUT_MIN         100
#define  PPMSV_GESTURE_TIMEOUT_MAX        4000
#define  PPMSV_GESTURE_TIMEOUT_DEF         900
#define  PPMSV_WRITING_TIMEOUT_MIN         100
#define  PPMSV_WRITING_TIMEOUT_MAX        6000
#define  PPMSV_WRITING_TIMEOUT_DEF        3000

#define  PPMSV_PAUSEBEEPFREQ_MIN           100
#define  PPMSV_PAUSEBEEPFREQ_MAX          4000
#define  PPMSV_PAUSEBEEPFREQ_DEF           100
#define  PPMSV_RECO_BEEP_ENABLE_MIN          0
#define  PPMSV_RECO_BEEP_ENABLE_MAX          1
#define  PPMSV_RECO_BEEP_ENABLE_DEF          1

#define  PPMSV_MINGESTUREPOINTS_MIN          1
#define  PPMSV_MINGESTUREPOINTS_MAX         20
#define  PPMSV_MINGESTUREPOINTS_DEF          5
#define  PPMSV_RECOPTRTIMER_MIN            200
#define  PPMSV_RECOPTRTIMER_MAX           2000
#define  PPMSV_RECOPTRTIMER_DEF            400

#define  PPMSV_STANDARDIZED_RESOLUTION_MIN 1000
#define  PPMSV_STANDARDIZED_RESOLUTION_MAX 1000
#define  PPMSV_STANDARDIZED_RESOLUTION_DEF 1000

#define  PPMSV_TOUCH_PAUSE_ENABLE_MIN        0
#define  PPMSV_TOUCH_PAUSE_ENABLE_MAX        1
#define  PPMSV_TOUCH_PAUSE_ENABLE_DEF        1
#define  PPMSV_PEN_PAUSE_ENABLE_MIN          0
#define  PPMSV_PEN_PAUSE_ENABLE_MAX          1
#define  PPMSV_PEN_PAUSE_ENABLE_DEF          1
#define  PPMSV_MOUSE_PAUSE_ENABLE_MIN        0
#define  PPMSV_MOUSE_PAUSE_ENABLE_MAX        1
#define  PPMSV_MOUSE_PAUSE_ENABLE_DEF        0
#define  PPMSV_OTHER_PAUSE_ENABLE_MIN        0
#define  PPMSV_OTHER_PAUSE_ENABLE_MAX        1
#define  PPMSV_OTHER_PAUSE_ENABLE_DEF        1
#define  PPMSV_INK_WIDTH_MIN                 1
#define  PPMSV_INK_WIDTH_MAX                 4
#define  PPMSV_INK_WIDTH_DEF                 1

/* HW pop preference equates  */
#define  HW_PAD_RULED     1       /* Ruled    */
#define  HW_PAD_UNRULED   2       /* Unruled  */
#define  HW_PAD_BOXED     3       /* Boxed    */

#define  PPMSV_HW_PAD_STYLE_MIN   HW_PAD_RULED
#define  PPMSV_HW_PAD_STYLE_MAX   HW_PAD_BOXED
#define  PPMSV_HW_PAD_STYLE_DEF   HW_PAD_BOXED

#define  PPMSV_HW_PAD_CONFIRM_MIN            0
#define  PPMSV_HW_PAD_CONFIRM_MAX            1
#define  PPMSV_HW_PAD_CONFIRM_DEF            1

#define  PPMSV_BASELINE_LEADING_MIN          0
#define  PPMSV_BASELINE_LEADING_MAX       1700     /* Max Y value */
#define  PPMSV_BASELINE_LEADING_DEF          0

#define  PPMSV_HW_PAD_POINT_SIZE_MIN         8
#define  PPMSV_HW_PAD_POINT_SIZE_MAX        72
#define  PPMSV_HW_PAD_POINT_SIZE_DEF        14

#define  PPMSV_TAP_PAUSE_MAPPING_MIN      PTO_BUTTON1
#define  PPMSV_TAP_PAUSE_MAPPING_MAX     (PTO_BUTTON_MASK + PTO_AUG_MASK)
#define  PPMSV_TAP_PAUSE_MAPPING_DEF     (PTO_BUTTON2 + PTO_SHIFT_AUG) /* Shift Button 2 (Move) */

#define  PPMSV_DBLTAP_PAUSE_MAPPING_MIN   PTO_BUTTON1
#define  PPMSV_DBLTAP_PAUSE_MAPPING_MAX  (PTO_BUTTON_MASK + PTO_AUG_MASK)
#define  PPMSV_DBLTAP_PAUSE_MAPPING_DEF  (PTO_BUTTON2 + PTO_CTRL_AUG) /* Ctrl Button 2 (Copy)  */

#define  PPMSV_BOXED_POINT_SIZE_MIN          8
#define  PPMSV_BOXED_POINT_SIZE_MAX         72
#define  PPMSV_BOXED_POINT_SIZE_DEF         14

#define  PPMSV_LINED_POINT_SIZE_MIN          8
#define  PPMSV_LINED_POINT_SIZE_MAX         72
#define  PPMSV_LINED_POINT_SIZE_DEF         16

#define  PPMSV_UNLINED_POINT_SIZE_MIN        8
#define  PPMSV_UNLINED_POINT_SIZE_MAX       72
#define  PPMSV_UNLINED_POINT_SIZE_DEF       14

#define  PPMSV_LINED_DY_MIN                  8
#define  PPMSV_LINED_DY_MAX                100
#define  PPMSV_LINED_DY_DEF                 25

#define  PPMSV_INCLUDE_RECT_MIN              0
#define  PPMSV_INCLUDE_RECT_MAX              1
#define  PPMSV_INCLUDE_RECT_DEF              1

#define  PPMSV_INCLUDE_RECT_CX_MIN         250
#define  PPMSV_INCLUDE_RECT_CX_MAX        8000
#define  PPMSV_INCLUDE_RECT_CX_DEF        4000      /* 4 inches default  */
#define  PPMSV_INCLUDE_RECT_CY_MIN         250
#define  PPMSV_INCLUDE_RECT_CY_MAX       10000
#define  PPMSV_INCLUDE_RECT_CY_DEF         500      /*  1/2 inch default */



APIRET APIENTRY WrtQueryInputDeviceVariable(PSZ     pszDriverName,
                                            PSZ     pszDeviceName,
                                            ULONG   ulValueID, /* See PPMID_* */
                                            PLONG   lValue,
                                            PULONG  ulDatalen,
                                            PVOID   pData);
APIRET APIENTRY WrtSetInputDeviceVariable(PSZ     pszDriverName,
                                          PSZ     pszDeviceName,
                                          ULONG   ulValueID, /* See PPMID_* */
                                          LONG    lValue,
                                          ULONG   ulDatalen,
                                          PVOID   pData);
/*************************************/
/* PPMID_* Input Device Variable IDs */
/*************************************/

/* Stored in Profile */
#define  PPMID_MOTION_STOP           0
#define  PPMID_MOTION_START          1
#define  PPMID_MINGESTURELENGTH      2
#define  PPMID_DISABLE_PROXIMITY     3
#define  PPMID_TOUCHOFFSET_X         4
#define  PPMID_TOUCHOFFSET_Y         5
#define  PPMID_BACKLIGHTBLANK        6
#define  PPMID_BACKLIGHTBLANK_INT    7
#define  PPMID_POINT_RATE            8
#define  PPMID_DIVIDE_RATE           9
#define  PPMID_DEVICE_PARMS         10
#define  PPMID_TAP_LENGTH           11
#define  PPMID_RSVD_2               12
#define  PPMID_RSVD_3               13

#define  PPMID_PROFILE_FIRST         0
#define  PPMID_PROFILE_LAST         13

#define  PPMID_NUM_PPMID            14


/***********************************/
/* PPMID_* Variables Range Values  */
/***********************************/
#define  PPMID_MOTION_STOP_MIN             14
#define  PPMID_MOTION_STOP_MAX            300
#define  PPMID_MOTION_STOP_DEF_PEN         28
#define  PPMID_MOTION_STOP_DEF_TOUCH       56
#define  PPMID_MOTION_STOP_DEF_MOUSE       28
#define  PPMID_MOTION_STOP_DEF_OTHER       28
#define  PPMID_MOTION_START_MIN             1
#define  PPMID_MOTION_START_MAX          1000
#define  PPMID_MOTION_START_DEF_PEN        64
#define  PPMID_MOTION_START_DEF_TOUCH     135
#define  PPMID_MOTION_START_DEF_MOUSE      52
#define  PPMID_MOTION_START_DEF_OTHER      25

#define  PPMID_MINGESTURELENGTH_MIN         0
#define  PPMID_MINGESTURELENGTH_MAX      8000
#define  PPMID_MINGESTURELENGTH_DEF_PEN   156
#define  PPMID_MINGESTURELENGTH_DEF_TOUCH 233
#define  PPMID_MINGESTURELENGTH_DEF_MOUSE 233
#define  PPMID_MINGESTURELENGTH_DEF_OTHER 233

#define  PPMID_DISABLE_PROXIMITY_MIN        0
#define  PPMID_DISABLE_PROXIMITY_MAX        1
#define  PPMID_DISABLE_PROXIMITY_DEF        1

#define  PPMID_BACKLIGHTBLANK_MIN           0
#define  PPMID_BACKLIGHTBLANK_MAX           1
#define  PPMID_BACKLIGHTBLANK_DEF           1
#define  PPMID_BACKLIGHTBLANK_INT_MIN      30
#define  PPMID_BACKLIGHTBLANK_INT_MAX  500000
#define  PPMID_BACKLIGHTBLANK_INT_DEF    1200

#define  PPMID_POINT_RATE_MIN              40
#define  PPMID_POINT_RATE_MAX             200
#define  PPMID_POINT_RATE_DEF             100

#define  PPMID_DIVIDE_RATE_MIN              1
#define  PPMID_DIVIDE_RATE_MAX              8
#define  PPMID_DIVIDE_RATE_DEF              2
#define  PPMID_DEVICE_PARMS_MIN             0
#define  PPMID_DEVICE_PARMS_MAX             1
#define  PPMID_DEVICE_PARMS_DEF             1

#define  PPMID_TOUCHOFFSET_X_MIN        -1000
#define  PPMID_TOUCHOFFSET_X_MAX         1000
#define  PPMID_TOUCHOFFSET_X_DEF           60
#define  PPMID_TOUCHOFFSET_Y_MIN        -1000
#define  PPMID_TOUCHOFFSET_Y_MAX         1000
#define  PPMID_TOUCHOFFSET_Y_DEF           10

#define  PPMID_TAP_LENGTH_MIN            PPMID_MOTION_START_MIN
#define  PPMID_TAP_LENGTH_MAX            PPMID_MOTION_START_MAX
#define  PPMID_TAP_LENGTH_DEF_PEN          73  /*91   7/28/94  JMZ*/
#define  PPMID_TAP_LENGTH_DEF_TOUCH       131  /* PPMID_MOTION_START_DEF_TOUCH */
#define  PPMID_TAP_LENGTH_DEF_MOUSE        79  /* PPMID_MOTION_START_DEF_MOUSE */
#define  PPMID_TAP_LENGTH_DEF_OTHER        62  /* PPMID_MOTION_START_DEF_OTHER */
/*                                                 */
/*  WM_WRT_SYSVALUECHANGED message mp1 values      */
/*                                                 */
#define   VT_PPMSV     1
#define   VT_PPMID     2


/************************/
/* WRTERR_* Error Codes */
/************************/
#define WRT_NO_ERROR                        0
#define WRTERR_START                   0x7000
#define WRTERR_ALREADY_ACTIVE          0x7001
#define WRTERR_NOT_ENOUGH_MEMORY       0x7002
#define WRTERR_CREATETHREAD_FAILED     0x7003
#define WRTERR_SETHOOK_FAILED          0x7004
#define WRTERR_RELEASEHOOK_FAILED      0x7005
#define WRTERR_NOT_ACTIVE              0x7006
#define WRTERR_NOT_OWNER               0x7007
#define WRTERR_INVALID_HWND            0x7008
#define WRTERR_INVALID_ID              0x7009
#define WRTERR_INVALID_PID             0x700A
#define WRTERR_TOO_MANY_MAPPERS        0x700B
#define WRTERR_WRITE_FAILED            0x700C
#define WRTERR_NOT_PEN_ENV             0x700D
#define WRTERR_BUFFER_TOO_SMALL        0x700E
#define WRTERR_STROKE_INVALID          0x700F
#define WRTERR_BxDOWN_IN_STATEB        0x7010
#define WRTERR_BxUP_IN_STATE0          0x7011
#define WRTERR_APPL_NOT_ACTIVE         0x7012

#define REDERR_RECO_SYS_ALREADY_REG    0x7013
#define REDERR_RECO_SUBSYS_ALREADY_REG 0x7014
#define REDERR_RECO_SUBSYS_NOT_REG     0x7015
#define REDERR_RECO_NAME_TOO_LONG      0x7016
#define REDERR_INVALID_RECO_NAME       0x7017
#define REDERR_EVENT_NAME_TOO_LONG     0x7018
#define REDERR_INVALID_EVENT_NAME      0x7019
#define REDERR_EVENT_NAME_NOT_REG      0x701A
#define REDERR_EVENT_NAME_ALREADY_REG  0x701B
#define REDERR_INVALID_EVENT_ID        0x701C
#define REDERR_INVALID_EVENT_PTR       0x701D
#define REDERR_RECO_DATA_INVALID       0x701E


#define WRTERR_SENSORRESOLUTION_UNAVAIL 0x701F
#define WRTERR_BxDOWN_IN_STATEA        0x7020
#define WRTERR_HITTEST_FAILED          0x7021
#define WRTERR_BACKLITE_CMD_FAILED     0x7022
#define WRTERR_INVALID_PARM            0x7023
#define WRTERR_INVALID_EXTENTS         0x7024
#define WRTERR_INVALID_OPTION          0x7025
#define WRTERR_PARM_OUT_OF_RANGE       0x7026
#define WRTERR_INVALID_BUFFER          0x7027
#define WRTERR_INVALID_DRIVER_NAME     0x7028
#define WRTERR_INVALID_DEVICE_NAME     0x7029
#define WRTERR_MORE_NAMES_AVAIL        0x702A
#define WRTERR_INVALID_POINTER         0x702B
#define WRTERR_INVALID_LOCATOR_ID      0x702C
#define WRTERR_INVALID_BUTTON_ID       0x702D
#define WRTERR_INVALID_DISPLAY_ID      0x702E
#define WRTERR_STRUCT_NOT_INIT         0x702F
#define WRTERR_VALUEID_DEVICE_MISMATCH 0x7030
#define WRTERR_NO_DEVICE_CAPABILITY    0x7031
#define WRTERR_INVALID_BUTTON_EVENT    0x7032
#define WRTERR_WAIT_TIMEOUT            0x7033
#define WRTERR_INVALID_MOUSE_EVENT     0x7034
#define WRTERR_INVALID_INPUT           0x7035
#define WRTERR_INVALID_COUNT           0x7036
#define WRTERR_ID_READ_ONLY            0x7037
#define WRTERR_PEN_NOT_INSTALLED       0x7038
#define WRTERR_NOT_ABLE_TO_CREATE_CONTROL 0x7039
#define WRTERR_NOT_ABLE_TO_CREATE_DLG  0x703A
#define WRTERR_INVALID_STROKE_BUFFER   0x703B
#define WRTERR_INVALID_MESSAGE         0x703C
#define WRTERR_NO_HOBJECT_FOUND        0x703D
#define WRTERR_EXCEEDED_MAX_SIZE       0x703E
#define WRTERR_CREATE_OBJECT_FAILED    0x703F
#define WRTERR_BAD_VERSION             0x7040
#define WRTERR_EXCEPTION               0x7041
#define WRTERR_BAD_BYTE_COUNT          0x7042
#define WRTERR_ALLOC_MEMORY_FAILURE    0x7043

/************************/
/* VKPERR_* Error Codes */
/************************/

#define VKP_NO_ERROR                   NO_ERROR
#define VKPERR_OUT_OF_MEMORY           0x7100
#define VKPERR_VKP_NOT_READY           0x7101
#define VKPERR_INVALID_KBD_NAME        0x7102
#define VKPERR_KBD_LOAD_FAILED         0x7103
#define VKPERR_KBD_LOAD_TIMEOUT        0x7104
#define VKPERR_INVALID_PARM            0x7105
#define VKPERR_WSWP_FAILED             0x7106
#define VKPERR_WQWP_FAILED             0x7107
#define VKPERR_INVALID_OWNER           0x7108

/***********************************/
/* Input Data Framework defines    */
/***********************************/
#include "penidf.h"         /* Some of these defines require SOM support and */
                            /* are only brought in when INCL_IDx is defined */

/* Start of handwriting control area */

#ifndef __HWXENTRY_INCLUDED__
#define __HWXENTRY_INCLUDED__

/*****************************************************************************/
/*                                                                           */
/* Class Name                                                                */
/*                                                                           */
/*****************************************************************************/

#define WC_HWXENTRY     ((PSZ)0xffff0029L)

/*****************************************************************************/
/*                                                                           */
/* Handwriting Control Style Bits                                            */
/*                                                                           */
/*****************************************************************************/

#define HX_BORDER       0x0001   /* Control displays a    border        */
#define HX_3DBORDER     0x0002   /* Control displays a 3D border        */

/******************************************************************************/
/*                                                                            */
/* Control Messages                                                           */
/*                                                                            */
/*  Messages sent to Handwriting Control from Owner                           */
/*                                                                            */
/******************************************************************************/

                                                      /* Non modal messages     */
#define HXM_QUERY_BACK_COLOR            WM_USER + 0x01 /* query background color */
#define HXM_SET_BACK_COLOR              WM_USER + 0x02 /* set background color   */
#define HXM_QUERY_BOX_COLOR             WM_USER + 0x03 /* query box color        */
#define HXM_SET_BOX_COLOR               WM_USER + 0x04 /* set box color          */
#define HXM_QUERY_BOX_SIZE              WM_USER + 0x05 /* query box size         */
#define HXM_SET_BOX_SIZE                WM_USER + 0x06 /* set box size           */
#define HXM_QUERY_CHAR_BOX              WM_USER + 0x07 /* query character        */
#define HXM_SET_CHAR_BOX                WM_USER + 0x08 /* set character          */
#define HXM_SET_DICTIONARY_COERCION     WM_USER + 0x09
#define HXM_QUERY_DICTIONARY_COERCION   WM_USER + 0x0a
#define HXM_QUERY_EXCLUDE_RECTL         WM_USER + 0x0b /* query exclusion rectls */
#define HXM_SET_EXCLUDE_RECTL           WM_USER + 0x0c /* set exclusion rectls   */
#define HXM_QUERY_WRITING_POINTER       WM_USER + 0x0d /* query writing pointer  */
#define HXM_SET_WRITING_POINTER         WM_USER + 0x0e /* set writing pointer    */
#define HXM_QUERY_INK_COLOR             WM_USER + 0x0f /* query ink color        */
#define HXM_SET_INK_COLOR               WM_USER + 0x10 /* set ink color          */
#define HXM_QUERY_INK_TYPE              WM_USER + 0x11 /* query ink type         */
#define HXM_SET_INK_TYPE                WM_USER + 0x12 /* set ink type           */
#define HXM_QUERY_INK_MIX               WM_USER + 0x13 /* query ink mix          */
#define HXM_SET_INK_MIX                 WM_USER + 0x14 /* set ink mix            */
#define HXM_QUERY_INK_WIDTH             WM_USER + 0x15 /* query ink width        */
#define HXM_SET_INK_WIDTH               WM_USER + 0x16 /* set ink width          */
#define HXM_QUERY_LINE_COLOR            WM_USER + 0x17 /* query line color       */
#define HXM_SET_LINE_COLOR              WM_USER + 0x18 /* set line color         */
#define HXM_QUERY_LINE_SPACING          WM_USER + 0x19 /* query line spacing     */
#define HXM_SET_LINE_SPACING            WM_USER + 0x1a /* set line spacing       */
#define HXM_QUERY_LINE_WIDTH            WM_USER + 0x1b /* query line width       */
#define HXM_SET_LINE_WIDTH              WM_USER + 0x1c /* set line width         */
#define HXM_QUERY_MAX_RESULTS           WM_USER + 0x1d /* query line width       */
#define HXM_SET_MAX_RESULTS             WM_USER + 0x1e /* set line width         */
#define HXM_QUERY_RECO_FORM             WM_USER + 0x21 /* query recognition style*/
#define HXM_SET_RECO_FORM               WM_USER + 0x22 /* set recognition style  */
#define HXM_QUERY_PROXIMITY_POINTER     WM_USER + 0x23 /* query proximity pointer*/
#define HXM_SET_PROXIMITY_POINTER       WM_USER + 0x24 /* set proximity pointer  */
#define HXM_QUERY_EOW_EVENTS            WM_USER + 0x25 /* query end of writing   */
#define HXM_SET_EOW_EVENTS              WM_USER + 0x26 /* set end of writing     */
#define HXM_QUERY_SYMBOL_SUBSET         WM_USER + 0x28 /* query symbol subset    */
#define HXM_SET_SYMBOL_SUBSET           WM_USER + 0x27 /* set symbol subset      */
#define HXM_QUERY_SMBOL_SUBSET_PRIORITY WM_USER + 0x29 /* query symbol subset priority*/
#define HXM_SET_SYMBOL_SUBSET_PRIORITY  WM_USER + 0x2a /* set symbol subset priority  */
#define HXM_QUERY_INKING_STYLE          WM_USER + 0x2d /* query inking style     */
#define HXM_SET_INKING_STYLE            WM_USER + 0x2e /* set inking style       */
#define HXM_SET_INCLUDE_RECTL           WM_USER + 0x2f
#define HXM_QUERY_INCLUDE_RECTL         WM_USER + 0x30
#define HXM_SET_GESTURE_SUBSET          WM_USER + 0x31
#define HXM_QUERY_GESTURE_SUBSET        WM_USER + 0x32
#define HXM_SET_DICTIONARY_POLICY       WM_USER + 0x33
#define HXM_QUERY_DICTIONARY_POLICY     WM_USER + 0x34

/*****************************************************************************/
/*                                                                           */
/*  Parent Notification Messages                                             */
/*                                                                           */
/*****************************************************************************/

#define HXN_STROKE_ADDED          WM_USER + 1   /* Stroke Add pending         */
#define HXN_GESTURE_PENDING       WM_USER + 2   /* Gesture pending            */
#define HXN_DELETE_PENDING        WM_USER + 3   /* Delete char pending        */
#define HXN_INSERT_PENDING        WM_USER + 4   /* Insert char pending        */
#define HXN_OVERFLOW              WM_USER + 5   /* Control text overflow      */
#define HXN_SETFOCUS              WM_USER + 6   /* Input focus activated      */
#define HXN_KILLFOCUS             WM_USER + 7   /* Input focus deactivated    */
#define HXN_CONTENTS_CHANGED      WM_USER + 8   /* Control Recognition change */
#define HXN_RECO_RESULTS          WM_USER + 9   /* recognition result message */
#define HXN_STATE_CHANGED         WM_USER + 10  /* HWX state changed          */
#define HXN_TERMINATE             WM_USER + 11  /* HWX state changed          */


/*****************************************************************************/
/*                                                                           */
/*  Result Data structure passed in WinSendMsg() for HXN_RECO_RESULTS        */
/*                                                                           */
/*****************************************************************************/

/**** Reco Text Array  *****/

#define CSF_CHAR_EMPTY          1               /* Character cell is empty   */
#define CSF_CHAR_VALID          2               /* Character cell has valid char*/
#define CSF_CHAR_NOT_RECOD      3               /* Character cell not reco'd */

typedef struct _HWXCHAR {                       /* HWX CHAR Definition       */
  unsigned char      ucChar;                    /* Character                 */
  unsigned char      ucCharStatus;              /* Character status flags    */
} HWXCHAR, *PHWXCHAR, **PPHWXCHAR;              /*                           */

/**** Results Type  *****/

typedef struct _HWXRECORESULTS {                /* HWX Reco Results          */
  ULONG            ulResultType;                /* Result type               */
  PSZ              pszRecoText;                 /* Recoed string             */
  PSZ              pszCtrlText;                 /* Ctrl string before update */
  PRECODATA        pRecoData;                   /* gesture data              */
  PHWXCHAR         pRecoTextArray;              /* Reco Text Array           */
  PIDR_SYM         pRecoResultsArray;           /* Ptr to Reco Results Array */
  ULONG            reserved;                    /* reserved                  */
} HWXRECORESULTS, *PHWXRECORESULTS, **PPHWXRECORESULTS;

                                                /* Reco Results               */
typedef struct _HWXPMCTRLRECORESULTS {          /* MLE/SLE,COMBOBOX/SPINBUTTON*/
  ULONG            ulResultType;                /* Result type                */
  PSZ              pszRecoText;                 /* Recoed string              */
  PRECODATA        pRecoData;                   /* gesture data               */
} HWXPMCTRLRECORESULTS, *PHWXPMCTRLRECORESULTS, **PPHWXPMCTRLRECORESULTS;

/*****************************************************************************/
/*                                                                           */
/*  Application return code for WinSendMsg() for HXN_RECO_RESULTS            */
/*                                                                           */
/*****************************************************************************/

#define RE_RECO_TEXT               0x00000003  /*recognize text              */
#define RE_RECO_GESTURE            0x00000004  /*recognize gesture           */
#define RE_RECO_GESTURE_NO_MAP     0x00000005  /*recognize not mapped gesture*/
#define RE_NO_RECO                 0x00000006  /*no further action           */

/*****************************************************************************/
/*                                                                           */
/*  MLE,SLE,COMBOBOX,SPINBUTTON interface                                    */
/*                                                                           */
/*****************************************************************************/

typedef struct _HXMI {                          /* HXMI     Definition       */
  MPARAM mp1;                                   /* mp1                       */
  MPARAM mp2;                                   /* mp2                       */
  ULONG  reserved;                              /* reserve                   */
} HXMI, *PHXMI, **PPHXMI;                       /*                           */

/*****************************************************************************/
/*                                                                           */
/*  Control Data structure passed in WinCreateWindow()                       */
/*                                                                           */
/*****************************************************************************/

typedef struct _BOX_LAYOUT {
          POINTL   ptlBox;         /*height and width of individual box     */
          LONG     lBoxColor;      /*Box Color                              */
      } BOX_LAYOUT;
typedef BOX_LAYOUT * PBOX_LAYOUT;

typedef struct _LINE_LAYOUT {
          LONG     lLineColor;     /*line Color                            */
          LONG     lSpacing;       /*line spacing                          */
          LONG     lWidth;         /*line width                            */
      } LINE_LAYOUT;
typedef LINE_LAYOUT * PLINE_LAYOUT;

typedef struct _HWX_IDRECT {
    ULONG  cbSize;                 /* number of exclude rectangle IDs         */
    ULONG  aulList[1];             /* array of exclude rectangle IDs          */
} HWX_IDRECT;
typedef HWX_IDRECT *PHWX_IDRECT;

/* defines for HWXCTLDATA.ulVersion */
#define PEN_VERSION_11  0x00000001 /* Pen for OS/2 version                    */
#define PEN_DIALOG_UNIT 0x80000000 /* identies if arclList, rclInclude,       */
                                   /* ptlBox, and lSpacing are in dialog      */
                                   /* units. If 0 they are in window          */
                                   /* coordinates.                            */
#define PEN_ID_RECT     0x40000000 /* identify if arclList is an id list or a */
                                   /* rectangle list. If 0 they are a         */
                                   /* rectangle list.                         */
#define PEN_FLAT        0x20000000 /* identify if the structure has flat      */
                                   /* layout or pointers. If 0 the structure  */
                                   /* has pointers.                           */
#define PEN_SPTR_PROXIMITY 0x10000000/* Proximity pointer is a system pointer */
#define PEN_SPTR_WRITING   0x08000000/* Writing   pointer is a system pointer */

#define PEN_IGNORE_OVERRIDES 0x04000000 /* Ignore overrides */

typedef struct _HWXCTLDATA {
           ULONG       cbSize;         /*size of HWX_CTLDATA structure        */
           ULONG       ulVersion;      /*Pen for OS/2 version                 */
                                       /*The low two bytes are for the pen    */
                                       /*version and the high two bytes are   */
                                       /*for control flags                    */
           BOX_LAYOUT  BoxLayout;      /*box layout                           */
           LINE_LAYOUT LineLayout;     /*Line layout                          */
           ULONG       ulRecoForm;     /*type of segmentation                 */
           ULONG       ulInkingStyle;  /*System or Window PS ink              */
           LONG        lInkColor;      /*ink color                            */
           LONG        lInkType;       /*cosmetic ink type                    */
           LONG        lInkMix;        /*ink color mix                        */
           LONG        lInkWidth;      /*ink color width                      */
           ULONG       idProximityPtr; /*in proximity, not writing            */
           ULONG       idWritingPointer;/*in proximity, writing               */
           ULONG       ulEOWevents;    /*events starting recognition          */
           RECTL       rclInclude;     /*valid touchdown region               */
           ULONG       ulCoercion;     /*degree of dictionary influence       */
           ULONG       ulPolicy;       /*text language modeling               */
           ULONG       ulSetMask;      /*symbol subset mask                   */
           union {
             ULONG cDLLName;        /* Number of bytes in DLL NAME pluse NULL */
             PSZ  pszDLLName;          /* Pointer to NULL terminated string   */
           } uDLLName;
           union {
             ULONG cRectList;                 /* number of exclude rectangles */
             PHWX_IDRECT phwxidrectList;             /* pointer to HWX_IDRECT */
             PIDM_RECTLIST pidmrectList;           /* pointer to IDM_RECTLIST */
           } uIDMRectList;
           union {
             ULONG cSymList;                         /* number of symbol list */
             PIDR_SYMSET pidrsymsetList;
           } uIDRSymSet;
           union {
             ULONG cGestList;                        /* Gesture List Count    */
             PIDR_SYMSET pidrsymsetGest;
           } uIDRGestSet;
           /*******************************************************************/
           /* (ulVersion & PEN_FLAT) == PEN_FLAT                              */
           /* and uDLLName.cDLLName is not zero                               */
           /* CHAR      aDLLName[1];*/        /* is a NULL termiante array    */
           /*                                                                 */
           /* (ulVersion & PEN_FLAT) == PEN_FLAT                              */
           /* and uIDRSymSet.cSymList not zero                                */
           /* IDR_SYM   asymList[1];                                          */
           /*                                                                 */
           /* (ulVersion & PEN_FLAT) == PEN_FLAT                              */
           /* and uIDRGestSet.cGestList not zero                              */
           /* IDR_SYM   asymGestList[1];                                      */
           /*                                                                 */
           /* (ulVersion & PEN_FLAT) == PEN_FLAT                              */
           /* and (ulVersion & PEN_ID_RECT) == PEN_ID_RECT                    */
           /* and uIDMRectList.cRectList not zero                             */
           /* ULONG     aulList[1];                                           */
           /*                                                                 */
           /* (ulVersion & PEN_FLAT) == PEN_FLAT                              */
           /* and (ulVersion & PEN_ID_RECT) == 0                              */
           /* and uIDMRectList.cRectList not zero                             */
           /* RECTL     arclLIst[1];                                          */
           /*******************************************************************/
      } HWXCTLDATA;
typedef HWXCTLDATA * PHWXCTLDATA;

/********************************/
/* HWX structure definitions    */
/********************************/
/* if you are using the PM control with a pointer to Pen control data */
/* set the .pHWXCtlData = PENPMDATA;                                  */
#define PENPMDATA (PVOID)0x00010000

/* if you are using the PM control with a structure to Pen control data */
/* set the .pHWXCtlData = PENPMDATAF;                                   */
#define PENPMDATAF (PVOID)0x80010000

#ifdef INCL_WINMLE
typedef struct _PENMLE
  {
    MLECTLDATA mleCtlData;
    PHWXCTLDATA pHwxCtlData;
  } PENMLE;
typedef struct _PENMLEF
  {
    MLECTLDATA mleCtlData;
    HWXCTLDATA HwxCtlData;
  } PENMLEF;
#endif /* !INCL_WINMLE */

#ifdef INCL_WINENTRYFIELDS
typedef struct _PENSLE
  {
    ENTRYFDATA   sleCtlData;
    PHWXCTLDATA pHwxCtlData;
  } PENSLE;
typedef struct _PENSLEF
  {
    ENTRYFDATA   sleCtlData;
    HWXCTLDATA   HwxCtlData;
  } PENSLEF;

typedef struct _PENCOMBO
  {
    COMBOCDATA   comboCtlData;
    PHWXCTLDATA pHwxCtlData;
  } PENCOMBO;
typedef struct _PENCOMBOF
  {
    COMBOCDATA   comboCtlData;
    HWXCTLDATA   HwxCtlData;
  } PENCOMBOF;

#endif /* !INCL_WINENTRYFIELDS */

#ifdef INCL_WINSTDDLGS

typedef struct _PENSPIN
  {
    SPBCDATA     spinCtlData;
    PHWXCTLDATA pHwxCtlData;
  } PENSPIN;
typedef struct _PENSPINF
  {
    SPBCDATA     spinCtlData;
    HWXCTLDATA   HwxCtlData;
  } PENSPINF;

#endif /* !INCL_WINSTDDLGS */

/*******************************************/
/* HWX  ERROR CODE                         */
/*******************************************/

#define HWX_ERROR_NO_CHAR            0

#endif                              /* ifndef __HWXENTRY_INCLUDED__ */

/* Start of Sketch area */

#ifndef __SKETCH_INCLUDED__
#define __SKETCH_INCLUDED__

/******************************************************************************/
/*                                                                            */
/* Class Name                                                                 */
/*                                                                            */
/******************************************************************************/

#define WC_SKETCH       ((PSZ)0xffff002AL)        /* sketch control */



/******************************************************************************/
/*                                                                            */
/* Control Style Bits                                                         */
/*                                                                            */
/******************************************************************************/

#define SKS_BORDER       0x0001   /* Control appearance is Bordered     */
#define SKS_3DBORDER     0x0002   /* Control appearance is 3D Bordered  */

/******************************************************************************/
/*                                                                            */
/* Control Messages                                                           */
/*                                                                            */
/*  Messages sent to Sketch Control from Owner                                */
/*                                                                            */
/* These constants below are used as messages to query and change             */
/* the state of a WC_SKETCH window.                                           */
/* They are sent to WC_SKETCH windows using WinSendMsg like so:               */
/*                                                                            */
/*    rc = WinSendMsg (hwndSketch, SKM_xxx, mp1, mp2);                        */
/*                                                                            */
/******************************************************************************/


                                                    /* Non modal messages     */
#define SKM_DELETE_ALL_STROKES        WM_USER +  1  /*                        */
#define SKM_GET_BITMAP                WM_USER +  2  /*                        */
#define SKM_UNDO_LAST_STROKE          WM_USER +  3  /*                        */
#define SKM_QUERY_BITMAP_SIZE         WM_USER +  4  /*                        */
#define SKM_RENDER_TO_CLIPBOARD       WM_USER +  5  /*                        */
                                                    /*                        */
                                                    /* Query control values   */
#define SKM_QUERY_CTL_DRAW_POINTER    WM_USER +  6  /*                        */
#define SKM_QUERY_CTL_INK_COLOR       WM_USER +  7  /*                        */
#define SKM_QUERY_CTL_INK_WIDTH       WM_USER +  8  /*                        */
#define SKM_QUERY_CTL_INVISIBLE_MODE  WM_USER +  9  /*                        */
#define SKM_QUERY_CTL_STROKE_COUNT    WM_USER + 10  /*                        */
                                                    /*                        */
                                                    /* Query stroke values    */
#define SKM_QUERY_STROKE_DATA         WM_USER + 11  /*                        */
#define SKM_QUERY_STROKE_INK_COLOR    WM_USER + 12  /*                        */
#define SKM_QUERY_STROKE_INK_WIDTH    WM_USER + 13  /*                        */
#define SKM_QUERY_STROKE_LENGTH       WM_USER + 14  /*                        */
                                                    /*                        */
                                                    /* Set control values     */
#define SKM_SET_CTL_AUXDATA_MODE      WM_USER + 15  /* level of aux data      */
#define SKM_SET_CTL_DRAW_POINTER      WM_USER + 16  /* draw pointer           */
#define SKM_SET_CTL_INK_COLOR         WM_USER + 17  /* ink color              */
#define SKM_SET_CTL_INK_WIDTH         WM_USER + 18  /* ink width              */
#define SKM_SET_CTL_INVISIBLE_MODE    WM_USER + 19  /* invisible mode         */
                                                    /*                        */
                                                    /* Set stroke values      */
#define SKM_SET_STROKE_INK_COLOR      WM_USER + 20  /*                        */
#define SKM_SET_STROKE_INK_WIDTH      WM_USER + 21  /*                        */

#define SKM_ADD_STROKE                WM_USER + 22  /* Add a stroke           */
#define SKM_QUERY_STANDARDIZED_DATA   WM_USER + 23  /* Query standardized data*/
#define SKM_SET_STANDARDIZED_DATA     WM_USER + 24  /* Set standardized data  */
#define SKM_SET_ERASER_MODE           WM_USER + 25  /* Set drawing mode       */
#define SKM_QUERY_ERASER_MODE         WM_USER + 26  /* Query drawing mode     */
#define SKM_SET_BACKGROUND_BITMAP     WM_USER + 27  /* Set background bitmap  */
#define SKM_QUERY_BACKGROUND_BITMAP   WM_USER + 28  /* Query background bitmap*/

/******************************************************************************/
/*                                                                            */
/* Parent Notification Messages                                               */
/*                                                                            */
/* The WC_SKETCH window will send a message to its parent                     */
/*                                                                            */
/*     WinSendMsg (hwndOwner, WM_CONTROL,                                     */
/*             MPFROM2SHORT (idSktWindow, usNotifyCode), mp2);                */
/*                                                                            */
/* These constants below are used as notification codes                       */
/* for WM_CONTROL messages. (These are usNotifyCode's.)                       */
/*                                                                            */
/******************************************************************************/

#define SKN_FOCUS_ACTIVATE        WM_USER + 100   /* Control has gained focus        */
#define SKN_FOCUS_DEACTIVATE      WM_USER + 102   /* Control has lost focus          */
#define SKN_STROKE_ADD            WM_USER + 103   /* Control is adding stroke        */
#define SKN_STROKE_UNDO           WM_USER + 104   /* Control is deleteting stroke    */
#define SKN_CONTROL_CLEARED       WM_USER + 105   /* Control is deleting all strokes */
#define SKN_INK_COLOR_CHANGE      WM_USER + 106   /* Control has changed ink color   */
#define SKN_INK_WIDTH_CHANGE      WM_USER + 107   /* Control has changed ink width   */

/******************************************************************************/
/*                                                                            */
/*     Structure used to retrieve stroke(i) from the stroke database.         */
/*                                                                            */
/*                                                                            */
/*             . ------------------------------- .                            */
/*           1 .                                 .                            */
/*             . SKETCHSTROKEDEF                 .                            */
/*             .                                 .                            */
/*             .         PSTROKEDATA             . -> 2                       */
/*             . ------------------------------- .                            */
/*           2 .                                 .                            */
/*             . STROKEDATA                      .                            */
/*             .                                 .                            */
/*             .         PPOINTL                 . -> 3                       */
/*             .                                 .                            */
/*             .         PAUXDATAINFO            . -> 4                       */
/*             .         PAUXSTROKEDATA          . -> 5                       */
/*             .                                 .                            */
/*             .         struct _STROKEDATA *    . -> NULL                    */
/*             .         struct _STROKEDATA *    . -> NULL                    */
/*             . ------------------------------- .                            */
/*           3 .                                 .                            */
/*             . X/Y POINTL array                .                            */
/*             .                                 .                            */
/*             . ------------------------------- .                            */
/*           4 .                                 .                            */
/*             . AUXDATAINFO                     .                            */
/*             .                                 .                            */
/*             . ------------------------------- .                            */
/*           5 .                                 .                            */
/*             . AUXPOINTDATA                    .                            */
/*             .                                 .                            */
/*             . ------------------------------- .                            */
/*                                                                            */
/******************************************************************************/

typedef struct _SKETCHSTROKEDEF              /* Sketch stroke data  */
{
   ULONG  ulStrokeColor;                     /* Color of the stroke */
   ULONG  ulStrokeWidth;                     /* Width of the Stroke */
   POINTL reserved1;                         /* reserved            */
   ULONG  reserved2;                         /* reserved            */
   ULONG  reserved3;                         /* reserved            */
   PSTROKEDATA pStroke;                      /* Pointer to STROKEDATA */
} SKETCHSTROKEDEF;

typedef SKETCHSTROKEDEF *PSKETCHSTROKEDEF;    /* Ptr->SKETCHSTROKEDEF         */
typedef PSKETCHSTROKEDEF *PPSKETCHSTROKEDEF;  /* Ptr->Ptr->SKETCHSTROKEDEF    */

#endif                                  /* ifndef __SKETCH_INCLUDED__ */

/* Start of RED area */

#ifndef PPMRED_INCLUDED
#define PPMRED_INCLUDED


#define REDHOBJ_OS2 (HOBJECT)(0xfffffffe)
#define REDHOBJ_WIN (HOBJECT)(0xfffffffd)
#define REDHOBJ_DOS (HOBJECT)(0xfffffffc)
#define REDHOBJ_PM  (HOBJECT)(0xfffffffb)
#define REDHOBJ_FLDR (HOBJECT)(0xfffffffa)
#define REDHOBJ_SYS  (HOBJECT)(0xfffffff9)


/*
 * Error defines for Red layer
 */
#define RED_NO_ERROR (0)
#define RED_INVALID_BUTTON        0x0000a000
#define RED_INVALID_BI            0x0000a001
#define RED_ERR_BUF_TOO_SMALL     0x0000a002
#define RED_INVALID_PARM          0x0000a003
#define RED_NO_MEMORY             0x0000a004
#define RED_NOT_FOUND             0x0000a005
#define RED_WRITE_ERR             0x0000a006
#define RED_PROFILE_ERROR         0x0000a007
#define RED_CMD_IS_BUILTIN        0x0000a008
#define RED_CMD_IS_TOO_LONG       0x0000a009

/*
 * Error defines for Red layer for 1.0 compatability
 */
#define REDERR_INVALID_BUTTON     0x0000a000
#define REDERR_INVALID_BI         0x0000a001
#define REDERR_BUF_TOO_SMALL      0x0000a002
#define REDERR_INVALID_PARM       0x0000a003
#define REDERR_NO_MEMORY          0x0000a004
#define REDERR_NOT_FOUND          0x0000a005
#define REDERR_WRITE_ERR          0x0000a006
#define REDERR_PROFILE_ERROR      0x0000a007
#define REDERR_CMD_IS_BUILTIN     0x0000a008


/*
 * declarations for RED functions
 */

#define MAX_RECO_COMMAND_NAME_LENGTH   11  /* Maximum characters length for  */
                                           /* a registered reco command      */
APIRET APIENTRY RedRegisterRecoCommand(PSZ pszCommand, ULONG id, HWND hwnd);

HWND APIENTRY RedQueryRecoCommand(UCHAR *cmdp);

APIRET APIENTRY RedDeregisterRecoCommand(HWND hwnd);


APIRET APIENTRY RedRecoDataFromEnv(PRECODATA prdEnv, ULONG *lenp);


APIRET APIENTRY RedQueryRecoHandle(PSZ     pszRecoSubsystem,
                                    PHRECO  phReco,
                                    PULONG pEventCount);


APIRET APIENTRY RedQueryRecoSubsystem(HRECO   hReco,
                                      PSZ     pszRecoSubsystem,
                                      PULONG pEventCount);


APIRET APIENTRY RedRecoIDFromName(HRECO   hReco,
                                  PSZ     pszEventName,
                                  PRECOID eventID);

APIRET APIENTRY RedRecoNameFromID(HRECO   hReco,
                                  RECOID  eventID,
                                  PSZ     pszEventName);

APIRET APIENTRY RedWriteObjectEventMap(HOBJECT hobj, UCHAR *fnamep);

APIRET APIENTRY RedReadObjectEventMap(HOBJECT hobj, UCHAR *fnamep);

APIRET APIENTRY RedHandleRecoEvent(RECODATA    *prdQ,
                                   BOOL        fSendRecoMsg,
                                   BOOL        fRecoPtr,
                                   BOOL        fRouteUpOwnerChain);
#endif      /* ifndef RED_INCLUDED */

/*****************************************************************************/
/* WrtHandwritingWindow define                                               */
/*****************************************************************************/

#ifndef WRT_HANDWRITING
#define WRT_HANDWRITING

/* Control style */
#define HW_BOX             0x0000000  /* default is BOX    */
#define HW_LINE            0x0010000  /* MLE with lines    */
#define HW_NOLINE          0x0030000  /* MLE with no lines */

/* Button or Button Group  */
#define HW_BUTTONMASK      0x03000000
#define HW_OK              0x00000000 /*defualt */ /* 1 button */
#define HW_OKCLEAR         0x01000000              /* 2 button */
#define HW_OKCANCEL        0x02000000              /* 2 button */
#define HW_OKCANCELCLEAR   0x03000000              /* 3 button */

/* Help button */
#define HW_HELP            0x04000000 /* add the Help button */
                                      /* default is no Help button */
/* Modality indicator */
#define HW_APPLMODAL       0x00000000 /*defualt */
#define HW_SYSTEMMODAL     0x10000000

/* Mobility indicator */
#define HW_MOVEABLE        0x20000000 /* default is not moveable */
#define HW_SIZABLE         0x40000000 /* default is not sizeable */

/* return code */
#define   HWID_OK           DID_OK
#define   HWID_CANCEL       DID_CANCEL
#define   HWID_TRUNCATED    3

ULONG APIENTRY WrtHandwritingWindow ( HWND hwndParent, HWND hwndOwner, PSZ pszTitle, PSZ pszText,
                                       ULONG ulMaxHWXLength, ULONG  ulWindow, ULONG  flStyle,
                                       ULONG flCtlStyle, PVOID pCtlData, PVOID pPresParams );
ULONG APIENTRY WrtGetDefHWXCTLDATA ( PHWXCTLDATA pctldata,
                                     PULONG pulctldatalen,
                                     ULONG ulRecoForm );
#endif      /* ifndef WRT_HANDWRITING */
#endif


