/*****************************************************************************/
/*                                                                           */
/*  File Name   : SLESAMP.C                                                  */
/*                                                                           */
/*  Copyright (C) 1995 IBM Corporation                                       */
/*                                                                           */
/*      DISCLAIMER OF WARRANTIES.  The following [enclosed] code is          */
/*      sample code created by IBM Corporation. This sample code is not      */
/*      part of any standard or IBM product and is provided to you solely    */
/*      for  the purpose of assisting you in the development of your         */
/*      applications.  The code is provided "AS IS", without                 */
/*      warranty of any kind.  IBM shall not be liable for any damages       */
/*      arising out of your use of the sample code, even if they have been   */
/*      advised of the possibility of such damages.                          */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#define INCL_PM
#include <os2.h>
#include <penpm.h>

#include "slesamp.h"
#include "globals.h"

/* temporary buffer used at various places. */
static CHAR szBuffer[200];

int main (void)
{
  HAB          hab;          /* Anchor block handle                          */
  HMQ          hmq;          /* handle to message queue                      */
  QMSG         qmsg;         /* message queue                                */
  HWND         hwndFrame,    /* handle frame window                          */
               hwndClient;   /*                                              */
  RECTL        rcl;
  LONG         PositionX,    /* bottom left x-coordinate of the window       */
               PositionY;    /* bottom left y-coordinate of the window       */
                             /* window class                                 */
  static PSZ  szClientClass="Travel Expense Form";
                             /* frame control flags                          */
  static ULONG flFrameFlags= FCF_TITLEBAR | FCF_SYSMENU | FCF_DLGBORDER |
                             FCF_MINBUTTON | FCF_TASKLIST;

/* initialize PM facilities.                                                 */
  hab = WinInitialize( 0 );

/* create a message queue with default size.                                 */
  hmq = WinCreateMsgQueue( hab, 0 );

  WinQueryWindowRect( HWND_DESKTOP, &rcl );
  ScreenWidth = rcl.xRight;
  ScreenHeight = rcl.yTop;

  WinRegisterClass (
                 hab,                /* Anchor block handle              */
                 szClientClass,      /* Name of class being registered   */
                 ClientWndProc,      /* Window procedure for class       */
                 CS_SIZEREDRAW,      /* Class style                      */
                 0);                 /* Extra bytes to reserve           */

  hwndFrame = WinCreateStdWindow (
                 HWND_DESKTOP,       /* Parent window handle             */
                 0L,                 /* Style of frame window            */
                 &flFrameFlags,      /* Pointer to control data          */
                 szClientClass,      /* Client window class name         */
                 szClientClass,      /* Title bar text                   */
                 0L,                 /* Style of client window           */
                 NULLHANDLE,         /* Module handle for resources      */
                 0,                  /* ID of resources                  */
                 &hwndClient) ;      /* Pointer to client window handle  */

/* when window is created, its size is calculated based on the resolution
   of the screen, stored in ScreenHeight and ScreenWidth, and the calculated
   size is stored in WindowHeight and WindowWidth.
*/

  PositionX = (rcl.xRight - WindowWidth) / 2;
  PositionY = (rcl.yTop - WindowHeight) / 2;
  rcl.xLeft = 0;
  rcl.xRight = WindowWidth;
  rcl.yBottom = 0;
  rcl.yTop = WindowHeight;

/* recalculate the size of the frame based on the new size of the window.    */
  WinCalcFrameRect( hwndFrame, &rcl, FALSE );

/* resize the window and display in the center.                              */

  WinSetWindowPos( hwndFrame,
                   HWND_TOP,
                   PositionX,
                   PositionY,
                   WindowWidth,
                   WindowHeight,
                   SWP_SIZE | SWP_MOVE | SWP_SHOW );

  WinSetFocus( HWND_DESKTOP, hwndClient );

  while ( WinGetMsg(hab, &qmsg, NULLHANDLE, 0, 0) )
      WinDispatchMsg( hab, &qmsg );

/* clean up.                                                                 */
  WinDestroyWindow( hwndFrame );
  WinDestroyMsgQueue( hmq );
  WinTerminate( hab );
  return 0;
}

/*****************************************************************************
* Name        : ClientWndProc                                                *
* Description : Client Window Procedure                                      *
*                                                                            *
*               This procedure handles all the messages receive by the       *
*               client window.                                               *
******************************************************************************/

MRESULT EXPENTRY ClientWndProc (HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
   ULONG msgProcessed=TRUE;
/* inProcess contains the id of the entry field being edited.                */
   static ULONG inProcess;

   switch( msg ) {
      case WM_SIZE:
              WindowWidth = SHORT1FROMMP( mp2 );
              WindowHeight = SHORT2FROMMP( mp2 );
              PositionFields( hWnd );
              break;
      case WM_CREATE:
              {
                FONTMETRICS fm ;
                HPS         hps ;
                COLOR       bkcolor;

/* calculate width and height of a character in pixels.                      */
                hps = WinGetPS( hWnd );
                GpiQueryFontMetrics( hps, sizeof(FONTMETRICS), &fm );
                AveCharWidth = fm.lAveCharWidth + 2;
                CharHeight = fm.lMaxBaselineExt;
                CharDesc = fm.lMaxDescender;
                WinReleasePS (hps) ;

/* change the background color of the window from default to palegray.       */
                bkcolor = CLR_PALEGRAY;
                WinSetPresParam( hWnd, PP_BACKGROUNDCOLORINDEX, sizeof(COLOR),
                                 &bkcolor );
              }

/* currently no entry field is being edited.                                 */
              inProcess = 0;

/* add labels and entry fields into the window.                              */
              AddFields( hWnd );
              break;
      case WM_ERASEBACKGROUND:
              {
                HPS      hps;
                RECTL    rc;
                HWND     WndSig;
                SWP      swpPos;

/*
   first clear the window background with palegray and then clear the signature
   fields with RGB(255, 255, 204).
*/
                hps = LONGFROMMP( mp1 );
                WinFillRect( hps, (PRECTL)PVOIDFROMMP(mp2), CLR_PALEGRAY );

                WndSig = WinWindowFromID( hWnd, ID_EMPSIG );
                WinQueryWindowPos( WndSig, &swpPos );
                rc.xLeft = swpPos.x;
                rc.xRight = swpPos.x + swpPos.cx;
                rc.yBottom = swpPos.y;
                rc.yTop = swpPos.y + swpPos.cy;
                GpiCreateLogColorTable( hps, LCOL_RESET, LCOLF_RGB,
                                        0L, 0L, NULL );
                WinFillRect( hps, &rc, ((ULONG)0xFF) << 16 |
                                       ((ULONG)0xFF) <<  8 |
                                       ((ULONG)0xCC) );

                WndSig = WinWindowFromID( hWnd, ID_APRVLSIG );
                WinQueryWindowPos( WndSig, &swpPos );
                rc.xLeft = swpPos.x;
                rc.xRight = swpPos.x + swpPos.cx;
                rc.yBottom = swpPos.y;
                rc.yTop = swpPos.y + swpPos.cy;
                WinFillRect( hps, &rc, ((ULONG)0xFF) << 16 |
                                       ((ULONG)0xFF) <<  8 |
                                       ((ULONG)0xCC) );
              }
              break;
      case WM_PAINT:
              {
                HPS    hps;
                RECTL  rc;
                HWND   WndSig;
                SWP    swpPos;

/*
   first clear the window background with palegray and then clear the signature
   fields with RGB(255, 255, 204).
*/
                hps = WinBeginPaint( hWnd, (HPS)NULL, &rc );
                WinFillRect( hps, &rc, CLR_PALEGRAY );

                WndSig = WinWindowFromID( hWnd, ID_EMPSIG );
                WinQueryWindowPos( WndSig, &swpPos );
                rc.xLeft = swpPos.x;
                rc.xRight = swpPos.x + swpPos.cx;
                rc.yBottom = swpPos.y;
                rc.yTop = swpPos.y + swpPos.cy;
                GpiCreateLogColorTable( hps, LCOL_RESET, LCOLF_RGB,
                                        0L, 0L, NULL );
                WinFillRect( hps, &rc, ((ULONG)0xFF) << 16 |
                                       ((ULONG)0xFF) <<  8 |
                                       ((ULONG)0xCC) );

                WndSig = WinWindowFromID( hWnd, ID_APRVLSIG );
                WinQueryWindowPos( WndSig, &swpPos );
                rc.xLeft = swpPos.x;
                rc.xRight = swpPos.x + swpPos.cx;
                rc.yBottom = swpPos.y;
                rc.yTop = swpPos.y + swpPos.cy;
                WinFillRect( hps, &rc, ((ULONG)0xFF) << 16 |
                                       ((ULONG)0xFF) <<  8 |
                                       ((ULONG)0xCC) );

                WinEndPaint( hps );
              }
              break;
      case WM_CHAR:
/*
   if keystroke is from one of the entry fields and it is RETURN or ENTER key
   then update the totals.
*/
              msgProcessed = FALSE;
              if ( FIELDID(inProcess) ) {
                 if ( SHORT1FROMMP(mp1) & KC_CHAR ) {
                    if ( SHORT1FROMMP(mp2) == 0xD ) {
                       UpdateTotals( hWnd, inProcess );
                       inProcess = 0;
                    }
                    msgProcessed = TRUE;
                 }
              }
              break;
      case WM_CLR_SIG:
              WinSendMsg( HWNDFROMMP(mp1), SKM_DELETE_ALL_STROKES, 0, 0 );
              break;
      case WM_COMMAND:
              switch( COMMANDMSG(&msg)->cmd ) {
                  case ID_PRINT:
                          Printout(  hWnd );
                          break;
                  case ID_OK:
                  case ID_CANCEL:
                          WinPostMsg( hWnd, WM_QUIT, 0, 0 );
                          break;
                  default:
                          msgProcessed = FALSE;
              }
              break;
      case WM_CONTROL:
              switch( SHORT1FROMMP(mp1) ) {
                  case ID_EMPSIG:
                  case ID_APRVLSIG:
                  case ID_EMPNAME:
                  case ID_EMPSRL:
                  case ID_EMPTEL:
                  case ID_EMPNODE:
                  case ID_EMPDEPT:
                  case ID_EMPLOC:
                  case ID_EMPZIP:
                  case ID_EMPBLDG:
                  case ID_EMPDATE:
                  case ID_APRVLNAME:
                  case ID_APRVLDATE:
                          break;
                  default:
/*
   if an entry field is begining to get focus, then save its id in inProcess.
   if an entry field is loosing focus, then update the totals.
*/
                          if ( FIELDID(SHORT1FROMMP(mp1)) ) {
                             switch( SHORT2FROMMP(mp1) ) {
                                case EN_SETFOCUS:
                                case EN_CHANGE:
                                        inProcess = SHORT1FROMMP(mp1);
                                        break;
                                case EN_KILLFOCUS:
                                        if ( inProcess ) {
                                           UpdateTotals( hWnd, inProcess );
                                           inProcess = 0;
                                        }
                             }
                          }
                          else
                             msgProcessed = FALSE;
              }
              break;
      case WM_CLOSE:
/* cause termination.                                                        */
              WinPostMsg( hWnd, WM_QUIT, 0, 0 );
              break;
      default:
              msgProcessed = FALSE;
   }

   if ( msgProcessed == FALSE )
      return WinDefWindowProc( hWnd, msg , mp1, mp2 );
   else
      return (MRESULT)FALSE;
}

/*****************************************************************************
* Name        : AddFields                                                    *
* Description : Adds labels and entry fields to client window.               *
******************************************************************************/

VOID AddFields (HWND hWnd)
{
   ULONG      x, y, i, j, k, NumCols, Width;
   SWP        swp1, swp2;
   HWND       hwndT;
   PENSLE     pensle;
   HWXCTLDATA hwxCtlData;
   ULONG      WinStyle, WinHeight;

   pensle.sleCtlData.cb = sizeof(ENTRYFDATA);
   pensle.sleCtlData.cchEditLimit = 128;
   pensle.sleCtlData.ichMinSel = 0;
   pensle.sleCtlData.ichMaxSel = 0;
   pensle.sleCtlData.pHWXCtlData = PENPMDATA;
   pensle.pHwxCtlData = &hwxCtlData;

   hwxCtlData.cbSize = sizeof(HWXCTLDATA);
   WrtGetDefHWXCTLDATA ( &hwxCtlData, &hwxCtlData.cbSize, IDR_RECOFORM_UNLINED);

   x = AveCharWidth;
   y = ScreenHeight - CharHeight - CharDesc - 2;
   NumCols = sizeof(EmpHeader1) / sizeof(PSZ);

/* display employee name, employee signature. . .                            */

   for (i=0; i < NumCols; ++i) {
       Width = strlen(EmpHeader1[i]) * AveCharWidth;
       hwndT = WinCreateWindow(
                      hWnd,                              /* Parent           */
                      WC_STATIC,                         /* Class            */
                      EmpHeader1[i],                     /* Text             */
                      SS_TEXT,                           /* Style            */
                      x, y,                              /* Position         */
                      Width,
                      CharHeight,
                      hWnd,                              /* Owner            */
                      HWND_BOTTOM,                       /* Placement        */
                      0,                                 /* ID               */
                      NULL,                              /* Ctrl Data        */
                      NULL );                            /* Pres Params      */

       WinQueryWindowPos( hwndT, &swp1 );
       EmpHeader1hWnd[i] = hwndT;

       if ( EmpID1[i] != ID_EMPSIG ) {
          hwxCtlData.ulSetMask = EmpHeader1Syms[i];
          hwndT = WinCreateWindow(
                      hWnd,                              /* Parent           */
                      WC_ENTRYFIELD,                     /* Class            */
                      "",                                /* Text             */
                      ES_AUTOSCROLL,                     /* Style            */
                      x, y-CharHeight-CharDesc-4,        /* Position         */
                      EmpWidth1[i] * AveCharWidth,       /* Width            */
                      CharHeight+4,                      /* Height           */
                      hWnd,                              /* Owner            */
                      HWND_BOTTOM,                       /* Placement        */
                      EmpID1[i],                         /* ID               */
                      &pensle,                           /* Ctrl Data        */
                      NULL );                            /* Pres Params      */
       }
       else {
          hwndT = WinCreateWindow(
                      hWnd,                              /* Parent           */
                      WC_SKETCH,                         /* Class            */
                      "",                                /* Text             */
                      0L,                                /* Style            */
                      x, y-CharHeight-CharDesc-4,        /* Position         */
                      EmpWidth1[i] * AveCharWidth,       /* Width            */
                      CharHeight+4,                      /* Height           */
                      hWnd,                              /* Owner            */
                      HWND_BOTTOM,                       /* Placement        */
                      EmpID1[i],                         /* ID               */
                      NULL,                              /* Ctrl Data        */
                      NULL );                            /* Pres Params      */

          OldEmpSigProc = WinSubclassWindow( hwndT, EmpSigWndProc );
       }

       WinQueryWindowPos( hwndT, &swp2 );

       if ( swp1.cx > swp2.cx )
          x += swp1.cx + 8;
       else
          x += swp2.cx + 8;

   }

/* display telephone, Node/ID, Department. . .                               */

   x = AveCharWidth;
   y = swp2.y - CharHeight - CharDesc - 4;
   NumCols = sizeof(EmpHeader2) / sizeof(PSZ);

   for (i=0; i < NumCols; ++i) {
       Width = strlen(EmpHeader2[i]) * AveCharWidth;
       hwndT = WinCreateWindow(
                      hWnd,                              /* Parent           */
                      WC_STATIC,                         /* Class            */
                      EmpHeader2[i],                     /* Text             */
                      SS_TEXT,                           /* Style            */
                      x, y,                              /* Position         */
                      Width,
                      CharHeight,
                      hWnd,                              /* Owner            */
                      HWND_BOTTOM,                       /* Placement        */
                      0,                                 /* ID               */
                      NULL,                              /* Ctrl Data        */
                      NULL );                            /* Pres Params      */

       WinQueryWindowPos( hwndT, &swp1 );
       EmpHeader2hWnd[i] = hwndT;

       hwxCtlData.ulSetMask = EmpHeader2Syms[i];
       hwndT = WinCreateWindow(
                      hWnd,                              /* Parent           */
                      WC_ENTRYFIELD,                     /* Class            */
                      "",                                /* Text             */
                      ES_AUTOSCROLL,                     /* Style            */
                      x, y-CharHeight-CharDesc-4,        /* Position         */
                      EmpWidth2[i] * AveCharWidth,       /* Width            */
                      CharHeight+4,                      /* Height           */
                      hWnd,                              /* Owner            */
                      HWND_BOTTOM,                       /* Placement        */
                      EmpID2[i],                         /* ID               */
                      &pensle,                           /* Ctrl Data        */
                      NULL );                            /* Pres Params      */

       WinQueryWindowPos( hwndT, &swp2 );

       if ( swp1.cx > swp2.cx )
          x += swp1.cx + 8;
       else
          x += swp2.cx + 8;
   }

/* display Date, Itinerary, Mileage. . .                                     */

   x = AveCharWidth;
   y = swp2.y - 3 * (CharHeight + CharDesc + 4);

   for (i=0; i < MAXCOLS; ++i) {
       Width = strlen(ValueHeader[i]) * AveCharWidth;
       hwndT = WinCreateWindow(
                      hWnd,                              /* Parent           */
                      WC_STATIC,                         /* Class            */
                      ValueHeader[i],                    /* Text             */
                      SS_TEXT,                           /* Style            */
                      x, y,                              /* Position         */
                      Width,
                      CharHeight,
                      hWnd,                              /* Owner            */
                      HWND_BOTTOM,                       /* Placement        */
                      0,                                 /* ID               */
                      NULL,                              /* Ctrl Data        */
                      NULL );                            /* Pres Params      */

       WinQueryWindowPos( hwndT, &swp1 );
       ValueHeaderhWnd[i] = hwndT;

       if ( WinClass[i] == WC_ENTRYFIELD ) {
          WinStyle = ES_AUTOSCROLL;
          WinHeight = CharHeight + 4;
       }
       else {
          WinStyle = CBS_DROPDOWN;
          WinHeight = CharHeight * (sizeof(Code)/sizeof(PSZ));
          y -=  (WinHeight - CharHeight - 4);
       }

       hwxCtlData.ulSetMask = ValueSyms[i];
       hwndT = WinCreateWindow(
                      hWnd,                              /* Parent           */
                      WinClass[i],                       /* Class            */
                      "0",                               /* Text             */
                      WinStyle,                          /* Style            */
                      x, y-CharHeight-CharDesc-4,        /* Position         */
                      ValueWidth[i] * AveCharWidth,      /* Width            */
                      WinHeight,                         /* Height           */
                      hWnd,                              /* Owner            */
                      HWND_BOTTOM,                       /* Placement        */
                      IDFROMROWCOL(1, i+1),              /* ID               */
                      &pensle,                           /* Ctrl Data        */
                      NULL );                            /* Pres Params      */

       WinQueryWindowPos( hwndT, &swpValue[i] );

       if ( WinClass[i] != WC_ENTRYFIELD ) {
          y +=  (WinHeight - CharHeight - 4);
          for (j=0; j < sizeof(Code)/sizeof(PSZ); ++j)
              WinSendDlgItemMsg( hWnd, IDFROMROWCOL(1, i+1), LM_INSERTITEM,
                                 MPFROM2SHORT(LIT_END, 0), MPFROMP(Code[j]) );
       }

       if ( swp1.cx > swpValue[i].cx ) {
          x += swp1.cx + 8;
          swpValue[i].cx = swp1.cx;
       }
       else
          x += swpValue[i].cx + 8;
   }

   x = swpValue[5].x;
   y = swpValue[6].y + 2 * (CharHeight + CharDesc + 4);

/* display the text Misc.                                                    */

   MischWnd = WinCreateWindow(
                      hWnd,                              /* Parent           */
                      WC_STATIC,                         /* Class            */
                      "Misc.",                           /* Text             */
                      SS_TEXT | DT_CENTER,               /* Style            */
                      x, y,                              /* Position         */
                      swpValue[5].cx+swpValue[6].cx+8,   /* Width            */
                      CharHeight,                        /* Height           */
                      hWnd,                              /* Owner            */
                      HWND_BOTTOM,                       /* Placement        */
                      0,                                 /* ID               */
                      NULL,                              /* Ctrl Data        */
                      NULL );                            /* Pres Param       */

/* display entry fields for Date, Itinerary. . .                             */

   swp1 = swpValue[0];
   for (i=0; i < MAXROWS-2; ++i) {
       x = AveCharWidth;
       y = swp1.y -  (CharHeight + CharDesc + 4);
       for (j=0; j < MAXCOLS; ++j) {
           if ( WinClass[j] == WC_ENTRYFIELD ) {
              WinStyle = ES_AUTOSCROLL;
              WinHeight = CharHeight + 4;
           }
           else {
              WinStyle = CBS_DROPDOWN;
              WinHeight = CharHeight * (sizeof(Code)/sizeof(PSZ));
              y -=  (WinHeight - CharHeight - 4);
           }

           hwxCtlData.ulSetMask = ValueSyms[j];
           hwndT = WinCreateWindow(
                           hWnd,                         /* Parent           */
                           WinClass[j],                  /* Class            */
                           "0",                          /* Text             */
                           WinStyle,                     /* Style            */
                           x, y,                         /* Position         */
                           ValueWidth[j] * AveCharWidth, /* Width            */
                           WinHeight,                    /* Height           */
                           hWnd,                         /* Owner            */
                           HWND_BOTTOM,                  /* Placement        */
                           IDFROMROWCOL(i+2, j+1),       /* ID               */
                           &pensle,                      /* Ctrl Data        */
                           NULL );                       /* Pres Params      */

           if ( WinClass[j] != WC_ENTRYFIELD ) {
              y +=  (WinHeight - CharHeight - 4);
              for (k=0; k < sizeof(Code)/sizeof(PSZ); ++k)
                 WinSendDlgItemMsg( hWnd, IDFROMROWCOL(i+2, j+1), LM_INSERTITEM,
                                    MPFROM2SHORT(LIT_END, 0), MPFROMP(Code[k]));
           }
           x += swpValue[j].cx + 8;
           if ( j == 0 )
              WinQueryWindowPos( hwndT, &swp1 );
       }
   }

/* display column totals.                                                    */

   x = AveCharWidth;
/*      y -= CharHeight + CharDesc + 4; */
   y = swp1.y - (CharHeight + CharDesc + 4);
   j = MAXCOLS + 1;
   for (i=0; i < MAXCOLS; ++i) {
       if ( TotColExist[i] ) {
          hwndT = WinCreateWindow(
                          hWnd,                          /* Parent           */
                          WC_ENTRYFIELD,                 /* Class            */
                          "0",                           /* Text             */
                          ES_AUTOSCROLL,                 /* Style            */
                          x, y,                          /* Position         */
                          ValueWidth[i] * AveCharWidth,  /* Width            */
                          CharHeight+4,                  /* Height           */
                          hWnd,                          /* Owner            */
                          HWND_BOTTOM,                   /* Placement        */
                          IDFROMROWCOL(MAXROWS, i+1),    /* ID               */
                          NULL,                          /* Ctrl Data        */
                          NULL );                        /* Pres Params      */

          if ( j > MAXCOLS )
             j = i;
       }

       x += swpValue[i].cx + 8;
   }

   x = swpValue[j].x - (6 * AveCharWidth);
   ColumnTotalhWnd= WinCreateWindow(
                            hWnd,                        /* Parent           */
                            WC_STATIC,                   /* Class            */
                            "Totals",                    /* Text             */
                            SS_TEXT,                     /* Style            */
                            x, y,                        /* Position         */
                            6 * AveCharWidth,            /* Width            */
                            CharHeight,                  /* Height           */
                            hWnd,                        /* Owner            */
                            HWND_BOTTOM,                 /* Placement        */
                            0,                           /* ID               */
                            NULL,                        /* Ctrl Data        */
                            NULL );                      /* Pres Params      */

/* display 0 for numeric entry fields and spaces for others.                 */

   for (i=0; i < MAXROWS-1; ++i) {
      for (j=0; j < MAXCOLS-1; ++j)
          if ( !TotColExist[j] )
             WinSetDlgItemText (hWnd, IDFROMROWCOL(i+1, j+1), "" );
   }

/* display grand total.                                                      */

   y -= CharHeight + CharDesc + 12;
   x = swpValue[MAXCOLS-1].x -
       (FIN_TOT_WIDTH * AveCharWidth - swpValue[MAXCOLS-1].cx);

   hwndT = WinCreateWindow(
                   hWnd,                                 /* Parent           */
                   WC_ENTRYFIELD,                        /* Class            */
                   "0",                                  /* Text             */
                   ES_AUTOSCROLL,                        /* Style            */
                   x, y,                                 /* Position         */
                   FIN_TOT_WIDTH * AveCharWidth,         /* Width            */
                   CharHeight+4,                         /* Height           */
                   hWnd,                                 /* Owner            */
                   HWND_BOTTOM,                          /* Placement        */
                   ID_FIN_TOTAL,                         /* ID               */
                   NULL,                                 /* Ctrl Data        */
                   NULL );                               /* Pres Params      */

   x -= 14 * AveCharWidth + 4;
   TotalhWnd = WinCreateWindow(
                       hWnd,                             /* Parent           */
                       WC_STATIC,                        /* Class            */
                       "Total Expenses",                 /* Text             */
                       SS_TEXT,                          /* Style            */
                       x, y,                             /* Position         */
                       14 * AveCharWidth,                /* Width            */
                       CharHeight,                       /* Height           */
                       hWnd,                             /* Owner            */
                       HWND_BOTTOM,                      /* Placement        */
                       0,                                /* ID               */
                       NULL,                             /* Ctrl Data        */
                       NULL );                           /* Pres Params      */

/* display approval name, approval signature and date.                       */

   x = AveCharWidth;
   y -= 2 * CharHeight + CharDesc;
   NumCols = sizeof(AprvlHeader) / sizeof(PSZ);
   for (i=0; i < NumCols; ++i) {
       Width = strlen(AprvlHeader[i]) * AveCharWidth;
       hwndT = WinCreateWindow(
                       hWnd,                             /* Parent           */
                       WC_STATIC,                        /* Class            */
                       AprvlHeader[i],                   /* Text             */
                       SS_TEXT,                          /* Style            */
                       x, y,                             /* Position         */
                       Width,
                       CharHeight,
                       hWnd,                             /* Owner            */
                       HWND_BOTTOM,                      /* Placement        */
                       0,                                /* ID               */
                       NULL,                             /* Ctrl Data        */
                       NULL );                           /* Pres Params      */

       WinQueryWindowPos( hwndT, &swp1 );
       AprvlHeaderhWnd[i] = hwndT;

       if ( AprvlID[i] != ID_APRVLSIG ) {
          hwxCtlData.ulSetMask = AprvlHeaderSyms[i];
          hwndT = WinCreateWindow(
                          hWnd,                          /* Parent           */
                          WC_ENTRYFIELD,                 /* Class            */
                          "",                            /* Text             */
                          ES_AUTOSCROLL,                 /* Style            */
                          x, y-CharHeight-CharDesc-4,    /* Position         */
                          AprvlWidth[i] * AveCharWidth,  /* Width            */
                          CharHeight+4,                  /* Height           */
                          hWnd,                          /* Owner            */
                          HWND_BOTTOM,                   /* Placement        */
                          AprvlID[i],                    /* ID               */
                          &pensle,                       /* Ctrl Data        */
                          NULL );                        /* Pres Params      */
      }
      else {
         hwndT = WinCreateWindow(
                         hWnd,                           /* Parent           */
                         WC_SKETCH,                      /* Class            */
                         "",                             /* Text             */
                         0L,                             /* Style            */
                         x, y-CharHeight-CharDesc-4,     /* Position         */
                         AprvlWidth[i] * AveCharWidth,   /* Width            */
                         CharHeight+4,                   /* Height           */
                         hWnd,                           /* Owner            */
                         HWND_BOTTOM,                    /* Placement        */
                         AprvlID[i],                     /* ID               */
                         NULL,                           /* Ctrl Data        */
                         NULL );                         /* Pres Params      */

          OldAprvlSigProc = WinSubclassWindow( hwndT, AprvlSigWndProc );
      }
      WinQueryWindowPos( hwndT, &swp2 );

      if ( swp1.cx > swp2.cx )
         x += swp1.cx + 8;
      else
         x += swp2.cx + 8;
   }

/*
   upto this point, evrything is drawn with respect to screen. now we calculate
   the actual width of the window needed.
*/

   WinQueryWindowPos( WinWindowFromID(hWnd, ID_FIN_TOTAL), &swp1 );
   WindowWidth = swp1.x + swp1.cx + AveCharWidth + 6;
   WinQueryWindowPos( WinWindowFromID(hWnd, ID_APRVLNAME), &swp1 );

/* display OK, CANCEL and PRINT buttons.                                     */

   x = 5 * AveCharWidth;
   y = swp1.y - 2 * CharHeight - 15;

   WinCreateWindow(
           hWnd,                                         /* Parent           */
           WC_BUTTON,                                    /* Class            */
           "OK",                                         /* Text             */
           BS_PUSHBUTTON | BS_DEFAULT,                   /* Style            */
           x, y,                                         /* Position         */
           8 * AveCharWidth,                             /* Width            */
           2 * CharHeight,                               /* Height           */
           hWnd,                                         /* Owner            */
           HWND_BOTTOM,                                  /* Placement        */
           ID_OK,                                        /* ID               */
           NULL,                                         /* Ctrl Data        */
           NULL );                                       /* Pres Params      */

   x = WindowWidth - 5 * AveCharWidth - 8 * AveCharWidth;

   WinCreateWindow(
           hWnd,                                         /* Parent            */
           WC_BUTTON,                                    /* Class             */
           "CANCEL",                                     /* Text              */
           BS_PUSHBUTTON,                                /* Style             */
           x, y,                                         /* Position          */
           8 * AveCharWidth,                             /* Width             */
           2 * CharHeight,                               /* Height            */
           hWnd,                                         /* Owner             */
           HWND_BOTTOM,                                  /* Placement         */
           ID_CANCEL,                                    /* ID                */
           NULL,                                         /* Ctrl Data         */
           NULL );                                       /* Pres Params       */

   x = ((WindowWidth - 26 * AveCharWidth) / 2) - (4 * AveCharWidth) + (13 * AveCharWidth);

   WinCreateWindow(
           hWnd,                                         /* Parent           */
           WC_BUTTON,                                    /* Class            */
           "PRINT",                                      /* Text             */
           BS_PUSHBUTTON,                                /* Style            */
           x, y,                                         /* Position         */
           8 * AveCharWidth,                             /* Width            */
           2 * CharHeight,                               /* Height           */
           hWnd,                                         /* Owner            */
           HWND_BOTTOM,                                  /* Placement        */
           ID_PRINT,                                     /* ID               */
           NULL,                                         /* Ctrl Data        */
           NULL );                                       /* Pres Params      */

/* now we calculate the actual height of the window needed.                  */

   WindowHeight = ScreenHeight - y + 30;
}

/*****************************************************************************
* Name        : UpdateTotals                                                 *
* Description : Updates the row and column totals.                           *
******************************************************************************/

VOID UpdateTotals (HWND hwndClient, ULONG id)
{
   ULONG row, col, newValue, Total, length;

   row = ROWFROMID( id );
   col = COLFROMID( id );

   if ( TotColExist[col-1] ) {
      length = WinQueryDlgItemText( hwndClient, id, 129, szBuffer );
      szBuffer[length] = 0;
      newValue = (ULONG)strtol( szBuffer, NULL, 10 );

      length = WinQueryDlgItemText( hwndClient, IDFROMROWCOL(MAXROWS, col),
                                    129, szBuffer );
      szBuffer[length] = 0;
      Total = (ULONG)strtol( szBuffer, NULL, 10 );
      sprintf( szBuffer, "%lu", Total+newValue-Values[row][col] );
      WinSetDlgItemText( hwndClient, IDFROMROWCOL(MAXROWS, col), szBuffer );

      length = WinQueryDlgItemText( hwndClient, IDFROMROWCOL(row, MAXCOLS),
                                    129, szBuffer );
      szBuffer[length] = 0;
      Total = (ULONG)strtol( szBuffer, NULL, 10 );
      sprintf( szBuffer, "%lu", Total+newValue-Values[row][col] );
      WinSetDlgItemText( hwndClient, IDFROMROWCOL(row, MAXCOLS), szBuffer );

      length = WinQueryDlgItemText( hwndClient, ID_FIN_TOTAL, FIN_TOT_WIDTH,
                                    szBuffer );
      szBuffer[length] = 0;
      Total = (ULONG)strtol( szBuffer, NULL, 10 );
      sprintf( szBuffer, "%lu", Total+newValue-Values[row][col] );
      WinSetDlgItemText( hwndClient, ID_FIN_TOTAL, szBuffer );

      Values[row][col] = newValue;
   }
   return;
}

/*****************************************************************************
* Name        : PositionFields                                               *
* Description : Repositions labels and fields when client window is resized. *
******************************************************************************/

VOID PositionFields (HWND hWnd)
{
   ULONG x, y, i, j, NumCols;
   SWP  swp1, swp2;

   x = AveCharWidth;
   y = WindowHeight - CharHeight - CharDesc - 2;
   NumCols = sizeof(EmpHeader1) / sizeof(PSZ);

   for (i=0; i < NumCols; ++i) {

       WinSetWindowPos( EmpHeader1hWnd[i],
                        HWND_TOP,
                        x,
                        y,
                        0,
                        0,
                        SWP_MOVE | SWP_SHOW );

       WinQueryWindowPos( EmpHeader1hWnd[i], &swp1 );

       WinSetWindowPos( WinWindowFromID(hWnd, EmpID1[i]),
                        HWND_TOP,
                        x,
                        y-CharHeight-CharDesc-4,
                        0,
                        0,
                        SWP_MOVE | SWP_SHOW );

       WinQueryWindowPos( WinWindowFromID(hWnd, EmpID1[i]), &swp2 );

       if ( swp1.cx > swp2.cx )
          x += swp1.cx + 8;
       else
          x += swp2.cx + 8;
   }

/* start new line */

   x = AveCharWidth;
   y = swp2.y - CharHeight - CharDesc - 4;
   NumCols = sizeof(EmpHeader2) / sizeof(PSZ);

   for (i=0; i < NumCols; ++i) {

       WinSetWindowPos( EmpHeader2hWnd[i],
                        HWND_TOP,
                        x,
                        y,
                        0,
                        0,
                        SWP_MOVE | SWP_SHOW );

       WinQueryWindowPos( EmpHeader2hWnd[i], &swp1 );

       WinSetWindowPos( WinWindowFromID(hWnd, EmpID2[i]),
                        HWND_TOP,
                        x,
                        y-CharHeight-CharDesc-4,
                        0,
                        0,
                        SWP_MOVE | SWP_SHOW );

       WinQueryWindowPos( WinWindowFromID(hWnd, EmpID2[i]), &swp2 );

       if ( swp1.cx > swp2.cx )
          x += swp1.cx + 8;
       else
          x += swp2.cx + 8;
   }

/* start new line */

   x = AveCharWidth;
   y = swp2.y - 3 * (CharHeight + CharDesc + 4);

   for (i=0; i < MAXCOLS; ++i) {

       WinSetWindowPos( ValueHeaderhWnd[i],
                        HWND_TOP,
                        x,
                        y,
                        0,
                        0,
                        SWP_MOVE | SWP_SHOW );

       WinQueryWindowPos( ValueHeaderhWnd[i], &swp1 );

       if ( WinClass[i] == WC_COMBOBOX )
          y -= (CharHeight * (sizeof(Code)/sizeof(PSZ))) - CharHeight - 4;

       WinSetWindowPos( WinWindowFromID(hWnd, IDFROMROWCOL(1, i+1)),
                        HWND_TOP,
                        x,
                        y-CharHeight-CharDesc-4,
                        0,
                        0,
                        SWP_MOVE | SWP_SHOW );

       WinQueryWindowPos( WinWindowFromID(hWnd, IDFROMROWCOL(1, i+1)),
                          &swpValue[i] );

       if ( WinClass[i] == WC_COMBOBOX )
          y += (CharHeight * (sizeof(Code)/sizeof(PSZ))) - CharHeight - 4;

       if ( swp1.cx > swpValue[i].cx ) {
          x += swp1.cx + 8;
          swpValue[i].cx = swp1.cx;
       }
       else
          x += swpValue[i].cx + 8;
   }

   x = swpValue[5].x;
   y = swpValue[6].y + 2 * (CharHeight + CharDesc + 4);

   WinSetWindowPos( MischWnd,
                    HWND_TOP,
                    x,
                    y,
                    0,
                    0,
                    SWP_MOVE | SWP_SHOW );

/* start new line */

   swp1 = swpValue[0];
   for (i=0; i < MAXROWS-2; ++i) {
       x = AveCharWidth;
       y = swp1.y -  (CharHeight + CharDesc + 4);

       for (j=0; j < MAXCOLS; ++j) {

          if ( WinClass[j] == WC_COMBOBOX )
             y -= (CharHeight * (sizeof(Code)/sizeof(PSZ))) - CharHeight - 4;

          WinSetWindowPos( WinWindowFromID(hWnd, IDFROMROWCOL(i+2, j+1)),
                           HWND_TOP,
                           x,
                           y,
                           0,
                           0,
                           SWP_MOVE | SWP_SHOW );

          x += swpValue[j].cx + 8;
          if ( WinClass[j] == WC_COMBOBOX )
             y += (CharHeight * (sizeof(Code)/sizeof(PSZ))) - CharHeight - 2;

          if ( j == 0 )
             WinQueryWindowPos( WinWindowFromID(hWnd, IDFROMROWCOL(i+2, j+1)),
                                &swp1 );
       }
   }

/* start new line */

   x = AveCharWidth;
/*      y -= CharHeight + CharDesc + 4; */
   y = swp1.y - (CharHeight + CharDesc + 4);
   j = MAXCOLS + 1;

   for (i=0; i < MAXCOLS; ++i) {
       if ( TotColExist[i] ) {
          WinSetWindowPos( WinWindowFromID(hWnd, IDFROMROWCOL(MAXROWS, i+1)),
                           HWND_TOP,
                           x,
                           y,
                           0,
                           0,
                           SWP_MOVE | SWP_SHOW );

          if ( j > MAXCOLS )
             j = i;
       }

       x += swpValue[i].cx + 8;
   }

   x = swpValue[j].x - (6 * AveCharWidth);
   WinSetWindowPos( ColumnTotalhWnd,
                    HWND_TOP,
                    x,
                    y,
                    0,
                    0,
                    SWP_MOVE | SWP_SHOW );

/* start new line */

   y -= CharHeight + CharDesc + 4;
   x = swpValue[MAXCOLS-1].x -
       (FIN_TOT_WIDTH * AveCharWidth - swpValue[MAXCOLS-1].cx);

   WinSetWindowPos( WinWindowFromID(hWnd, ID_FIN_TOTAL),
                    HWND_TOP,
                    x,
                    y,
                    0,
                    0,
                    SWP_MOVE | SWP_SHOW );

   x -= 14 * AveCharWidth + 4;
   WinSetWindowPos( TotalhWnd,
                    HWND_TOP,
                    x,
                    y,
                    0,
                    0,
                    SWP_MOVE | SWP_SHOW );

/* start new line */

   x = AveCharWidth;
   y -= 2 * CharHeight + CharDesc;
   NumCols = sizeof(AprvlHeader) / sizeof(PSZ);
   for (i=0; i < NumCols; ++i) {

       WinSetWindowPos( AprvlHeaderhWnd[i],
                        HWND_TOP,
                        x,
                        y,
                        0,
                        0,
                        SWP_MOVE | SWP_SHOW );

       WinQueryWindowPos( AprvlHeaderhWnd[i], &swp1 );

       WinSetWindowPos( WinWindowFromID(hWnd, AprvlID[i]),
                        HWND_TOP,
                        x,
                        y-CharHeight-CharDesc-4,
                        0,
                        0,
                        SWP_MOVE | SWP_SHOW );

       WinQueryWindowPos( WinWindowFromID(hWnd, AprvlID[i]), &swp2 );

       if ( swp1.cx > swp2.cx )
          x += swp1.cx + 8;
       else
          x += swp2.cx + 8;
   }

/* start new line */

   x = 5 * AveCharWidth;
   y = swp2.y - 2 * CharHeight - 15;

   WinSetWindowPos( WinWindowFromID(hWnd, ID_OK),
                    HWND_TOP,
                    x,
                    y,
                    0,
                    0,
                    SWP_MOVE | SWP_SHOW );

   x = WindowWidth - 5 * AveCharWidth - 8 * AveCharWidth;

   WinSetWindowPos( WinWindowFromID(hWnd, ID_CANCEL),
                    HWND_TOP,
                    x,
                    y,
                    0,
                    0,
                    SWP_MOVE | SWP_SHOW );

   x = ((WindowWidth - 26 * AveCharWidth) / 2) - (4 * AveCharWidth) + (13 * AveCharWidth);

   WinSetWindowPos( WinWindowFromID(hWnd, ID_PRINT),
                    HWND_TOP,
                    x,
                    y,
                    0,
                    0,
                    SWP_MOVE | SWP_SHOW );
}

/*****************************************************************************
* Name        : EmpSigWndProc                                                *
* Description : Employee signature window procedure                          *
*
*               The procedure translates PEN button 2 down to mouse button 2 *
*               down message. On mouse button 2 down, it informs the client  *
*               window to clear the employee signature window.               *
******************************************************************************/

MRESULT EXPENTRY EmpSigWndProc (HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
   PWRTEVENTDATA pwrtevInfo;
   ULONG ulButton;

   if ( msg == WM_BUTTON2DOWN )
      WinPostMsg( WinQueryWindow(hWnd, QW_OWNER), WM_CLR_SIG, MPFROMHWND(hWnd),
                  (MPARAM)0 );
   else
      if ( msg == WM_TOUCHDOWN ) {
         WinSetActiveWindow( HWND_DESKTOP, hWnd );
         pwrtevInfo = (PWRTEVENTDATA)PVOIDFROMMP( mp2 );
         ulButton = pwrtevInfo->flEventStatus & WRT_MBUTTON_MASK;
         if ( ulButton & WRT_MBUTTON2 )
            return (MRESULT)TDN_IMMEDIATE;
      }

   return OldEmpSigProc( hWnd, msg, mp1, mp2 );
}

/*****************************************************************************
* Name        : AprvlSigWndProc                                              *
* Description : Approval signature window procedure                          *
*
*               The procedure translates PEN button 2 down to mouse button 2 *
*               down message. On mouse button 2 down, it informs the client  *
*               window to clear the approval signature window.               *
******************************************************************************/

MRESULT EXPENTRY AprvlSigWndProc (HWND hWnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
   PWRTEVENTDATA pwrtevInfo;
   ULONG ulButton;

   if ( msg == WM_BUTTON2DOWN )
      WinPostMsg( WinQueryWindow(hWnd, QW_OWNER), WM_CLR_SIG, MPFROMHWND(hWnd),
                  (MPARAM)0 );
   else
      if ( msg == WM_TOUCHDOWN ) {
         WinSetActiveWindow( HWND_DESKTOP, hWnd );
         pwrtevInfo = (PWRTEVENTDATA)PVOIDFROMMP( mp2 );
         ulButton = pwrtevInfo->flEventStatus & WRT_MBUTTON_MASK;
         if ( ulButton & WRT_MBUTTON2 )
            return (MRESULT)TDN_IMMEDIATE;
      }

   return OldAprvlSigProc( hWnd, msg, mp1, mp2 );
}

/*****************************************************************************
* Name        : Printout                                                     *
* Description : Procedure to print the form to default printer.              *
******************************************************************************/

VOID Printout (HWND hWnd)
{
  PSZ    pszHeader="Travel Expense Account";
  HDC    hdcPrinter;
  HPS    hpsPrinter;
  HAB    hab;
  HBITMAP hbm;
  BITMAPINFOHEADER bmphdr;
  FATTRS fat;
  SIZEL  sizel;
  FONTMETRICS fm;
  POINTL pt;
  ULONG  i;

  hab = WinQueryAnchorBlock( hWnd );
  hdcPrinter = OpenDefaultPrinterDC( hab );
  if ( hdcPrinter == DEV_ERROR )
     return;

  sizel.cx = 0;
  sizel.cy = 0;
  hpsPrinter = GpiCreatePS( hab, hdcPrinter, &sizel, PU_ARBITRARY |
                            GPIF_DEFAULT | GPIT_MICRO | GPIA_ASSOC );
  fat.usRecordLength  = sizeof(FATTRS);
  fat.fsSelection     = FATTR_SEL_BOLD;
  fat.lMatch          = 0;
  fat.idRegistry      = 0;
  fat.usCodePage      = GpiQueryCp( hpsPrinter );
  fat.lMaxBaselineExt = 12L;
  fat.lAveCharWidth   = 12L;
  fat.fsType          = 0;
  fat.fsFontUse       = FATTR_FONTUSE_NOMIX;

  strcpy( fat.szFacename, "Courier" );
  GpiCreateLogFont( hpsPrinter, NULL, LCID_PRNFONT, &fat );
  GpiSetCharSet( hpsPrinter, LCID_PRNFONT );
  GpiQueryPS (hpsPrinter, &sizel );
  GpiQueryFontMetrics( hpsPrinter, sizeof(FONTMETRICS), &fm );

  DevEscape( hdcPrinter, DEVESC_STARTDOC, strlen(pszHeader), pszHeader,
             NULL, NULL );

  pt.x = 0;
  pt.y = sizel.cy - fm.lMaxBaselineExt;
  WinLoadString( hab, NULLHANDLE, IDS_EMPHDR1, 200L, szBuffer );
  GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

  pt.y -= 2 * fm.lMaxBaselineExt;
  WinQueryWindowText( WinWindowFromID(hWnd, EmpID1[0]), 25L, szBuffer );
  szBuffer[25] = NULL;
  GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

  pt.x += 27 * fm.lAveCharWidth;
  WinQueryWindowText( WinWindowFromID(hWnd, EmpID1[1]), 6L, szBuffer );
  szBuffer[6] = NULL;
  GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

  pt.x += 8 * fm.lAveCharWidth;
  WinQueryWindowText( WinWindowFromID(hWnd, EmpID2[0]), 13L, szBuffer );
  szBuffer[13] = NULL;
  GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

  pt.x += 15 * fm.lAveCharWidth;
  WinQueryWindowText( WinWindowFromID(hWnd, EmpID2[2]), 7L, szBuffer );
  szBuffer[7] = NULL;
  GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

  pt.x += 9 * fm.lAveCharWidth;
  WinQueryWindowText( WinWindowFromID(hWnd, EmpID2[3]), 4L, szBuffer );
  szBuffer[4] = NULL;
  GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

  pt.x = 0;
  pt.y -= 4 * fm.lMaxBaselineExt;
  WinLoadString( hab, NULLHANDLE, IDS_DATAHDR, 200L, szBuffer );
  GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

  for (i=0; i < MAXROWS-1; ++i) {
     pt.x = 0;
     pt.y -= 2 * fm.lMaxBaselineExt;
     WinQueryWindowText( WinWindowFromID(hWnd, IDFROMROWCOL(i+1, 1)), 10L,
                         szBuffer );
     szBuffer[10] = NULL;
     GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

     pt.x += 12 * fm.lAveCharWidth;
     WinQueryWindowText( WinWindowFromID(hWnd, IDFROMROWCOL(i+1, 2)), 23L,
                         szBuffer );
     szBuffer[23] = NULL;
     GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

     pt.x += 25 * fm.lAveCharWidth;
     WinQueryWindowText( WinWindowFromID(hWnd, IDFROMROWCOL(i+1, 3)), 8L,
                         szBuffer );
     szBuffer[8] = NULL;
     GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

     pt.x += 10 * fm.lAveCharWidth;
     WinQueryWindowText( WinWindowFromID(hWnd, IDFROMROWCOL(i+1, 4)), 6L,
                         szBuffer );
     szBuffer[6] = NULL;
     GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

     pt.x += 8 * fm.lAveCharWidth;
     WinQueryWindowText( WinWindowFromID(hWnd, IDFROMROWCOL(i+1, 5)), 8L,
                         szBuffer );
     szBuffer[8] = NULL;
     GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

     pt.x += 10 * fm.lAveCharWidth;
     WinQueryWindowText( WinWindowFromID(hWnd, IDFROMROWCOL(i+1, 7)), 6L,
                         szBuffer );
     szBuffer[6] = NULL;
     GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

     pt.x += 8 * fm.lAveCharWidth;
     WinQueryWindowText( WinWindowFromID(hWnd, IDFROMROWCOL(i+1, 8)), 5L,
                         szBuffer );
     szBuffer[5] = NULL;
     GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );
  }

  pt.x = 0;
  pt.y -= 4 * fm.lMaxBaselineExt;
  WinLoadString( hab, NULLHANDLE, IDS_EMPHDR2, 200L, szBuffer );
  GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

  hbm = (HBITMAP)WinSendMsg( WinWindowFromID(hWnd, ID_EMPSIG), SKM_GET_BITMAP,
                             (MPARAM)0, (MPARAM)0 );

  pt.y -= 2 * fm.lMaxBaselineExt;
  PrintSignature( hab, hbm, hdcPrinter, hpsPrinter, pt, hWnd );

  pt.x = 67 * fm.lAveCharWidth;
  WinQueryWindowText( WinWindowFromID(hWnd, ID_EMPDATE), 10L,
                      szBuffer );
  szBuffer[10] = NULL;
  GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

  GpiQueryBitmapParameters( hbm, &bmphdr );

  pt.x = 0;
  pt.y -= (4 * fm.lMaxBaselineExt) + bmphdr.cy;
  WinLoadString( hab, NULLHANDLE, IDS_APRVLHDR, 200L, szBuffer );
  GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

  hbm = (HBITMAP)WinSendMsg( WinWindowFromID(hWnd, ID_APRVLSIG), SKM_GET_BITMAP,
                             (MPARAM)0, (MPARAM)0 );

  pt.y -= 2 * fm.lMaxBaselineExt;
  PrintSignature( hab, hbm, hdcPrinter, hpsPrinter, pt, hWnd );

  pt.x = 26 * fm.lAveCharWidth;
  WinQueryWindowText( WinWindowFromID(hWnd, ID_APRVLNAME), 25L,
                      szBuffer );
  szBuffer[25] = NULL;
  GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

  pt.x = 67 * fm.lAveCharWidth;
  WinQueryWindowText( WinWindowFromID(hWnd, ID_APRVLDATE), 10L,
                      szBuffer );
  szBuffer[10] = NULL;
  GpiCharStringAt( hpsPrinter, &pt, strlen(szBuffer), szBuffer );

  DevEscape( hdcPrinter, DEVESC_NEWFRAME, 0L, NULL, NULL, NULL );
  DevEscape( hdcPrinter, DEVESC_ENDDOC, 0L, NULL, NULL, NULL );

Error:
  GpiDeleteSetId( hpsPrinter, LCID_PRNFONT );
  GpiDestroyPS( hpsPrinter );
  DevCloseDC( hdcPrinter );
}

/*****************************************************************************
* Name        : OpenDefaultPrinterDC                                         *
* Description : Procedure to get DC of default printer.                      *
******************************************************************************/

HDC OpenDefaultPrinterDC (HAB hab)

{
  static CHAR     szPRNInfo[256];
  CHAR            szPRNName[33], *pszDelimiter;
  static DRIVDATA drivData;
  DEVOPENSTRUC    dop = { 0 };

/*  Obtain default printer name and remove semicolon */

  PrfQueryProfileString (HINI_PROFILE, "PM_SPOOLER", "PRINTER", ";", szPRNName,
                         sizeof(szPRNName) );

  pszDelimiter = strchr( szPRNName, ';' );
  if ( pszDelimiter )
     *pszDelimiter = '\0' ;

  if ( szPRNName[0] == '\0' )
     return DEV_ERROR ;

/*  Obtain information on default printer  */

  PrfQueryProfileString( HINI_PROFILE, "PM_SPOOLER_PRINTER",
                         szPRNName, ";;;;", szPRNInfo, sizeof(szPRNInfo) ) ;

  pszDelimiter = strchr( szPRNInfo, ';' );
  if ( !pszDelimiter )
     return DEV_ERROR ;

  dop.pszDriverName = pszDelimiter + 1;

  pszDelimiter = strchr( dop.pszDriverName, ';' );
  if ( !pszDelimiter )
       return DEV_ERROR ;

  dop.pszLogAddress = pszDelimiter + 1 ;

  *( dop.pszLogAddress + strcspn(dop.pszLogAddress, ",;") ) = '\0';
  *( dop.pszDriverName + strcspn(dop.pszDriverName, ",;") ) = '\0';

/*  Fill DRIVDATA structure if necessary  */

  pszDelimiter = strchr( dop.pszDriverName, '.' );
  if ( pszDelimiter ) {
     drivData.cb = sizeof( drivData );
     *pszDelimiter = '\0' ;
     strncpy( drivData.szDeviceName, pszDelimiter + 1,
              sizeof(drivData.szDeviceName) );
     dop.pdriv = &drivData ;
  }
  else
     dop.pdriv = NULL ;

/*  Set data type to "std"  */

  dop.pszDataType = "PM_Q_STD" ;

/*  Open printer device context  */

  return DevOpenDC( hab, OD_QUEUED, "*", 4L, (PDEVOPENDATA) &dop, 0L );
}

/*****************************************************************************
* Name        : PrintSignature                                               *
* Description : Procedure to print bitmap of the signature.                  *
******************************************************************************/

VOID PrintSignature (HAB hab, HBITMAP hbm, HDC hdcPrinter, HPS hpsPrinter,
                     POINTL pt, HWND hWnd)
{
  HDC    hdcMem;
  HPS    hpsMem, hpsWnd;
  BITMAPINFOHEADER bmphdr;
  SIZEL  sizel, sizelWnd;
  POINTL aptDest[4];

  hdcMem = DevOpenDC( hab, OD_MEMORY, "*", 0L, NULL, hdcPrinter );

  sizel.cx = 0;
  sizel.cy = 0;
  hpsMem = GpiCreatePS( hab, hdcMem, &sizel, PU_PELS | GPIF_DEFAULT |
                                             GPIT_MICRO | GPIA_ASSOC );

  GpiSetBitmap( hpsMem, hbm );
  GpiSetBitmap( hpsMem, NULL );

  bmphdr.cbFix = sizeof( BITMAPINFOHEADER );
  GpiQueryBitmapParameters( hbm, &bmphdr );

  hpsWnd = WinGetPS( hWnd );
  GpiQueryPS( hpsWnd, &sizelWnd );
  WinReleasePS( hpsWnd );
  GpiQueryPS( hpsPrinter, &sizel );

  aptDest[0].x = pt.x;
  aptDest[0].y = pt.y - (bmphdr.cy * (sizel.cy / sizelWnd.cy + 1));
  aptDest[1].x = pt.x + (bmphdr.cx * (sizel.cx / sizelWnd.cx + 1));
  aptDest[1].y = pt.y;
  aptDest[2].x = 0;
  aptDest[2].y = 0;
  aptDest[3].x = bmphdr.cx;
  aptDest[3].y = bmphdr.cy;
  GpiWCBitBlt( hpsPrinter, hbm, 4L, aptDest, ROP_SRCCOPY, BBO_OR );

  GpiDestroyPS( hpsMem );
  DevCloseDC( hdcMem );
  return;
}
