/******************************************************************************
*                                                                             *
*  File Name   : WRTSAMPL.C                                                   *
*                                                                             *
*  Description : Example of WRT APIs                                          *
*                                                                             *
*  Function:  This is a sample of all Wrt APIs                                *
*                                                                             *
*  Copyright (C) 1993 IBM Corporation                                         *
*                                                                             *
*      DISCLAIMER OF WARRANTIES.  The following [enclosed] code is            *
*      sample code created by IBM Corporation. This sample code is not        *
*      part of any standard or IBM product and is provided to you solely      *
*      for  the purpose of assisting you in the development of your           *
*      applications.  The code is provided "AS IS", without                   *
*      warranty of any kind.  IBM shall not be liable for any damages         *
*      arising out of your use of the sample code, even if they have been     *
*      advised of the possibility of such damages.                            *
*                                                                             *
*******************************************************************************/

#define INCL_DOS
#define INCL_PM
#define INCL_GPI
#define INCL_DOSMEMMGR
#define  INCL_WINLOAD
#define  INCL_DOSPROCESS
#define  INCL_DOSSEMAPHORES
#include <os2.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <process.h>

#include <penpm.h>

#define  WRTSAMPL_MAIN
#include "wrtsampl.h"

HWND       hwndClient;
HWND       hwndFrame;
SHORT      sLastP = 0;           /* Last pressure of the point on the line */
SHORT      sCurrP = 0;           /* Current pressure of the point on the line */
LONG       lLastWidth = 1L;      /* Last width of line drawned */

int main()
{
  HMQ   hmqMain;
  QMSG  qmsgMain;
  ULONG flCreate;

  habMain = WinInitialize( (ULONG) NULL );
  hmqMain = WinCreateMsgQueue( habMain, 0L );

  WinRegisterClass( habMain,
                    "MyClient",
                    (PFNWP) ClientWndProc,
                    CS_SIZEREDRAW,
                    0UL );

  flCreate =  FCF_STANDARD & ~FCF_SHELLPOSITION;
  hwndFrame = WinCreateStdWindow( HWND_DESKTOP,
                                  0L,
                                  (PULONG) &flCreate,
                                  "MyClient",
                                  "wrtsampl",
                                  0L,
                                  (HMODULE) NULL,
                                  ID_MAIN,
                                  (PHWND) &hwndClient );

  /****************************************************************************/
  /* Tell PM to display our window at the designated size and place           */
  /****************************************************************************/
  WinSetWindowPos( hwndFrame,
                   HWND_TOP,
                   50, 50, 550, 380,
                   SWP_SIZE | SWP_MOVE | SWP_ACTIVATE | SWP_SHOW );


  while( WinGetMsg( habMain, &qmsgMain, (HWND) NULL, 0, 0 ) )
  {
    WinDispatchMsg( habMain, &qmsgMain );
  }

  WinDestroyWindow( hwndFrame );
  WinDestroyMsgQueue( hmqMain );
  WinTerminate( habMain );
  return( 0 );
}

MRESULT EXPENTRY ClientWndProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
ULONG             ulBuflen, ulOptions, flAuxData;
PSTROKEDATA       psdCurStroke;
ULONG             ulfButton;
APIRET            irc;               /* return code from Wrt APIs           */
ULONG             ulLocatorID;       /* variable for Locator IDs            */
WRTDISPLAYDEVINFO dcInfo;            /* Display capabilities structure      */
ULONG             ulDisplayID;       /* variable for Display IDs            */
WRTSYSTEMINFO     scInfo;            /* System capabilities structure       */
CHAR              cbuffer[MAXCBUFFER];
LONG              lvalue;            /* variable to hold system value       */
ULONG             ulid;
INT               i;
POINTL            ptlXY;
SIZEL             Size;
HPS               hpsPaint;
static HWND       hwndMenu;
static SWP        swpClient;
static HPOINTER   hptrClient;
static HPS        hpsClient;
static ULONG      MouMovCount = 0L;
static ULONG      MouseDivide =1L;
static ULONG      ulSegNo = 0;
static BOOL       bTouchDown = FALSE;
static BOOL       fButton1Down = FALSE;
static BOOL       fButton2Down = FALSE;

  switch( msg )
  {
    case WM_CREATE:
    {
      HAB   hab;
      HDC   hdc;

      /************************************************************************/
      /* The APIs being used in this procedure depend on the Pen for OS/2     */
      /* subsystem being correctly initialized.  Thus we check to make sure   */
      /* Pen for OS/2 is up before we do anything else.                       */
      /************************************************************************/
      if( ( irc = WrtWaitActive ( WRT_IMMEDIATE_RETURN ) ) != WRT_NO_ERROR )
      {
        /**********************************************************************/
        /* If Pen for OS/2 is NOT active then put up a message box and        */
        /* terminate the program.                                             */
        /**********************************************************************/
        WinLoadString ( habMain,
                        NULLHANDLE,
                        WRTERR_NOT_ACTIVE,
                        MAXCBUFFER,
                        cbuffer );
        WinMessageBox( HWND_DESKTOP,
                       hwnd,
                       cbuffer,
                       NULL, 1, MB_OK );
        WinSendMsg ( hwnd, WM_CLOSE, NULL, NULL );
        return( 0 );
      } /* end of if( ( irc = WrtWaitActive ...  */

      hwndMenu = WinWindowFromID( WinQueryWindow(hwnd, QW_PARENT), FID_MENU);

      /* initialize structure size to save time */
      apdExtra.cbStructSize = sizeof ( AUXPOINTDATA );
      edInfo.cbStructSize = sizeof ( WRTEVENTDATA );

      /************************************************************************/
      /* Initialize the ldiarr with all the locator information so it can be  */
      /* used during WM_SENSOR_MOVE                                           */
      /************************************************************************/
      /************************************************************************/
      /*  Find out how many locators the system has by querying the           */
      /*  locator capabilities API with ulLocatorID = 0L.                     */
      /*  The API will write the number of locator IDs in the system in       */
      /*  the variable ulLocatorID.                                           */
      /************************************************************************/
      ulLocatorID = 0L;
      irc = WrtQueryLocatorCaps ( &ulLocatorID, NULL );
      if ( irc != WRT_NO_ERROR )
      {
         /***********************************************************/
         /* ERROR from API                                          */
         /***********************************************************/
         DisplayError ( hwnd, irc );
      } else
      {
        /**************************************************************/
        /* Find out if locator are greater than zero                  */
        /**************************************************************/
        if ( ulLocatorID == 0L )
        {
           WinMessageBox( HWND_DESKTOP,
                           hwnd,
                           "WrtQueryLocatorCaps return 0 locators",
                           NULL, 1, MB_OK );
        } else
        {
          /***************************************************************/
          /* Display the Capabilities for all the locator in the system  */
          /***************************************************************/
          for ( ; ulLocatorID > 0L; ulLocatorID-- )
          {
            /*************************************************************/
            /* The cbStructureSize variable in the WRTLOCATORDEVINFO     */
            /* structure must be initialize to the size of the           */
            /* WRTLOCATORDEVINFO structure before querying the locator   */
            /* capabilities.                                             */
            /*************************************************************/
            ldiarr[ulLocatorID].cbStructSize = sizeof ( ldiarr[ulLocatorID] );

            /*************************************************************/
            /* ulLocatorID variable has the ID of the locator that you   */
            /* want to query                                             */
            /*************************************************************/
            irc = WrtQueryLocatorCaps ( &ulLocatorID, &ldiarr[ulLocatorID] );

            if ( irc == WRT_NO_ERROR )
            {
              /**********************************************************/
              /*  good return from API                                  */
              /*  mark good locator array as good                       */
              /**********************************************************/
              blocarr[ulLocatorID] = TRUE;
            } else
            {
              /**********************************************************/
              /* ERROR from API                                         */
              /**********************************************************/
              DisplayError ( hwnd, irc );
            } /* endif */
          } /* end of for ( ; ulLocatorID > 0L; ulLocatorID-- ) */
        } /* end of else */
      } /* end of else */

      /******************************************************************/
      /* post message to see what is active in the system               */
      /******************************************************************/
      WinPostMsg(hwnd, ID_DISP, NULL, NULL);

      /*********************************************************************/
      /* Load the "Blob" pointer we are going to use instead of the system */
      /* pointer when the pointer is over the client area.                 */
      /*********************************************************************/
      hptrClient = WinLoadPointer( HWND_DESKTOP, 0L, PTR_BLOB );

      hab = WinQueryAnchorBlock ( hwnd );
      hdc = WinOpenWindowDC ( hwnd );
      DevQueryCaps ( hdc, CAPS_WIDTH, 1L, &Size.cx );
      DevQueryCaps ( hdc, CAPS_HEIGHT, 1L, &Size.cy );
      hpsClient = GpiCreatePS ( hab,
                                hdc,
                                &Size,
                                PU_PELS      |
                                GPIT_NORMAL  |
                                GPIF_LONG    |
                                GPIA_ASSOC );
      GpiSetDrawingMode ( hpsClient, DM_DRAWANDRETAIN );
      break;
    } /* of WM_CREATE */

    case WM_TOUCHDOWN:
    {
      /*******************************************************************/
      /* Ensure that the client window is active.                        */
      /*******************************************************************/
      WinSetActiveWindow( HWND_DESKTOP, hwnd );

      /*******************************************************************/
      /* We are going to keep a count of mouse moves that make will make */
      /* up this stroke.                                                 */
      /*******************************************************************/
      MouMovCount  = 0L;

      /*******************************************************************/
      /* Set the lLastWidth to 1 pel wide to start with                  */
      /*******************************************************************/
      lLastWidth = 1L;

      /******************************************************************/
      /* Tell PM that ALL pointer messages should come to this window.  */
      /******************************************************************/
      WinSetCapture( HWND_DESKTOP, hwnd ) ;

      /******************************************************************/
      /* Determine where on the DESKTOP the client window is currently. */
      /* This will be used to make sure we do not draw strokes that go  */
      /* outside our client window.                                     */
      /******************************************************************/
      WinQueryWindowPos( hwnd, &swpClient );

      /****************************************************************/
      /* Get event data pointer from mp2                              */
      /****************************************************************/
      pedInfo = (PWRTEVENTDATA) PVOIDFROMMP ( mp2 );

      /****************************************************************/
      /* Check to see if we have the LOCATOR information set up       */
      /****************************************************************/
      if ( blocarr[pedInfo->ulLocatorID] == TRUE )
      {
         ulocindx = pedInfo->ulLocatorID;

         /* Set the MouseDivide variable */
         switch ( ldiarr[ulocindx].ulLocatorType )
         {
            case LT_PEN:
                    MouseDivide = (ldiarr[ulocindx].ulValidityFlags & ADF_SCREENZ) ? 4L : 1L ;
                    break;
            default:
                    MouseDivide = 1L;
                    break;
         } /* endswitch */
      } /* endif */

      /******************************************************************/
      /* Mask out the mouse button from the EventStatus flag.           */
      /******************************************************************/
      ulfButton = pedInfo->flEventStatus & WRT_MBUTTON_MASK;

      if( ulfButton & WRT_MBUTTON2 )
      {
        /****************************************************************/
        /* Set the Mouse state to mouse button 2 down                   */
        /****************************************************************/
        fButton1Down  = FALSE;
        fButton2Down  = TRUE;
      }
      else
      {
        /****************************************************************/
        /* Set the Mouse state to mouse button 1 down                   */
        /****************************************************************/
        fButton1Down  = TRUE;
        fButton2Down  = FALSE;
      }

      /******************************************************************/
      /* Get the mouse coordinates                                      */
      /******************************************************************/
      ptlXY.x = (SHORT) MOUSEMSG(&msg)->x ;
      ptlXY.y = (SHORT) MOUSEMSG(&msg)->y ;

      /**************************************************************************/
      /* if fButton1Down is set to zero it means that fButton2Down is to to one */
      /* and we are in erase mode. Set the ink to the background to erase       */
      /**************************************************************************/
      GpiOpenSegment(hpsClient, ++ulSegNo);
      GpiSetColor( hpsClient, fButton1Down ? CLR_NEUTRAL : CLR_BACKGROUND );
      GpiMove( hpsClient, &ptlXY );

      /************************************************************************/
      /* Tell the Pen for OS/2 subsystem that we want                         */
      /* 1) no button down message TDN_INFINITE,                              */
      /* 2) as many mouse moves message as possible TDN_HIFREQ_MOUSEMOVE, and */
      /* 3) that we will take care of inking any strokes TDN_NO_INK_STROKE.   */
      /************************************************************************/
      return((MRESULT) (TDN_INFINITE | TDN_NO_INK_STROKE |
                        TDN_HIFREQ_MOUSEMOVE ));
    }

    case WM_LIFTOFF:
    {
      /******************************************************************/
      /* Tell PM to resume routing pointer messages to the windows that */
      /* the pointer goes over.                                         */
      /******************************************************************/
      WinSetCapture( HWND_DESKTOP, (HWND) NULL ) ;

      GpiCloseSegment(hpsClient);
      /******************************************************************/
      /* Get the WRT event Data to see if the STROKE DATA is valid      */
      /******************************************************************/
      pedInfo = (PWRTEVENTDATA) PVOIDFROMMP ( mp2 );

      if( ( pedInfo->flEventStatus & WRT_STROKE_AVAIL ) )
      {
        /******************************************************************/
        /* There is stroke data, query the size of the data, when put in  */
        /* a POINTL array of screen coordinates.                          */
        /******************************************************************/
        ulOptions = QSD_SCALE;     /* return in screen resolution */
        ulBuflen = 0L;

        /*********************************************************************************/
        /* Get the screen Z information for every point                                  */
        /* if ldiarr[ulocindx].ulValidityFlags & ADF_SCREENZ is true else do not get any */
        /*********************************************************************************/
        flAuxData = (ldiarr[ulocindx].ulValidityFlags & ADF_SCREENZ) ? ADF_SCREENZ : 0L ;

        irc = WrtQueryStrokeData ( NULL, &ulBuflen, hwnd, ulOptions,
                                   0UL, 0UL,
                                   flAuxData   /* get the AUX data per point identify in flAuxData */
                                   );
        if( irc != WRT_NO_ERROR )
        {
           DisplayError ( hwnd, irc );
        }
        else
        {
          if( !ulBuflen )
          {
            WinMessageBox( HWND_DESKTOP, hwnd, "Zero ulBuflen", NULL, 1, MB_OK);
          }
          else
          {
            if( ( psdCurStroke = (PSTROKEDATA) malloc( ulBuflen ) ) == NULL )
            {
               WinMessageBox( HWND_DESKTOP, hwnd, "malloc error",
                              NULL, 1, MB_OK );
            }
            else
            {
              psdCurStroke->cbStructSize = sizeof ( STROKEDATA );
              irc = WrtQueryStrokeData ( (PBYTE)psdCurStroke,
                                         &ulBuflen, hwnd, ulOptions,
                                         0UL, 0UL,   /*  scaled to screen */
                                         flAuxData   /* get the AUX data per point identify in flAuxData */
                                         );
              if( irc != WRT_NO_ERROR )
              {
                DisplayError ( hwnd, irc );
              }
              else
              {
                 GiveLiftOff(psdCurStroke, fButton1Down);
              } /* endif */
            } /* endif */
          } /* endif */
        } /* endif */
      } /* endif */

      /******************************************************************/
      /* Reset our flags                                                */
      /******************************************************************/
      fButton1Down = FALSE;
      fButton2Down = FALSE;
      bTouchDown = FALSE;
      MouMovCount = 0;
      return( (MRESULT) LO_STROKE_PROCESSED );
    } /* of WM_LIFTOFF */

    case WM_MOUSEMOVE:
    {
      /******************************************************************/
      /* We are over our client window, change pointer bitmap to our    */
      /* "BLOB" pointer bitmap.                                         */
      /******************************************************************/
      WinSetPointer( HWND_DESKTOP, hptrClient );

      if( !fButton1Down && !fButton2Down )
      {
        return( (MRESULT) FALSE );
      } /* endif */

      /************************************************************/
      /* get the event data to check if the stroke buffer is full */
      /* if the stroke buffer is full. The stroke buffer is 1000  */
      /* which is about nine(9) seconds of data                   */
      /************************************************************/
      /*******************************************************//* initialize structure size at WM_CREATE to save time */
      /* edInfo.cbStructSize = sizeof ( WRTEVENTDATA );      */
      /*******************************************************/
      WrtQueryEventData ( &edInfo );
      pedInfo = &edInfo;
      if ( pedInfo->flEventStatus & WRT_BUFFER_OVERRUN )
      {
        return( (MRESULT) FALSE ); /* if the buffer is full do not draw any more points */
      } /* endif */

      MouMovCount++;
      if ( (MouMovCount % MouseDivide ) )
      {
        return( (MRESULT) TRUE );
      } /* endif */

      /******************************************************************/
      /* Get the mouse coordinates                                      */
      /******************************************************************/
      ptlXY.x = (SHORT) MOUSEMSG ( &msg )->x;
      ptlXY.y = (SHORT) MOUSEMSG ( &msg )->y;
      GpiLine ( hpsClient, &ptlXY );
      return( (MRESULT) TRUE );
    } /* of WM_MOUSEMOVE, WM_SENSOR_MOVE */

    case WM_WRT_SYSVALUECHANGED:
    {
      ulid = LONGFROMMP (mp2);
      switch ( LONGFROMMP (mp1) )
      {
         case VT_PPMSV:
         {
            /* some program changed a system value */
            for ( i = 0; i < QSVALUESIZE; i++ )
            {
               if ( qsvalue[i].valueid == ulid )
               {
                  break;
               }
            }  /* endfor */
            if ( i < QSVALUESIZE )
            {
              sprintf ( cbuffer, "%s [%ld] System Variable got changed",
                       qsvalue[i].valname, ulid );
            } else
            {
              sprintf ( cbuffer, "Error [%ld] is not a valid System Variable ID",
                        ulid );
            }
            WinMessageBox( HWND_DESKTOP,
                           hwnd,
                           cbuffer,
                           NULL, 1, MB_OK );
            break;
         }  /* of case VT_PPMSV */

         case VT_PPMID:
         {
            /* some program change a Device Variable */
            for ( i = 0; i < PPMIDSIZE; i++ )
            {
               if ( ppmid[i].ValueID == ulid )
               {
                  break;
               }
            }
            if ( i < PPMIDSIZE )
            {
              sprintf ( cbuffer, "%s [%ld] Device Variable got changed",
                       ppmid[i].name, ulid );
            } else
            {
              sprintf ( cbuffer, "Error [%ld] is not a valid Device Variable ID",
                        ulid );
            }
            WinMessageBox( HWND_DESKTOP,
                           hwnd,
                           cbuffer,
                           NULL, 1, MB_OK );
            break;
         }  /* of VT_PPMID */
      } /* end of switch */
      break;
    }
    case WM_PAINT:
    {
      hpsPaint = WinBeginPaint( hwnd, hpsClient, NULL );
      GpiErase(hpsPaint);
      GpiDrawChain( hpsPaint );
      WinEndPaint( hpsPaint );
      break;
    }

    case ID_DISP:
    {
       /***********************************************************/
       /* 1.Find out if there are any Button Caps                 */
       /*   if no button, disable the query button menu           */
       /* 2.Find out if there are any Locator Caps                */
       /*   if no locator, disable query Locator Caps             */
       /***********************************************************/
       scInfo.cbStructSize = sizeof ( scInfo );
       irc = WrtQuerySystemCaps ( &scInfo );
       if ( irc == WRT_NO_ERROR )
       {
          if ( scInfo.ulNumButtons == 0L )
          {
             /***************************************************/
             /* disable the query Button menu                   */
             /***************************************************/
             WinSendMsg( hwndMenu,
                         (ULONG) MM_SETITEMATTR,
                         MPFROM2SHORT ( ID_BCAPS, TRUE ),
                         MPFROM2SHORT ( MIA_DISABLED, MIA_DISABLED ) );
          } /* endif */

          if ( scInfo.ulNumLocators == 0L )
          {
             /****************************************************/
             /* disable the query Locator menu                   */
             /****************************************************/
             WinSendMsg( hwndMenu,
                         (ULONG) MM_SETITEMATTR,
                         MPFROM2SHORT ( ID_LCAPS, TRUE ),
                         MPFROM2SHORT ( MIA_DISABLED, MIA_DISABLED ) );
          } /* endif */
       } else
       {
         /*********************************************************/
         /* ERROR from API                                        */
         /*********************************************************/
         DisplayError ( hwnd, irc );
       } /* endif */
       break;
    } /* of ID_DISP */

    case WM_COMMAND:
    {
      switch ( SHORT1FROMMP(mp1) )
      {
         case ID_CLEAR:
         {
           /***************************************************************/
           /* If there was a clear cmd, then tell the procedure to clear  */
           /* the client window, and clear the memory bitmap now.         */
           /***************************************************************/
           GpiDeleteSegments( hpsClient, 1, ulSegNo );
           WinInvalidateRect( hwndClient, NULL, TRUE );
           break;
         }

         case ID_EXIT:
           GpiDestroyPS( hpsClient );
           WinSendMsg(hwnd, WM_CLOSE, NULL, NULL);
           break;

        /***************************************************************/
        /* System Capability                                           */
        /***************************************************************/
        case ID_SCAPS:
        {
           /************************************************************/
           /* The cbStructureSize variable in the WRTSYSTEMINFO        */
           /* structure must be initialize to the size of the          */
           /* WRTSYSTEMINFO structure before querying the System       */
           /* capabilities.                                            */
           /************************************************************/
           scInfo.cbStructSize = sizeof ( scInfo );
           irc = WrtQuerySystemCaps ( &scInfo );
           if ( irc == WRT_NO_ERROR )
           {
              /*********************************************************/
              /*  good return from API                                 */
              /*  display the System capabilities                      */
              /*********************************************************/
              WinDlgBox(HWND_DESKTOP,
                        hwnd,
                        SysCapDlgProc,
                        0UL,
                        IDD_SCDLG,
                        &scInfo);
           } else
           {
              /*********************************************************/
              /* ERROR from API                                        */
              /*********************************************************/
              DisplayError ( hwnd, irc );
           } /* endif */
           break;
        }  /* of ID_SCAPS */

        case ID_LCAPS:      /* Locator Capability          */
        case ID_BCAPS:      /* Button  Capability          */
        case ID_QSVAL:      /* Query System Values         */
        case ID_SSVAL:      /* Set System Values           */
        case ID_WSVAL:      /* Write System Values         */
        case ID_QDNAM:      /* Query Input Device Names    */
        case ID_SDVAR:      /* Set Input Device Variable   */
        {
           PFNWP  pfnP;
           ULONG  ulDlgID;

           switch(SHORT1FROMMP(mp1))
           {
              case ID_LCAPS: /* Locator Capability */
                 pfnP = LocCapDlgProc;
                 ulDlgID = IDD_LCDLG;
                 break;

              case ID_BCAPS: /* Button  Capability */
                 pfnP = ButCapDlgProc;
                 ulDlgID = IDD_BCDLG;
                 break;

              case ID_QSVAL: /* Query System Values */
                 pfnP = SysValDlgProc;
                 ulDlgID = IDD_SVDLG;
                 break;

              case ID_SSVAL: /* Set System Values */
                 pfnP = SSysVlDlgProc;
                 ulDlgID = IDD_SSDLG;
                 break;

              case ID_WSVAL: /* Write System Values */
                 pfnP = WSysVlDlgProc;
                 ulDlgID = IDD_WSDLG;
                 break;

              case ID_QDNAM: /* Query Input Device Names */
                 pfnP = QInDevNDlgProc;
                 ulDlgID = IDD_DNDLG;
                 break;

              case ID_SDVAR: /* Set Input Device Variable */
                 pfnP = SInpDevDlgProc;
                 ulDlgID = IDD_SNDLG;
                 break;
           }  /* of sub switch */

           WinDlgBox(HWND_DESKTOP,
                     hwnd,
                     pfnP,
                     0UL,
                     ulDlgID,
                     NULL );
           break;
        } /* of sub cases */

        /*****************************************************************/
        /* Enumerate Input Drivers                                       */
        /*****************************************************************/
        case ID_ENUDR:
        {
            /* Find out how many input driver there are */
            ulNumDrivers = 0L;
            irc = WrtEnumInputDrivers ( &ulNumDrivers, NULL );
            if ( irc != WRT_NO_ERROR )
            {
               DisplayError ( hwnd, irc );
            } else
            {
              if ( ulNumDrivers > 10L )
              {
                 sprintf ( cbuffer, "Too many drivers[%ld] for my ARRAY",
                         ulNumDrivers );
                 WinMessageBox( HWND_DESKTOP,
                                hwnd,
                                cbuffer,
                                NULL, 1, MB_OK );
              } else
              {
                if ( ulNumDrivers == 0L )
                {
                   WinMessageBox( HWND_DESKTOP,
                                  hwnd,
                                  "WrtEnumInputDrivers return 0 drivers",
                                  NULL, 1, MB_OK );
                } else
                {
                  irc = WrtEnumInputDrivers ( &ulNumDrivers, szDriverName );
                  if ( irc == WRT_NO_ERROR )
                  {
                     WinDlgBox(HWND_DESKTOP,
                               hwnd,
                               EnumDrvDlgProc,
                               0UL,
                               IDD_EIDLG,
                               NULL );
                  } else
                  {
                     DisplayError ( hwnd, irc );
                  } /* endif */
                } /* endif */
              } /* endif */
            } /* endif */
           break;
        }  /* of ID_ENUDR */
        break;
      }
      break;
    }

    default:
    {
      return( WinDefWindowProc( hwnd, msg, mp1, mp2 ) );
    }
  }
  return( (MRESULT) FALSE );
} /* of ClientWndProc */

VOID GiveLiftOff( PSTROKEDATA psdCurStroke, BOOL fButton1Down)
{
  INT          i;
  HPS          hpsTClient;
  POINTL       *pxyCurStroke;
  SHORT        *psCurrP;             /* pointer to get the pressure */

    hpsTClient = WinGetPS ( hwndClient );
    pxyCurStroke = psdCurStroke->pXY;

    /**************************************************************************/
    /* if fButton1Down is set to zero it means that fButton2Down is to to one */
    /* and we are in erase mode. Set the ink to the background to erase       */
    /**************************************************************************/
    GpiSetColor( hpsTClient, fButton1Down ? CLR_BLUE : CLR_BACKGROUND );

    /**************************************************************/
    /* Move the staring point of the the line to the beging of    */
    /* the stroke                                                 */
    /**************************************************************/
    GpiMove( hpsTClient, &pxyCurStroke[0] );

    /*******************************************/
    /* find out if AUXPOINTDATA.sScrZ is valid */
    /*******************************************/
    if ( (ldiarr[ulocindx].ulValidityFlags & ADF_SCREENZ) )
    {
      /***********************************************************/
      /* Since I only requested one AUX data                     */
      /* psdCurStroke->pAuxInfo->cbAuxDesc == 24                 */ /* 4*/
      /* psdCurStroke->pAuxInfo->ulNumElements == 1              */ /* 8*/
      /* psdCurStroke->pAuxInfo->ulAuxSize == 2                  */ /*12*/
      /* psdCurStroke->pAuxInfo->addAuxDesc[0].id == ADT_SCREENZ */ /*16*/
      /* psdCurStroke->pAuxInfo->addAuxDesc[0].offset == 0L      */ /*20*/
      /* psdCurStroke->pAuxInfo->addAuxDesc[0].cb == 2L          */ /*24*/
      /***********************************************************/

      /* get the first SCREENZ information */
psCurrP = (SHORT *)&(psdCurStroke->pAuxData->bAuxData[0][psdCurStroke->pAuxInfo->addAuxDesc[0].offset]);
      sLastP = *psCurrP;    /* this is done becuase bAuxData is unsign CHAR and SCREENZ is SHORT */

      /* for all the point in the stroke after the first point */
      for ( i = 1L; i < psdCurStroke->ulNumPoints ; i++ )
      {
psCurrP = (SHORT *)&(psdCurStroke->pAuxData->bAuxData[ (i*psdCurStroke->pAuxInfo->ulAuxSize) ][psdCurStroke->pAuxInfo->addAuxDesc[0].offset]);
         sCurrP = *psCurrP;

         /***********************************************************************/
         /*  the smaller AUXPOINTDATA.sScrZ the closer you are to the surface   */
         /* once you go negative value you are applying pressure to the screen  */
         /***********************************************************************/
         lLastWidth += (sLastP > sCurrP) ? 1 : -1;

         /***********************************************************************/
         /* for this example I do not want the width to get bigger than 16 pels */
         /* and teh width should never be allow to be less than 1 pel.          */
         /***********************************************************************/
         lLastWidth = ( lLastWidth <= 16L ) ? lLastWidth : 16L;
         lLastWidth = ( lLastWidth > 0L ) ? lLastWidth : 1L;

         sLastP = sCurrP;      /* set the last pressure to the current pressure */

         /*********************************************/
         /* Begins a figure that has  geometric width */
         /*********************************************/
         GpiBeginPath ( hpsTClient, 1L );

         /*************************************************/
         /* Set the current geometric line-width.         */
         /* This width is only used in the GpiStrokePath  */
         /*************************************************/
         GpiSetLineWidthGeom ( hpsTClient, lLastWidth );

         /************************/
         /* The figure is a line */
         /************************/
         GpiLine ( hpsTClient, &pxyCurStroke[i] );

         /********************************************/
         /* Ends the figure that has geometric width */
         /********************************************/
         GpiEndPath ( hpsTClient );

         /***************************************************/
         /* draws the figures with the geometric line-width */
         /* and fills it in                                 */
         /***************************************************/
         GpiStrokePath ( hpsTClient, 1L, 0L );
      } /* endfor */
    } else       /* the Locator does not support SCREENZ */
      {
          GpiPolyLine( hpsTClient,
                       (LONG)(psdCurStroke->ulNumPoints - 1L ),
                       &pxyCurStroke[1] );
      } /* endif */
    WinReleasePS ( hpsTClient );
    free( psdCurStroke );
} /* of GiveLiftOff */

