/******************************************************************************
*                                                                             *
*  File Name   : WRTSMDLG.C                                                   *
*                                                                             *
*  Description : Dialog functions for the WRTSAMPL.exe                        *
*                                                                             *
*  Function:  This file has all the dialog procedures for the wrtsampl.exe    *
*                                                                             *
*  Copyright (C) 1993 IBM Corporation                                         *
*                                                                             *
*      DISCLAIMER OF WARRANTIES.  The following [enclosed] code is            *
*      sample code created by IBM Corporation. This sample code is not        *
*      part of any standard or IBM product and is provided to you solely      *
*      for  the purpose of assisting you in the development of your           *
*      applications.  The code is provided "AS IS", without                   *
*      warranty of any kind.  IBM shall not be liable for any damages         *
*      arising out of your use of the sample code, even if they have been     *
*      advised of the possibility of such damages.                            *
*                                                                             *
******************************************************************************/

#define  INCL_DOS
#define  INCL_PM
#define  INCL_GPI
#define  INCL_DOSMEMMGR
#define  INCL_WINLOAD
#define  INCL_DOSPROCESS
#define  INCL_DOSSEMAPHORES
#include <os2.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <penpm.h>
#include "wrtsampl.h"

ULONG    ulNumDevices;
CHAR     aDeviceName[10][32];
SHORT    index3;
HWND     hwndCombo1, hwndCombo2, hwndCombo3;
LONG     startvalue;
SHORT    index;

/*******************************************************************************
*                                                                              *
* Function Name: QInDevNDlgProc                                                *
*                                                                              *
* Description :                                                                *
*                                                                              *
*   Displays the Device names supported by each driver                         *
*                                                                              *
* Input:                                                                       *
*                                                                              *
* Output: Display the Device names supported by each driver.                   *
*                                                                              *
*******************************************************************************/
MRESULT EXPENTRY QInDevNDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
APIRET    irc;
CHAR      cbuffer[MAXCBUFFER];
CHAR      achDriverName[32];
ULONG     aDeviceType[10];
ULONG     aDeviceID[10];
HWND      hwnd_temp;
WNDPARAMS comboprams;
INT       i;

   switch(msg)
   {
      case WM_INITDLG:
      {
         WinSendDlgItemMsg ( hwnd,
                             wrtelc04,
                             EM_SETTEXTLIMIT,
                             MPFROM2SHORT(4, 0),
                             0 );
        init_invar ( hwnd, TRUE );
        return(FALSE);
      } /* of WM_INITDLG */

      case WM_COMMAND:
      {
         /* Only EXIT button gives this message */
         WinDismissDlg ( hwnd, FALSE );
         return ( FALSE );
      }

    case WM_CONTROL:
    {
       switch ( SHORT2FROMMP ( mp1 ) )
       {
          case CBN_ENTER:
          {
            hwnd_temp = HWNDFROMMP ( mp2 );
            hwndCombo1 = WinWindowFromID( hwnd, wrtelc01);
            if ( hwnd_temp == hwndCombo1 )
            {
              SHORT index1;

              /*******************************************************************/
              /* Get the Driver selected and find:                               */
              /* 1: devices supported by the driver                              */
              /*******************************************************************/
              /*******************************************************************/
              /* find the index of the selected item                             */
              /*******************************************************************/
              index1 = (SHORT) WinSendDlgItemMsg ( hwnd,
                                                   wrtelc01,
                                                   LM_QUERYSELECTION,
                                                   0L,
                                                   0L );
              if ( index1 != LIT_NONE )
              {
                /*****************************************************************/
                /* get the name of the driver choosen.                           */
                /*****************************************************************/
                WinSendDlgItemMsg ( hwnd,
                                    wrtelc01,
                                    LM_QUERYITEMTEXT,
                                    MPFROM2SHORT ( index1, 32 ),
                                    MPFROMP ( achDriverName ) );
                /*****************************************************************/
                /* Clear out the Devices list box and get the devices for the    */
                /* driver.                                                       */
                /*****************************************************************/
                WinSendDlgItemMsg ( hwnd,
                                    wrtelc02,
                                    LM_DELETEALL,
                                    0L,
                                    0L );
                /*****************************************************************/
                /* Find out how many input device there are                      */
                /*****************************************************************/
                GetInputDevices(hwnd, achDriverName);

              } /* end of if ( index1 != LIT_NONE )        */
            } /* end of if ( hwnd_temp == hwndCombo1 )     */
            break;
          } /* of case CBN_ENTER */

          default:
             return(WinDefDlgProc(hwnd, msg, mp1, mp2));
       }  /* end of WM_CONTROL subswitch */
       break;
    }  /* of WM_CONTROL */

    default:
       return(WinDefDlgProc(hwnd, msg, mp1, mp2));
   } /* endswitch */

   return ( FALSE );
}  /* of QInDevNDlgProc */

/*******************************************************************************
*                                                                              *
* Function Name: SInpDevDlgProc                                                *
*                                                                              *
* Description :                                                                *
*                                                                              *
*   Set the Input Device variable Pen for OS/2                                 *
*                                                                              *
* Input:                                                                       *
*                                                                              *
* Output: Set the Input variable.                                              *
*                                                                              *
******************************************************************************/
MRESULT EXPENTRY SInpDevDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
APIRET          irc;
CHAR            cbuffer[MAXCBUFFER];
HWND            hwnd_temp;
INT             i;
INT             cbox;
static SHORT    idx1, idx2, idx3;

   switch(msg)
   {
      case WM_INITDLG:
         init_invar ( hwnd, FALSE );
         return(FALSE);

      case WM_COMMAND:
         WinDismissDlg ( hwnd, FALSE );
         return ( FALSE );

      case WM_CONTROL:
         switch ( SHORT2FROMMP ( mp1 ) )
         {
            case CBN_ENTER:
               hwnd_temp = HWNDFROMMP ( mp2 );
               updatecntrl ( hwnd, hwnd_temp );
               break;

            default:
               return(WinDefDlgProc(hwnd, msg, mp1, mp2));
         }  /* endswitch */
         return(FALSE);

       default:
          return(WinDefDlgProc(hwnd, msg, mp1, mp2));
   } /* endswitch */
}  /* of SInpDevDlgProc */

/*******************************************************************************
*                                                                              *
* Function Name: EnumDrvDlgProc                                                *
*                                                                              *
* Description :                                                                *
*                                                                              *
*   Displays the Enumiration of the Pen for OS/2 input drivers.                *
*                                                                              *
* Input:                                                                       *
*                                                                              *
* Output: Display the Enumiration of the Pen for OS/2 drivers.                 *
*                                                                              *
*******************************************************************************/
MRESULT EXPENTRY EnumDrvDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
INT i;

   switch(msg)
   {
      case WM_INITDLG:
        WinSendDlgItemMsg(hwnd,          /* Delete all items in the list box */
                          wrtelc01,
                          LM_DELETEALL,
                          0UL,
                          0UL);

        for ( i = 0; i < ulNumDrivers; i++ )
        {
              WinSendDlgItemMsg(hwnd,   /* put the Driver name to listbox */
                                wrtelc01,
                                LM_INSERTITEM,
                                MPFROM2SHORT( LIT_END, 0 ),
                                MPFROMP( szDriverName[i] )
                                );
        } /* endfor */
        return(FALSE);

      case WM_COMMAND:
         WinDismissDlg ( hwnd, FALSE );
         return ( FALSE );

      default:
         return(WinDefDlgProc(hwnd, msg, mp1, mp2));
   } /* endswitch */
}  /* of EnumDrvDlgProc */

/*******************************************************************************
*                                                                              *
* Function Name: WSysVlDlgProc                                                 *
*                                                                              *
* Description :                                                                *
*                                                                              *
*   Displays and Write   System values.                                        *
*                                                                              *
* Input:                                                                       *
*                                                                              *
* Output: Write a     system values                                            *
*                                                                              *
*******************************************************************************/
MRESULT EXPENTRY WSysVlDlgProc ( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
APIRET irc;

   switch(msg)
   {
     case WM_INITDLG:
        init_sysval( hwnd, 0 );
        return(FALSE);

     case WM_CONTROL:
        switch ( SHORT2FROMMP ( mp1 ) )
        {
           case SPBN_UPARROW:
           case SPBN_DOWNARROW:
              updwn_sysval ( hwnd );
              break;

           case CBN_ENTER:
              enter_sysval( hwnd, 0 );
              return ( FALSE );

           default:
              return(WinDefDlgProc(hwnd, msg, mp1, mp2));
        } /* end of WM_CONTROL subswitch */

     case WM_COMMAND:
        switch ( SHORT1FROMMP ( mp1 ) )
        {
            case EXIT:
               WinDismissDlg ( hwnd, FALSE );
               break;

            case SET:
               set_sysval ( hwnd );

               /***************************************************************/
               /* Write the system value from memory to Pen for OS/2 profile  */
               /***************************************************************/
               irc = WrtWriteSysValue ( HWND_DESKTOP,
                                        qsvalue[index+1].valueid );
               DisplayError ( hwnd, irc );
               break;
        } /* endswitch */
        return ( FALSE );

        default:
           return ( WinDefDlgProc(hwnd, msg, mp1, mp2) );
   } /* end of switch */
}  /* of WSysVlDlgProc */

/*******************************************************************************
*                                                                              *
* Function Name: SSysVlDlgProc                                                 *
*                                                                              *
* Description :                                                                *
*                                                                              *
*   Displays and set the System values.                                        *
*                                                                              *
* Input:                                                                       *
*                                                                              *
* Output: Set     the system values                                            *
*                                                                              *
*******************************************************************************/
MRESULT EXPENTRY SSysVlDlgProc ( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
   switch(msg)
   {
      case WM_INITDLG:
        init_sysval( hwnd, 0 );
        return(FALSE);

     case WM_CONTROL:
        switch ( SHORT2FROMMP ( mp1 ) )
        {
           case SPBN_UPARROW:
           case SPBN_DOWNARROW:
              updwn_sysval ( hwnd );
              break;

           case CBN_ENTER:
              enter_sysval( hwnd, 0 );
              return ( FALSE );

           default:
              return(WinDefDlgProc(hwnd, msg, mp1, mp2));
        } /* endswitch */

     case WM_COMMAND:
        switch ( SHORT1FROMMP ( mp1 ) )
        {
            case EXIT:
               WinDismissDlg ( hwnd, FALSE );
               break;

            case SET:
               set_sysval ( hwnd );
               break;
        } /* endswitch */
        return ( FALSE );

     default:
        return ( WinDefDlgProc(hwnd, msg, mp1, mp2) );
   } /* endswitch */
}  /* end of SSysVlDlgProc */

/******************************************************************************
*                                                                             *
* Function Name: SysValDlgProc                                                *
*                                                                             *
* Description :                                                               *
*                                                                             *
*   Displays the System Values                                                *
*                                                                             *
* Input:                                                                      *
*                                                                             *
* Output: Display the system values                                           *
*                                                                             *
******************************************************************************/
MRESULT EXPENTRY SysValDlgProc ( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
CHAR  cbuffer[MAXCBUFFER];
CHAR  cname[30], cvalue[15];
INT   i;

   switch(msg)
   {
      case WM_INITDLG:
        for (i = 0; i < QSVALUESIZE ; i++)
        {
          qsvalue[i].value = WrtQuerySysValue ( HWND_DESKTOP,
                                                qsvalue[i].valueid );

          if ( qsvalue[i].value == WRTERR_INVALID_PARM )
          {
            sprintf ( cvalue, "%15s","Invalid Parm" );
          } else
          {
            sprintf ( cvalue, "%05li", qsvalue[i].value );
          } /* endif */

          sprintf ( cname, "%-30s", qsvalue[i].valname );
          sprintf ( cbuffer, "%-30.30s%-15.15s", cname, cvalue );
          WinSendDlgItemMsg(hwnd,
                            wrtelc01,
                            LM_INSERTITEM,
                            MPFROM2SHORT(LIT_END, 0),
                            MPFROMP( cbuffer ) );
        } /* endfor */
        return(FALSE);

      case WM_COMMAND:
         WinDismissDlg ( hwnd, FALSE );
         return ( FALSE );

      default:
         return(WinDefDlgProc(hwnd, msg, mp1, mp2));
   }
}  /* of SysValDlgProc */

/*******************************************************************************
*                                                                              *
* Function Name: ButCapDlgProc                                                 *
*                                                                              *
* Description :                                                                *
*                                                                              *
*   Displays the capabilities of the Buttons                                   *
*                                                                              *
* Input:                                                                       *
*                                                                              *
* Output: Display the capabilities of the system                               *
*                                                                              *
*******************************************************************************/
MRESULT EXPENTRY ButCapDlgProc ( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
CHAR              cbuffer[MAXCBUFFER];
WRTBUTTONDEVINFO  bcInfo;
ULONG             ulButtonID;
ULONG             ulButTmp;
WNDPARAMS         comboprams;
APIRET            irc;
SHORT             index1;

   switch(msg)
   {
      case WM_INITDLG:
      {
         /********************************************************************/
         /*  Find out how many buttons the system has by querying the        */
         /*  button capabilities API with ulButtonID = 0L.                   */
         /*  The API will write the number of button IDs in the system in    */
         /*  the variable ulButtonID.                                        */
         /********************************************************************/
         ulButtonID = 0L;
         irc = WrtQueryButtonCaps ( &ulButtonID, NULL );
         if ( irc != WRT_NO_ERROR )
         {
           /******************************************************************/
           /* ERROR from API                                                 */
           /******************************************************************/
           DisplayError ( hwnd, irc );
         } else
         {
           /******************************************************************/
           /* Find out if return greater than zero buttons                   */
           /******************************************************************/
           if ( ulButtonID == 0L )
           {
             WinMessageBox( HWND_DESKTOP,
                             hwnd,
                             "WrtQueryButtonCaps return 0 buttons",
                             NULL, 1, MB_OK );
           } else
           {
             /****************************************************************/
             /* Got some locator to display                                  */
             /****************************************************************/
             hwndCombo1 = WinWindowFromID ( hwnd, wrtelc05 );
             for ( ulButTmp = 1L; ulButTmp <= ulButtonID; ulButTmp++ )
             {
               /**************************************************************/
               /* The cbStructureSize variable in the WRTBUTTONDEVINFO       */
               /* structure must be initialize to the size of the            */
               /* WRTBUTTONDEVINFO structure before querying the button      */
               /* capabilities.                                              */
               /**************************************************************/
               bcInfo.cbStructSize = sizeof ( bcInfo );

               /**************************************************************/
               /* ulButTmp   variable has the ID of the button that you      */
               /* want to query                                              */
               /**************************************************************/
               irc = WrtQueryButtonCaps ( &ulButTmp, &bcInfo );

               if ( irc == WRT_NO_ERROR )
               {
                 /************************************************************/
                 /*  good return from API                                    */
                 /************************************************************/
                 sprintf ( cbuffer, "%s %s",
                           bcInfo.pszDriverName, bcInfo.pszDeviceName );
                 index1 = (SHORT) WinSendMsg ( hwndCombo1,
                                               LM_INSERTITEM,
                                               MPFROMSHORT ( LIT_END ),
                                               MPFROMP ( cbuffer ) );
               } else
               {
                 /************************************************************/
                 /* ERROR from API                                           */
                 /************************************************************/
                 DisplayError ( hwnd, irc );
               } /* endif */
             } /* endfor */
             /****************************************************************/
             /* Set the last  item on the list at the top.                   */
             /****************************************************************/
             WinSendMsg ( hwndCombo1, LM_SETTOPINDEX,
                          MPFROMSHORT ( index1 ), 0 );

             comboprams.fsStatus = WPM_TEXT;           /* change window text */
             comboprams.cchText = sizeof ( cbuffer );  /* text size          */
             comboprams.pszText = cbuffer ;            /* text to display    */
             WinSendMsg ( hwndCombo1, WM_SETWINDOWPARAMS, &comboprams, 0 );
             disp_butcaps ( hwnd, &bcInfo );
           } /* endif */
         } /* endif */
         return(FALSE);
      } /* of case WM_INITDLG */

      case WM_CONTROL:
         switch ( SHORT2FROMMP (mp1) )
         {
            case CBN_ENTER:
              /*****************************************************************/
              /* find the index of the selected item                           */
              /*****************************************************************/
              index1 = (SHORT) WinSendMsg ( hwndCombo1, LM_QUERYSELECTION, 0L, 0L );
              if ( index1 != LIT_NONE )
              {
                ulButTmp = (ULONG)(index1 + 1L);
                bcInfo.cbStructSize = sizeof ( bcInfo );
                irc = WrtQueryButtonCaps ( &ulButTmp, &bcInfo );
                if ( irc == WRT_NO_ERROR )
                {
                  disp_butcaps (hwnd, &bcInfo );
                } /* endif */
              } /* endif */
              break;

            default:
               return(WinDefDlgProc(hwnd, msg, mp1, mp2));
         } /* endswitch */
         return ( FALSE );

      case WM_COMMAND:
         WinDismissDlg ( hwnd, FALSE );
         return ( FALSE );

      default:
         return(WinDefDlgProc(hwnd, msg, mp1, mp2));
   }
}  /* of ButCapDlgProc */

/*******************************************************************************
*                                                                              *
* Function Name: LocCapDlgProc                                                 *
*                                                                              *
* Description :                                                                *
*                                                                              *
*   Displays the capabilities of the Locaotors                                 *
*                                                                              *
* Input:                                                                       *
*                                                                              *
* Output: Display the capabilities of the system                               *
*                                                                              *
*******************************************************************************/
MRESULT EXPENTRY LocCapDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
WRTLOCATORDEVINFO lcInfo;
CHAR              cbuffer[MAXCBUFFER];
ULONG             ulLocatorID;
ULONG             ulLocTmp;
WNDPARAMS         comboprams;
APIRET            irc;
SHORT             index1;

   switch(msg)
   {
      case WM_INITDLG:
      {
         /******************************************************************/
         /*  Find out how many locators the system has by querying the     */
         /*  locator capabilities API with ulLocatorID = 0L.               */
         /*  The API will write the number of locator IDs in the system in */
         /*  the variable ulLocatorID.                                     */
         /******************************************************************/
         ulLocatorID = 0L;
         irc = WrtQueryLocatorCaps ( &ulLocatorID, NULL );
         if ( irc != WRT_NO_ERROR )
         {
           /****************************************************************/
           /* ERROR from API                                               */
           /****************************************************************/
           DisplayError ( hwnd, irc );
         } else
         {
           /****************************************************************/
           /* Find out if locator are greater than zero                    */
           /****************************************************************/
           if ( ulLocatorID == 0L )
           {
             WinMessageBox( HWND_DESKTOP,
                             hwnd,
                             "WrtQueryLocatorCaps return 0 locators",
                             NULL, 1, MB_OK );
           } else
           {
             /* Got some locator to display */
             hwndCombo1 = WinWindowFromID ( hwnd, wrtelc23 );
             /**************************************************************/
             /* Display the Capabilities for all the locator in the system */
             /**************************************************************/
             for ( ulLocTmp = 1L ; ulLocTmp <= ulLocatorID ; ulLocTmp++ )
             {
               /************************************************************/
               /* The cbStructureSize variable in the WRTLOCATORDEVINFO    */
               /* structure must be initialize to the size of the          */
               /* WRTLOCATORDEVINFO structure before querying the locator  */
               /* capabilities.                                            */
               /************************************************************/
               lcInfo.cbStructSize = sizeof ( lcInfo );

               /************************************************************/
               /* ulLocTmp    variable has the ID of the locator that you  */
               /* want to query                                            */
               /************************************************************/
               irc = WrtQueryLocatorCaps ( &ulLocTmp, &lcInfo );

               if ( irc == WRT_NO_ERROR )
               {
                 /**********************************************************/
                 /*  good return from API                                  */
                 /**********************************************************/
                 sprintf ( cbuffer, "%s %s",
                           lcInfo.pszDriverName, lcInfo.pszDeviceName );
                 index1 = (SHORT) WinSendMsg ( hwndCombo1, LM_INSERTITEM,
                                   MPFROMSHORT ( LIT_END ),
                                   MPFROMP ( cbuffer ) );
               } else
               {
                 /**********************************************************/
                 /* ERROR from API                                         */
                 /**********************************************************/
                 DisplayError ( hwnd, irc );
               } /* endif */
             } /* endfor */
             /**************************************************************/
             /* Set the last  item on the list at the top.                 */
             /**************************************************************/
             WinSendMsg ( hwndCombo1, LM_SETTOPINDEX,
                          MPFROMSHORT ( index1 ), 0 );

             comboprams.fsStatus = WPM_TEXT;               /* change window text */
             comboprams.cchText = sizeof ( cbuffer );      /* text size          */
             comboprams.pszText = cbuffer ;                /* text to display    */
             WinSendMsg ( hwndCombo1, WM_SETWINDOWPARAMS, &comboprams, 0 );
             disp_loccaps ( hwnd, &lcInfo );
           } /* endif */ /* end of display locator information */
         } /* endif */
         return(FALSE);
      }  /* of case WM_INITDLG */

      case WM_CONTROL:
        switch ( SHORT2FROMMP (mp1) )
        {
           case CBN_ENTER:
             /*****************************************************************/
             /* find the index of the selected item                           */
             /*****************************************************************/
             index1 = (SHORT)WinSendMsg ( hwndCombo1, LM_QUERYSELECTION, 0L, 0L );
             if ( index1 != LIT_NONE )
             {
               ulLocTmp = (ULONG)(index1 + 1L);
               lcInfo.cbStructSize = sizeof ( lcInfo );
               irc = WrtQueryLocatorCaps ( &ulLocTmp, &lcInfo );
               if ( irc == WRT_NO_ERROR )
               {
                 disp_loccaps ( hwnd, &lcInfo );
               } /* endif */
             } /* endif */
             break;

           default:
              return(WinDefDlgProc(hwnd, msg, mp1, mp2));
        } /* endswitch */
        return ( FALSE );

      case WM_COMMAND:
        WinDismissDlg ( hwnd, FALSE );
        return ( FALSE );

      default:
        return(WinDefDlgProc(hwnd, msg, mp1, mp2));
   }
}  /* of LocCapDlgProc */

/*******************************************************************************
*                                                                              *
* Function Name: SysCapDlgProc                                                 *
*                                                                              *
* Description :                                                                *
*                                                                              *
*   Displays the capabilities of the system.                                   *
*                                                                              *
* Input:                                                                       *
*                                                                              *
* Output: Display the capabilities of the system                               *
*                                                                              *
*******************************************************************************/
MRESULT EXPENTRY SysCapDlgProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
PWRTSYSTEMINFO pscInfo;
WRTSYSINFO     wrtInfo;
CHAR           cbuffer[MAXCBUFFER];
INT            i, j;

   switch(msg)
   {
      case WM_INITDLG:
        pscInfo = PVOIDFROMMP ( mp2 );
        wrtInfo = * (WRTSYSINFO *) pscInfo;
        j = wrtelc01;
        for (i = 0; i < 11; ++i)
        {
           sprintf ( cbuffer, "%ld", wrtInfo.ulWrtSysInfo[i] );
           WinSetDlgItemText ( hwnd, j++, cbuffer );
        }
        return(FALSE);

     case WM_COMMAND:
        WinDismissDlg ( hwnd, FALSE );
        return ( FALSE );

     default:
        return(WinDefDlgProc(hwnd, msg, mp1, mp2));
   }
}  /* of SysCapDlgProc */

/******************************************************************************/
/*  Error reporting functions                                                 */
/******************************************************************************/
VOID DisplayError ( HWND hwnd, APIRET errcode )
{
CHAR errbuff[MAXCBUFFER];
CHAR loadbuff[MAXCBUFFER];

   if ( errcode != WRT_NO_ERROR )
   {
      switch ( errcode )
      {
         case WRTERR_INVALID_BUTTON_ID:
         case WRTERR_INVALID_POINTER:
         case WRTERR_INVALID_BUFFER:
         case WRTERR_STRUCT_NOT_INIT:
         case WRTERR_NOT_ACTIVE:
         case WRTERR_INVALID_LOCATOR_ID:
         case WRTERR_INVALID_DISPLAY_ID:
         case WRTERR_INVALID_HWND:
         case WRTERR_WRITE_FAILED:
         case WRTERR_INVALID_PARM:
         case WRTERR_INVALID_ID:
         case WRTERR_PARM_OUT_OF_RANGE:
         case WRTERR_MORE_NAMES_AVAIL:
         case WRTERR_STROKE_INVALID:
            WinLoadString ( habMain,
                            NULLHANDLE,
                            errcode,
                            MAXCBUFFER,
                            errbuff );
            break;

         default:
            WinLoadString ( habMain,
                            NULLHANDLE,
                            IDMSG_UNKOWN,
                            MAXCBUFFER,
                            loadbuff );
            sprintf ( errbuff, loadbuff, errcode );
            break;
      }  /* end of switch */

      WinMessageBox( HWND_DESKTOP,
                     hwnd,
                     errbuff,
                     NULL, 1, MB_OK );
   } /* endif */
}  /* of DisplayError */

VOID init_sysval( HWND hwnd, INT r_sw )
{
CHAR       cbuffer[MAXCBUFFER];
WNDPARAMS  comboprams;
INT        i;

   /**********************************************************************/
   /* initialize the entries in the list box                             */
   /**********************************************************************/
   hwndCombo1 = WinWindowFromID ( hwnd , wrtelc01 );

   for ( i = 1; i < (QSVALUESIZE - 2); i++ )
   {
      WinSendDlgItemMsg ( hwnd,
                          wrtelc01,
                          LM_INSERTITEM,
                          MPFROMSHORT ( LIT_END ),
                          MPFROMP ( qsvalue[i].valname ) );
   }
   /**********************************************************************/
   /* Set the first item on the list at the top.                         */
   /**********************************************************************/
   index = 0;                            /* intialize the index to zeror */
   WinSendDlgItemMsg ( hwnd,
                       wrtelc01,
                       LM_SETTOPINDEX,
                       MPFROMSHORT ( index ),
                       0 );

   comboprams.fsStatus = WPM_TEXT;      /* change window text            */
   comboprams.cchText = sizeof ( qsvalue[index+1].valname );/* text size*/
   comboprams.pszText = qsvalue[index+1].valname;  /* text to display   */
   WinSendDlgItemMsg ( hwnd,
                       wrtelc01,
                       WM_SETWINDOWPARAMS,
                       &comboprams,
                       0 );
   /**********************************************************************/
   /* query the system value to set the spin box.                        */
   /**********************************************************************/
   startvalue = WrtQuerySysValue ( HWND_DESKTOP,
                                   qsvalue[index + 1].valueid );

   /**********************************************************************/
   /* initialize all the spin button to the system value that where      */
   /* query. If the query reutrn an error (WRTERR_INVALID_PARM) SET      */
   /* the spin button to zerro.                                          */
   /**********************************************************************/
   if ( startvalue == WRTERR_INVALID_PARM )
   {
      startvalue = 0L;
   } /* endif */
   if ( r_sw )
   {            /* r-sw == 1 is a read */
      /**********************************************************************/
      /* Set the entry field value to the value that was query from         */
      /* the system.                                                        */
      /**********************************************************************/
      sprintf ( cbuffer, "%li", startvalue);
      WinSetDlgItemText ( hwnd, wrtelc02, cbuffer );
   } else       /* r_sw == 0is a set or write */
   {
      /************************************************************/
      /* check to see if the value can be set to 0 (disabled)     */
      /************************************************************/
      WinSendDlgItemMsg ( hwnd,
                          wrtelc02,
                          SPBM_SETLIMITS,
                          MPFROMLONG ( qsvalue[index+1].valuemax ),
                          MPFROMLONG ( (qsvalue[index + 1].disable == 1 ) ?
                                       0L : qsvalue[index+1].valuemin ) );
      /**********************************************************************/
      /* Set the spin button value to the value that was query from         */
      /* the system.                                                        */
      /**********************************************************************/
      WinSendDlgItemMsg ( hwnd,
                          wrtelc02,
                          SPBM_SETCURRENTVALUE,
                          MPFROMLONG ( startvalue ),
                          0L );
   } /* endif */
}  /* of init_sysval */

VOID enter_sysval( HWND hwnd, INT r_sw )
{
  CHAR cbuffer[MAXCBUFFER];

   /******************************************************************/
   /* find the index of the selected item                            */
   /******************************************************************/
   index = (SHORT) WinSendMsg ( hwndCombo1, LM_QUERYSELECTION, 0L, 0L );
   if ( index != LIT_NONE )
   {
     /****************************************************************/
     /* get the sytem value to change                                */
     /****************************************************************/
     startvalue = WrtQuerySysValue ( HWND_DESKTOP,
                                qsvalue[index+1].valueid );

     /****************************************************************/
     /* initialize all the spin button to the system value that where*/
     /* query. If the query reutrn an error (WRTERR_INVALID_PARM) SET*/
     /* the spin button to zerro.                                    */
     /****************************************************************/
     if ( startvalue == WRTERR_INVALID_PARM )
     {
       startvalue = 0L;
     } /* endif */

     if ( r_sw )
     {             /* if r_sw == 1 is read */
       /****************************************************************/
       /* Set the entry field value to the value that was query from   */
       /* the system.                                                  */
       /****************************************************************/
       sprintf ( cbuffer, "%li", startvalue);
       WinSetDlgItemText ( hwnd, wrtelc02, cbuffer );
     } else
     {             /* if r_sw == 0 is set or write */
       /************************************************************/
       /* check to see if the value can be set to 0 (disabled)     */
       /************************************************************/
       if ( qsvalue[index+1].disable == 1 )
       {
         WinSendMsg ( WinWindowFromID ( hwnd, wrtelc02 ),
                      SPBM_SETLIMITS,
                      MPFROMLONG ( qsvalue[index+1].valuemax ),/* MAX    */
                      MPFROMLONG ( 0L ) );
       } else
       {
         WinSendMsg ( WinWindowFromID ( hwnd, wrtelc02 ),
                      SPBM_SETLIMITS,
                      MPFROMLONG ( qsvalue[index+1].valuemax ),/* MAX    */
                      MPFROMLONG ( qsvalue[index+1].valuemin ) );
       } /* endif */
       /****************************************************************/
       /* Set the spin button value to the value that was query from   */
       /* the system.                                                  */
       /****************************************************************/
       WinSendMsg ( WinWindowFromID ( hwnd, wrtelc02 ),
                    SPBM_SETCURRENTVALUE,
                    MPFROMLONG ( startvalue ),
                    0L );
       /****************************************************************/
       /* set the spin button's maximum and minimum values for the     */
       /* users to choose from.                                        */
       /****************************************************************/
       WinSendMsg ( WinWindowFromID ( hwnd, wrtelc02 ),
                    SPBM_OVERRIDESETLIMITS,
                    MPFROMLONG ( qsvalue[index+1].valuemax ),/* MAX    */
                    MPFROMLONG ( qsvalue[index+1].valuemin ) );

     } /* endif */
   } /* endif */
} /* of enter_sysval */

VOID updwn_sysval ( HWND hwnd )
{
   if ( qsvalue[index+1].disable == 1 )
   {
      /*********************************************************/
      /* Query the value of the spin button and put it in      */
      /* ppmid[].value                                         */
      /*********************************************************/
      WinSendMsg ( WinWindowFromID ( hwnd, wrtelc02 ),
                   SPBM_QUERYVALUE,
                   MPFROMLONG ( &startvalue ),
                   MPFROM2SHORT ( NULL, SPBQ_DONOTUPDATE ) );
      if ( startvalue == qsvalue[index+1].valuemin ||
           startvalue == qsvalue[index+1].valuemax  )
      {
        WinSendMsg ( WinWindowFromID ( hwnd, wrtelc02 ),
                     SPBM_OVERRIDESETLIMITS,
                     MPFROMLONG ( qsvalue[index+1].valuemax ),/* MAX*/
                     MPFROMLONG ( 0L ) );
      } else
      {
        if ( startvalue < qsvalue[index+1].valuemin )
        {
          WinSendMsg ( WinWindowFromID ( hwnd, wrtelc02 ),
                       SPBM_SETCURRENTVALUE,
                       MPFROMLONG ( 0L ),
                       0L );
          WinSendMsg ( WinWindowFromID ( hwnd, wrtelc02 ),
                       SPBM_OVERRIDESETLIMITS,
                       MPFROMLONG ( qsvalue[index+1].valuemax ),/* MAX*/
                       MPFROMLONG ( qsvalue[index+1].valuemin ) );
        } /* endif */
      } /* endif */
   } /* endif */
} /* of updwn_sysval */

VOID set_sysval ( HWND hwnd )
{
LONG   value;
APIRET irc;

   /***************************************************************/
   /* Query the value of the spin button and if the value got     */
   /* change set the new system value.                            */
   /***************************************************************/
   WinSendMsg ( WinWindowFromID ( hwnd, wrtelc02 ),
                SPBM_QUERYVALUE,
                MPFROMLONG ( &value ),
                MPFROM2SHORT ( NULL, SPBQ_DONOTUPDATE ) );
   if ( startvalue != value )
   {
      /************************************************************/
      /* value got change must set the value                      */
      /************************************************************/
      irc = WrtSetSysValue ( HWND_DESKTOP,
                             qsvalue[index+1].valueid, value );
      if ( irc == WRT_NO_ERROR )
      {
        /**********************************************************/
        /* Broadcast a post message to all descendants of         */
        /* HWND_DESKTOP notifying which system value got changed  */
        /* so they can update if need be                          */
        /**********************************************************/
        WinBroadcastMsg ( HWND_DESKTOP,
                          WM_WRT_SYSVALUECHANGED,
                          (MPARAM)VT_PPMSV,
                          (MPARAM)qsvalue[index+1].valueid,
                          BMSG_DESCENDANTS | BMSG_POST );
      } else
      {
         DisplayError ( hwnd, irc );
      } /* endif */
   } /* endif */
}  /* of set_sysval */

VOID disp_loccaps ( HWND hwnd, PWRTLOCATORDEVINFO lcInfo )
{
CHAR            cbuffer[MAXCBUFFER];
WRTLOCDEVINFO   wrtLocDevInfo;
INT             i, j;

   wrtLocDevInfo = *(WRTLOCDEVINFO *) lcInfo;
   j = wrtelc01;
   for (i = 0; i < 22; ++i)
   {
      if (i == 10)
      {
         continue;
      }
      sprintf ( cbuffer, "%ld", wrtLocDevInfo.ulWrtLocDevInfo[i] );
      WinSetDlgItemText ( hwnd, j++, cbuffer );
   }
   sprintf ( cbuffer, "%ld", wrtLocDevInfo.ulWrtLocDevInfo[10] );
   WinSetDlgItemText ( hwnd, wrtelc22, cbuffer );

}  /* of disp_loccaps */

VOID disp_butcaps ( HWND hwnd, PWRTBUTTONDEVINFO bcInfo )
{
  CHAR cbuffer[MAXCBUFFER];

   sprintf ( cbuffer, "%ld", bcInfo->cbStructSize );
   WinSetDlgItemText ( hwnd, wrtelc01, cbuffer );

   sprintf ( cbuffer, "%ld", bcInfo->ulButtonID );
   WinSetDlgItemText ( hwnd, wrtelc02, cbuffer );

   sprintf ( cbuffer, "%ld", bcInfo->ulNumButtons );
   WinSetDlgItemText ( hwnd, wrtelc03, cbuffer );

   sprintf ( cbuffer, "%#lx", bcInfo->ulButtonMask );
   WinSetDlgItemText ( hwnd, wrtelc04, cbuffer );
}  /* of disp_butcaps */

VOID init_invar( HWND hwnd, BOOL fEarly )
{
  APIRET    irc;
  ULONG     aDeviceType[10];
  ULONG     aDeviceID[10];
  ULONG     ulDatalen;
  CHAR      Data[512];
  WNDPARAMS comboprams;
  INT       i;
  INT       cbox;
  CHAR      cbuffer[MAXCBUFFER];

  /*****************************************************/
  /* Find out how many input drivers there are         */
  /*****************************************************/
  ulNumDrivers = 0L;
  irc = WrtEnumInputDrivers ( &ulNumDrivers, NULL );
  if ( irc != WRT_NO_ERROR )
  {
    DisplayError ( hwnd, irc );
  } else
  {
     if ( ulNumDrivers > 10L )
     {
       sprintf(cbuffer,"Too many drivers[%ld] for my ARRAY", ulNumDrivers );
       WinMessageBox( HWND_DESKTOP,
                      hwnd,
                      cbuffer,
                      NULL, 1, MB_OK );
     } else
     {
        if ( ulNumDrivers == 0L )
        {
          sprintf(cbuffer, "No drivers found" );
          WinMessageBox( HWND_DESKTOP,
                         hwnd,
                         cbuffer,
                         NULL, 1, MB_OK );
        } else
        {
           /****************************************************************/
           /* get array of driver names                                    */
           /****************************************************************/
           irc = WrtEnumInputDrivers ( &ulNumDrivers,
                                       szDriverName );
           if ( irc != WRT_NO_ERROR )
           {
               DisplayError ( hwnd, irc );
           } else
           {
              /**************************************************************/
              /* initialize the entries in the Drivers list box             */
              /**************************************************************/
              for ( i = 0; i < ulNumDrivers ; i++ )
              {
                 WinSendDlgItemMsg ( hwnd,
                                     wrtelc01,
                                     LM_INSERTITEM,
                                     MPFROMSHORT ( LIT_END ),
                                     MPFROMP ( szDriverName[i] ) );
              } /* endfor */
              /**************************************************************/
              /* Set the first item on the list at the top.                 */
              /**************************************************************/
              WinSendDlgItemMsg ( hwnd,
                                  wrtelc01,
                                  LM_SETTOPINDEX,
                                  MPFROMSHORT ( 0 ),
                                  0 );

              comboprams.fsStatus = WPM_TEXT;          /* change window text */
              comboprams.cchText = sizeof ( szDriverName[0] );  /* text size */
              comboprams.pszText = szDriverName[0];        /*text to display */
              WinSendDlgItemMsg ( hwnd,
                                  wrtelc01,
                                  WM_SETWINDOWPARAMS,
                                  &comboprams,
                                  0 );

              /*************************************************************/
              /* Find out how many input device there are                  */
              /*************************************************************/
              irc = GetInputDevices(hwnd, szDriverName[0]);

              if (fEarly)
              {
                 return;
              }

              /**************************************************************/
              /* initialize the value ID valid for this driver and device   */
              /**************************************************************/
              cbox = 0;
              for ( i = 0; i < (PPMIDSIZE - 1); i++ )
              {
                irc = WrtQueryInputDeviceVariable ( szDriverName[0],
                                                    aDeviceName[0],
                                                    ppmid[i].ValueID,
                                                    &ppmid[i].value,
                                                    NULL,
                                                    NULL );
                if ( irc == WRT_NO_ERROR )
                {
                   /**********************************************************/
                   /* Mark the input device variable as suported.            */
                   /**********************************************************/
                   ppmid[i].active = 1;
                   cbox++;

                   /**********************************************************/
                   /*Set the input device variable in the list to choose from*/
                   /**********************************************************/
                    WinSendDlgItemMsg ( hwnd,
                                        wrtelc03,
                                        LM_INSERTITEM,
                                        MPFROMSHORT ( LIT_END ),
                                        MPFROMP ( ppmid[i].name ) );
                } else
                {
                   /**********************************************************/
                   /* Mark the input device variable as not active.          */
                   /**********************************************************/
                   ppmid[i].active = 0;
                } /* endif */
              } /* endfor */
              if ( cbox == 0 )
              {
                 WinEnableWindow ( (WinWindowFromID ( hwnd , wrtlc03 )), FALSE );
                 WinEnableWindow ( (WinWindowFromID ( hwnd , wrtelc03 )), FALSE );
                 WinEnableWindow ( (WinWindowFromID ( hwnd , wrtlc04 )), FALSE );
                 WinEnableWindow ( (WinWindowFromID ( hwnd , wrtelc04 )), FALSE );
                 sprintf(cbuffer,"No active variables Drv[%s] Name[%s]",
                                  szDriverName[0], aDeviceName[0] );
                 WinMessageBox( HWND_DESKTOP,
                                hwnd,
                                cbuffer,
                                NULL, 1, MB_OK );
              } else
              {
                 /************************************************************/
                 /* Find the first active input device variable to gets      */
                 /* its name                                                 */
                 /************************************************************/
                 for ( i = 0; i < (PPMIDSIZE - 1); i++)
                 {
                    if ( ppmid[i].active )
                    {
                       break;
                    }
                 } /* endfor */

                 /************************************************************/
                 /* Set the first item on the list at the top.               */
                 /************************************************************/
                 WinSendDlgItemMsg ( hwnd,
                                     wrtelc03,
                                     LM_SETTOPINDEX,
                                     MPFROMSHORT ( 0 ),
                                     0 );

                 comboprams.fsStatus = WPM_TEXT;       /* change window text */
                 comboprams.cchText = sizeof ( ppmid[i].name ); /* text size */
                 comboprams.pszText = ppmid[i].name;      /* text to display */
                 WinSendDlgItemMsg ( hwnd,
                                     wrtelc03,
                                     WM_SETWINDOWPARAMS,
                                     &comboprams,
                                     0 );

                 sprintf ( cbuffer, "%li", ppmid[i].value );
                 WinSendDlgItemMsg ( hwnd,
                                     wrtelc04,
                                     EM_SETTEXTLIMIT,
                                     MPFROM2SHORT(4, 0),
                                     0 );
                 WinSetDlgItemText(hwnd,
                                   wrtelc04,
                                   cbuffer );

              } /* endif */

              /**************************************************************/
              /* Find out if the "Driver" is supported.                     */
              /* If it is set "Driver as a choice in combo 2                */
              /**************************************************************/
              ulDatalen = 512;
              irc = WrtQueryInputDeviceVariable( szDriverName[0],
                                                 "Driver",
                                                 ppmid[PPMIDSIZE - 1].ValueID,
                                                 &ppmid[PPMIDSIZE - 1].value,
                                                 &ulDatalen,
                                                 Data );
              if ( irc == WRT_NO_ERROR )
              {
                 ppmid[PPMIDSIZE - 1].active = 1;
                 WinSendDlgItemMsg ( hwnd,
                                     wrtelc02,
                                     LM_INSERTITEM,
                                     MPFROMSHORT ( LIT_END ),
                                     MPFROMP ( "Driver" ) );
              } else
              {
                 ppmid[PPMIDSIZE - 1].active = 0;
              }  /* endif */
           } /* endif */
        } /* endif */
     } /* endif */
  } /* endif */
}  /* of init_invar */

VOID updatecntrl ( HWND hwnd, HWND hwnd_temp )
{
APIRET       irc;
CHAR         szDrvName[32];
ULONG        aDeviceType[10];
ULONG        aDeviceID[10];
ULONG        ulDatalen;
CHAR         Data[512];
WNDPARAMS    comboprams;
INT          i;
INT          cbox;
CHAR         cbuffer[MAXCBUFFER];
CHAR         achDriverName[32];
static SHORT index1 = 0, index2 = 0;

   hwndCombo1 = WinWindowFromID(hwnd, wrtelc01);
   hwndCombo2 = WinWindowFromID(hwnd, wrtelc02);
   hwndCombo3 = WinWindowFromID(hwnd, wrtelc03);

   WinEnableWindow( WinWindowFromID( hwnd, wrtlc04 ), TRUE );
   WinEnableWindow( WinWindowFromID( hwnd, wrtlc03 ), TRUE );
   WinEnableWindow( WinWindowFromID( hwnd, wrtelc03), TRUE );
   WinEnableWindow( WinWindowFromID( hwnd, wrtelc04), TRUE );

   if ( hwnd_temp == hwndCombo1 )
   {
     /*******************************************************************/
     /* Get the Driver selected and find:                               */
     /* 1: devices supported by the driver                              */
     /* 2: ValueID supported by the 1st device                          */
     /* 3: Value of the 1st ValueID                                     */
     /*******************************************************************/
     /*******************************************************************/
     /* find the index of the selected item                             */
     /*******************************************************************/
     index1 = (SHORT) WinSendMsg ( hwndCombo1,
                                   LM_QUERYSELECTION,
                                   (MPARAM) 0L,
                                   (MPARAM) 0L );
     if ( index1 != LIT_NONE )
     {
       /*****************************************************************/
       /* get the name of the driver choosen.                           */
       /*****************************************************************/
       WinSendMsg ( hwndCombo1,
                    LM_QUERYITEMTEXT,
                    MPFROM2SHORT ( index1, 32 ),
                    MPFROMP ( achDriverName ) );

       /*****************************************************************/
       /* Clear out the Devices list box and get the devices for the    */
       /* driver.                                                       */
       /*****************************************************************/
       WinSendMsg ( hwndCombo2,
                    EM_CLEAR,
                    (MPARAM) 0L,
                    (MPARAM) 0L );
       WinSendMsg ( hwndCombo2,
                    LM_DELETEALL,
                    (MPARAM) 0L,
                    (MPARAM) 0L );
       WinSendMsg ( hwndCombo3,
                    EM_CLEAR,
                    (MPARAM) 0L,
                    (MPARAM) 0L );
       WinSendMsg ( hwndCombo3,
                    LM_DELETEALL,
                    (MPARAM) 0L,
                    (MPARAM) 0L );
       WinSendMsg ( hwndCombo3,
                    LM_SETTOPINDEX,
                    MPFROMSHORT ( 0 ),
                    0 );

       comboprams.fsStatus = WPM_TEXT;            /* change window text */
       comboprams.cchText = sizeof ( "     " );   /*text size*/
       comboprams.pszText = "     ";              /* text to display */
       WinSendMsg ( hwndCombo3,
                    WM_SETWINDOWPARAMS,
                    &comboprams,
                    0 );

       /*****************************************************************/
       /* Find out how many input device there are                      */
       /*****************************************************************/
       irc = GetInputDevices(hwnd, achDriverName);

       if (irc == WRT_NO_ERROR)
       {
          /*************************************************************/
          /* initialize the entries in the Value ID list box           */
          /*************************************************************/
          hwndCombo3 = WinWindowFromID ( hwnd , wrtelc03 );

          update3 ( hwnd, index1, 0 );
          /***********************************************************/
          /* Find out if the "Driver" is supported.                  */
          /* If it is set "Driver as a choice in combo 2             */
          /***********************************************************/
          ulDatalen = 512;
          irc = WrtQueryInputDeviceVariable ( szDriverName[index1],
                                              "Driver",
                                              ppmid[ PPMIDSIZE - 1 ].ValueID,
                                              &ppmid[ PPMIDSIZE - 1 ].value,
                                              &ulDatalen,
                                              Data );
          if ( irc == WRT_NO_ERROR )
          {
            ppmid[ PPMIDSIZE - 1 ].active = 1;
            WinSendMsg ( hwndCombo2, LM_INSERTITEM,
                         MPFROMSHORT ( LIT_END ),
                         MPFROMP ( "Driver" ) );
          } else
          {
            ppmid[ PPMIDSIZE - 1 ].active = 0;
          }  /* endif */
          /* endif */
       }  /* of if (irc == WRT_NO_RETURN) */
     } /* endif */
   } else
   {
     if ( hwnd_temp == hwndCombo2 )
     {
       /*****************************************************************/
       /* Get the Device selected and find:                             */
       /* 1: ValueID supported by the 1st device                        */
       /* 2: Value of the 1st ValueID                                   */
       /*******************************************************************/
       /*****************************************************************/
       /* get the name of the driver choosen.                           */
       /*****************************************************************/

       comboprams.fsStatus = WPM_TEXT;            /* get window text    */
       comboprams.cchText = sizeof ( szDrvName ); /* text size          */
       comboprams.pszText = szDrvName;            /* buffer to put text */
       WinSendMsg ( hwndCombo1,
                    WM_QUERYWINDOWPARAMS,
                    &comboprams,
                    0 );
       /*****************************************************************/
       /* find the index of the selected item                           */
       /*****************************************************************/
       index2 = (SHORT) WinSendMsg( hwndCombo2,
                                    LM_QUERYSELECTION,
                                    0L,
                                    0L );
       if ( index2 != LIT_NONE )
       {
         /***************************************************************/
         /* get the name of the device choosen.                         */
         /***************************************************************/
         WinSendMsg ( hwndCombo2,
                      LM_QUERYITEMTEXT,
                      MPFROM2SHORT ( index2, 32 ),
                      MPFROMP ( aDeviceName[index2] ) );

         /***************************************************************/
         /* Clear out the ValueID list box and get the devices for the  */
         /* driver.                                                     */
         /***************************************************************/
         WinSendMsg ( hwndCombo3,
                      EM_CLEAR,
                      0L,
                      0L );
         WinSendMsg ( hwndCombo3,
                      LM_DELETEALL,
                      0L,
                      0L );
         index3 = 0;                     /* intialize the index to zero */
         WinSendMsg ( hwndCombo3,
                      LM_SETTOPINDEX,
                      MPFROMSHORT ( index3 ),
                      0 );

         comboprams.fsStatus = WPM_TEXT;          /* change window text */
         comboprams.cchText = sizeof ( "       " );/*text size          */
         comboprams.pszText = "       ";           /* text to display   */
         WinSendMsg ( hwndCombo3,
                      WM_SETWINDOWPARAMS,
                      &comboprams,
                      0 );
         /***************************************************************/
         /* if the device name is "Driver" then only do driver          */
         /* else do all variables                                       */
         /***************************************************************/
         if ( strcmp ( "Driver", aDeviceName[index2] )  == 0 )
         {
           ulDatalen = 512;
           for ( i=0; i < ulDatalen ; i++)
           {
             Data[i] = 0;                         /* clear out old data */
           } /* endfor */
           irc = WrtQueryInputDeviceVariable ( szDrvName,
                                                "Driver",
                                                ppmid[ PPMIDSIZE - 1 ].ValueID,
                                                &ppmid[ PPMIDSIZE - 1 ].value,
                                                &ulDatalen,
                                                Data
                                               );
           if ( irc == WRT_NO_ERROR )
           {
             sprintf ( cbuffer, "Data length is %i", ulDatalen );
             WinSetDlgItemText(hwnd,
                               wrtelc04,
                               cbuffer );
             if ( ulDatalen == 0L)
             {
               WinEnableWindow ( (WinWindowFromID ( hwnd , wrtlc03 )),
                                     FALSE );
               WinEnableWindow ( (WinWindowFromID ( hwnd , wrtelc03 )),
                                     FALSE );
               WinEnableWindow ( (WinWindowFromID ( hwnd , wrtlc04 )),
                                     FALSE );
             } /* endif */
           } else
           {
             sprintf(cbuffer,"WrtQueryInputDeviceVariable with Driver Failed");
             WinMessageBox( HWND_DESKTOP,
                            hwnd,
                            cbuffer,
                            NULL, 1, MB_OK );
           } /* endif */
         } else
         {
           update3 ( hwnd, index1, index2 );
         } /* endif */
       } /* endif */
     } else
     {
       if ( hwnd_temp == hwndCombo3 )
       {
         /***************************************************************/
         /* Get the valueID selected and find:                          */
         /* 2: Value of the ValueID                                     */
         /***************************************************************/
         /***************************************************************/
         /* find the index of the selected item                         */
         /***************************************************************/
         index3 = (SHORT) WinSendDlgItemMsg( hwnd,
                                             wrtelc03,
                                             LM_QUERYSELECTION,
                                             0L,
                                             0L);
         if ( index3 != LIT_NONE )
         {
           /*************************************************************/
           /*Find the ID coresponding to the index                      */
           /*************************************************************/
           cbox = -1;
           for ( i = 0; i < (PPMIDSIZE - 1); i++)
           {
             if ( ppmid[i].active )
             {
                if (++cbox == index3)
                   break;
             } /* endif */
           } /* endfor */
           sprintf ( cbuffer, "%li", ppmid[i].value );
           WinSetDlgItemText(hwnd,
                             wrtelc04,
                             cbuffer );
         } /* endif */
       } else
       {
          sprintf(cbuffer,"No MATCH mp2[%ld] 1[%ld]2[%ld]3[%ld]",
                  hwnd_temp, hwndCombo1, hwndCombo2, hwndCombo3 );
          WinMessageBox( HWND_DESKTOP,
                         hwnd,
                         cbuffer,
                         NULL, 1, MB_OK );
       } /* endif */
     } /* endif */
   } /* endif */
}  /* of updatecntrl */

VOID update3 (HWND hwnd, SHORT index1, SHORT index2 )
{
  APIRET    irc;
  WNDPARAMS comboprams;
  INT       i;
  INT       cbox;
  CHAR      cbuffer[MAXCBUFFER];

  /*************************************************************/
  /* initialize the value ID valid for this driver and device  */
  /*************************************************************/
    cbox = 0;
    for ( i= 0; i < (PPMIDSIZE - 1) ; i++ )
    {
      irc = WrtQueryInputDeviceVariable ( szDriverName[index1],
                                          aDeviceName[index2],
                                          ppmid[i].ValueID,
                                          &ppmid[i].value,
                                          NULL,
                                          NULL );
      if ( irc == WRT_NO_ERROR )
      {
        /*******************************************************/
        /* Mark the input device variable as suported.         */
        /*******************************************************/
        ppmid[i].active = 1;
        cbox++;

        /*******************************************************/
        /* Set the input device variable in the list to choose */
        /* from                                                */
        /*******************************************************/
        index3 = (SHORT) WinSendDlgItemMsg ( hwnd,
                                             wrtelc03,
                                             LM_INSERTITEM,
                                             MPFROMSHORT ( LIT_END ),
                                             MPFROMP ( ppmid[i].name ) );

      } else
      {
        /*******************************************************/
        /* Mark the input device variable as not active.       */
        /*******************************************************/
        ppmid[i].active = 0;
      } /* endif */
    } /* endfor */
    if ( cbox == 0)
    {
      WinEnableWindow ( (WinWindowFromID ( hwnd , wrtlc03 )), FALSE );
      WinEnableWindow ( (WinWindowFromID ( hwnd , wrtelc03 )), FALSE );
      WinEnableWindow ( (WinWindowFromID ( hwnd , wrtlc04 )), FALSE );
      sprintf(cbuffer,"No active variables Drv[%s] Name[%s]",
      szDriverName[index1], aDeviceName[index2] );
      WinMessageBox( HWND_DESKTOP,
                     hwnd,
                     cbuffer,
                     NULL, 1, MB_OK );
    } else
    {
      /*********************************************************/
      /* Find the first active input device variable to gets   */
      /* its name                                              */
      /*********************************************************/
      for ( i= 0; i < (PPMIDSIZE - 1); i++)
      {
        if ( ppmid[i].active )
        {
           break;
        } /* endif */
      } /* endfor */
      /*********************************************************/
      /* Set the first item on the list at the top.            */
      /*********************************************************/
      index3 = 0;                /* intialize the index to zero */
      WinSendDlgItemMsg ( hwnd,
                          wrtelc03,
                          LM_SETTOPINDEX,
                          MPFROMSHORT ( index3 ),
                          0 );

      comboprams.fsStatus = WPM_TEXT;    /* change window text */
      comboprams.cchText = sizeof ( ppmid[i].name );/*text size*/
      comboprams.pszText = ppmid[i].name;   /* text to display */
      WinSendDlgItemMsg ( hwnd,
                          wrtelc03,
                          WM_SETWINDOWPARAMS,
                          &comboprams,
                          0 );

      sprintf ( cbuffer, "%li", ppmid[i].value );
      WinSetDlgItemText(hwnd,
                        wrtelc04,
                        cbuffer );
    } /* endif */
}   /* of update3 */

APIRET GetInputDevices(HWND hwnd, CHAR achDriverName[32])
{
APIRET    irc;
CHAR      cbuffer[MAXCBUFFER];
ULONG     aDeviceType[10];
ULONG     aDeviceID[10];
WNDPARAMS comboprams;
INT       i;

   ulNumDevices = 0L;
   irc = WrtQueryInputDeviceNames ( achDriverName,
                                    &ulNumDevices,
                                    NULL, NULL, NULL );
   if ( irc != WRT_NO_ERROR )
   {
      DisplayError( hwnd, irc );
   }
   else
   {
      if ( ulNumDevices > 10L )
      {
         sprintf(cbuffer,"Too many devices[%ld] Driver[%s] for my ARRAY",
                 ulNumDevices, achDriverName );
         WinMessageBox( HWND_DESKTOP,
                        hwnd,
                        cbuffer,
                        NULL, 1, MB_OK );
      } else
      {
         /*************************************************************/
         /* Get array of device names                                 */
         /*************************************************************/
         irc = WrtQueryInputDeviceNames ( achDriverName,
                                          &ulNumDevices,
                                          aDeviceName,
                                          aDeviceType,
                                          aDeviceID );
         if ( irc != WRT_NO_ERROR )
         {
            DisplayError( hwnd, irc );
         }
         else
         {
            hwndCombo2 = WinWindowFromID(hwnd, wrtelc02);

            /***********************************************************/
            /* initialize the entries in the list box                  */
            /***********************************************************/
            for ( i = 0; i < ulNumDevices ; i++ )
            {
               WinSendDlgItemMsg ( hwnd,
                                   wrtelc02,
                                   LM_INSERTITEM,
                                   MPFROMSHORT ( LIT_END ),
                                   MPFROMP ( aDeviceName[i] ) );
            } /* endfor */

            /***********************************************************/
            /* Set the first item on the list at the top.              */
            /***********************************************************/
            WinSendDlgItemMsg ( hwnd,
                                wrtelc02,
                                LM_SETTOPINDEX,
                                MPFROMSHORT ( 0 ),
                                0 );

            comboprams.fsStatus = WPM_TEXT;
            comboprams.cchText = sizeof ( aDeviceName[0] );
            comboprams.pszText = aDeviceName[0];
            WinSendDlgItemMsg ( hwnd,
                                wrtelc02,
                                WM_SETWINDOWPARAMS,
                                &comboprams,
                                0 );

         } /* end of if ( irc != WRT_NO_ERROR ) */
      } /* end of if ( ulNumDevices > 10L )     */
   } /* end of if ( irc != WRT_NO_ERROR )       */
   return(irc);
}  /* of GetInputDevices */
